/****************************************************************************/
/*                                                                          */
/*  COPYRIGHT (c) 1993-1999 BY                                              */
/*  Digital Equiptment/Compaq Computer Corporation.                         */
/*  ALL RIGHTS RESERVED.                                                    */
/*                                                                          */
/*  This file is part of the Iprobe performance suite for Alpha processors. */
/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 2 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License       */
/*  along with this program; if not, write to the Free Software             */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307    */
/*                                                                          */
/****************************************************************************/

/*
 * Abstract:	This file contains the buffer descriptor
 *		kernel routines based on the IPROBE API
 *              kernel rationalization (reorganization).
 *
 * Author:		Bradley M. Waters
 * Creation Date:	18-Jul-1994
 *
 * Modified by:
 *
 * X-4  BMW             Bradley M. Waters       25-Oct-1994
 *      Added clear_bufdsc_address routine and call from BdDelete.
 *
 * X-3	BMW		Bradley M. Waters	10-Oct-1994
 *	Cleanup of code.
 *
 * X-2	BMW		Bradley M. Waters	17-Aug-1994
 *	Updated based on code review input.
 *
 * X-1	BMW		Bradley M. Waters	18-Jul-1994
 *	Original Version.
 *
*/

#ifdef __NT
#  include "ntddk.h"
#endif

#include "ipr_kernel_int_defs.h"

#ifdef __NT
#  include "ipr_driver_nt.h"
#endif

#ifdef   TRCI
#  include "TraceDrv.h"
#endif

#ifdef __LINUX__
#include <linux/kernel.h>
#endif

// This array is initialized in KIprInitSysInfo in OS specific kernel_general
// extern IPR_BUFDSC **pCPUDataArray;

IPR_BUFHDR bdesc_template = {
        IPR_PROD_IDENT,
	0,		// 2nd field (size) is filled in by allocation routine
	IPR_BUFDSC_TYPE,
	IPR_BUFDSC_SUBTYPE,
	IPR_BUFDSC_KIND
	};

IPR_BUFHDR pcrng_template = {
        IPR_PROD_IDENT,
	0,			// 2nd field (size) is filled in by allocation routine
	IPR_PBUFFER_TYPE,
	IPR_PBUFFER_SUBTYPE,
	IPR_PBUFFER_KIND
	};

IPR_BUFHDR skip_template = {
        IPR_PROD_IDENT,
	0,			// 2nd field (size) is filled in by allocation routine
	IPR_IBUFFER_TYPE,
	IPR_IBUFFER_SUBTYPE,
	IPR_IBUFFER_KIND
	};

/*****************************************************************************/
/* get_bufdsc_address returns the address of the buffer descriptor for the   */
/* specified CPU.  Since the location of the pointer to the buffer descriptor*/
/* is OS-specific, it calls an OS-specific routine to find this pointer      */
/* and then dereferences it.                                                 */
/*****************************************************************************/

extern IPR_STATUS get_bufdsc_address (IN IPR_CPUNUM cpu, OUT IPR_BUFDSC **retptr)
{
   IPR_BUFDSC *tretptr;

#ifdef TRC0
    TrcPrint("get_bufdsc_address enter, cpu = %d\n",
       cpu);
#endif
    if (cpu > IPR_MAX_CPUS)	// NOTE: cpu is an unsigned int
       return IPR__INVCPU;
   if (!retptr)
   {
#ifdef TRCE
      TrcPrint("get_bufdsc_address Error: retptr is null\n");
#endif
      return IPR__NULLPTR;
   }

#ifdef TRCB
   TrcPrint("CPUDataArray = %x\n",
       pCPUDataArray [cpu]);
#endif
   if (!pCPUDataArray || !pCPUDataArray[cpu])
   {
      *retptr = (IPR_BUFDSC *) NULL;
      return IPR__BDNOTFND;

   }

   *retptr = pCPUDataArray[cpu];
   tretptr = *retptr;
   if ((tretptr->header.prod_ident != IPR_PROD_IDENT) ||
      (tretptr->header.bufkind != IPR_BUFDSC_KIND))
   {
#ifdef TRCE
      TrcPrint("get_bufdsc_address Error: found buffer prod %Lx kind %x when looking for %Lx %x\n",
         tretptr->header.prod_ident,(*retptr)->header.bufkind,
         IPR_PROD_IDENT,IPR_BUFDSC_KIND);
#endif
      return IPR__INVBUFKIND;
   }
   return IPR__SUCCESS;
}

static IPR_STATUS put_bufdsc_address (IN IPR_CPUNUM cpu, IN IPR_BUFDSC *bufptr)
{
#ifdef TRC0
    TrcPrint("put_bufdsc_address enter, cpu = %d\n",
       cpu);
    TrcPrint("CPUDataArray = %x\n",
       pCPUDataArray [cpu]);
#endif

   if (!bufptr)
   {
#ifdef TRCE
      TrcPrint("put_bufdsc_address Error: bufptr is null\n");
#endif
      return IPR__NULLPTR;
   }
   if ((bufptr->header.prod_ident != IPR_PROD_IDENT) ||
      (bufptr->header.bufkind != IPR_BUFDSC_KIND))
   {
#ifdef TRCE
      TrcPrint("put_bufdsc_address Error: found buffer prod %Lx kind %x when looking for %Lx %x\n",
         bufptr->header.prod_ident,bufptr->header.bufkind,
         IPR_PROD_IDENT,IPR_BUFDSC_KIND);
#endif
      return IPR__INVBUFKIND;
   }
   if (cpu > IPR_MAX_CPUS)	// NOTE: cpu is an unsigned int
      return IPR__INVCPU;

   // Make sure that the buffer descriptor doesn't already exist

   if (pCPUDataArray[cpu])
      return IPR__BDEXISTS;
   pCPUDataArray[cpu] = bufptr;
   return IPR__SUCCESS;
}

static IPR_STATUS clear_bufdsc_address (IN IPR_CPUNUM cpu)
{
#ifdef TRC0
    TrcPrint("clear_bufdsc_address enter, cpu = %d\n",
       cpu);
    TrcPrint("CPUDataArray = %x\n",
       pCPUDataArray [cpu]);
#endif

   if (cpu > IPR_MAX_CPUS)	// NOTE: cpu is an unsigned int
      return IPR__INVCPU;

   // Update proper entry in the buffer descriptor table
   pCPUDataArray[cpu] = NULL;
   return IPR__SUCCESS;

}

extern IPR_STATUS KIprBdCreate (IN IPR_CPUNUM cpu, OUT IPR_BUFDSC **retptr)
{

   IPR_STATUS status;
   IPR_BUFDSC
      *aptr;
   IPR_BUFDSC
      *BDptr;
   int allocSize;

#ifdef TRC0
    TrcPrint("KIprBdCreate enter, cpu = %d\n",
       cpu);
#endif
   if (!retptr)
   {
#ifdef TRCE
      TrcPrint("KIprBdCreate Error: retptr is null\n");
#endif
      return IPR__NULLPTR;
   }
   *retptr = NULL;
   status = get_bufdsc_address (cpu,&aptr);
#ifdef TRCB
   TrcPrint("return from get_bufdsc_address, status= %x, &aptr= %x, aptr= %x \n",
      status,&aptr,aptr);
#endif

   // Check to see if the buffer descriptor exist.  If it exists,
   // return NULL, otherwise create it and return a pointer to it.
//   if ((status == IPR__BDNOTFND) && (!aptr))
   if ((status != IPR__BDNOTFND) || (aptr))
      return IPR__BDEXISTS;   // buffer descriptor exists, return NULL
  //
  // A new buffer descriptor is needed.  Create it and fill it in.
  // We will copy the data from any old buffer descriptor later.
  //
   allocSize = sizeof (IPR_BUFDSC);
   status = allocate_memory (allocSize,&bdesc_template,&BDptr);
   if (!IPR_SUCCESS(status))
   {
#ifdef TRCE
      TrcPrint("KIprBdCreate Error return from allocate_memory, status = %x \n",
         status);
#endif
      return status;
   }
   if (!BDptr)
      return IPR__ALLOCFAIL;
#ifdef TRCB
   TrcPrint("return from allocate_memory, BDptr = %x \n",
      BDptr);
#endif

   all_bd_mem_cnt++;
   BDptr -> pSBuf = NULL;
   BDptr -> pHBuf = NULL;
   BDptr -> pRoutine = NULL;
   BDptr -> pPBuf = NULL;
   BDptr -> pIBuf = NULL;
   BDptr -> sampleaction = 0;
   BDptr -> numcounters = 0;
   BDptr -> fullaction = IPR_FULL_NONE;
   BDptr -> targetpid = 0;
   BDptr -> validcounters = BIprAll1 (BDptr -> validcounters);
   *retptr = BDptr;
   status = put_bufdsc_address (cpu,BDptr);	// Add the buffer descriptor to the list
   return status;
}

extern IPR_STATUS KIprBdDelete (IN IPR_CPUNUM cpu)
{

   IPR_STATUS status;
   IPR_BUFDSC
      *BDptr;

#ifdef TRC0
    TrcPrint("KIprBdDelete enter, cpu = %d\n",
       cpu);
#endif

   status = get_bufdsc_address (cpu,&BDptr);
   if (status == IPR__BDNOTFND)
      return IPR__SUCCESS;
   if (!IPR_SUCCESS(status))
   {
#ifdef TRCE
      TrcPrint("KIprBdDelete Error return from get_bufdsc_address, status %x\n",
         status);
#endif
      return status;
   }
   if (!BDptr)
      return IPR__INVCPU;
   // Don't delete if a sample buffer or histogram buffer exists.
   // Sample and histogram buffers must be deallocated before the
   // buffer descriptor can be released.
//   if ((!BDptr -> pSBuf) && (!BDptr-> pHBuf))
   if ((BDptr -> pSBuf) || (BDptr-> pHBuf))
   {
#ifdef TRCE
     TrcPrint("KIprBdDelete Error buff exists, sample %x, histogram %x\n",
        BDptr -> pSBuf,BDptr-> pHBuf);
#endif
      return IPR__BUFEXISTS;
   }

   // First deallocate subblocks of the buffer descriptor such as the
   // PC range and skip buffers if they exist.

   if (BDptr -> pPBuf)
   {
      //  deallocate PC range buffer
      deallocate_memory (BDptr -> pPBuf);
      deall_pb_mem_cnt++;
   }
   if (BDptr -> pIBuf)
   {
      //  deallocate PC range buffer
      deallocate_memory (BDptr -> pIBuf);
      deall_sk_mem_cnt++;
   }
      // clear the entry of the buffer descriptor table
      status = clear_bufdsc_address(cpu);
      if (!IPR_SUCCESS(status))
      {
#ifdef TRCE
         TrcPrint("KIprBdDelete Error return from clear_bufdsc_addres for cpu %d\n",
            cpu);
#endif
      }
   // Now, deallocate the rest of the buffer descriptor
   deallocate_memory (BDptr);
   deall_bd_mem_cnt++;
   return IPR__SUCCESS;
}

extern IPR_STATUS KIprBdSetNumCounters (IN IPR_CPUNUM cpu, IN IPR_COUNT numctrs)
{

   IPR_STATUS status;
   IPR_BUFDSC
      *BDptr;

#ifdef TRC0
    TrcPrint("KIprBdSetNumCounters enter, cpu = %d, numctrs = %d\n",
       cpu,numctrs);
#endif
   status = get_bufdsc_address (cpu,&BDptr);
   if (!IPR_SUCCESS(status))
   {
#ifdef TRCE
      TrcPrint("KIprBdSetNumCounters Error return from get_bufdsc_address, status %x\n",
         status);
#endif
      return status;
   }
   if (!BDptr)
      return IPR__INVCPU;

   // check to see if sample buffer, histogram buffers, or skip count exists
//   if ((!BDptr -> pSBuf) && (!BDptr -> pHBuf) && (!BDptr -> pIBuf))
   if ((BDptr -> pSBuf) || (BDptr -> pHBuf) || (BDptr -> pIBuf))
   {
      // one of the above mentioned structures exist, return unhappy
#ifdef TRCE
      TrcPrint("KIprBdSetNumCounters Error, 1 or more buffs. exist, samp %x, hist %x, skip %x\n",
         BDptr->pSBuf,BDptr->pHBuf,BDptr->pIBuf);
#endif
      return IPR__BUFEXISTS;
   }

   // skip structure doesn't exist, ok to set counters
   BDptr -> numcounters = numctrs;
#ifdef TRCB
   TrcPrint("BDptr= %x, BDptr->numcounters= %d\n",
      BDptr,BDptr->numcounters);
#endif
   return IPR__SUCCESS;
}

extern IPR_STATUS KIprBdSetAction (IN IPR_CPUNUM cpu,
			IN int action_type, 	// ACTION_SAMPLE or ACTION_FULL - need to define
			IN int action,
			IN int (*targetroutine) (),
			IN unsigned int targetpid)
{

   IPR_STATUS status;
   IPR_BUFDSC
      *BDptr;

#ifdef TRC0
    TrcPrint("KIprBdSetAction enter, cpu = %d, action_type = %x, action = %x\n",
       cpu,action_type,action);
    TrcPrint("target_routine addr = %x, targetpid = %x\n",
       targetroutine,targetpid);
#endif
   status = get_bufdsc_address (cpu,&BDptr);
   if (!IPR_SUCCESS(status))
   {
#ifdef TRCE
      TrcPrint("KIprBdSetAction Error return from get_bufdsc_address, status %x\n",
         status);
#endif
      return status;
   }
   if (!BDptr)
      return IPR__INVCPU;

   if (action_type == IPR_ACTION_FULL)
   {
      BDptr -> fullaction = action;
      BDptr -> pRoutine = targetroutine;
      BDptr -> targetpid = targetpid;
   }
   else if (action_type == IPR_ACTION_SAMPLE)
   {
      BDptr -> sampleaction = action;
      BDptr -> pRoutine = targetroutine;
      BDptr -> targetpid = targetpid;
   }
   else
      return IPR__INVSETACT;
#ifdef DBGB
   DbgPrint("Action type %x set to %x on BD %x\n",
      action_type,action,BDptr);
   DbgBreakPoint();
#endif
   return IPR__SUCCESS;

}

extern IPR_STATUS KIprBdSetActiveCounters (IN IPR_CPUNUM cpu, IN IPR_CTRMASK ctrmask)
{

   IPR_STATUS status;
   IPR_BUFDSC
      *BDptr;

#ifdef TRC0
   TrcPrint("KIprBdSetActiveCounters enter, cpu = %d, ctrmask = %x\n",
       cpu,ctrmask);
#endif
   status = get_bufdsc_address (cpu, &BDptr);
   if (!IPR_SUCCESS(status))
   {
#ifdef TRCE
      TrcPrint("KIprBdSetActiveCounters Error return from get_bufdsc_address, status %x\n",
         status);
#endif
      return status;
   }
   if (!BDptr)
      return IPR__INVCPU;

   BDptr -> validcounters = ctrmask;

   return IPR__SUCCESS;

}

extern IPR_STATUS KIprBdAddPcRange (IN IPR_CPUNUM cpu, IN IPR_PC_RANGE *pcrange)
{

   IPR_STATUS status;
   IPR_BUFDSC
      *BDptr;
   IPR_PBUFFER
      *PBptr;
   IPR_PC_RANGE
      *tPCRptr,*fPCRptr;
   int allocSize;
   unsigned int cnt,mcnt,tmp;

#ifdef TRC0
   TrcPrint("KIprBdAddPcRange enter, cpu = %d, pcrange addr = %x\n",
       cpu,pcrange);
#endif

   if (!pcrange)
   {
#ifdef TRCE
      TrcPrint("KIprBdAddPcRange Error: pcrange is null\n");
#endif
      return IPR__NULLPTR;
   }
   status = get_bufdsc_address (cpu,&BDptr);
   if (!IPR_SUCCESS(status))
   {
#ifdef TRCE
      TrcPrint("KIprBdAddPcRange Error return from get_bufdsc_address, status %x\n",
         status);
#endif
      return status;
   }
   if (!BDptr)
      return IPR__INVCPU;

   // PC range function - look for existing PC ranges to see if
   // one matches the pcrange passed into this function

   if (!BDptr -> pPBuf)
   {
      // no PC range exists, create structures and add specified PC range
      allocSize = sizeof (IPR_PBUFFER) + (IPR_PCRangeGroup * sizeof(IPR_PC_RANGE));
      status = allocate_memory (allocSize,&pcrng_template,&PBptr);
      if (!IPR_SUCCESS(status))
      {
#ifdef TRCE
         TrcPrint("KIprBdAddPcRange Error return from allocate_memory, status %x\n",
            status);
#endif
         return status;
      }
      if (!PBptr)
      	  return IPR__ALLOCFAIL;

      all_pb_mem_cnt++;
      BDptr -> pPBuf = PBptr;

      // Fill in the PC range PBUFFER structure
      PBptr -> rangecnt = 1;
      PBptr -> curmaxrangecnt = IPR_PCRangeGroup;
      PBptr -> sample_always = FALSE;
#ifdef TRCB
      TrcPrint("1st PC range structure start %x, size %d, rangecnt=%d, maxrange=%d\n",
         PBptr,PBptr -> header.size,PBptr -> rangecnt,PBptr -> curmaxrangecnt);
#endif

      // Fill in the PC RANGE structure - 1st get starting address of
      // PC ranges (they begin after to IPR_PBUFFER header).  Integer
      // arithmetic is used to avoid any irregularities that may occur
      // due to alignment.
      tPCRptr = (IPR_PC_RANGE *)(PBptr + 1);
#ifdef TRCB2
      TrcPrint("New PC range slot is %x\n",
         tPCRptr);
#endif
      *tPCRptr = *pcrange;
#ifdef TRCB
      TrcPrint("Created PC range buffer, start= %x, end= %x, range=%d, curmaxrangecnt=%d\n",
         tPCRptr->StartAddress,tPCRptr->EndAddress,PBptr->rangecnt,PBptr->curmaxrangecnt);
#endif
   }
   else
   {
      // Check to see if their is more space in the current PC range buffer.
      // If not, allocate more PC range space.
      if (BDptr->pPBuf->rangecnt == BDptr->pPBuf->curmaxrangecnt)
      {
         // Allocate a larger PC range buffer and move ranges from old buffer
         // to new buffer.  Also fill in new header and update the buffer desc. ptr.
         allocSize = sizeof (IPR_PBUFFER) + ((BDptr->pPBuf->curmaxrangecnt + IPR_PCRangeGroup)
            * sizeof(IPR_PC_RANGE));
         status = allocate_memory (allocSize,&pcrng_template,&PBptr);
         if (!IPR_SUCCESS(status))
         {
#ifdef TRCE
            TrcPrint("KIprBdAddPcRange Error return from allocate_memory, status %x\n",
               status);
#endif
            return status;
         }
         if (!PBptr)
      	    return IPR__ALLOCFAIL;

         all_pb_mem_cnt++;

         // Fill in new PC range PBUFFER structure from old structure
         PBptr -> rangecnt = BDptr->pPBuf->rangecnt;
         PBptr -> curmaxrangecnt = BDptr->pPBuf->curmaxrangecnt + IPR_PCRangeGroup;
         PBptr -> sample_always = BDptr->pPBuf->rangecnt;
#ifdef TRCB
         TrcPrint("New PC range structure start %x, size %d, rangecnt=%d, maxrange=%d\n",
            PBptr,PBptr -> header.size,PBptr -> rangecnt,PBptr -> curmaxrangecnt);
#endif
         // Move the PC Ranges
         fPCRptr = (IPR_PC_RANGE *)(BDptr->pPBuf + 1);
         tPCRptr = (IPR_PC_RANGE *)(PBptr + 1);
#ifdef TRCB2
         TrcPrint("Starting PC range loc: old=%x, new=%x\n",
            fPCRptr,tPCRptr);
#endif
         for (cnt=0; cnt < BDptr->pPBuf->rangecnt; cnt++)
         {
            *tPCRptr = *fPCRptr;
            fPCRptr++;
            tPCRptr++;
#ifdef TRCB2
            TrcPrint("Next PC range loc: old=%x, new=%x\n",
               fPCRptr,tPCRptr);
#endif
         }
         // Deallocate old PC range block
         deallocate_memory(BDptr->pPBuf);
         deall_pb_mem_cnt ++;
         // Update buffer descriptor with new PC range block
         BDptr -> pPBuf = PBptr;
      }
      else
         PBptr = BDptr->pPBuf;
      // Space exists in current PC range buffer for another range,
      // insert this range in the proper place.
      tPCRptr = (IPR_PC_RANGE *)(PBptr + 1);
#ifdef TRCB2
      TrcPrint("Adding to buffer, tPCRptr=%x, PBptr=%x\n",
         tPCRptr,PBptr);
      TrcPrint("new SA=%Lx, old SA=%Lx\n",
         pcrange->StartAddress,tPCRptr->StartAddress);
#endif
      for (cnt=0; cnt < BDptr->pPBuf->rangecnt; cnt++)
      {
#ifdef TRCB2
         TrcPrint("Range %d, SA=%Lx, EA=%#16Lx\n",
            cnt,tPCRptr->StartAddress,tPCRptr->EndAddress);
#endif
         if (pcrange->StartAddress < tPCRptr->StartAddress)
         {
             // New range goes here, current and subsequent ranges back to
             // make room for the new rang
             fPCRptr = (IPR_PC_RANGE *)(PBptr + 1);
             fPCRptr += BDptr->pPBuf->rangecnt-1;
             tmp = BDptr->pPBuf->rangecnt * sizeof(IPR_PC_RANGE);
	     tPCRptr = (IPR_PC_RANGE *)(PBptr + 1);
	     tPCRptr += BDptr->pPBuf->rangecnt;
#ifdef TRCB2
             TrcPrint("rangecnt= %d, curmaxrangecnt= %d, IPR_PC_RANGE size= %d, tmp=%d\n",
                 BDptr->pPBuf->rangecnt,BDptr->pPBuf->curmaxrangecnt,sizeof(IPR_PC_RANGE),tmp);
             TrcPrint("Adding PC range at slot %d, new PC range loc is %x, last PC range loc is %x\n",
                 cnt,tPCRptr,fPCRptr);
#endif
             for (mcnt = cnt; mcnt < BDptr->pPBuf->rangecnt; mcnt++)
             {
                 *tPCRptr = *fPCRptr;
                 tPCRptr = fPCRptr;
                 fPCRptr--;
#ifdef TRCB2
                 TrcPrint("Next new PC range loc is %x, prev PC range loc is %x\n",
                    tPCRptr,fPCRptr);
#endif
             }
             // Now put the new PC range in its slot
	     tPCRptr = (IPR_PC_RANGE *)(PBptr + 1);
	     tPCRptr += cnt;
             *tPCRptr = *pcrange;
             PBptr -> rangecnt++;
#ifdef TRCB2
             TrcPrint("New PC range slot is %d, loc is %x, SA=%Lx\n",
                cnt,tPCRptr,tPCRptr->StartAddress);
             TrcPrint("There are now %d PC ranges in buffer with max. of %d ranges\n",
                PBptr->rangecnt,PBptr->curmaxrangecnt);
#endif
             return IPR__SUCCESS;
         }
         tPCRptr++;
#ifdef TRCB2
         TrcPrint("Next PC range starts at %x\n",
            tPCRptr);
#endif
      }  // end - for
      // This PC range is the largest range, add to end of buffer
#ifdef TRCB2
      TrcPrint("Range start address larger than all previous.\n");
      TrcPrint("rangecnt= %d, curmaxrangecnt= %d, IPR_PC_RANGE size= %d\n",
         BDptr->pPBuf->rangecnt,BDptr->pPBuf->curmaxrangecnt,sizeof(IPR_PC_RANGE));
      TrcPrint("Adding PC range at slot %d, new PC range loc is %x\n",
         cnt,tPCRptr);
#endif
      tPCRptr = (IPR_PC_RANGE *)(PBptr + 1);
      tPCRptr += PBptr->rangecnt;
      *tPCRptr = *pcrange;
      PBptr -> rangecnt++;
#ifdef TRCB2
      TrcPrint("New PC range slot is %d, loc is %x, SA=%Lx\n",
         (PBptr -> rangecnt-1),tPCRptr,tPCRptr->StartAddress);
      TrcPrint("There are now %d PC ranges in buffer with max. of %d ranges\n",
         PBptr->rangecnt,PBptr->curmaxrangecnt);
#endif
   }

   return IPR__SUCCESS;
}

extern IPR_STATUS KIprBdSetSkipCount (IN IPR_CPUNUM cpu, IN IPR_CTRNUMBER ctr,
                                      IN int skipvalue)
{

   IPR_STATUS status;
   IPR_BUFDSC
      *BDptr;
   IPR_IBUFFER
      *Iptr;
   IPR_ISKIP
      *ISptr;
   unsigned int cnt;
   int allocSize;

#ifdef TRC0
   TrcPrint("KIprBdSetSkipCount enter, cpu = %d, ctr = %d, skip = %d\n",
       cpu,ctr,skipvalue);
#endif

   status = get_bufdsc_address (cpu, &BDptr);
   if (!IPR_SUCCESS(status))
   {
#ifdef TRCE
      TrcPrint("KIprBdSetSkipCount Error return from get_bufdsc_address, status %x\n",
         status);
#endif
      return status;
   }
   if (!BDptr)
      return IPR__INVCPU;

   if (BDptr -> numcounters == 0)
   {
#ifdef TRCE
      TrcPrint("KIprBdSetSkipCount Error BDptr = %x, BDptr->numcounters = %d\n",
          BDptr,BDptr->numcounters);
#endif
      return IPR__INVACTION;	// numcounters must have been specified already
   }
   if (!BDptr -> pIBuf)
   {
      // skip count structures don't exist, create them
      allocSize = sizeof (IPR_IBUFFER) + (BDptr -> numcounters * sizeof (IPR_ISKIP));
      status = allocate_memory (allocSize,&skip_template,&Iptr);
      if (!IPR_SUCCESS(status))
      {
#ifdef TRCE
         TrcPrint("KIprBdSetSkipCount Error return from allocate_memory, status %x\n",
            status);
#endif
         return status;
      }
      if (!Iptr)
      	  return IPR__ALLOCFAIL;
      all_sk_mem_cnt++;
      BDptr -> pIBuf = Iptr;

      // Fill in the skip count IBUFFER structure
      Iptr -> numcounters = BDptr -> numcounters;
      Iptr -> skipmask = 1 << ctr;
      ISptr = (IPR_ISKIP *) (Iptr + 1);
#ifdef TRCB
      TrcPrint("New skip range structure start %x, size %d, numcounters=%d, BDptr->numcount %d\n",
         Iptr,Iptr->header.size,Iptr->numcounters,BDptr->numcounters);
      TrcPrint("Start of skip records %x\n",
         ISptr);
#endif
//
// Really ought to be grabbing a lock or mutex here to prevent problems
// with the ISR - problem is, we can't really coordinate with the
// ISR because it is running at elevated IPL.  We can't block it and
// can't even insure that it won't use intermediate values unless we
// grab some sort of mutex and tell it to simply do nothing if an update
// to a key data structure such as this is in progress.  - BMW
//
// This is ok, the API will insure that the counters are inactive before
// data structures touched by the API such as this one are modified.
//
      for (cnt = 0; cnt < BDptr -> numcounters; cnt++)
      {
#ifdef TRCB
         TrcPrint("Set skip threshold for counter %d at %x to %d, skipmask %x, BDptr %x, BDptr->numcount %d, Iptr %x\n",
            cnt,ISptr,ISptr -> intspersample,Iptr->skipmask,BDptr,BDptr->numcounters, Iptr);
#endif
         if (cnt == ctr)
         {
             ISptr -> intspersample = skipvalue;
             ISptr -> intssincelast = 0;
         }
         else
         {
             ISptr -> intspersample = 0;
             ISptr -> intssincelast = 0;
         }
#ifdef TRCB
         TrcPrint("Set skip threshold for counter %d at %x to %d, skipmask %x, BDptr %x, BDptr->numcount %d, Iptr %x\n",
            cnt,ISptr,ISptr -> intspersample,Iptr->skipmask,BDptr,BDptr->numcounters, Iptr);
#endif
         ISptr++;
#ifdef TRCB
         TrcPrint("Set skip threshold for counter %d at %x to %d, skipmask %x, BDptr %x, BDptr->numcount %d, Iptr %x\n",
            cnt,ISptr,ISptr -> intspersample,Iptr->skipmask,BDptr,BDptr->numcounters, Iptr );
#endif
      }
   }
   else
   {
      // skip count structure exists, update the apppropriate counter
      Iptr = BDptr -> pIBuf;
      Iptr -> skipmask = ((Iptr -> skipmask) | (1 << ctr));
      ISptr = (IPR_ISKIP *) (Iptr + 1);
      ISptr += ctr;
#ifdef TRCB
      TrcPrint("Skip structure exists, Iptr %x, Start of skip records for counter %d at  %x, new skip mask %x\n",
         Iptr,ISptr,ctr,Iptr->skipmask);
#endif
      ISptr -> intspersample = skipvalue;
      ISptr -> intssincelast = 0;
   }

   return IPR__SUCCESS;
}

