/****************************************************************************/
/*                                                                          */
/*  COPYRIGHT (c) 1993-1999 BY                                              */
/*  Digital Equiptment/Compaq Computer Corporation.                         */
/*  ALL RIGHTS RESERVED.                                                    */
/*                                                                          */
/*  This file is part of the Iprobe performance suite for Alpha processors. */
/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 2 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License       */
/*  along with this program; if not, write to the Free Software             */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307    */
/*                                                                          */
/****************************************************************************/

/*
 * Abstract:	This file contains the general
 *		kernel routines based on the IPROBE API kernel
 *              rationalization (reorganization).
 *
 * Author:		Bradley M. Waters
 * Creation Date:	21-Jul-1994
 *
 * Modified by:
 *
 * X-4  BMW             Bradley M. Waters       25-Oct-1994
 *      Updated some of the trace messages and conditionals.
 *
 * X-3	BMW		Bradley M. Waters	10-Oct-1994
 *	Code cleanup
 *
 * X-2	BMW		Bradley M. Waters	17-Aug-1994
 *	Updated based on code review input.
 *
 * X-1	BMW		Bradley M. Waters	21-Jul-1994
 *	Original Version.  Some code pulled from old pfm_routines.c.
 *
*/

#ifdef __NT
#  include "ntddk.h"
#endif

#include "ipr_kernel_int_defs.h"
#ifdef __NT
#  include "ipr_driver_nt.h"
#endif

#ifdef   TRCI
#include "TraceDrv.h"
#endif

// Counters for pool allocation and deallocation

uint64	all_bd_mem_cnt = 0,	// buffer descriptors
	deall_bd_mem_cnt = 0,
	all_hb_mem_cnt = 0,	// histogram buffers
	deall_hb_mem_cnt = 0,
	all_sb_mem_cnt = 0,     // sample buffers
	deall_sb_mem_cnt = 0,
	all_sk_mem_cnt = 0,     // skip count structures
	deall_sk_mem_cnt = 0,
	all_pb_mem_cnt = 0,     // PC range buffer
	deall_pb_mem_cnt = 0;

IPR_NPP_LIST npp_table;	// Structure containing pointers all nonpaged currently allocated

extern IPR_STATUS AddToPoolList(IN PVOID alloc_area)
{
   int cnt;

   for (cnt=0; cnt < IPR_MAX_NPP_BLOCKS; cnt++)
   {
      if (npp_table.start[cnt] == NULL)
      {
         npp_table.start[cnt] = alloc_area;
#ifdef TRCP
         TrcPrint("Adding to pool list location %d loc=%x\n",
            cnt,npp_table.start[cnt]);
#endif
#ifdef DBGP
         DbgPrint("Adding to pool list location %d loc=%x\n",
            cnt,npp_table.start[cnt]);
#endif
         return IPR__SUCCESS;
      }
   }
#ifdef TRCE
   TrcPrint("AddToPoolList Error adding %x to pool list, NPP list overflow, increase list size currently %d\n",
      alloc_area,IPR_MAX_NPP_BLOCKS);
#endif
   return IPR__NPPLOVFLW;
}

extern IPR_STATUS DelFromPoolList(IN PVOID alloc_area)
{
   int cnt;

   for (cnt=0; cnt < IPR_MAX_NPP_BLOCKS; cnt++)
   {
      if (npp_table.start[cnt] == alloc_area)
      {
#ifdef TRCP
         TrcPrint("Deleting from pool list location %d loc=%x\n",
                 cnt,npp_table.start[cnt]);
#endif
         npp_table.start[cnt] = NULL;
         return IPR__SUCCESS;
      }
   }
#ifdef TRCE
   TrcPrint("AddToPoolList Error deleting %x from pool list, entry not found\n",
      alloc_area);
#endif
   return IPR__NOTINNPP;
}

/* Round value up to next multiple of granularity */

extern int round_value_up (int xval, int gran)
{
   int tmp;

#ifdef TRCG
    TrcPrint("round_value_up enter, xval = %x, gran = %x\n",
       xval,gran);
#endif
   // avoid divide by 0.

   if (gran == 0)
      return 0;
   tmp = (xval + gran - 1) - ((xval + gran - 1) % gran);
#ifdef TRCG
    TrcPrint("rounded value = %x\n",
       tmp);
#endif
   return tmp;
}

// NOTE WELL!!!  Do not under any circumstances attempt to coalesce (combine)
// chuncks of nonpaged pool allocated by allocate_memory.  Doing such a thing
// will screw up the system's bookkeeping of nonpaged pool and eventually have
// problems (ie, crash).  For each block of memory allocated with
// allocate memory, call deallocate memory when the memory is no longer needed.

// Allocate the specified amount of resident memory

extern IPR_STATUS allocate_memory (IN int bytecnt,IN IPR_BUFHDR *hdr,
                                   OUT void *pRet)
{

   IPR_STATUS
      status;
   void **retptr;
   int
      retsize;
   IPR_BUFHDR *areaptr;
   IPR_BOOLEAN allocZone;

   retptr = pRet;
#ifdef TRC0
   TrcPrint("allocate_memory: bytecnt=%d, retptr %x, hdr %x, sysdata %x\n",
       bytecnt,retptr,hdr,&sysdata);
#endif
   if (!retptr)
   {
#ifdef TRCE
      TrcPrint("allocate_memory Error: retptr is null\n");
#endif
      return IPR__NULLPTR;
   }
   if (!hdr)
   {
#ifdef TRCP
      TrcPrint("hdr ptr is null, no header used\n");
#endif
   }
#ifdef TRCP
    TrcPrint("sysdata.affinity = %x, sysdata.allocation_gran = %x \n",
       sysdata.affinity_cpu_mask,sysdata.allocation_granularity);
#endif
    *retptr = NULL;
    if (hdr && (bytecnt < sizeof(IPR_BUFHDR)))
    {
       return IPR__ALLOCSMALL;
    }
    if ((bytecnt = round_value_up (bytecnt, ALLOCATION_GRANULARITY))
        > IPR_MAX_BUFFER_SIZE)
    {
#ifdef TRCE
       TrcPrint("allocate_memory Error allocating %d bytes, max. "
                "allocation size is %d\n",
          bytecnt,IPR_MAX_BUFFER_SIZE);
#endif
       return IPR__ALLOCBIG;
    }
#ifdef TRCP
    TrcPrint("return from round_value_up, bytecnt = %x\n",
       bytecnt);
#endif

   // Check to see if a header was passed, if not, the allocation cannot
   // be from a zone because we have no way of knowing how big the packet
   // is on deallocation
   if (hdr)
      allocZone = TRUE;
   else
      allocZone = FALSE;

   status = ALONONPAGED (bytecnt, allocZone, &retsize, &areaptr);
#ifdef TRCP
    TrcPrint("return from ALONONPAGED, status = %x, retsize = %d, areaptr = %x\n",
       status,retsize,areaptr);
#endif

   if (!IPR_SUCCESS (status))
      return status;
   if (retsize != bytecnt)
   {
#ifdef TRCW
      TrcPrint("allocate_memory WARNING: allocated %d bytes when %d requested\n",
         retsize,bytecnt);
#endif
   }
   if (hdr)
   {
      *areaptr = *hdr;
      areaptr->size = retsize;
   }
   *retptr = areaptr;

#ifdef TRCP
   TrcPrint("allocate_memory: allocated %d bytes at %x\n",
      bytecnt,areaptr);
#endif

   return IPR__SUCCESS;
}

// Deallocate the specified resident memory area

extern void deallocate_memory (IN void *area)
{

#ifdef TRC0
    TrcPrint("deallocate_memory enter...\n");
#endif
   if (!area)
   {
#ifdef TRCE
      TrcPrint("deallocate_memory Error: area ptr is null\n");
#endif
      return;
   }

   (void) DEANONPAGED (area);

#ifdef DBGP
   DbgPrint("deallocate_memory: deallocated %x\n",
      area);
#endif
}
