/****************************************************************************/
/*                                                                          */
/*  COPYRIGHT (c) 1993-1999 BY                                              */
/*  Digital Equiptment/Compaq Computer Corporation.                         */
/*  ALL RIGHTS RESERVED.                                                    */
/*                                                                          */
/*  This file is part of the Iprobe performance suite for Alpha processors. */
/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 2 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License       */
/*  along with this program; if not, write to the Free Software             */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307    */
/*                                                                          */
/****************************************************************************/
/*
 * Abstract:	This file contains the sample
 *		kernel routines based on the IPROBE API
 *              kernel rationalization (reorganization).
 *
 * Author:		Bradley M. Waters
 * Creation Date:	20-Jul-1994
 *
 * Modified by:
 *
 * X-3	BMW		Bradley M. Waters	10-Oct-1994
 *	Code cleanup
 *
 * X-2	BMW		Bradley M. Waters	17-Aug-1994
 *	Updated based on code review input.
 *
 * X-1	BMW		Bradley M. Waters	20-Jul-1994
 *	Original Version.
 *
*/

#ifdef __NT
#  include "ntddk.h"
#endif

#include "ipr_kernel_int_defs.h"

#ifdef __NT
#  include "ipr_driver_nt.h"
#endif

#ifdef   TRCI
#include "TraceDrv.h"
#endif

// global variables

IPR_BUFHDR sample_template = {
	IPR_PROD_IDENT,
	0,		// 2nd field (size) is filled in by allocation routine
	IPR_SBUFFER_TYPE,
	IPR_SBUFFER_SUBTYPE,
	IPR_SBUFFER_KIND
	};

extern IPR_STATUS KIprSbCreate (IN IPR_CPUNUM cpu,
			IN IPR_SIZE size,
			IN unsigned int numbuffs,
			OUT IPR_SDESC *descrip,
// contains data and dataquad mask as input
// and is updated with sample info. for output
			OUT IPR_SBUFFER **retptr)
{
   IPR_STATUS Status;
   IPR_BUFDSC
      *BDptr;
   IPR_SBUFFER
      *Sptr;
   IPR_SIZE asize,allocSize,cnt,DataSize;
   unsigned int createdbuff = 0;
   unsigned int pad;
   IPR_WrtList *list_ptr;

#ifdef TRC0
   TrcPrint("KIprSbCreate enter...\n");
#endif
   if (!retptr)
   {
#ifdef TRCE
      TrcPrint("KIprSbCreate Error: retptr is null\n");
#endif
      return IPR__NULLPTR;
   }
   *retptr = NULL;
   if (!descrip)
   {
#ifdef TRCE
      TrcPrint("KIprSbCreate Error: descrip ptr is null\n");
#endif
      return IPR__NULLPTR;
   }

   Status = get_bufdsc_address (cpu, &BDptr);
   if (!IPR_SUCCESS(Status))
      return Status;
   if (!BDptr)
      return IPR__INVCPU;

   // If one or more sample buffers already exists, insure that the
   // the datamask and dataquad mask match that of the sample buffers
   // already present.  Because of this restriction, we only need
   // to check the 1st buffer on the list.

   // First clear out any excess bits in the quad mask, to ensure it is a
   // subset of the data mask

   descrip->dataquadmask.ipr_dmask_l_bits &=
      descrip->datamask.ipr_dmask_l_bits;
   if (BDptr->pSBuf)
      if ((BDptr->pSBuf->data.ipr_dmask_l_bits
           != descrip->datamask.ipr_dmask_l_bits) ||
          (BDptr->pSBuf-> dataquad.ipr_dmask_l_bits
           != descrip->dataquadmask.ipr_dmask_l_bits))
         // data mismatch, return unhappy
         return IPR__NOMATCH;

   // Make sure the masks are valid

   if (descrip->datamask.ipr_dmask_l_bits >= IPR_DMASK_M_MAXIMUM)
   {
#ifdef TRCE
      TrcPrint("Data mask %x invalid, maximum bit in mask is %x\n",
         descrip->datamask.ipr_dmask_l_bits,IPR_DMASK_M_MAXIMUM);
#endif
      return IPR__INVMASK;
   }
   if (descrip->dataquadmask.ipr_dmask_l_bits >= IPR_DMASK_M_MAXIMUM)
   {
#ifdef TRCE
      TrcPrint("Data mask %x invalid, maximum bit in mask is %x\n",
         descrip->dataquadmask.ipr_dmask_l_bits,IPR_DMASK_M_MAXIMUM);
#endif
      return IPR__INVMASK;
   }

   // Determine the size necessary to hold the desired data.  It is calculated
   // as (number-of-ints * sizeof-int) + (number-of-quads * sizeof-quad)

   DataSize =
      (sizeof (uint32) *
         BIprCnt (BIprClr (descrip -> datamask . ipr_dmask_l_bits,
                           descrip -> dataquadmask . ipr_dmask_l_bits))) +
      (sizeof (uint64) *
         BIprCnt (BIprBoth (descrip -> datamask . ipr_dmask_l_bits,
                            descrip -> dataquadmask . ipr_dmask_l_bits)));

   if (!DataSize)
      return IPR__EMPTYSAMP;

   // Round requested buffer size up to IPR_SAMPBUF_GRAN units

   allocSize = round_value_up (size, IPR_SAMPBUF_GRAN);
   if (allocSize > IPR_MAX_BUFFER_SIZE)
   {
#ifdef TRCW
      TrcPrint("Warning, buffer size %d (after rounding to gran %d) requested, max size is %d, using max size\n",
         allocSize,IPR_SAMPBUF_GRAN,IPR_MAX_BUFFER_SIZE);
#endif
      allocSize = IPR_MAX_BUFFER_SIZE;
   }

   // Compute pad for data size so samples containing a quadword
   // are aligned on quadword boundaries
   // Note that, at this point, excess bits in the quad mask have been cleared

   pad = 0;
   if (descrip->dataquadmask.ipr_dmask_l_bits)
   {
      pad = (sizeof (uint64) - (DataSize % sizeof (uint64))) % sizeof (uint64);
      DataSize += pad;
   }

   // set up areasize to hold an exact number of samples based on the space
   // remaining in the sample buffer (after the header is accounted for)

   asize = allocSize - sizeof (IPR_SBUFFER);
   asize -= asize % DataSize;

#ifdef TRCS
   TrcPrint("Actual sample data size %d, area size is %d, total sample buffer size is %d\n",
      DataSize,asize,allocSize);
#endif

   cnt = 0;
   while (createdbuff < numbuffs)
   {
      // Create buffer
      Status = allocate_memory (allocSize,&sample_template,&Sptr);
      if (!IPR_SUCCESS(Status))
      {
#ifdef TRCE
         TrcPrint("KIprSbCreate Error return from allocate_memory, Status %x\n",
            Status);
#endif
         return Status;
      }

      if (!Sptr)
         return IPR__ALLOCFAIL;
      all_sb_mem_cnt++;
      // Fill in the rest of the sample buffer SBUFFER structure
      Sptr -> begintime = 0;
      Sptr -> areasize = asize;
      Sptr -> full = IPR_BUFFER_EMPTY;
      Sptr -> cpuid = cpu;
      Sptr -> data = descrip->datamask;
      Sptr -> dataquad = descrip->dataquadmask;
      Sptr -> missedcnt = 0;
      Sptr -> areaoffset = asize + 1;
      Sptr -> pad = pad;
      Status = allocate_wrtQEntry(Sptr, &list_ptr);
      if (!IPR_SUCCESS(Status))
      {
#ifdef TRCE
         TrcPrint("KIprSbCreate Error: Failure allocating memory for the write list entry: buffer %x, status = %x\n",
           Sptr,Status );
#endif
         deallocate_memory(Sptr);
         return Status;
      }
      Sptr -> wrtQEntry = list_ptr;
#ifdef TRCS
      TrcPrint("Samp buff alloc, prod ident %Lx, kind %x, size %d, asize is %d, aoff %d, pad %d, lmask %x, qmask %x\n",
         Sptr->header.PROD_IDENT_FIELD.PROD_IDENT,Sptr->header.bufkind,Sptr->header.size,
         Sptr->areasize,Sptr->areaoffset,Sptr->pad,Sptr->data,Sptr->dataquad);
#endif

      descrip->size = Sptr->header.size;
      descrip->begintime = Sptr->begintime;
      descrip->areasize = Sptr->areasize;
      descrip->full = Sptr->full;
      descrip->cpuid = Sptr->cpuid;
      descrip->missedcnt = Sptr->missedcnt;
      descrip->areaoffset = Sptr->areaoffset;

      createdbuff++;

      // Put buffer in proper place in the buffer chain
      if (!BDptr -> pSBuf)
      {
         // No sample buffers exist yet, this one will be 1st on the list
         Sptr->pNext = Sptr;	// Initially the 1st buffer pts to itself
         BDptr->pSBuf = Sptr;
      }
      else
      {
         // One or more sample buffers already exist,
         // make this buffer the 2nd in the ring (to make it easy to
         // update pointers - this way it isn't necessary to traverse
         // the sample buffer pool looking for the 1st buffer)
         Sptr->pNext = BDptr->pSBuf->pNext;
         BDptr->pSBuf->pNext = Sptr;
      }
#ifdef TRCS
      TrcPrint("1st sample buffer %x, this buffer %x, next %x, total size %d, size for samples=%d\n",
         BDptr->pSBuf,Sptr,Sptr->pNext,Sptr->header.size,asize);
      TrcPrint("prodident %Lx, type %d, subtype %d, kind %d\n",
         Sptr->header.prod_ident,Sptr->header.type,Sptr->header.subtype,Sptr->header.bufkind);
      TrcPrint("Start of samples %x, mask=%x, quadmask=%x\n",
         ((char *)Sptr+Sptr->areaoffset),Sptr -> data,Sptr -> dataquad);
#endif

      cnt++;
      if (cnt > IPR_MAX_LOOP_CNT)
      {
#ifdef TRC0
         TrcPrint("Went through loop %d times before finishing adding sample buffers, exiting\n",
            cnt);
#endif
         return IPR__PTRLOOPLIM;
      }
   }  // end - while
   *retptr = Sptr;  // return pointer of last buffer created.
   return IPR__SUCCESS;
}

extern IPR_STATUS KIprSbDelete (IN IPR_CPUNUM cpu,
                                IN IPR_SBUFFER *bufptr)
{
   IPR_STATUS Status;
   IPR_BUFDSC
      *BDptr;
   IPR_SBUFFER
      *SOptr,*SPptr,*SLptr;
   int cnt;

#ifdef TRCS
   TrcPrint("KIprSbDelete enter...\n");
#endif
   if (!bufptr)
   {
#ifdef TRCE
      TrcPrint("KIprSbDelete Error: bufptr is null\n");
#endif
      return IPR__NULLPTR;
   }
   if ((bufptr->header.prod_ident != IPR_PROD_IDENT) ||
      (bufptr->header.bufkind != IPR_SBUFFER_KIND))
   {
#ifdef TRCE
      TrcPrint("KIprSbDelete Error: found buffer prod %Lx kind %x, bufptr %x, when looking for %Lx %x\n",
         bufptr->header.prod_ident,bufptr->header.bufkind,bufptr,
         IPR_PROD_IDENT,IPR_SBUFFER_KIND);
#endif
      return IPR__INVBUFKIND;
   }
   Status = get_bufdsc_address (cpu, &BDptr);
   if (!IPR_SUCCESS(Status))
      return Status;
   if (!BDptr)
      return IPR__INVCPU;

   if (!BDptr -> pSBuf)
   {
      // No sample buffers to delete
      return IPR__FAILURE;
   }  // if - then

   Status = SynchClose();
   if (!IPR_SUCCESS(Status))
      return Status;

   // Search for the buffer in the ring
#ifdef TRCS
   TrcPrint("Searching for buffer %x, in ring, 1st %x, 2nd %x, 3rd %x\n",
      bufptr,BDptr->pSBuf,(BDptr->pSBuf)->pNext,((BDptr->pSBuf)->pNext)->pNext);
#endif
   SOptr = BDptr -> pSBuf;
   if (SOptr == bufptr)
   {
      // Have a match, delete 1st buffer in ring
      if (BDptr->pSBuf == SOptr->pNext)
         {
#ifdef TRCS
         TrcPrint("Deleting 1st and last buffer in ring, BDpSBuf %x\n",
            BDptr->pSBuf);
#endif
         BDptr->pSBuf = NULL;	// deleting last buffer in the ring
      }
      else
      {
         BDptr -> pSBuf = SOptr -> pNext;	// more buffers in the ring
         // Look for last buffer in ring so its pointer can be updated to new first buffer
         SLptr = BDptr -> pSBuf;
	 cnt = 0;
         while (SLptr->pNext != bufptr)
         {
#ifdef TRCS
            TrcPrint("SLptr %x, SLptr->pNext %x \n",
               SLptr,SLptr->pNext);
#endif
            SLptr = SLptr->pNext;
            cnt++;
            if (cnt > IPR_MAX_LOOP_CNT)
            {
#ifdef TRC0
               TrcPrint("Went through loop %d times before match, exiting\n",
                  cnt);
#endif
	       return IPR__PTRLOOPLIM;
            }
         }
         // OK, found last buffer, update its pNext field
         SLptr->pNext = BDptr->pSBuf;
#ifdef TRCS
         TrcPrint("BDpSBuf %x, SLptr->pNext now %x \n",
            BDptr -> pSBuf,SLptr->pNext);
#endif
      }
#ifdef TRCS
      if (BDptr->pSBuf)
         TrcPrint("Deleted 1st buffer in ring, bufptr %x, new BDpSBuf %x, 2nd %x, 3rd %x\n",
            bufptr,BDptr->pSBuf,(BDptr->pSBuf)->pNext,((BDptr->pSBuf)->pNext)->pNext);
      else
         TrcPrint("Deleted 1st buffer in ring, bufptr %x, new BDpSBuf %x\n",
            bufptr,BDptr->pSBuf);
#endif
#ifdef DBG2
      DbgPrint("SbDelete, deleting 1st sample buffer in ring at %x, new BDpSBuf %x\n\n",
         	bufptr,BDptr->pSBuf);
      DbgBreakPoint();
#endif
      if (bufptr->wrtQEntry)
         deallocate_memory(bufptr->wrtQEntry);
      deallocate_memory (bufptr);
      deall_sb_mem_cnt++;
      return IPR__SUCCESS;
   }  // if -then
   else
   {
      // not the 1st buffer in the ring, search for the buffer
      SPptr = SOptr;
      SOptr = SOptr -> pNext;
      cnt = 0;
      while (SOptr != BDptr->pSBuf)
      {
         if (SOptr == bufptr)
         {
            // Have a match, delete non 1st buffer in ring, can't be the last buffer left in ring
#ifdef TRCS
            TrcPrint("Deleting non-1st buffer in ring, bufptr %x, BDpSBuf %x, prev %x, next %x\n",
               bufptr,BDptr->pSBuf,SPptr,SOptr->pNext);
#endif
#ifdef DBG2
            DbgPrint("SbDelete, deleting non-1st sample buffer in ring %x, BDpSBuf %x\n\n",
               bufptr,BDptr->pSBuf);
            DbgBreakPoint();
#endif
            SPptr -> pNext = SOptr -> pNext;
#ifdef TRCS
            TrcPrint("non-1st buffer in ring deleted, new BDpSBuf %x, 2nd %x, 3rd %x\n",
               BDptr->pSBuf,(BDptr->pSBuf)->pNext,((BDptr->pSBuf)->pNext)->pNext);
#endif
            if (bufptr->wrtQEntry)
               deallocate_memory(bufptr->wrtQEntry);
            deallocate_memory (bufptr);
            deall_sb_mem_cnt++;
            return IPR__SUCCESS;
         }   // if - then
         SPptr = SOptr;
         SOptr = SOptr -> pNext;
         cnt++;
         if (cnt > IPR_MAX_LOOP_CNT)
         {
#ifdef TRC0
            TrcPrint("Went through loop %d times before match, exiting\n",
               cnt);
#endif
            return IPR__PTRLOOPLIM;
         }
      }  // while
      // Didn't find the buffer, return unhappy
   }  // if - else

   return IPR__FAILURE;
}

extern IPR_STATUS KIprSbSetFull(IN IPR_SBUFFER *bufptr,
                                IN IPR_FULL_STATE full_val)
{
#ifdef TRC0
   TrcPrint("KIprSbSetFull, full_val %d enter...\n",
      full_val);
#endif
   if (!bufptr)
   {
#ifdef TRCE
      TrcPrint("KIprSbSetFull Error: bufptr is null\n");
#endif
      return IPR__NULLPTR;
   }
   if ((bufptr->header.prod_ident != IPR_PROD_IDENT) ||
      (bufptr->header.bufkind != IPR_SBUFFER_KIND))
   {
#ifdef TRCE
      TrcPrint("KIprSbSetFull Error: found buffer prod %Lx kind %x when looking for %Lx %x\n",
         bufptr->header.prod_ident,bufptr->header.bufkind,
         IPR_PROD_IDENT, IPR_SBUFFER_KIND);
#endif
      return IPR__INVBUFKIND;
   }
   bufptr -> full = full_val;
   return IPR__SUCCESS;

}

extern IPR_STATUS KIprSbGetInfo(IN IPR_SBUFFER *bufptr, OUT IPR_SDESC *descrip)
{
#ifdef TRC0
   TrcPrint("KIprSbGetInfo enter...\n");
#endif
   if (!bufptr)
   {
#ifdef TRCE
      TrcPrint("KIprSbGetInfo Error: bufptr is null\n");
#endif
      return IPR__NULLPTR;
   }
   if ((bufptr->header.prod_ident != IPR_PROD_IDENT) ||
      (bufptr->header.bufkind != IPR_SBUFFER_KIND))
   {
#ifdef TRCE
      TrcPrint("KIprSbGetInfo Error: found buffer prod %Lx kind %x when looking for %Lx %x\n",
         bufptr->header.prod_ident,bufptr->header.bufkind,
         IPR_PROD_IDENT,IPR_SBUFFER_KIND);
#endif
      return IPR__INVBUFKIND;
   }
   descrip->size = bufptr->header.size;
   descrip->begintime = bufptr->begintime;
   descrip->areasize = bufptr->areasize;
   descrip->full = bufptr->full;
   descrip->cpuid = bufptr->cpuid;
   descrip->missedcnt = bufptr->missedcnt;
   descrip->areaoffset = bufptr->areaoffset;
   return IPR__SUCCESS;
}

extern IPR_STATUS KIprSbFind(IN IPR_CPUNUM cpu,
			OUT IPR_SDESC *descrip,
			IN IPR_SBUFFER *previous_buffer,
			OUT IPR_SBUFFER **retptr)
{

   IPR_STATUS Status;
   IPR_BUFDSC
      *BDptr;
   IPR_SBUFFER
      *fbuf;

#ifdef TRC0
   TrcPrint("KIprSbFind enter...\n");
#endif
   if (previous_buffer)
   {

      // Verify that the previous buffer is really a sample buffer

      if ((previous_buffer->header.prod_ident != IPR_PROD_IDENT) ||
         (previous_buffer->header.bufkind != IPR_SBUFFER_KIND))
      {
#ifdef TRCE
         TrcPrint("KIprSbFind Error: found buffer prod %Lx kind %x when looking for %Lx %x\n",
            previous_buffer->header.prod_ident,previous_buffer->header.bufkind,
            IPR_PROD_IDENT,IPR_SBUFFER_KIND);
#endif
         return IPR__INVBUFKIND;
      }
      fbuf = previous_buffer->pNext;
   }
   else
   {
      Status = get_bufdsc_address (cpu, &BDptr);
      if (!IPR_SUCCESS(Status))
         return Status;
      if (!BDptr)
         return IPR__INVCPU;

      fbuf = BDptr -> pSBuf;
   }

   // We have a buffer to return.  Validate it first.
#ifdef TRCS
   TrcPrint("KIprSbFind: NoPrev, fbuf %x, BDptr %x\n",fbuf,BDptr);
#endif

   if (!fbuf)
   {
      // Nothing found
#ifdef TRCS
      TrcPrint("KIprSbFind: buffer not found, returning %x\n",IPR__SBNOTFND);
#endif
      return IPR__SBNOTFND;
   }
   if ((fbuf->header.prod_ident != IPR_PROD_IDENT) ||
       (fbuf->header.bufkind != IPR_SBUFFER_KIND))
      return IPR__INVBUFKIND;
   *retptr = fbuf;
   descrip->size = fbuf->header.size;
   descrip->begintime = fbuf->begintime;
   descrip->areasize = fbuf->areasize;
   descrip->full = fbuf->full;
   descrip->cpuid = fbuf->cpuid;
   descrip->missedcnt = fbuf->missedcnt;
   descrip->areaoffset = fbuf->areaoffset;
   return IPR__SUCCESS;
}
