/****************************************************************************/
/*                                                                          */
/*  COPYRIGHT (c) 1993-1999 BY                                              */
/*  Digital Equiptment/Compaq Computer Corporation.                         */
/*  ALL RIGHTS RESERVED.                                                    */
/*                                                                          */
/*  This file is part of the Iprobe performance suite for Alpha processors. */
/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 2 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License       */
/*  along with this program; if not, write to the Free Software             */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307    */
/*                                                                          */
/****************************************************************************/

/*
 * Program: Alpha/Linux Iprobe driver.
 *
 * Abstract: This contains all of the linux module specific stuff.
 *
 * Author: Phillip Ezolt
 * Creation Date:  10-Aug-1998
 *
 * Modification history:
 *
 */


/* THIS LINE MUST BE FIRST. */
#include <linux/module.h>
#include <linux/sched.h>

#include <linux/fs.h>
#include <linux/version.h>
#include <linux/mm.h>


#include <asm/irq.h>


#include "stdtypes.h"

#include "ipr_kernel_int_defs.h"
#include "ipr_driver_linux.h"


#define NON_DATA_VALUE 0xffffffffffffffffL


/* LINUX:  Is this right??? */

extern unsigned int ipr_cp_mask, ipr_is_cp_mask; /*active CPU's mask */


/* This is the same device number of one version of the pentium status
   register monitors.  It should be safe to assume no one is going to
   use this on the alpha. 
   */

#define DEFAULT_MAJOR_NUMBER 61


#define DEVICE_NAME "ipr"

/* Forward Definitions... Should be in a header file.  PHIL */

void ipr_interrupt(unsigned long vector, struct pt_regs *regs); 

void do_full_action (IPR_SBUFFER *pSB, IPR_BUFDSC *bD, IPR_CPUNUM cdb);


boolean InUse=FALSE;

void (*temp_perf_irq)(unsigned long, struct pt_regs *)=NULL;

/* What is the size of this int? */

int MajorNumber=DEFAULT_MAJOR_NUMBER; 



/* This is a pointer to the original caller of the iprobe driver. */
extern struct task_struct *caller;

int ipr_open(struct inode * inode, struct file * filp)
{

  /* We tried to open a device that is already open. */
  if (InUse)
    {
      printk("Error: Iprobe device cannot be opened twice.\n");
      return -EBUSY; /* Correct? */

    }
  
  
  MOD_INC_USE_COUNT;
  InUse = TRUE;

  return 0;
} 


/* Below is a list of the IOCTL call defined for the IPR_DRIVER 
   The following device controls are recognized for the IPROBE API part of
    the performance counter driver:

    IOCTL_IPR_GetChipInfo - Return chip info
    IOCTL_IPR_UpdatePERFMON - Change PERFMON settings
    IOCTL_IPR_GetCurrentIpid - Return current internal PID
    IOCTL_IPR_SetAffinity - Update processor affinity
    IOCTL_IPR_SetPme - Update PME bit

    IOCTL_IPR_BdCreate - Buffer descriptor create
    IOCTL_IPR_BdDelete - Buffer descriptor delete
    IOCTL_IPR_BdSetNumCounters - Set number of perf. counters
    IOCTL_IPR_BdSetAction - Setup action to occur at particular time
    IOCTL_IPR_BdSetActiveCounters - Set active counter mask
    IOCTL_IPR_BdAddPcRange - Add a PC range
    IOCTL_IPR_BdSetSkipCount - Set number of interrupts to skip between samples

    IOCTL_IPR_SbCreate - Sample buffer create
    IOCTL_IPR_SbDelete - Sample buffer delete
    IOCTL_IPR_SbSetFull - Set full bit for sample buffer
    IOCTL_IPR_SbGetInfo - Returns info. about the sample buffer
    IOCTL_IPR_SbFind - Returns pointer to sample buffer

    IOCTL_IPR_HbElements - Returns number of elements required for given 
                           histogram parameters
    IOCTL_IPR_HbIndex - Returns array index for the specified value and 
                        counter for given histogram parameters
    IOCTL_IPR_HbValue - Return value at given histogram index
    IOCTL_IPR_HbMaxElements - Return maximum number of histogram elements
    IOCTL_IPR_HbCreate - Create histogram buffer
    IOCTL_IPR_HbDelete - Delete histogram buffer
    IOCTL_IPR_HbFind - Return next histogram buffer that matches given criteria
    IOCTL_IPR_HbRead -  Read given elements from histogram buffer
    IOCTL_IPR_HbClear - Clear given elements in histogram buffer
   
   */



/* The following is a clever way of changing the generic ioarg pointer into
   a somewhat readable state.  
   */

#define DChipInfo             	((IPR_ARGChipInfo *)ioarg)
#define DUpdatePERFMON        	((IPR_ARGUPerfmon *)ioarg)
#define DIpid 		      	((IPR_ARGIpid *)ioarg)
#define DAffinity		((IPR_ARGAffinity *)ioarg)
#define DPme			((IPR_ARGPme *)ioarg)
#define DBDCreate		((IPR_ARGBDCreate *)ioarg)
#define DBDelete		((IPR_ARGBDelete *)ioarg)
#define DBDNumctrs		((IPR_ARGBDNumctrs *)ioarg)
#define DBDAction		((IPR_ARGBDAction *)ioarg)
#define DBDActctrs		((IPR_ARGBDActctrs *)ioarg)
#define DBDPcrng		((IPR_ARGBDPcrng *)ioarg)
#define DBDSkip			((IPR_ARGBDSkip *)ioarg)
#define DSBCreate		((IPR_ARGSBCreate *)ioarg)
#define DSBDelete		((IPR_ARGSBDelete *)ioarg)
#define DSBSetFull		((IPR_ARGSBSetFull *)ioarg)
#define DSBGetInfo		((IPR_ARGSBGetInfo *)ioarg)
#define DSBFind			((IPR_ARGSBFind *)ioarg)
#define DHBElements		((IPR_ARGHBElements *)ioarg)
#define DHBMaxElem		((IPR_ARGHBMaxElem *)ioarg)
#define DHBIndex		((IPR_ARGHBIndex *)ioarg)
#define DHBValue		((IPR_ARGHBValue *)ioarg)
#define DHBCreate		((IPR_ARGHBCreate *)ioarg)
#define DHBDelete		((IPR_ARGHBDelete *)ioarg)
#define DHBFind			((IPR_ARGHBFind *)ioarg)
#define DHBRead			((IPR_ARGHBRead *)ioarg)
#define DHBClear		((IPR_ARGHBClear *)ioarg)
#define DGetBuffer		((IPR_ARGGetBuffer *)ioarg)
#define DCpuConf		((IPR_ARGCpuConf *) ioarg)
#define DAllocate		((IPR_ARGAllocate *)ioarg)
#define DDeallocate		((IPR_ARGDeallocate *)ioarg)

int ipr_ioctl(struct inode *inode, struct file *file, unsigned int iocmd, unsigned long ioarg)
{

   /* Process commands */
         
   switch( iocmd ){

     case IOCTL_IPR_GetChipInfo:
       {	
	   
	   DChipInfo->status = KIprGetChipInfo(&(DChipInfo->chip_info));
      	   break;
       }
     case IOCTL_IPR_UpdatePERFMON:
       {
	 
	   DUpdatePERFMON->status=KIprUpdatePerfmon(DUpdatePERFMON->cpu,
						    DUpdatePERFMON->arg1,
						    DUpdatePERFMON->arg2,
						    &(DUpdatePERFMON->retval));
	   break;
       }
     case IOCTL_IPR_GetCurrentIpid:
       {
       	   DIpid->status = KIprGetCurrentIpid(&(DIpid->Ipid));
	   break;
       }
     case IOCTL_IPR_SetAffinity:
       {
	   DAffinity->status = KIprSetAffinity(DAffinity->cpu,
					       DAffinity->setting);
	   break;
       }
     case IOCTL_IPR_SetPme:
       {
	   DPme->status = KIprSetPme(DPme->pid,DPme->state, DPme->child,
				     &(DPme->prev_state));
	   break;
       }
     case IOCTL_IPR_BdCreate:
       {
	   DBDCreate->status = KIprBdCreate(DBDCreate->cpu,
					    &(DBDCreate->retptr));
	   break;
       }
     case IOCTL_IPR_BdDelete:
       {
	   DBDelete->status = KIprBdDelete(DBDelete->cpu);
	   break;
       }
     case IOCTL_IPR_BdSetNumCounters:
       {
	   DBDNumctrs->status = KIprBdSetNumCounters(DBDNumctrs->cpu,
					     DBDNumctrs->numctrs);
	   break;
       }
     case IOCTL_IPR_BdSetAction:
       {
	   DBDAction->status = KIprBdSetAction(DBDAction->cpu,
					  DBDAction->action_type,
					  DBDAction->action,
					  DBDAction->targetroutine,
					  DBDAction->targetpid);
	   break;
       }
     case IOCTL_IPR_BdSetActiveCounters:
       {
	   DBDActctrs->status = KIprBdSetActiveCounters(DBDActctrs->cpu,
							DBDActctrs->ctrmask);
	   break;
       }
     case IOCTL_IPR_BdAddPcRange:
       {     
	   DBDPcrng->status = KIprBdAddPcRange(DBDPcrng->cpu,
					       &(DBDPcrng->pcrange));
	   break;
       }
     case IOCTL_IPR_BdSetSkipCount:
       {	
	   DBDSkip->status = KIprBdSetSkipCount(DBDSkip->cpu,
						DBDSkip->ctr,
						DBDSkip->skipvalue);
	   break;
       }

/* Sample buffer shells */
     case IOCTL_IPR_SbCreate:
       {
	   DSBCreate->status = KIprSbCreate(DSBCreate->cpu,
				       DSBCreate->size,
				       DSBCreate->numbuffs,
				       &(DSBCreate->descrip),
				       &(DSBCreate->retptr));
	   break;
       }
     case IOCTL_IPR_SbDelete:
       {
	   DSBDelete->status = KIprSbDelete(DSBDelete->cpu,DSBDelete->bufptr);
	   break;
       }
     case IOCTL_IPR_SbSetFull:
       {
	   DSBSetFull->status = KIprSbSetFull(DSBSetFull->bufptr,
					      DSBSetFull->full_val);
	   break;
       }
     case IOCTL_IPR_SbGetInfo:	
       {
	   DSBGetInfo->status = KIprSbGetInfo(DSBGetInfo->bufptr,
						 &(DSBGetInfo->descrip));
	   break;
       }
     case IOCTL_IPR_SbFind:
       {
	   DSBFind->status = KIprSbFind(DSBFind->cpu,
				     &(DSBFind->descrip),
				     DSBFind->previous_buffer,  
				     &(DSBFind->retptr));
	   break;
       }	
/* Histogram Buffer Shells */
     case IOCTL_IPR_HbMaxElements:
       {
	   DHBMaxElem->status = KIprHbMaxElements(&(DHBMaxElem->elem));
	   break;
       }
     case IOCTL_IPR_HbCreate:
       {
	   DHBCreate->status = KIprHbCreate(DHBCreate->cpu,
				       &(DHBCreate->descrip),
				       &(DHBCreate->retptr));
	   break;
       }
     case IOCTL_IPR_HbDelete:
       {
	   DHBDelete->status = KIprHbDelete(DHBDelete->cpu, DHBDelete->bufptr);
	   break;
       }
     case IOCTL_IPR_HbFind:
       {
	   DHBFind->status =  KIprHbFind(DHBFind->cpu,
					 DHBFind->previous_buffer,
					 &(DHBFind->descrip),
					 &(DHBFind->retptr));
	   break;
       }
     case IOCTL_IPR_HbRead:
       {
	   DHBRead->status = KIprHbRead(DHBRead->cpu,
					DHBRead->bufptr,
					DHBRead->start,
					DHBRead->cnt,
					DHBRead->retptr);
	   break;
       }
     case IOCTL_IPR_HbClear:
       {
	   DHBClear->status = KIprHbClear(DHBClear->cpu,
					  DHBClear->bufptr,
					  DHBClear->start,
					  DHBClear->cnt);
	   break;
       }
     case IOCTL_IPR_GetBuffer:
       {
           DGetBuffer->status = KIprGetBuffer(DGetBuffer->srcbuffer,
					      DGetBuffer->usrbuffer);
           break;
       }
     case IOCTL_IPR_CpuConf:
       {
	   DCpuConf->status = KIprGetSysInfo(&(DCpuConf->Usysstrc));
	   break;
       }
     case IOCTL_IPR_Allocate:
       {
	   DAllocate->status = KIprAllocate();
	   DAllocate->ipr_is_cp_mask = ipr_is_cp_mask;
	   DAllocate->ipr_cp_mask = ipr_is_cp_mask;
	   break;
       }
     case IOCTL_IPR_Deallocate:
       {
	   DDeallocate->status =KIprDeallocate();
	   break;
       }
     default:
       {
	   return IPR__FAILURE;
       }
   }
   return 0;

}



int ipr_close( struct inode *inode, struct file *filp )
{
  if (!InUse)
    {
      printk("Error: Tried to close non-open iprobe device.\n");
      return; 
    }

  MOD_DEC_USE_COUNT;
  InUse=FALSE;

} 

/* The FLUSH was added for the later kernels. ie, 2.1.12X and beyond */

struct file_operations ipr_fops = {
	NULL,		/* lseek */
	NULL,       	/* read */
	NULL,      	/* write */
	NULL,		/* readdir */
	NULL,		/* Poll */
	ipr_ioctl,	/* ioctl */
	NULL,		/* mmap */
	ipr_open,	/* just a selector for the real open */
	NULL,           /* Flush */
	ipr_close,	/* release */
	NULL		/* fsync */
};

/* I believe that this takes the place of ipr_init for modules. */
	
int init_module(void) 
{

 
  if (register_chrdev(MajorNumber, DEVICE_NAME, &ipr_fops))
      {
	/* If our default major number didn't work, automagically
	   assign one. 
	   */

	MajorNumber = register_chrdev(0,DEVICE_NAME, &ipr_fops);
	
	if (MajorNumber <= 0 )
	  {
	    printk("Failed to insert Iprobe/Linux Module.\n");
	    return -1;
	  }

      }
  
  /* Store a pointer to the old irq handler */

  temp_perf_irq = perf_irq;

  /* Install the new handler. */
  perf_irq = ipr_interrupt;
  

 return 0;
} 

void cleanup_module(void)
{
  
  perf_irq=temp_perf_irq;

  if (unregister_chrdev(MajorNumber, DEVICE_NAME)!=0)
    {
      printk("Could not remove Iprobe Module.\n");
    }
  else 
    {
      printk( "Iprobe Module Removed.\n");
    }


}



#define GET_BD_PTR(cnum) (pCPUDataArray [cnum])

unsigned int skipHist;

IPR_BUFDSC *pCPUDataArray[IPR_MAX_CPUS];


/* LINUX: This points to the current counter number. */

#define THIS_CTR() (regs->trap_a2)

/*LINUX: These SHOULD be right. */

#define THIS_PC() (regs->pc)
#define THIS_PS() (regs->ps)
#define THIS_CPU() (0)
#define THIS_PID() (current->pid)

/* LINUX: These MAY be right. */
#define THIS_PRIORITY() (current->priority)
#define THIS_IPL() (getipl())

#define BARRIER() mb()

/*LINUX: This SHOULD be replaced with gettimeofday */

#define THIS_TIME() (*(uint64 *) &xtime)

/* LINUX: These are all wrong. */
#define THIS_MODE() (mode_number(regs->ps, current->pid))

/************************************************************************************/
/* Name: mode_number                                                                */
/*                                                                                  */
/* Purpose:  This returns the number of the mode we are currently in.               */
/*                                                                                  */
/* Arguments: cdb:                                                                  */
/*            input_ps:                                                             */
/*            pid:                                                                  */
/*                                                                                  */
/* Notes:  This may not apply to linux.                                             */
/*                                                                                  */
/* Outputs:  None                                                                   */
/*                                                                                  */
/* Function Value: The number of the mode we are currently in.                      */
/*                                                                                  */
/************************************************************************************/                       


/* In OSF we will look for three states for starters with a possible fourth 
   later 0) Kernel Mode 1) User Mode 2) Idle State and 4) interupting an 
   interupt this would be nice if we have time.
*/


/* LINUX: We need an alternate way of doing this. */

int mode_number (uint64 ps, uint64 pid)
{

  /* How do we check if we are in the idle 'process'? */


   /* If we are in User mode, return 1 */
   /* Otherwise return 0, kernel space. */

   if (ps&8)
     return 1;

   return 0;

 }

/************************************************************************************/
/* Name: PCinRange                                                                  */
/*                                                                                  */
/* Purpose: This functions tells us whether or not we are skipping the current      */
/*          interrupt.                                                              */
/*                                                                                  */
/* Arguments: PCrangeBuf: This is a pointer to all of the given PC ranges.          */
/*            pc: This is the current pc.                                           */
/*                                                                                  */
/* Notes:  None.                                                                    */
/*                                                                                  */
/* Outputs:  None                                                                   */
/*                                                                                  */
/* Function Value: This returns the number of the first range in which the PC fits. */
/*                 or -1 if it fits in none of the ranges.                          */
/*                                                                                  */
/************************************************************************************/                       

IPR_BOOLEAN PCinRange(IPR_PBUFFER *PCrangeBuf,IN uint64 pc)
{
   register IPR_PC_RANGE *pRange;
   int first, middle, last;
   
   // If no ranges exist the exit
   if(!PCrangeBuf)
     return -1;

   pRange = (IPR_PC_RANGE *)(PCrangeBuf + 1);

   // For now we cannot assume that the PC range list is well behaved,
   // so return the first range into which the candidate PC fits.

   for (first= 0; first < PCrangeBuf->rangecnt; first++)
      if (pc >= pRange[first].StartAddress && 
	  pc <= pRange[first].EndAddress)
      {
	  return first;
      }
   // None found

   return -1;

}

/************************************************************************************/
/* Name: SkipInt                                                                    */
/*                                                                                  */
/* Purpose: This functions tells us whether or not we are skipping the current      */
/*          interrupt.                                                              */
/*                                                                                  */
/* Arguments: SkipBufPtr: This contains information about the previously skipped    */
/*                        interrupts.                                               */
/*            Counter: This is the number of the counter that interrupted.          */
/*                                                                                  */
/* Notes:  None.                                                                    */
/*                                                                                  */
/* Outputs:  None                                                                   */
/*                                                                                  */
/* Function Value: If we are supposed to skip this interrupt, return TRUE,          */
/*                 otherwise, return FALSE.                                         */
/*                                                                                  */
/************************************************************************************/                       

IPR_BOOLEAN SkipInt(IPR_IBUFFER *SkipBufPtr, int Counter)
{
   register IPR_ISKIP *pSkip;

   // NOTE: Following code reorged.
   if (!SkipBufPtr)
     {
       return FALSE;
     }

   if(!(SkipBufPtr->skipmask & (1 << Counter)))
     {
       return FALSE;
     }

   /* THIS IS REALLY UGLY CODE.  IT SHOULD BE REMOVED... Adding one to a pointer? */

   pSkip = (IPR_ISKIP *) (SkipBufPtr + 1);
   pSkip += Counter;

   // Log a sample only if the skip counter has reached threshold

   if (pSkip->intspersample &&
       pSkip[Counter].intssincelast < pSkip[Counter].intspersample)
   {
      pSkip[Counter].intssincelast++;
      return TRUE;
   }
   else
   {
      pSkip[Counter].intssincelast = 0;
      return FALSE;
   }
}


int ipr_do_signal(void *sarg1, void *sarg2)
{
    send_sig(SIGUSR1,caller,1);
}


/************************************************************************************/
/* Name: add_to_histogram                                                           */
/*                                                                                  */
/* Purpose:                                                                         */
/*                                                                                  */
/* Arguments: pHB:                                                                  */
/*            ctr:                                                                  */
/*            numcounters:                                                          */
/*            v:                                                                    */
/*                                                                                  */
/* Notes:  None.                                                                    */
/*                                                                                  */
/* Outputs:  None                                                                   */
/*                                                                                  */
/* Function Value: None.                                                            */
/*                                                                                  */
/************************************************************************************/                       

void add_to_histogram (IPR_HBUFFER *pHB, IPR_CTRNUMBER ctr,
                              IPR_COUNT numcounters, IPR_HISTVALUE v)
{
   int hidx;
   IPR_HISTELEM *pHE;
   IPR_STATUS status;

   pHE = (IPR_HISTELEM *) (pHB + 1);
   if (v != NON_DATA_VALUE)
   {
    status = IprHbIndex (v, ctr, numcounters, pHB -> minval, pHB -> pcgran,  &hidx);

      if (IPR_SUCCESS (status) && status != IPR__NOTINHIST && hidx < pHB -> elemcnt)
	{
	  pHE [hidx]++;
	}
      else
	{
         skipHist++;
	}
   }
}


/************************************************************************************/
/* Name: ipr_interrupt                                                              */
/*                                                                                  */
/* Purpose:  This is the performance interrupt handler.                             */
/*                                                                                  */
/* Arguments: irq: this is the number of the irq, irrelavent for performance        */
/*            dev_id: an unimportant id.                                            */
/*            regs: a pointer to all of the registers.                              */
/*                                                                                  */
/* Notes:  None.                                                                    */
/*                                                                                  */
/* Outputs:  None.                                                                  */
/*                                                                                  */
/* Function Value: It is bad to do all of this work in the interrupt.        */
/*                 However, if we don't, all things will start to skip.  Ie. */
/*		   We need all of this work to be atomic.                    */
/*                                                                           */
/************************************************************************************/                       



void ipr_interrupt(unsigned long vector, struct pt_regs *regs)  
{

   IPR_HBUFFER *pHB;
   IPR_BUFDSC *pBD;
   IPR_HISTVALUE datum;
   register int cpu_no;
   int hIndex, ridx;
   uint32 *Data32;
   uint64 *Data64, PCval;
   register unsigned int dummy;
   IPR_BOOLEAN collect_sample, collect_histograms;
   IPR_PC_RANGE *pRange;


   cpu_no=THIS_CPU();
   pBD = GET_BD_PTR(cpu_no);
   if(!pBD)
   {
     printk(KERN_DEBUG "BD Pointer is NULL.\n");
     return;
   }

   if(!(BIprIsSet(pBD->validcounters, THIS_CTR())))
   {	 
     //LINUX
     //     printk(KERN_DEBUG "validcounters:%d THIS_Contorl:%d\n",pBD->validcounters, THIS_CTR());

     return;
   }
   // Update the skip count and skip this interupt if necessary

   if(SkipInt (pBD->pIBuf, THIS_CTR()))
     {
       printk(KERN_DEBUG "We are supposed to skip this one.\n");
       return;
     }


   // determine if we are to log data for this PC value

   PCval = THIS_PC();
   collect_sample = pBD->pPBuf ? pBD->pPBuf->sample_always : TRUE;
   collect_histograms = collect_sample;
   pRange = pBD->pPBuf ? ((IPR_PC_RANGE *) (pBD->pPBuf + 1)) : NULL;



   // Do the address histograms first.  Stop searching the range list
   // when the PC is less than the start address, since the range list
   // is sorted by start address.

   for (ridx = PCinRange (pBD->pPBuf, PCval);
       (ridx>=0 && ridx<pBD->pPBuf->rangecnt
           && PCval>=pRange[ridx].StartAddress);
        ridx++)
   {

      // For now, since we have no guarantees about the end address order,
      // check the end address before proceeding with saving the data.


      if (PCval <= pRange[ridx].EndAddress)
      {

         // Add to the address histogram if it exists

         pHB = pRange[ridx].pHBuf;
         if (pHB)
            add_to_histogram (pHB, THIS_CTR(), pBD -> numcounters, PCval);

         // Update the general collection flags

         collect_histograms = collect_histograms || pRange[ridx].histogram;
         collect_sample = collect_sample || pRange[ridx].sample;
      }
   }



   if (collect_histograms && pBD -> pHBuf)
   {
     //
     // CREATING HISTOGRAMS
     //

     //
     // NOTE: The code below assumes that the buffers are linear, i.e.
     //       that the last one points to null.
     //
     //       Also note that the data array is organized with all the elements
     //       corresponding to a single value are located contiguously.
     //       For example, an element sequence on a chip with three counters
     //       might be val 1 ctr 0, val 1 ctr 1, val 1 ctr 2, val 2 ctr 0,
     //       val 2 ctr 1, val 2 ctr 2, etc.
     //
     //       All subscripting is done manually.
     //

        // Histogram buffers to fill in?

      for (pHB = pBD -> pHBuf;
           pHB && pHB -> item.ipr_hmask_l_bits != IPR_HMASK_M_ADDRESS;
           pHB = pHB -> pNext)
      {
         switch (pHB -> item.ipr_hmask_l_bits)
         {
            case IPR_HMASK_M_TOTAL :
               datum = 0;
               break;
            case IPR_HMASK_M_IPL :
               datum = THIS_IPL();
               break;
            case IPR_HMASK_M_MODE :
               datum = THIS_MODE();
               break;
            case IPR_HMASK_M_ADDRESS :
               datum = THIS_PC();
               break;
            case IPR_HMASK_M_PRIORITY :
               datum = THIS_PRIORITY();
               break;
            default :
               datum = NON_DATA_VALUE;
               break;
         }

         add_to_histogram (pHB, THIS_CTR(), pBD -> numcounters, datum);
      }
   }      

   // Sample buffers to fill in?

   if (collect_sample && pBD -> pSBuf)
   {
      ////////////////////////
      // COLLECTING SAMPLES //
      ////////////////////////

      IPR_SBUFFER *BufPtr;
      IPR_SBUFFER *marked_buffer;
      IPR_SBUFFER *wptr;
      IPR_BOOLEAN dropSamp;
      unsigned int
         mask32,
         mask64,
         bmask;

      dropSamp = FALSE;
      marked_buffer = NULL;
      BufPtr = pBD -> pSBuf;
      if (BufPtr -> areaoffset >= BufPtr -> areasize)
      {
         //
         // The location is out of range, which means we have finished with
         // this buffer.  Mark it and try to move on to the next buffer.
         //
         if (BufPtr -> areaoffset == BufPtr -> areasize)
         {
	   BARRIER ();
            BufPtr -> full = IPR_BUFFER_FULL;
            BufPtr -> areaoffset++;
            marked_buffer = BufPtr;
         }
         //
         // The current buffer is full.  Check if the next buffer can
         // be used.
         //
         if (BufPtr -> pNext -> full == IPR_BUFFER_FULL)
         {
            // The next buffer is full, so count a missed sample

            BufPtr->missedcnt++;

            if (marked_buffer)
               do_full_action (marked_buffer, pBD, cpu_no);

            return;
         }

         // The next buffer is empty, so switch buffers

         BufPtr = BufPtr -> pNext;
         BufPtr -> full = IPR_BUFFER_EMPTY;
         BufPtr -> begintime = THIS_TIME();
         BufPtr -> missedcnt = 0;
         BufPtr -> areaoffset = 0;
         pBD -> pSBuf = BufPtr;
      }
      //
      // Save the requested information.  Start with the quadword data
      // to allow alignment easily.
      //
      Data64 = (uint64 *) ((char *) ((BufPtr) + 1) + BufPtr->areaoffset);

      // Convert the masks to non-overlapping form

      mask32 = BufPtr->data.ipr_dmask_l_bits;
      mask64 = mask32 & BufPtr->dataquad.ipr_dmask_l_bits;
      mask32 &= ~mask64;

      // Walk the list of data bits in the quadword mask

      /* THIS SHOULD BE REWRITTEN..... Something LIKE this... Phil */
      /*
	


	if (bmask & IPR_DMASK_M_TIME)  
        *Data64++ = THIS_TIME();     

	if (bmask & IPR_DMASK_M_PS)  
        *Data64++ = THIS_PS();     
	
	if (bmask & IPR_DMASK_M_PC)
            *Data64++ = THIS_PC();

	if (bmask & IPR_DMASK_M_COUNTER)
            *Data64++ = THIS_CTR();

        if (bmask & IPR_DMASK_M_PID)
            *Data64++ = THIS_PID();

        if (bmask & IPR_DMASK_M_PRIORITY)
	*Data64++ = THIS_PRIORITY();
       
	if (bmask & IPR_DMASK_M_TIME & IPR_DMASK_M_PS & IPR_DMASK_M_PC & 
	    IPR_DMASK_M_COUNTER & IPR_DMASK_M_PID & IPR_DMASK_M_PRIORITY)
            *Data64++ = 0;

	  */


      for (bmask = BIprLowest (mask64); bmask;
           mask64 = BIprClr (mask64, bmask), bmask = BIprLowest (mask64))
      {
         if (bmask == IPR_DMASK_M_TIME)
            *Data64++ = THIS_TIME();
         else if (bmask == IPR_DMASK_M_PS)
            *Data64++ = THIS_PS();
         else if (bmask == IPR_DMASK_M_PC)
            *Data64++ = THIS_PC();
         else if (bmask == IPR_DMASK_M_COUNTER)
            *Data64++ = THIS_CTR();
         else if (bmask == IPR_DMASK_M_PID)
            *Data64++ = THIS_PID();
         else if (bmask == IPR_DMASK_M_PRIORITY)
            *Data64++ = THIS_PRIORITY();
         else
            *Data64++ = 0;
      }

      // Walk the list of data bits in the longword mask


      Data32 = (uint32 *) Data64;
      for (bmask = BIprLowest (mask32); bmask;
           mask32 = BIprClr (mask32, bmask), bmask = BIprLowest (mask32))
      {
         if (bmask == IPR_DMASK_M_TIME)
            *Data32++ = THIS_TIME();
         else if (bmask == IPR_DMASK_M_PS)
            *Data32++ = THIS_PS();
         else if (bmask == IPR_DMASK_M_PC)
            *Data32++ = THIS_PC();
         else if (bmask == IPR_DMASK_M_COUNTER)
            *Data32++ = THIS_CTR();
         else if (bmask == IPR_DMASK_M_PID)
            *Data32++ = THIS_PID();
         else if (bmask == IPR_DMASK_M_PRIORITY)
            *Data32++ = THIS_PRIORITY();
         else
            *Data32++ = 0;
      }
      dummy = ((char *) Data32 + BufPtr->pad - (char *) (BufPtr + 1));
      BufPtr -> areaoffset = dummy;

      // Notify the process if necessary

       if (marked_buffer)
	 {
	   do_full_action (marked_buffer, pBD, cpu_no);
	 }
   }  // end - sample code
   
   return;
}

/*****************************************************************************/
/* Name: do_full_action                                                      */
/*                                                                           */
/* Purpose: This sends a signal back to "iprobe" that we need flush the      */
/*          buffers.                                                         */
/*                                                                           */
/* Arguments: pSB: Sample Buffer (Contains a chunk of samples. )             */
/*            bD:Buffer Descriptor                                           */
/*            cdb: Which CPU this is full for.                               */
/*                                                                           */
/* Notes:  None.                                                             */
/*                                                                           */
/* Outputs:  None                                                            */
/*                                                                           */
/* Function Value: None.                                                     */
/*                                                                           */
/*****************************************************************************/

void do_full_action (IPR_SBUFFER *pSB, IPR_BUFDSC *bD, IPR_CPUNUM cdb)
{	
    switch(bD -> fullaction)
    {
    case IPR_FULL_WAKE:
    case IPR_FULL_AST:
	{
	  send_sig(SIGUSR1,caller,1);
	  break;
	}
    case IPR_FULL_NONE:
	break;
    }

}



