/****************************************************************************/
/*                                                                          */
/*  COPYRIGHT (c) 1993-1999 BY                                              */
/*  Digital Equiptment/Compaq Computer Corporation.                         */
/*  ALL RIGHTS RESERVED.                                                    */
/*                                                                          */
/*  This file is part of the Iprobe performance suite for Alpha processors. */
/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 2 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License       */
/*  along with this program; if not, write to the Free Software             */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307    */
/*                                                                          */
/****************************************************************************/


/*
 * Abstract:	This file contains the Linux-dependent kernel
 *		routines needed to support the API 
 *
 *                      Phil Ezolt	19-Aug-1998
 *             This is the osf version ported to linux.
 *             
 *
 */

/* This is for the current process info. */
#include <linux/sched.h>
/* This is information about the CPU. */
#include <asm/hwrpb.h> 


/* This contains information about the version of the system and the hostname.*/
#include <linux/utsname.h>


/* This contains the size of the memory in the machine. */
#include <linux/mm.h>

/* This has information about how to allocate kernel memory. */
#include <linux/malloc.h>

#include <asm/system.h>

#include "ipr_kernel_int_defs.h"
#include"ipr_api_int_defs.h"
#include "ctr_int_defs.h"
#include "stdtypes.h"


/* This contains informations about the hardware that we are running on. */

extern struct hwrpb_struct *hwrpb; 
/* This is a pointer to the "iprobe" process.  We send a signal to it
   when buffers are full. */

struct task_struct * caller=NULL;

/* This flag tells us when we first allocate the counters for our use. */

static int64 IPR_IS_ALLOCATED=-1;

/* This appears to be the CPU masks. */
unsigned int ipr_cp_mask=0, ipr_is_cp_mask=0;


/**********Begin Non-Paged memory Routines. *****************/

/* Routine to initialize the non paged pool zones at startup */

extern IPR_STATUS IprInitMemory(void)
{
    int cnt;
    for(cnt=0;cnt <IPR_MAX_NPP_BLOCKS;cnt++)
	    npp_table.start[cnt]=NULL;
    return IPR__SUCCESS;
}    


/* Routine to Allocate non-paged memory */
extern IPR_STATUS ALONONPAGED(IN int bytecnt, IN IPR_BOOLEAN allocZone,
		       OUT int *retsize, OUT IPR_BUFHDR **areaptr)
{
  IPR_STATUS Status;

  /*LINUX:  This function used to round to the nearest page.  It is not a good idea to do
            that in linux. 
	    I don't know if GFP_KERNEL is the proper priority.. if this called from the interrupt 
	    handler, GFP_ATOMIC would be better. 
  */

  /*
  printk("ALNONPAGED: This is the bytecnt: %d\n", bytecnt);
  */

  if(!areaptr){
    /*
    printk("ALONONPAGED Error: areaptr is null\n");
    */

    return IPR__NULLPTR;
  }
  if(!retsize){
    /*
	printk("ALONONPAGED Error: retsize is null\n");
	*/
	return IPR__NULLPTR;
    }

  /* Allocate the wired down non-pageable memory  */
  *areaptr = (IPR_BUFHDR*)kmalloc(bytecnt,GFP_KERNEL);


/* return the size of mem allocated rounded to the nearest page */
    if (*areaptr != 0) {
 	*retsize = (int)bytecnt;
	Status = AddToPoolList(*areaptr);
	if(!IPR_SUCCESS(Status))
	    return Status;
	return	IPR__SUCCESS;
    }
    else {
	*retsize = 0;
	return IPR__ALLOCFAIL;
    }

}
 
/* Routine to free non-paged memory */

extern IPR_STATUS DEANONPAGED(IN IPR_BUFHDR *areaptr)
{
    int cnt;
    
    if(!areaptr){

//        printk("DEANONPAGED Error: areaptr is null\n");

        return IPR__NULLPTR;
    }
    
    
    /* Check to see if the block is allocated for a single NPP block
       --- If not it is part of a zone */

    for(cnt=0; cnt < IPR_MAX_NPP_BLOCKS; cnt++){
    
      if(npp_table.start[cnt] == areaptr)
	{
	  
	  /*       found a block, deallocate it directly from NPP
		   Free the memory of size ssize bytes
		   */
	  
	  kfree(areaptr);
	  
	  npp_table.start[cnt] = NULL;
	  
	  return IPR__SUCCESS;
        }
      
    }

//    printk(" DEANONPAGED ERROR: Block not a known block\n");
    return IPR__FAILURE;       
    
}

/***** END NON-PAGED MEMORY ROUTINES ******/


/* This is the wrapper function to set the pme bit for a process
 * in this case state is crucial for getting or setting the PME bit.
 * if the state is passed in as -1 then the pme bit for the process 
 * is obtained. Otherwise state is used to set or clear the PME bit 
 */

/* NOTE: Linux doesn't have any PME bit functionality.  */
/*       This function is simply a dummy function.      */


/* DONE */

extern IPR_STATUS KIprSetPme(IN unsigned int pid, IN int state, 
			     IN IPR_BOOLEAN children, OUT IPR_BOOLEAN *p_state)
{
  return IPR__SUCCESS;
}


/*** END PME BIT FUNCTIONS ****/


/* Find out what chip we are running on */
/* :12/13/97:phil: This fills in the Given ChInfo structure with information 
   about the chip that we are running on. 
   */


/* DONE */
extern IPR_STATUS KIprGetChipInfo(OUT IPR_CHIPINFO *ChInfo)
{
  /* This Little Magic comes from /usr/src/linux/arch/alpha/kernel/setup.c */  
  struct percpu_struct *CpuInfo; 
  CpuInfo = (struct percpu_struct*)((char*)hwrpb + hwrpb->processor_offset); 


  /* SHADY VALUES: The minor number appears to be bogus, however it is unused by */
  /* Higher levels of the API.   The Cpuid seems to think I am a EV4.  Which I think is ok. */

  ChInfo->majorIdent = CpuInfo->type;
  ChInfo->minorIdent = CpuInfo->variation;

  return IPR__SUCCESS;
}

/* This routine updates the performance counters using a pal-code call */ 

/* DONE */
extern IPR_STATUS KIprUpdatePerfmon(IN IPR_CPUNUM cpu,
				    IN uint64 arg1, IN uint64 arg2,
				    OUT uint64 *retptr)
{
    *retptr = wrperfmon(arg1,arg2);  
    return IPR__SUCCESS;
}
    


/* Get the PID of the current process. */

/* DONE */
extern IPR_STATUS KIprGetCurrentIpid(OUT unsigned int *Ipid)
{
  (*Ipid) = current->pid;
  return IPR__SUCCESS;
}

/* Shell routine for setting affinity to a particular pocess */
/* 12/13/97:phil: This is irrelavent right now, seeing as AXP/Linux
                  does not support SMP.*/

/* DONE */
IPR_STATUS KIprSetAffinity(IN IPR_CPUNUM cpu, IN IPR_BOOLEAN setting)
{
  return IPR__SUCCESS;
}

/* routine for allocating the ipr device to the counters */
 
IPR_STATUS KIprAllocate (void)
{
   IPR_STATUS status;

   int i=0; 

   //   printk(KERN_CRIT "This is IPR_IS_ALLOCATED:%d.\n",IPR_IS_ALLOCATED);

    if(IPR_IS_ALLOCATED == -1){
	status =  IprInitMemory();
	IPR_IS_ALLOCATED = 0;
	for(i=0;i<IPR_MAX_CPUS;i++) pCPUDataArray[i]=NULL;   
    }
    if (IPR_IS_ALLOCATED == 0)
	    status = KIprGetCurrentIpid((uint *)&IPR_IS_ALLOCATED);
    else
	    return IPR__UNAVAILABLE;

    caller = current;
    ipr_is_cp_mask = 1;
    ipr_cp_mask = 1;

    return IPR__SUCCESS;
}

/* This deallocates all of the structures that were allocated above. */

IPR_STATUS KIprDeallocate (void) 
{
  IPR_IS_ALLOCATED = -1;
  
  caller = NULL;

  return IPR__SUCCESS;
}

/*****************************************************************************/
/* Name: KIprGetSysInfo */ 
/*  */
/* Purpose: This will fill the system specific information for the machine. */
/* */
/* Arguments:  Usysstrc: The structure where the system info will be held.   */
/* */
/* Output:  Usysstrc: The structure where the system info will be held.   */
/* */
/* Function Value: This contains whether or not the call succeeded or failed.*/
/* */
/* Comments:  This should be TRIVIAL to port to linux. The relevant structure*/
/* to be filled is in:   ipr_api_base_defs_linux.h Just use /proc! */
/* */
/*****************************************************************************/
     

extern IPR_STATUS KIprGetSysInfo(IPR_CPU_CONF *SystemInfo)
{

  /* This contains information about the size of the physical memory. */

  struct sysinfo SystemInformation;

  /* This Little Magic comes from /usr/src/linux/arch/alpha/kernel/setup.c */  
  struct percpu_struct *CpuInfo; 
  CpuInfo = (struct percpu_struct*)((char*)hwrpb + hwrpb->processor_offset); 

  
  /* This fills in the information about the memory of the machine. */
  si_meminfo(&SystemInformation);
  

  /* SHADY VALUES: The minor number appears to be bogus, however it is unused by */
  /* Higher levels of the API.   The Cpuid seems to think I am a EV4.  Which I think is ok. */

  SystemInfo->cpu_maj = CpuInfo->type;
  SystemInfo->cpu_min = CpuInfo->variation;

  SystemInfo->sys_type = hwrpb->sys_type;
  SystemInfo->cur_phys_mem = SystemInformation.totalram;
  
  SystemInfo->clockspeed = hwrpb->cycle_freq;

  SystemInfo->page_sz = hwrpb->pagesize;

    /* NOTE: Right now this is hard coded to one, because AlphaLinux/SMP
       doesn't exist yet.  
       */

    SystemInfo->num_cpus = 1;
    SystemInfo->active_cpu_count = 1;


    strncpy(SystemInfo->hardware_name,system_utsname.machine,SYS_NMLN);
    strncpy(SystemInfo->sysname,system_utsname.nodename,SYS_NMLN);
    strncpy(SystemInfo->Osname,system_utsname.sysname,SYS_NMLN);
    strncpy(SystemInfo->Osrelease,system_utsname.release,SYS_NMLN);
    strncpy(SystemInfo->Osver,system_utsname.version,SYS_NMLN);

    return IPR__SUCCESS;

}

extern IPR_STATUS KIprHbValidRange(IN IPR_HDESC *descrip)
{
  return IPR__SUCCESS;
}

