#ifndef __IPREDUCE_LOADED
#define __IPREDUCE_LOADED 1
/****************************************************************************/
/*                                                                          */
/*  COPYRIGHT (c) 1993-1999 BY                                              */
/*  Digital Equiptment/Compaq Computer Corporation.                         */
/*  ALL RIGHTS RESERVED.                                                    */
/*                                                                          */
/*  This file is part of the Iprobe performance suite for Alpha processors. */
/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 2 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License       */
/*  along with this program; if not, write to the Free Software             */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307    */
/*                                                                          */
/****************************************************************************/

/*
 *  Patent Pending, Digital Equipment Corporation
 *
 *  Module:   IPREDUCE.H
 *
 *  Version:  NONE
 *
 *
 *  Abstract: This header file contains definitions used in the sampling data
 *            reduction program IPREDUCE.
 *
 *  Author:         Brian F. Wibecan
 *  Creation Date:  20-Jun-1995
 *
 *  Modified by:
 *
 *  NONE        Nelson Montalvo    
 *
 *  X-1   BFW   Brian F. Wibecan    20-Jun-1995
 *    Original version.
 *
 */

 /*** #includes ***/

#include <time.h>
#include <stdio.h>
#include <stdlib.h>

#ifdef nt
#define WIN32_LEAN_AND_MEAN 1
#include <windows.h>
#endif

#include "stdtypes.h"
#include "ipr_api_ext_defs.h"
#include "ipr_sample.h"
#include "ipr_events.h"
#include "getopt.h"

/*** #defines ***/

#define HELP_INDENT     10
#define MAX_HELP_LENGTH 75

#define ADDRFILEDEF "address.res"

#ifdef nt
#define IPLNAME     "irql"
#define IPLNAMEC    "IRQL"
#else
# define IPLNAME     "ipl"
# define IPLNAMEC    "IPL"
#endif

#define BREAK_RANGE(s, r1, r2)      BreakGenericRange(s, r1, r2, 0)
#define BREAK_HEX_RANGE(s, r1, r2)  BreakGenericRange(s, r1, r2, HEXBASE)

/* Need to make sure strings get converted to 64-bit values. */

#ifdef vms
# define STRTOUL strtoull
#else
# define STRTOUL strtoul
#endif

#define BADTIME(TimeString) { \
  fprintf(stderr, "Bad time string format: \"%s\", ignored\n", TimeString); \
  return 0; }

#define HANDLE FILE *

#define FILE_SIGNATURE "IPROBE"
#define SEPCHAR ':'

#define OS_VMS  1
#define OS_OSF  2
#define OS_NT   3
#define OS_LINUX 4


#define EOF_STATUS          IPR__INVALID
#define OVERFLOW_STATUS     IPR__UNAVAILABLE

#define BLOCK_SIZE          512
#define BUFFER_SIZE         256
#define INSTRUCTION_SIZE    4
#define NUMBER_SIZE         32

#define ELEMENT_CHUNK       512
#define IMAGE_CHUNK         16
#define FUNCTION_CHUNK      128

#define STATISTICS_BARSIZE  100
#define PROFILE_BARSIZE     50

#define PAGE_MASK           8191
#define CACHE_SHIFT         5

#define GLOBAL_INDENT       2
#define MAX_CCTRS           8
#define NAME_SIZE           40
#define HEXBASE             16

#define ARGS_DONE           -2
#define ARGS_INVALID        -1
#define ARGS_HELP            2
#define ARGS_ADDRESS_NEEDED  1
#define ARGS_NOADDRESS       0

#define ADDRESS_DIGITS 16

/* For scanning in and outputting the addresses in hex. */

#ifdef nt
#define PC_RANGE_INPUT "%I64x:%I64x"
#define SCAN_INPUT  "%s %s %I64x %I64x"
#define SCAN_OUTPUT "%016I64X"
#endif

#ifdef unix
#define PC_RANGE_INPUT "%16lx:%16lx"
#define SCAN_INPUT  "%s %s %16lx %16lx"
#define SCAN_OUTPUT  "%016lX"
#endif

#ifdef vms
#define PC_RANGE_INPUT "%16Lx:%16Lx"
#define SCAN_INPUT  "%s %s %Lx %Lx"
#define SCAN_OUTPUT "%016LX"
#endif

#ifdef vms

/* "Int is interrupt mode and Null is idle mode. */

#define MODE_LIST_INITIALIZER {"Kernel", "Exec", "Super", "User", "Int","Null", "\?\?\?\?"}
#define DATE_TIME_FORMAT "|!DB-!MAAU-!Y4|!H04:!M0:!S0.!C4|"
#endif

#ifdef unix
#define MODE_LIST_INITIALIZER {"Kernel", "User", "Idle", "\?\?\?\?"}
#endif

#ifdef nt
#define MODE_LIST_INITIALIZER {"Kernel", "User", "Idle", "\?\?\?\?"}
#endif

#define ERRMSG(x)     fprintf(stderr, x)
#define STRNEQ(s1,s2) (strcmp(s1, s2) != 0)
#define STREQ(s1,s2)  (strcmp (s1, s2) == 0)

/* make_ftype creates a new typedef with all structure elements having the */
/* same type fdtype.                                                       */

#define make_ftype(fname,fdtype) \
typedef struct _ ## fname \
{ \
   fdtype Total; \
   fdtype Time; \
   fdtype PC; \
   fdtype IPL; \
   fdtype Mode; \
   fdtype PID; \
   fdtype Counter; \
   fdtype CPU; \
   fdtype Cache; \
   fdtype Filter; \
   fdtype Entries; \
   fdtype Events; \
   fdtype Priority; \
} fname

/* Make two types... */

make_ftype(filtertype,IPR_COUNT);
make_ftype(filterflags,int64);


/*** typedefs ***/

/* The following enum enumerates each possible argument into an index. */
/* See the function GetCommandArgs to get an idea of how this is used. */

typedef enum
{
  AI_HELP,          /* -h               */
  AI_QUESTION_MARK, /* -?               */
  AI_INPUT,         /* -input_file      */
  AI_OUTPUT,        /* -output_file     */
  AI_ADDRESS,       /* -address_file    */
  AI_REPORT,        /* -report_type     */
  AI_R,             /* -r               */
  AI_DISPLAY,       /* -display_options */
  AI_TIME_BEGIN,    /* -tb              */
  AI_TIME_END,      /* -te              */
  AI_PC,            /* -pc              */
  AI_ROUTINE,       /* -routine         */
  AI_IPL,           /* -ipl             */
  AI_IRQL,          /* -irql            */
  AI_MODE,          /* -mode            */
  AI_PRIORITY,      /* -pri             */
  AI_PID,           /* -pid             */
  AI_COUNTER,       /* -counter         */
  AI_C,             /* -c               */
  AI_EVENT,         /* -event           */
  AI_CPU,           /* -cpu             */
  AI_CACHE_BLOCK,   /* -cache_block     */
  AI_CB,            /* -cb              */
  AI_ENTRY,         /* -entry_point     */
  AI_PTHRESH,       /* -pthresh         */
  AI_THRESH,        /* -thresh          */
#ifdef DEBUG
  AI_FORCE,
#endif
  AI_MAX,
  AI_COUNT
} argument_index;

                                      /* The struct option is defined in  */
                                      /* getopts.h and is used for        */
typedef struct option argument_type;  /* obtaining command line arguments.*/


/* Sample information is stored in this structure. */

typedef struct _datarecord
{
  uint64 Time;
  uint64 PC;
  uint64 PSval;

  uint32 IPL;
  uint32 Mode;
  uint32 PID;
  uint32 Counter;
  uint32 CPU;
  uint32 Cache;
  uint32 Priority;
} datarecord;


/* File header for an IPROBE sample file. */

typedef struct _fileheader
{
                          /* File header signature identifying IPROBE sample */
  char Signature[8];      /* files. 					     */
  uint64 CpuSpeed;        /* Cpu speed of current system.                    */
  uint64 TimeStamp;       /* Starting time in VMS format.                    */
  uint64 CpuType;         /* Alpha chip type.                                */

  int32 NumberOfCounters; /* The number of counters present on the system.   */
  IPR_CTRMASK CounterMask;/* Counter mask on current system.                 */
  int32 AvailableCpuMask; /* CPU mask for current system.                    */

  int32 OSType;           /* OS_VMS, OS_NT, OS_OSF1, etc.                    */
  int32 OSMajorVersion;   /* OS major version number.                        */
  int32 OSMinorVersion;   /* OS minor version number.                        */

  time_t StartTime;       /* Starting time in UNIX format.                   */

  int32 DataType;         /* Sampling data, histogram data, etc.             */
  IPR_COUNT RangeCount;   /* Number of ranges.                               */
  int32 NullProcessID;    /* Null process ID for current system.             */

  IPR_DMASK Data;
  IPR_DMASK DataQuad;
} fileheader;


typedef struct _filenames
{
  char *pAddressFileName;
  char *pSamplesFileName;
  char *pOutputFileName;
} filenames;   


/* Each function (entry point) will have information stored */
/* in a functiontype structure.                             */

/* In the following image is used very loosely to refer to the shared */
/* library, program, or kernel that a function "belongs" to.          */

typedef struct _functiontype
{
   uint64 BaseAddress;  /* Base address of this routine.         */
   char *FunctionName;  /* Name of the routine.                  */
                        /* Index in an image arry. Which library */
   int32 ImageIndex;    /*  does function belong to?             */
   IPR_COUNT *Counter;  /* Number of samples per HW counter.     */
} functiontype;


typedef struct _functionsinfo
{
  functiontype *pFunctions;
  int64 FunctionsArraySize;
  int64 MaxFunctionsArraySize;
} functionsinfo;


typedef struct _imagetype
{
   uint64 BaseAddress;  /* Base address of the image.        */
   char *ImageName;     /* Name of the image.                */
   int32 NewIndex;      /* Used in permutation inversion.    */
   IPR_COUNT *Counter;  /* Number of samples per HW counter. */
} imagetype;


typedef struct _imagesinfo
{
  imagetype *pImages;
  int64 ImagesArraySize;
  int64 MaxImagesArraySize;
  uint64 MaxImagesStringSize;
} imagesinfo;


typedef struct _piddata
{
  struct
  {
    uint32 MinPID;
    uint32 MaxPID;
  } *Range;

  int64 Count;
} piddata;


typedef struct _reporttype
{
  boolean ProfileReport;    /* Print a profile report?    */
  boolean StatisticsReport; /* Print a statistics report? */
} reporttype;


typedef struct _samplingdata
{
  int64 BufferCount;
  int64 MissedSampleCount;
  int64 MaxRecordCount;
  int64 MinRecordCount;

  uint64 SamplingStartTime;
  uint64 SamplingStopTime;

  filtertype *pCounterSpecificTotals;
  filtertype SamplingTotals;
  IPR_COUNT MissedSampleBreakdown[IPR_MAX_CPUS];

  IPR_COUNT PreviousMissedSampleCount;
} samplingdata;


typedef struct _selections
{
  datarecord MinSelections;
  datarecord MaxSelections;

  uint64 ModeSelections;

  /* Selections for final output. */

  reporttype ReportSelections;

  piddata PIDSelections;

  IPR_CTRMASK CounterSelections;
  IPR_CPUMASK CPUSelections;
  
  double ReportPctThreshold;
  IPR_COUNT ReportThreshold;

  filterflags DataDisplayFlags;
  filterflags Filters;

  /* Function or image name used to limit the PC range examined. */

  char *pUserSpecifiedFunction;

  IPR_COUNT NumberOfCounters;
  IPR_COUNT EntryInstructionRange;

  IPR_COUNT ForcedCpuCount;
  IPR_COUNT NumberOfCpus;

  IPR_EVENTSET *EventRequests;

} selections;


typedef struct _valelement
{
  uint64 HistogramValue;  /* Histogram element value.                 */
  uint32 SampleCount;     /* Number of samples in this statistics bar.*/
} valelement;


typedef struct _valarray
{
  valelement *pElem;    /* Bars.                  */
  uint32 ECount;        /* Number of bars.        */
  uint32 MaxECount;     /* Size of the bar array. */
} valarray;

typedef struct _dataarrays
{
  valarray *PIDArray;
  valarray *IPLArray;
  valarray *PRIORITYArray;
  valarray *MODEArray;
  valarray *COUNTERArray;
  valarray *CPUArray;
  valarray *CACHEArray;
  valarray *PCArray;
} dataarrays;

/* Variable part of the sample file header */

typedef IPR_EVENT FHact_event;
typedef IPR_FREQ FHact_freq;
typedef int32 FHskip;
typedef uint64 FHrange;

typedef struct _fileinfo
{
  FHact_event *pFHact_events;
  FHact_freq *pFHact_freqs;
  FHskip *pFHskips;
  FHrange *pFHranges;
} fileinfo;


#ifdef vms
#include "psldef.h"

#pragma member_alignment save
#pragma member_alignment

typedef union psldef PSL;
typedef union _psl_quad
{
   PSL xpsl;
   uint64 xquad;
} PSL_QUAD;

#pragma member_alignment restore
#endif


/*** Function Prototypes ***/

extern char *DateToString(uint64 Date);
extern uint64 StringToDate(char *pDateString);
extern uint64 GetModeSelection(char *pModeRequest);
extern char *ParseDataArea(IPR_SBUFFER *pBufHdr, datarecord *pRecord, char *pBuf);
extern void ParsePS(datarecord *pRecor, fileheader *pFileHeader);

IPR_STATUS GetRecord (FILE * pfile, datarecord *pRecord, fileheader *pFileHeader);

/* Specific to ipreduce.c */

static void BreakAddressRange(char *pAddressString, uint64 *pMinRange, uint64 *pMaxRange);
static void BreakGenericRange(char *pRangeString, uint32 *pMinRange, uint32 *pMaxRange, uint64 Base);
static IPR_COUNT CalculateDataCategoryThreshold(const valarray *pValueArray);
static uint64 ConvertHex(char *AddressString);
static void CopyString(char **pDestination, const char *pSource);
static int64 FilterSum(filtertype ISample, filterflags *ilters);
static functiontype *FindFunction(uint64 PC);
static void FPrintAddress(FILE * pOutputFile, uint64 Address);
static int FunctionsBsearchCompare(functiontype *pKey, functiontype *pCmp);
static int FunctionsQsortCompare(const void *pFirstFunction, const void *pSecondFunction);
static IPR_STATUS GetAddressRecord(FILE * pInputFile, char **pImageName, char **pFunctionName, uint64 *pStartingAddress, uint64 *pEndingAddress);
static int64 GetCommandArgs(int argc, char **argv);
static int ImagesQsortCompare(const void *pFirstImage, const void *pSecondImage);
static void InitStatsArray(valarray StatsArray[], uint64 ArraySize);
static void InitStatsArrays(uint64 ArraySize);
static void InvertPermutation(imagetype *ImagesArray, int64 ImageSize);
static IPR_BOOLEAN MatchesPID(uint64 MatchPID);
static FILE * OpenDataFile (char *FileName, fileheader *pHdr);
static int64 ParseDataDisplayFlags(char *pDisplayArguments, argument_type *pDisplays);
static double PercentValue(uint64 Numerator, uint64 Denominator);
static void PrintAddress(uint64 Address);
static void PrintBar(FILE * pOutputFile, double xpct, int64 BarSize);
static void PrintBox(FILE * pOutputFile, char *pOutputString, char *pOutputString2);
static char *PrintEventName(IPR_EVENT Event);
static void PrintHelp(const char *pHelpString);
static void PrintRange(FILE * pOutputFile, char *pOutputString, uint64 MinValue, uint64 MaxValue);
static void PrintSuppressReason(int64 EventIndex);
static IPR_STATUS ProcessAddressFile(FILE * pAddressFile);
static int64 ReadVariableData(FILE * pInputFile, void *pBuffer, int64 SizeOfDataToRead);
static uint64 SignExtendAddress(uint64 Address);
static void StoreArray(valarray *pValues, uint64 StorageValue, char *ArrayName);
static void ValidateDataDisplayFlags(filterflags *pCollected, filterflags *pDataDisplayFlags);
static void ValidateFilter(filterflags *pFilters, filterflags *pCollected, fileheader *pFileHeader, FHact_event *pEvents);
static void WriteCountersUsed(FILE *pOutputFile, uint64 CounterMask);
static void WritePercentBar(FILE * pOutputFile, uint64 Numerator, uint64 Denominator, int64 BarSize);
static IPR_STATUS WriteProfileReport(FILE * pOutputFile, fileheader *pFileHeader, int64 EventID, const char *Event);
static IPR_STATUS WriteReportHeading(FILE * pOutputFile, fileheader *pFileHeader, FHact_event *pEvents, uint64 FMask);
static IPR_STATUS WriteStatisticsReport(FILE * pOutputFile, const fileheader *pFileHeader, int64 EventID, const char *Event);
static void WriteStatsLines(FILE * pOutputFile, const char *pPrintFormat, const valarray *pValueArray, uint64 TotalNumberOfEvent);
static void WriteStatsNamesLines(FILE * pOutputFile, const char *PrintFormat, const valarray *ValueArray, const char **Names, uint64 NumberOfNames, uint64 Total);


/*** Global Data Accessor Functions ***/

/* Data File Info */

static fileinfo *GetFileInfoPointer(void);
static FHact_event *GetFHact_events(void);
static void MallocFHact_events(uint64 MallocSize);
static FHact_freq *GetFHact_freqs(void);
static void MallocFHact_freqs(uint64 MallocSize);
static FHskip *GetFHskips(void);
static void MallocFHskips(uint64 MallocSize);
static FHrange *GetFHranges(void);
static void MallocFHranges(uint64 MallocSize);

/* File Names */

static filenames *GetFileNamesPointer(void);
static char *GetAddressFileName(void);
static char *GetOutputFileName(void);
static char *GetSamplesFileName(void);
static void SetAddressFileName(const char *pNewFileName);
static void SetOutputFileName(const char *pNewFileName);
static void SetSamplesFileName(const char *pNewFileName);

/* Functions */

static functionsinfo *GetFunctionsPointer(void);
static IPR_STATUS Add2Functions(char *pNewFunctionName, uint64 Base, int64 ImageIndex);
static functiontype *GetFunctions(void);
static int64 GetFunctionsArraySize(void);

/* Images */

static imagesinfo *GetImagesPointer(void);
static IPR_STATUS Add2Images(char *pNewImageName, uint64 BaseAddress);
static imagetype *GetImages(void);
static int64 GetImagesArraySize(void);
static uint64 GetMaxImagesStringSize(void);

/* Sampling data */

static samplingdata *GetSamplingDataPointer(void);
static int64 GetBufferCount(void);
static filtertype *GetCounterSpecificTotals(void);
static int64 GetMaxRecordCount(void);
static int64 GetMinRecordCount(void);
static IPR_COUNT *GetMissedSampleBreakdown(void);
static IPR_COUNT GetMissedSampleCount(void);
static IPR_COUNT GetPreviousMissedSampleCount(void);
static uint64 GetSamplingStartTime(void);
static uint64 GetSamplingStopTime(void);
static filtertype *GetSamplingTotals(void);
static void MallocCounterSpecificTotals(uint64 MallocSize);
static void SetBufferCount(int64 NewBufferCount);
static void SetMaxRecordCount(int64 NewMaxRecordCount);
static void SetMinRecordCount(int64 NewMinRecordCount);
static void SetMissedSampleCount(int64 NewMissedSampleCount);
static void SetPreviousMissedSampleCount(IPR_COUNT NewPreviousMissedSampleCount);
static void SetSamplingStartTime(uint64 NewSamplingStartTime);
static void SetSamplingStopTime(uint64 NewSamplingStopTime);

/* Selections */

static selections *GetSelectionsPointer(void);
static IPR_CTRMASK GetCounterSelections(void);
static IPR_CPUMASK GetCPUSelections(void);
static filterflags *GetDataDisplayFlags(void);
static IPR_COUNT GetEntryInstructionRange(void);
static IPR_EVENTSET *GetEventRequests(void);
static filterflags *GetFilterFlags(void);
static IPR_COUNT GetForcedCpuCount(void);
static datarecord *GetMinSelections(void);
static datarecord *GetMaxSelections(void);
static uint64 GetModeSelections(void);
static IPR_COUNT GetNumberOfCounters(void);
static IPR_COUNT GetNumberOfCpus(void);
static piddata *GetPIDSelections(void);
static boolean GetProfileReportSelection(void);
static double GetReportPctThreshold(void);
static IPR_COUNT GetReportThreshold(void);
static reporttype *GetReportSelections(void);
static boolean GetStatisticsReportSelection(void);
static char *GetUserSpecifiedFunction(void);
static void MallocEventRequests(uint64 EventSetSize);
static void SetCounterSelections(IPR_CTRMASK NewCounterSelections);
static void SetCPUSelections(IPR_CPUMASK NewCPUSelections);
static void SetEntryInstructionRange(IPR_COUNT NewEntryInstructionRange);
static void SetModeSelections(uint64 NewModeSelections);
static void SetNumberOfCounters(IPR_COUNT NewNumberOfCounters);
static void SetNumberOfCpus(IPR_COUNT NewNumberOfCpus);
static void SetReportPctThreshold(double NewReportPctThreshold);
static void SetReportThreshold(IPR_COUNT NewReportThreshold);
static void SetProfileReportSelection(boolean NewSelection);
static void SetStatisticsReportSelection(boolean NewSelection);
static void SetUserSpecifiedFunction(char *pNewUserSpecifiedFunction);

/* Stats arrays. */

static dataarrays *GetStatsArraysPointer(void);
static valarray **GetPIDArray(void);
static valarray **GetIPLArray(void);
static valarray **GetPRIORITYArray(void);
static valarray **GetMODEArray(void);
static valarray **GetCOUNTERArray(void);
static valarray **GetCPUArray(void);
static valarray **GetCACHEArray(void);
static valarray **GetPCArray(void);

#endif /* ifndef __IPREDUCE_LOADED */


