#ifndef __IPR_HDR_LOADED
#define __IPR_HDR_LOADED
/****************************************************************************/
/*                                                                          */
/*  COPYRIGHT (c) 1993-1999 BY                                              */
/*  Digital Equiptment/Compaq Computer Corporation.                         */
/*  ALL RIGHTS RESERVED.                                                    */
/*                                                                          */
/*  This file is part of the Iprobe performance suite for Alpha processors. */
/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 2 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License       */
/*  along with this program; if not, write to the Free Software             */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307    */
/*                                                                          */
/****************************************************************************/
/*
 * Program: 
 *
 *
 * Abstract:	This contains the structure used for storing the state of
 *		iprobe. Some fields in this structure are filled up by OS
 *		specific routines and they are available in ipr_os_vms.c.
 * 
 * Author: 
 *
 * Creation Date: 
 * 
 * Modification History:
 *   Phil Ezolt, 11-Jul-1997
 *     Update Function names to new coding spec.
 *
 *   Ravindra Kulkarni, X-Aug-1994  
 *     Included the file iprobe_struct.h
 *
 *   Ravindra Kulkarni, X-Apr-1994
 *      Wrote original code.
 *
 */

#include "iprobe_struct.h"

/* function prototypes */
extern int32 ReadUserOptions(IPR_EVENT *Events,       /* events to sample */
			     IPR_FREQ *Frequiencies,         /* frequency of sampling */
			     IPR_COUNT *Skips,        /* skip value vector */
			     int32 NumberOfEvents,           /* length of prev 3 vectors */
			     IPR_MMASK ModeMask,      /* modes to measure in */
			     IPR_DMASK DataMask,      /* data mask for collecting samples */
			     IPR_DMASK QuadDataMask,  /* data mask for collecting samples */
			     IPR_COUNT BufferCount,  /* # of bufs specified */
			     IPR_COUNT BufferSize,   /* size of buffers specified */
			     time_t Delay,            /* start delay */
			     time_t Duration,         /* run duration */
			     IPR_COUNT Interval,      /* output interval */
			     struct pc_ranges *PCRange, /*  PC range information    */
			     char *OutputFilename,           /* outfile name */
			     IPR_COUNT method,        /* measuring method */
			     char *CommandLine);      /* command to issue */

extern int32 DisplayUserOptions(FILE *FilePointer);
extern void DisplaySystemHeader(FILE *FilePointer);
extern int32 WriteHeaderInfo(void);

typedef struct system_info_t{

  /* Time stamps */
  uint64 timestamp;       /* High-accuracy, OS-specific format time stamp */
  uint64 cpu_type;
  time_t start_time;
  
  /* Range descriptions (keep rangeptr aligned on 64-bit boundary) */
  uint64 *rangeptr;       /* range pointer  */
  IPR_COUNT rangecnt;       /* range information */

  /*  Machine specific information */

  char *nodename;         /* node name of the current system */
  char *osname;           /* OS name and version of current system */
  char *modelname;        /* The name of current model */
  int32 NumCpus;            /* Number of CPU's present on current system */
  int32 memorysize;         /* The physical number of pages available */
  int32 pagelength;         /* The size of each page in bytes */
  int32 num_ctrs;           /* The number of counters present on the system */
  uint64 cpu_speed;       /* cpu speed of current system */

  /*  Counter specific information */

  IPR_EVENT *sel_events;    /* events selected by user */
  IPR_FREQ *sel_freqs;      /* frequencies selected by the user */
  int32 NumEvents;          /* Number of events selected */
  IPR_EVENT *act_events;    /* actual selected events on the current system
                               These are as returned by pfmEventCounterMap()
			       */
  IPR_FREQ *act_freqs;      /* actual selected freqs on the current system
                               These are as returned by pfmFreqCounterMap() */

  IPR_CTRMASK counter_mask; /* counter mask on current system */
  int32 null_proc_id;       /* Null process id for current system */
  int32 avail_cpu_mask;     /* cpu mask for current system */

  /*  command line arguments */

  time_t delay;           /* Amount of delay in seconds */
  time_t duration;        /* Duration of intended operation */
  IPR_COUNT buffer_count;   /* Number of buffers used */
  IPR_COUNT buffer_size;    /*  size of sample buffers used */
  IPR_COUNT interval;       /* The interval for counting mode */
  char *outfile;          /* Output file name */
  IPR_COUNT method;         /* Method of measurement */
  char *CommandLine;		/* command line to issue */
  time_t start;           /* Start time of measurement */
  time_t cur_time;        /* Present time */
  IPR_COUNT *skips;         /* Sample skip info */
  IPR_COUNT *act_skips;         /* Sample skip info */
  IPR_MMASK modemask;      /* modes to measure in */
  IPR_DMASK datamask;      /* data mask for collecting samples. */
  IPR_DMASK quaddatamask;      /* data mask for collecting samples. */
  struct pc_ranges *pc_range; /*  PC ranges entered by user */
  
  /*   OS  related information */

  int32 OS_type;           /* OS_VMS, OS_NT, OS_OSF1 etc */
  int32 OS_ver_major;      /* OS major version number */
  int32 OS_ver_minor;      /* OS minor version number */

  /* Signature for sample file */
  char signature[8];
} system_info;


#endif /* ifndef __IPR_HDR_LOADED */





