#ifndef __IPR_KEYLIB_LOADED
#define __IPR_KEYLIB_LOADED
/****************************************************************************/
/*                                                                          */
/*  COPYRIGHT (c) 1993-1999 BY                                              */
/*  Digital Equiptment/Compaq Computer Corporation.                         */
/*  ALL RIGHTS RESERVED.                                                    */
/*                                                                          */
/*  This file is part of the Iprobe performance suite for Alpha processors. */
/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 2 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License       */
/*  along with this program; if not, write to the Free Software             */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307    */
/*                                                                          */
/****************************************************************************/

/*    Program: iprobe_keylib.h
 *
 *    Abstract: Structures and prototypes for the keyword table manipulation routines.
 *
 *    Author: Greg Tarsa, 
 *
 *    Creation Date: Jun-94
 *
 *    Modification History:
 *
 *    Phil Ezolt: 11-Jul-1997
 *     Update Function names to new coding spec.
 *
 *    Ravindra Kulkarni: Jun-94,
 *         Added keywords T_COLLECT, T_NOCOLLECT, T_ADD
 *
 *    Greg Tarsa:  May-94
 *         Author.
 */


#include "ipr_api_ext_defs.h"
#include "getopt.h"

/* This should probably be "void *" */
#ifndef ADDR
#define ADDR uint64
#endif

/* option types */
#define T_NONE       0       /* no associated value */
#define T_REQ_NUM    1       /* has required numeric value */
#define T_REQ_FILE   2       /* has required file name */
#define T_REQ_METH   3       /* has required method keyword */
#define T_REQ_MODE   4       /* has required mode keyword */
#define T_REQ_RSPEC  5       /* has required range spec */
#define T_REQ_STRING 6       /* has required string argument */

/* option attributes */
#define A_NOATTR    0       /* no special attrs */
#define A_INVISIBLE (1<<1)      /* is invisible */
#define A_REPEATING (1<<2)      /* can be repeated */

struct keywords
    {
    char *name;     /* text name of option */
    int32 num;       /* internal code for option */
    uint32 type;      /* arg type of option */
    int32 attr;      /* attributes (i.e., visible, repeating) */
    char *def_val;  /* default value of argument, if applicable */
    char *desc;     /* short description of option */
    };

/*
    pc_ranges are stored in a singly linked list.
*/
struct pc_ranges
    {
    ADDR start;
    ADDR end;
    uint64 granularity;
    struct pc_ranges *next;
    };

/* define the acceptable keyword tags */
#define T_BUFFER_COUNT   1
#define T_BEGIN_TIME     2
#define T_END_TIME   3
#define T_INTERVAL   4
#define T_OUTPUT     5
#define T_METHOD     6
#define T_MODE       7
#define T_HELP       8
#define T_QUIET      9
#define T_DEBUG     10
#define T_PERCPU    11
#define T_PEREVENT  12
#define T_RANGE     13
#define T_BUFSIZ    14
#define T_INPUT     15
#define T_COLLECT   16
#define T_NOCOLLECT 17
#define T_ADD       18
#define T_COMMAND   19
#define T_NO_INTERMEDIATE_RESULTS 20

/* define the acceptable methods */
#define METH_SAMPLE (1<<1)
#define METH_COUNT  (1<<2)
#define METH_TOTAL  (1<<3)
#define METH_IPL    (1<<4)
#define METH_MODE   (1<<5)
#define METH_ADDR   (1<<6)
#define METH_PRIORITY (1<<7)

#define METH_ALL ( METH_TOTAL | METH_IPL | METH_MODE | METH_ADDR | \
    METH_PRIORITY )
/* define the acceptable modes */
#define MODE_DEFAULT    0
#define MODE_K      IPR_MMASK_M_KERNEL
#define MODE_E      IPR_MMASK_M_EXECUTIVE
#define MODE_S      IPR_MMASK_M_SUPERVISOR
#define MODE_U      IPR_MMASK_M_USER
#define MODE_PAL    IPR_MMASK_M_PAL
#define MODE_SEL    IPR_MMASK_M_SELECTED

/* function prototypes */
struct keywords *FindByName(struct keywords *table, char *ostr);
struct keywords *FindByNumber(struct keywords *table, int32 num);
struct keywords *FindOptionByName(char *ostr);
struct keywords *FindOptionByNumber(int32 num);
struct keywords *FindMethodByName(char *ostr);
struct keywords *FindMethodByNumber(int32 num);
struct keywords *FindModeByName(char *ostr);
struct keywords *FindModeByNumber(int32 num);
int32 CaseInsensitiveStringCompare(char *s1, char *s2);
int32 ParseEventString(char *src_espec, IPR_EVENT *evtptr, IPR_FREQ *freqptr, IPR_COUNT *skipptr);
int32 ParsePCRange(char *src_pspec, ADDR *begaddr, ADDR *endaddr, uint64 *granularity);

#endif // ifndef __IPR_KEYLIB_LOADED
