#ifndef __IPR_STRUCT_LOADED
#define __IPR_STRUCT_LOADED
/****************************************************************************/
/*                                                                          */
/*  COPYRIGHT (c) 1993-1999 BY                                              */
/*  Digital Equiptment/Compaq Computer Corporation.                         */
/*  ALL RIGHTS RESERVED.                                                    */
/*                                                                          */
/*  This file is part of the Iprobe performance suite for Alpha processors. */
/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 2 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License       */
/*  along with this program; if not, write to the Free Software             */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307    */
/*                                                                          */
/****************************************************************************/

/*
 *   Program name: iprobe_struct.h
 *
 *   Abstract: Data structures and definitions for the IPROBE program.
 *
 *   Modification History:
 *
 *    Phil Ezolt: 11-Jul-1997
 *     Update Function names to new coding spec, removed some unecessary #ifdefs.
 *
 *    Ravindra Kulkarni: Sep-94
 *     Changed default granularity to 2 from 1
 *
 *    Ravindra Kulkarni: Aug-94
 *     Added conditional inclusion of ipr_struct.h itself.
 *     Included time.h in place of sys/time.h in case of NT.
 *     Added macros for printf formats.
 *     Defined CVTINPADDR as strtoq in case of vms and strtoul in case of
 *     osf and nt.
 *     Redefined access as _access in case of nt.
 *
 *    Ravindra Kulkarni: Jun-94
 *     Added buffer_size argument in do_sampling.
 *
 *    Ravindra Kulkarni: May-94
 *     Added min_value in the declaration of do_histogram.
 *
 *    Greg Tarsa: May-94
 *     Added declarations for multiple method support and mode support.
 *
 *    Ravindra Kulkarni: Apr-94
 *     Moved the struct sys_info to ipr_hdr_vms.h
 *     Included the file ipr_hdr_vms.h
 *
 *    Greg Tarsa: Mar-94
 *     Added ipr__dbg flag.
 *     Removed EVTARRSIZE and friends.
 *     Added prototype for set_default_events.
 *
 *    Ravindra Kulkarni: Feb-94
 *     Fixed prototype defs; Added to eventdesc structure.
 *
 *    Greg Tarsa: Dec-93
 *     Wrote original code.
 *
 */

#define OS_VMS 1
#define OS_OSF1 2
#define OS_NT 3
#define OS_LINUX 4

#if defined vms || defined unix
#include <sys/time.h>
#endif

#ifdef nt
#include <time.h>
#include <windows.h>
#include <wchar.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include "iprobe_keylib.h"
#include "ipr_api_ext_defs.h"
#include "ipr_events.h"

#define MAX_ELEMENTS 2560

typedef volatile uint32 FLAG;
typedef unsigned long FLAGS;

#define FLAG_CHAR1 '-'

#define EVT_SEP '.'     /* eventspec field separator */

/* limit values */
#define MAX_EVENTSPECS IPR_MAX_COUNTERS
#define MIN_BUFFER_COUNT 1
#define MAX_BUFFER_COUNT 2048   /* arbitrary */
#define MIN_INTERVAL 1
#define MAX_INTERVAL 32767
#define MIN_TIME 0
#define MAX_TIME ((unsigned) -1)

/* default values for Command line options */
#define DEF_EVENT       IPR_EVT_CYCLES
#define DEF_BUFFER_COUNT    3
#define DEF_BUFFER_SIZE     8192
#define DEF_BEGIN_TIME      0
#define DEF_END_TIME        0
#define DEF_INTERVAL        1
#define DEF_OUTPUT      "pcsample.dat"
#define DEF_INPUT       "iprobe.cmd"
#define DEF_METHOD      METH_COUNT
#define DEF_MODE        0 /* this may not be valid */
#define DEF_IPR_DMASK    (0XFF)
#define DEF_GRANULARITY     2

/* Define the command line (CL) keyword bit flags */
#define CL_BUFFER_COUNT (1<<1)
#define CL_BEGIN_TIME   (1<<2)
#define CL_END_TIME     (1<<3)
#define CL_INTERVAL     (1<<4)
#define CL_OUTFILE      (1<<5)
#define CL_METHOD       (1<<6)
#define CL_MODE         (1<<6)
#define CL_HELP         (1<<7)
#define CL_QUIET        (1<<8)
#define CL_PEREVENT     (1<<9)
#define CL_RANGE        (1<<10)
#define CL_BUFSIZ       (1<<11)
#define CL_INPUT        (1<<12)
#define CL_COLLECT      (1<<13)
#define CL_NOCOLLECT    (1<<14)
#define CL_ADD          (1<<15)

/*  define the bit flags for data mask. */
#define CL_TIME         (1<<1)

#define DEF_END_ADDRESS FREQ_DEF

#define FREQ_OFF    0
#define FREQ_LOW    IPR_FREQUENCY_LOWEST
#define FREQ_HIGH   IPR_FREQUENCY_HIGHEST
#define FREQ_MIN    (1<<31)
#define FREQ_MAX    (1<<1)

/*  This code takes care of different formats on vms osf1 and nt */

#if defined vms || defined nt
#define FREQ_DEF    0XFFFFFFFF
#define PRINT_FORMAT1 " %12Ld %9Ld"
#define ADDRFORMAT "%x"
#define PRINT_FORMAT2 " %12Ld"
#define PRINT_FORMAT3 "%x"
#endif

#ifdef unix
#define FREQ_DEF 0XFFFFFFFFFFFFFFFF
#define PRINT_FORMAT1 " %14lu %9lu"
#define ADDRFORMAT "%lx"
#define PRINT_FORMAT2 " %14lu"
#define PRINT_FORMAT3 "%lx"
#define DEFAULT_DATA ( IPR_DMASK_M_COUNTER | IPR_DMASK_M_PID )
#define DEFAULT_QUADDATA (IPR_DMASK_M_PS | IPR_DMASK_M_PC)
#endif

#ifdef vms
#define CVTINPADDR strtoq
#define DEFAULT_DATA (IPR_DMASK_M_PS | IPR_DMASK_M_PC | IPR_DMASK_M_COUNTER |\
                      IPR_DMASK_M_PID)
#define DEFAULT_QUADDATA (0)
#endif

#if defined unix || defined nt
#define CVTINPADDR strtoul
#endif

#ifdef nt
#define DEFAULT_DATA ( IPR_DMASK_M_PS | IPR_DMASK_M_PC | IPR_DMASK_M_COUNTER |\
     IPR_DMASK_M_PID )
#define DEFAULT_QUADDATA (0)
#define access _access
#endif


/* Following definitions are to be deleted after they have been defined in API */

struct eventspec    {
    char *name;     /* string name of event */
    IPR_EVENT num;    /* numeric name of event */
    IPR_FREQ freq;    /* frequency to measure event */
    };

struct eventdesc    {
    char *name;     /* string name of event */
    IPR_EVENT num;    /* numeric name of event */
    };

/* global debug flag */
extern int64 ipr__dbg;

/* function prototypes */

IPR_EVENT FindEventNumberByName(char *ename, IPR_BOOLEAN print_message);
IPR_STATUS DoSampling (IPR_EVENT *selected_evts,    /* array of evts to measure */
		       IPR_FREQ *selected_freqs,    /* freqs to measure at */
		       IPR_COUNT *selected_skips,   /* which interrupts to measure */
		       IPR_COUNT selection_count,   /* size of the prev 2 arrays */
		       IPR_MMASK modes2use,    /* prog modes to measure in */
		       IPR_DMASK datamask,     /*  data mask to collect samples */
		       IPR_DMASK quaddatamask,     /*  data mask to collect quad samples */
		       time_t start_time,     /* # secs till start of meas */
		       time_t end_time,       /* # secs to measure */
		       IPR_COUNT buffer_count,  /* number of buffers to use */
		       IPR_COUNT buffer_size,   /* size of buffers to use */
		       IPR_COUNT range_count,        /* Number of pc ranges */
		       struct pc_ranges *pc_range, /* Pointer to addr range */
		       char *sfile_name);     /* sampling file name */

IPR_STATUS DoCounting (IPR_EVENT *selected_evts, /* array of evts to measure */
		       IPR_FREQ *selected_freqs, /* freqs to measure at */
		       IPR_COUNT selection_count, /* size of the prev 2 arrays */
		       IPR_MMASK modes2use, /* prog modes to measure in */
		       time_t start_time,  /* # secs till start of meas */
		       time_t end_time,    /* # secs to measure */
		       time_t interval    /* output interval */
		       );

int32 ProcessArguments(int32 argc, char **argv, /* command line info */
		       FLAGS *flags,      /* ret flags for which options set */
		       IPR_COUNT *buffer_count, /* ret # of buffers specified */
		       IPR_COUNT *buffer_size,  /* ret size of buffers specified */
		       time_t *start,     /* ret # of secs till start */
		       time_t *finish,        /* ret # of secs to measure */
		       IPR_COUNT *interval,     /* ret length of interval */
		       char **outfile,        /* ret name of sampling file */
		       FLAGS *method,     /* ret method of measuring */
		       IPR_MMASK *modemask,    /* ret prog modes to measure */
		       IPR_DMASK *datamask,    /* data mask for collecting samples */
		       IPR_DMASK *quaddatamask,    /* data mask for collecting samples */
		       struct pc_ranges *pc_range,/* ptr to list of pc_ranges */
		       IPR_EVENT *ret_evtarr,   /* ret array of events to measure */
		       IPR_FREQ *ret_freqarr,   /* ret array of freqs to measure at */
		       IPR_COUNT *ret_skiparr,  /* ret array of skip values */
		       IPR_COUNT *ret_arrsize, /* ret length of prev 2 arrays */
		       char *CommandLine,	/* pointer to command line buffer */
		       size_t CommandLineLength); /* usable size of CommandLine */

 void WaitForTime(time_t wait_time) ; /* sleep until wait_time */

/*
 Signal handling routines that have OS-dependent implementation.
 See ipr_os_osf1.c, ipr_os_vms.c, and ipr_os_nt.c for details.
*/

IPR_STATUS OSDeclareWakeUpHandler(FLAG *flag);
IPR_STATUS OSDeclareCntrlCHandler(FLAG *flag);
IPR_STATUS OSDeclarePFMHandler(FLAG *flag);
void OSCancelWakeUpHandler();
void OSCancelCntrlCHandler();
void OSCancelPFMHandler();
IPR_STATUS OSScheduleWakeUpTime(time_t wait_time);
void OSWaitForSignal();
IPR_STATUS OSClearRepeatingWakeUp();
IPR_STATUS OSScheduleRepeatingWakeUp (time_t delta_time);
void OSGetSystemInformation(void);
IPR_STATUS IssueCommandLine(char *);
void InitializeModes(void);
void UpdateTime(void);

char **OSGetModeNames(void);
struct keywords *OSGetModeTable(void);
struct keywords *OSGetSampleDataTable(void);
struct keywords *OSGetSampleQuadDataTable(void);


IPR_STATUS DoHistogram (IPR_HMASK hist_type,  /* histogram type */
			IPR_EVENT *selected_evts,     /* array of evts to measure */
			IPR_FREQ *selected_freqs,     /* freqs to measure at */
			IPR_COUNT selection_count,    /* size of the prev 2 arrays */
			IPR_MMASK modes2use,     /* prog modes to measure in */
			IPR_COUNT start_time,     /* # secs till start of meas */
			IPR_COUNT end_time,       /* # secs to measure */
			IPR_COUNT interval,       /* Output interval */
			uint32 range_count,   /*  Number of pc ranges */
			struct pc_ranges *pc_range,/*  Pointer to the actual ranges */
			int32 histogram_count);      /* number of histograms entered */

int bstreql(char *s1, char *s2);

#endif // ifndef __IPR_STRUCT_LOADED

