/****************************************************************************/
/*                                                                          */
/*  COPYRIGHT (c) 1993-1999 BY                                              */
/*  Digital Equiptment/Compaq Computer Corporation.                         */
/*  ALL RIGHTS RESERVED.                                                    */
/*                                                                          */
/*  This file is part of the Iprobe performance suite for Alpha processors. */
/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 2 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License       */
/*  along with this program; if not, write to the Free Software             */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307    */
/*                                                                          */
/****************************************************************************/
/*
 * Program: iprobe_glocal.c
 *
 * Abstract: Functions that are wrappers for former global variables.       
 *
 * Author: Phillip Ezolt
 *
 * Creation Date: May-94
 *
 * Modification History:
 *
 */

#ifdef nt
#define WIN32_LEAN_AND_MEAN 1
#endif
#include "stdtypes.h"
#include "iprobe_hdr.h"

/************************************************************************************/
/* Name:  GetSystemInformation                                                      */
/*                                                                                  */
/* Purpose:  This is a wrapper for a former global variable.                        */
/*                                                                                  */
/* Arguments:  None.                                                                */
/*                                                                                  */
/* Outputs: None                                                                    */
/*                                                                                  */
/* Function Value: A pointer to the system information.                             */
/*                                                                                  */
/************************************************************************************/


system_info *GetSystemInformation(void){
  static system_info SystemInformation;
  return &SystemInformation;
}

/************************************************************************************/
/* Name:  OSGetModeTable                                                            */
/*                                                                                  */
/* Purpose:  This is a wrapper for a former global variable.                        */
/*                                                                                  */
/* Arguments:  None.                                                                */
/*                                                                                  */
/* Outputs: None                                                                    */
/*                                                                                  */
/* Function Value: A pointer to the FLAG that determines whether or not the child   */
/*                 process has exited.                                              */
/*                                                                                  */
/************************************************************************************/
FLAG *GetStartedProcessExited(void)
{
  static FLAG StartedProcessExited = 0;

  return &StartedProcessExited;
}

/************************************************************************************/
/* Name: GetStartedCommandPidPointer																*/
/*                                                                                  */
/* Purpose: Return a pointer to the started command process PID							*/
/*                                                                                  */
/* Arguments:  None.                                                                */
/*                                                                                  */
/* Outputs: None                                                                    */
/*                                                                                  */
/* Function Value: a pointer to the started process PID										*/
/*                                                                                  */
/************************************************************************************/
int *GetStartedCommandPidPointer(void)
{
  static int StartedProcessPid = 0;

  return &StartedProcessPid;
}

/************************************************************************************/
/* Name:  GetTotalNumberOfCounters                                                  */
/*                                                                                  */
/* Purpose:  This is a wrapper for a former global variable.                        */
/*                                                                                  */
/* Arguments:  None.                                                                */
/*                                                                                  */
/* Outputs: None                                                                    */
/*                                                                                  */
/* Function Value: A pointer to the total number of on-chip counters.               */
/*                                                                                  */
/************************************************************************************/

uint32 *GetTotalNumberOfCounters(void)
{
  static uint32 TotalNumberOfCounters;
  return &TotalNumberOfCounters;
}

/************************************************************************************/
/* Name:  GetTotalNumberOfCPUs                                                      */
/*                                                                                  */
/* Purpose:  This is a wrapper for a former global variable.                        */
/*                                                                                  */
/* Arguments:  None.                                                                */
/*                                                                                  */
/* Outputs: None                                                                    */
/*                                                                                  */
/* Function Value: A pointer to the total number of CPUs.                           */
/*                                                                                  */
/************************************************************************************/

uint32 *GetTotalNumberOfCPUs(void)
{
  static uint32 TotalNumberOfCPUs;
  return &TotalNumberOfCPUs;
}

/************************************************************************************/
/* Name:  GetOptionTable                                                            */
/*                                                                                  */
/* Purpose:  This is a wrapper for a former global variable.                        */
/*                                                                                  */
/* Arguments:  None.                                                                */
/*                                                                                  */
/* Outputs: None                                                                    */
/*                                                                                  */
/* Function Value: A pointer to the table of options.                               */
/*                                                                                  */
/************************************************************************************/
/* Table of option keywords */
struct keywords *GetOptionTable(void){

  static struct keywords OptionTable[] =
    {
    { "buffer_count", T_BUFFER_COUNT, T_REQ_NUM, A_NOATTR, "3",
        "number of buffers to allocate for PC samples (sampling only)"},

    { "debug",        T_DEBUG,    T_NONE,    A_INVISIBLE, "",
        "Internal option to incr debug flag by one"},

    { "delay_start",  T_BEGIN_TIME,   T_REQ_NUM, A_NOATTR, "0",
        "# of seconds between tool start and start of measurements"},

    { "duration",     T_END_TIME, T_REQ_NUM, A_NOATTR, "0",
        "length of time, in seconds, for tool to measure"},

    { "help",     T_HELP,     T_NONE,    A_NOATTR, NULL,
        "Display event, help, switch and/or usage help"},

    { "interval",     T_INTERVAL, T_REQ_NUM, A_NOATTR, "1",
        "number of seconds between reports (counting only)"},

    { "method",       T_METHOD,   T_REQ_METH,A_REPEATING, "count",
        "type of measurement to perform"},

    { "command",       T_COMMAND,   T_REQ_STRING, A_NOATTR , "",
        "command line for starting a process to profile"},

    { "mode",     T_MODE,     T_REQ_MODE,A_REPEATING, "all",
        "record events in the specified mode"},

    { "output",       T_OUTPUT,   T_REQ_FILE,A_NOATTR, "pcsample.dat",
        "name of output file in which to place information"},

    { "pc_range",     T_RANGE,    T_REQ_RSPEC,A_REPEATING, "0-;8192",
        "add an addr range and granularity to the addr histogram list"},

    { "per_cpu",      T_PERCPU,   T_NONE,    A_INVISIBLE, "",
        "Print counts all events on one line, with one line for each CPU"},

    { "per_event",    T_PEREVENT, T_NONE,    A_INVISIBLE, "",
        "Print counts all cpus on one line, with one line for each event"},

    { "quiet",            T_QUIET,    T_NONE,    A_NOATTR, "",
        "Do not print machine/argument summary prior to measurement"},

    { "no_intermediate",            T_NO_INTERMEDIATE_RESULTS,    T_NONE,    A_NOATTR, "",
        "Do not print intermediate results during measurement (applies only to counting at present)"},

    { "buffer_size",  T_BUFSIZ,  T_REQ_NUM, A_NOATTR,    "page length",
        "Size of sample buffers to be allocated"},

    { "input",        T_INPUT,    T_REQ_FILE,A_NOATTR, "iprobe.cmd",
        "name of file to receive command line arguments from "},

    { "collect", T_COLLECT, T_REQ_FILE, A_NOATTR, "",
        "Collect the following data in sampling"},

    { "nocollect", T_NOCOLLECT, T_REQ_FILE, A_NOATTR, "",
        "Do not collect the following data in sampling"},

    { "add", T_ADD, T_REQ_FILE, A_NOATTR, "",
        "Collect the following data in addition to default data"},

    {0,          0,      0,  0,  0, 0 }
    };

  return OptionTable;

}

/************************************************************************************/
/* Name:  GetMethodTable                                                            */
/*                                                                                  */
/* Purpose:  This is a wrapper for a former global variable.                        */
/*                                                                                  */
/* Arguments:  None.                                                                */
/*                                                                                  */
/* Outputs: None                                                                    */
/*                                                                                  */
/* Function Value: A pointer to the table of methods.                               */
/*                                                                                  */
/************************************************************************************/

/* Table of valid methods */

struct keywords *GetMethodTable(void)
{

  static struct keywords MethodTable[] =
    {
/*  keyword name    keyword symbol */
    { "sample",       METH_SAMPLE,    T_NONE,    A_NOATTR, "",
    "Create a file of PC samples based on the specified events"},
    { "count",        METH_COUNT, T_NONE,    A_NOATTR, "",
    "Periodically print the number of specified evts/second"},
    { "total",        METH_TOTAL, T_NONE,    A_NOATTR, "",
    "Periodically print histogram of the specified evts/second"},

#ifdef nt
    { "irql",      METH_IPL,   T_NONE,    A_NOATTR, "",
    "Periodically print histogram of the # of evts at each irql level"},
#else
    { "ipl",      METH_IPL,   T_NONE,    A_NOATTR, "",
    "Periodically print histogram of the # of evts at each ipl level"},
#endif

    { "mode",     METH_MODE,  T_NONE,    A_NOATTR, "",
    "Periodically print histogram of the # of events in each mode"},
    { "address",      METH_ADDR,  T_NONE,    A_NOATTR, "",
    "Print a histogram of the specified address ranges"},
    { "priority",     METH_PRIORITY,    T_NONE, A_NOATTR, "",
    "Print a histogram of system based on priority"},
    { 0,          0,      0,  0,  0, 0 }
    };

 return MethodTable;

}

/************************************************************************************/
/* Name:  GetUnknownKeyword                                                         */
/*                                                                                  */
/* Purpose:  This is a wrapper for a former global variable.                        */
/*                                                                                  */
/* Arguments:  None.                                                                */
/*                                                                                  */
/* Outputs: None                                                                    */
/*                                                                                  */
/* Function Value: A pointer to table entry of an unknown keyword.                  */
/*                                                                                  */
/************************************************************************************/

struct keywords *GetUnknownKeyword(void)
{
  static struct keywords UnknownKeyword =
    {"*UNKNOWN*", 0, T_NONE, A_NOATTR, "", "unrecognized keyword" };

  return &UnknownKeyword;
}

/************************************************************************************/
/* Name:  IsDebug                                                                   */
/*                                                                                  */
/* Purpose: Returns a pointer to the debug flag													*/
/*                                                                                  */
/* Arguments:  None.                                                                */
/*                                                                                  */
/* Outputs: None                                                                    */
/*                                                                                  */
/* Function Value: A pointer to the debug flag													*/
/*                                                                                  */
/************************************************************************************/
boolean *IsDebug(void)
{
  static boolean DebugFlag = FALSE;

  return &DebugFlag;
}

/************************************************************************************/
/* Name: IsQuiet																							*/
/*                                                                                  */
/* Purpose: Returns a pointer to the quiet flag													*/
/*                                                                                  */
/* Arguments:  None.                                                                */
/*                                                                                  */
/* Outputs: None                                                                    */
/*                                                                                  */
/* Function Value: A pointer to the quiet flag													*/
/*                                                                                  */
/************************************************************************************/
boolean *IsQuiet(void)
{
  static boolean QuietFlag = FALSE;

  return &QuietFlag;
}
/************************************************************************************/
/* Name: IsNoIntermediate																				*/
/*                                                                                  */
/* Purpose: Returns a pointer to the don't print intermediate results flag				*/
/*                                                                                  */
/* Arguments:  None.                                                                */
/*                                                                                  */
/* Outputs: None                                                                    */
/*                                                                                  */
/* Function Value: A pointer to the flag															*/
/*                                                                                  */
/************************************************************************************/
boolean *IsNoIntermediate(void)
{
  static boolean NoIntermediateFlag = FALSE;

  return &NoIntermediateFlag;
}

