/****************************************************************************/
/*                                                                          */
/*  COPYRIGHT (c) 1993-1999 BY                                              */
/*  Digital Equiptment/Compaq Computer Corporation.                         */
/*  ALL RIGHTS RESERVED.                                                    */
/*                                                                          */
/*  This file is part of the Iprobe performance suite for Alpha processors. */
/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 2 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License       */
/*  along with this program; if not, write to the Free Software             */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307    */
/*                                                                          */
/****************************************************************************/
/*
 *  Program Name:  iprobe_time
 *
 *  Abstract:  Platform independent time functions.
 *    All code presumes the use of UNIX standard time, which is the number of
 *    seconds since midnight, 1-JAN-1970.
 *
 * Modification History:
 *
 * Phillip Ezolt: 11-Jul-97
 *  Renamed variables and functions, and added comments. 
 *
 * Ravindra Kulkarni: Aug-94
 *    Conditionally included sys/time.h in case of osf and vms and time.h
 *    in case of nt.
 *    Changed sleep call to Sleep in case of NT.
 *    Changed argument to Sleep as Sleep reads the time im milliseconds.
 *
 * Ravindra Kulkarni: Feb-94
 *    Added include of signal.h to resolve info messages about types.
 *
 * Greg Tarsa: Jan-94
 *    Wrote original code.
 */

#ifdef nt
#define WIN32_LEAN_AND_MEAN 1
#endif
#include <signal.h>
#include <sys/types.h>
#include "stdtypes.h"

#if defined vms || defined unix
#include <sys/time.h>
#endif

#ifdef nt
#include <time.h>
#endif

#include "iprobe_struct.h"

/************************************************************************************/
/* Name: WaitForTime                                                                */
/*                                                                                  */
/* Purpose: Sleep until the absolute time specified by the caller.  Return          */
/*          immediately if that time has passed.                                    */
/*                                                                                  */
/* Arguments: None                                                                  */
/*                                                                                  */
/* Output: None                                                                     */
/*                                                                                  */
/* Function Value: None                                                             */
/*                                                                                  */
/* Notes: Doesn't NT have a sleep() function?                                       */
/*        How about a macro?  If we are NT,  define sleep(a) Sleep(1000*A)          */
/*                                                                                  */
/************************************************************************************/

void WaitForTime(time_t TimeToWait)
{
  time_t TimeDifference;
  time_t CurrentTime;
  
  /*  Get current time and calculate the amount of sleep time left. */

  CurrentTime = time(0);
  TimeDifference = TimeToWait - CurrentTime;
  
  /* If there is time left to wait, then sleep */

  if (TimeDifference > 0)
    {
#ifdef nt
    Sleep(TimeDifference * 1000);
#endif

#if defined vms || defined unix
  (void) sleep(TimeDifference);
#endif
    }

}/* end WaitForTime() */

