/****************************************************************************/
/*                                                                          */
/*  COPYRIGHT (c) 1993-1999 BY                                              */
/*  Digital Equiptment/Compaq Computer Corporation.                         */
/*  ALL RIGHTS RESERVED.                                                    */
/*                                                                          */
/*  This file is part of the Iprobe performance suite for Alpha processors. */
/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 2 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License       */
/*  along with this program; if not, write to the Free Software             */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307    */
/*                                                                          */
/****************************************************************************/
/*
 *   Program Name: ipr_os.c (OSF version)
 *
 *   Abstract: Operating system specific code.
 *
 *   Creation Date: Jan-1994
 *
 *   Author: Greg Tarsa   
 *
 *   Modification History:
 *
 *   Phillip Ezolt: 11-Jul-97
 *    Renamed variables and functions, and added comments.
 *        
 *   Greg Tarsa: Jan-1994
 *    Wrote original code.
 */

#include "stdtypes.h"
#include "iprobe_hdr.h"
#include "iprobe_global.h"


#define OS_OSF1 2
  
/************************************************************************************/
/* Name:  OSGetSampleQuadDataTable                                                  */
/*                                                                                  */
/* Purpose:  This is a wrapper for a former global variable.                        */
/*                                                                                  */
/* Arguments:  None.                                                                */
/*                                                                                  */
/* Outputs: None                                                                    */
/*                                                                                  */
/* Function Value: A pointer to the keyword table.                                  */
/*                                                                                  */
/************************************************************************************/

struct keywords *OSGetSampleQuadDataTable(void)
{
  static struct keywords SampleQuadDataTable[] =
    {
      { "ps", IPR_DMASK_M_PS, T_NONE, A_NOATTR, "64",
	  "Collect PS full quad word" } ,
      { "pc", IPR_DMASK_M_PC, T_NONE, A_NOATTR, "64",
	  "Collect PC full quad word" } ,
      { "time", IPR_DMASK_M_TIME, T_NONE, A_NOATTR, "64",
	  "Collect the sampling time" } ,
      { "pid", IPR_DMASK_M_PID, T_NONE, A_NOATTR, "",
	  "Collect current process id" } ,
      { "ctr", IPR_DMASK_M_COUNTER, T_NONE, A_NOATTR, "",
	  "Collect counter number" } ,
      { "pri", IPR_DMASK_M_PRIORITY, T_NONE, A_NOATTR, "",
	  "Collect current process priority" } ,
      { 0, 0, 0, 0, 0, 0 }
    };
  return SampleQuadDataTable;
}


/************************************************************************************/
/* Name:  OSGetModeNames                                                            */
/*                                                                                  */
/* Purpose:  This is a wrapper for a former global variable.                        */
/*                                                                                  */
/* Arguments:  None.                                                                */
/*                                                                                  */
/* Outputs: None                                                                    */
/*                                                                                  */
/* Function Value: A pointer to an array of mode names.                             */
/*                                                                                  */
/************************************************************************************/

char **OSGetModeNames(void)
{
  static char *ModeNames[IPR_MODE_COUNT] = {"Kernel", "User", "Idle"};
  return ModeNames;
}

/************************************************************************************/
/* Name:  OSGetModeTable                                                            */
/*                                                                                  */
/* Purpose:  This is a wrapper for a former global variable.                        */
/*                                                                                  */
/* Arguments:  None.                                                                */
/*                                                                                  */
/* Outputs: None                                                                    */
/*                                                                                  */
/* Function Value: A pointer to the table of modes.                                 */
/*                                                                                  */
/************************************************************************************/

struct keywords *OSGetModeTable(void){
  
  /* Table of valid modes */
  static struct keywords ModeTable[] =
    {
      /*  keyword name    keyword symbol */
      { "kernel",     MODE_K,     T_NONE,    A_NOATTR, "" ,
	  "Measure events in kernel mode" } ,
      { "user",       MODE_U,     T_NONE,    A_NOATTR, "",
	  "Measure events in user mode" } ,
      { "pal",        MODE_PAL,   T_NONE,    A_NOATTR, "",
	  "Measure events in PAL mode" } ,
    { "selected",       MODE_SEL,   T_NONE,    A_INVISIBLE, "",
	"Measure events in instrumented processes" } ,
      { 0,            0,      0,  0,  0, 0 }
    } ;
  
  return ModeTable;
}

/************************************************************************************/
/* Name:  OSGetSampleDataTable                                                      */
/*                                                                                  */
/* Purpose:  This is a wrapper for a former global variable.                        */
/*                                                                                  */
/* Arguments:  None.                                                                */
/*                                                                                  */
/* Outputs: None                                                                    */
/*                                                                                  */
/* Function Value: A pointer to the SampleDataTable.                                */
/*                                                                                  */
/************************************************************************************/


struct keywords *OSGetSampleDataTable(void){

  static struct keywords SampleDataTable[] =
    {
      
      { "ps", IPR_DMASK_M_PS, T_NONE, A_NOATTR, "",
	  "Collect PS low longword" } ,
      { "pc", IPR_DMASK_M_PC, T_NONE, A_NOATTR, "",
	  "Collect PC low longword" } ,
      { "time", IPR_DMASK_M_TIME, T_NONE, A_NOATTR, "",
	  "Collect the sampling time" } ,
      { "pid", IPR_DMASK_M_PID, T_NONE, A_NOATTR, "32",
	  "Collect current process id" } ,
      { "ctr", IPR_DMASK_M_COUNTER, T_NONE, A_NOATTR, "32",
	  "Collect counter number" } ,
      { "pri", IPR_DMASK_M_PRIORITY, T_NONE, A_NOATTR, "32",
	  "Collect current process priority" } ,
      { 0, 0, 0, 0, 0, 0,}
    };
  
  return SampleDataTable;
}


/************************************************************************************/
/* Name: OSGetSystemInformation                                                     */
/*                                                                                  */
/* Purpose: This fills in the SystemInformation array with the relavent information */
/*          about the hardware and software features of the current machine.        */
/*                                                                                  */
/* Arguments: None                                                                  */
/*                                                                                  */
/* Output: None                                                                     */
/*                                                                                  */
/* Function Value: None                                                             */
/*                                                                                  */
/* Notes: None.                                                                     */
/*                                                                                  */
/************************************************************************************/

void OSGetSystemInformation(void)
{
  IPR_CPU_CONF OSSystemParameters;
  IPR_STATUS Status;
  int CurrentCPU;

  uint32 *TotalNumberOfCPUs;

  system_info *SystemInformation;

  TotalNumberOfCPUs = GetTotalNumberOfCPUs();

  SystemInformation = GetSystemInformation();


  Status = IprGetSysInfo(&OSSystemParameters);

#ifdef linux
  SystemInformation->OS_type = OS_LINUX;
#else
  SystemInformation->OS_type = OS_OSF1;
#endif


  SystemInformation->NumCpus = OSSystemParameters.active_cpu_count;
  (*TotalNumberOfCPUs) = OSSystemParameters.active_cpu_count;
  if (SystemInformation->NumCpus > 16)
    {
      fprintf(stderr, "Too many CPUS: %ld\n", SystemInformation->NumCpus);
      exit(0);
    }
  SystemInformation->num_ctrs = IprNumCounters();
  SystemInformation->cpu_speed = OSSystemParameters.clockspeed;
  SystemInformation->pagelength = OSSystemParameters.page_sz;
  SystemInformation->memorysize = OSSystemParameters.cur_phys_mem;
  SystemInformation->cpu_type = OSSystemParameters.cpu_maj;
  
  SystemInformation->modelname = (char *) malloc(sizeof(OSSystemParameters.hardware_name));
  (void) strcpy(SystemInformation->modelname, OSSystemParameters.hardware_name);
  SystemInformation->nodename = (char *) malloc(sizeof(OSSystemParameters.sysname));
  (void) strcpy(SystemInformation->nodename, OSSystemParameters.sysname);
  SystemInformation->osname = (char *) malloc(64);
  (void) sprintf(SystemInformation->osname, "%s %s-%s",
		 OSSystemParameters.Osname, OSSystemParameters.Osrelease, OSSystemParameters.Osver);
  
#ifdef linux
  Status = sscanf(OSSystemParameters.Osrelease, "%d.%d.%d",
		  &(SystemInformation->OS_ver_major), &(SystemInformation->OS_ver_minor));

#else
  Status = sscanf(OSSystemParameters.Osrelease, "%*c %d.%d",
		  &(SystemInformation->OS_ver_major), &(SystemInformation->OS_ver_minor));
#endif


  
  SystemInformation->timestamp = 0;
  SystemInformation->start_time = SystemInformation->start = 0;
  
  SystemInformation->NumEvents = 0;
  
  SystemInformation->counter_mask = 0;
  SystemInformation->avail_cpu_mask = 0;

  for (CurrentCPU = 0; CurrentCPU < SystemInformation->NumCpus; CurrentCPU++)
    {
      SystemInformation->avail_cpu_mask += 1 << CurrentCPU;
    }

  SystemInformation->null_proc_id = 0;
  SystemInformation->rangecnt = 0;
  SystemInformation->rangeptr = 0;
  SystemInformation->buffer_count = 0;
  SystemInformation->delay = 0;
  SystemInformation->duration = 0;
  SystemInformation->interval = 0;
  SystemInformation->outfile = 0;
  SystemInformation->method = 0;
  return;
}

/************************************************************************************/
/* Name: UpdateTime                                                                 */
/*                                                                                  */
/* Purpose:  This resets the start_time to the current time.                        */
/*                                                                                  */
/* Arguments: None                                                                  */
/*                                                                                  */
/* Output: None                                                                     */
/*                                                                                  */
/* Function Value: None                                                             */
/*                                                                                  */
/* Notes: None.                                                                     */
/*                                                                                  */
/************************************************************************************/

void UpdateTime(void)
{
  system_info *SystemInformation;
  SystemInformation = GetSystemInformation();
 
  SystemInformation->start_time = time(0);
}
/************************************************************************************/
/* Name: InitiateOS                                                                 */
/*                                                                                  */
/* Purpose: This would normally set up any needed OS specific features.             */
/*                                                                                  */
/* Arguments: None                                                                  */
/*                                                                                  */
/* Output: None                                                                     */
/*                                                                                  */
/* Function Value: None                                                             */
/*                                                                                  */
/* Notes: No functions are needed.                                                  */
/*                                                                                  */
/************************************************************************************/
void InitiateOS(void)
{

#ifdef linux
 init_ipr_lib();
#endif

}
/************************************************************************************/
/* Name: InitializeModes                                                            */
/*                                                                                  */
/* Purpose: This sets up the ModeNames array so that it contains the proper names. */
/*                                                                                  */
/* Arguments: None                                                                  */
/*                                                                                  */
/* Output: ModeNames: An array of names describing the different modes that        */
/*                     iprobe can run in. (Kernel, User, Idle)                      */
/*                                                                                  */
/* Function Value: None                                                             */
/*                                                                                  */
/* Notes: None.                                                                     */
/*                                                                                  */
/************************************************************************************/
void InitializeModes(void)
{
  char **ModeNames;

  ModeNames = OSGetModeNames(); 
  ModeNames[IPR_MODE_KERNEL] = "Kernel";
  ModeNames[IPR_MODE_USER] = "User";
  ModeNames[IPR_MODE_IDLE] = "Idle";
}


