/****************************************************************************/
/*                                                                          */
/*  COPYRIGHT (c) 1993-1999 BY                                              */
/*  Digital Equiptment/Compaq Computer Corporation.                         */
/*  ALL RIGHTS RESERVED.                                                    */
/*                                                                          */
/*  This file is part of the Iprobe performance suite for Alpha processors. */
/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 2 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License       */
/*  along with this program; if not, write to the Free Software             */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307    */
/*                                                                          */
/****************************************************************************/

/*
 * Abstract:	This file contains the EV4-specific performance counter
 *		support routines.
 *
 * Author:		Brian F. Wibecan
 * Creation Date:	16-Dec-1993
 *
 * Modified by:
 *
 * X-22	BFW		Brian F. Wibecan 	23-Aug-1994
 *	Modified based on new KIprUpdatePerfmon routine.
 *
 * X-21	BMW		Bradley M. Waters	18-Jul-1994
 *	Modified based on kernel rationalization.
 *
 * X-20	BMW		Bradley M. Waters	15-Jul-1994
 *	Added include for api_routines_nt.h to define API routines to ctr...
 *
 * X-19	BMW		Bradley M. Waters	06-Jul-1994
 *	Ported to NT - added NT include file and other NT related stuff.
 *
 * X-18	BFW1151		Brian F. Wibecan	01-Jul-1994
 *	Removed inclusion of PFMEDEF.
 *
 * X-17	BFW1150		Brian F. Wibecan	01-Jul-1994
 *	Broke up PERFMONDEF.
 *
 * X-16	BFW1101		Brian F. Wibecan	20-May-1994
 *	Fixed issues vs. cycles confusion.
 *
 * X-15	BFW1099		Brian F. Wibecan	18-May-1994
 *	Added Ev4SelectEventsFromArray.
 *
 * X-14	BFW1085		Brian F. Wibecan	09-May-1994
 *	Moved assert.h inclusion to OS-specific header files.
 *
 * X-13	BFW1080		Brian F. Wibecan	04-May-1994
 *	Modified #pragma module to be compatible with CC on OSF/1.
 *
 * X-12	BFW1073		Brian F. Wibecan	06-Apr-1994
 *	Added #pragma module statement.  OSF/1 merge.
 *
 * X-11	BFW1068		Brian F. Wibecan	24-Mar-1994
 *	Add bits to the FUNC mask.
 *
 * X-10	BFW1056		Brian F. Wibecan	14-Mar-1994
 *	Fix parallel initialization bug.
 *
 * X-9	BFW1051		Brian F. Wibecan	13-Mar-1994
 *	Changed terminator key for MUX constant numbers.
 *
 * X-8	BFW1040		Brian F. Wibecan	11-Mar-1994
 *	Add FreqCounterMap.
 *
 * X-7	BFW1035		Brian F. Wibecan	08-Mar-1994
 *	Modified event_to_mux and mux_to_event to use an array of events.
 *
 * X-6	BFW1029		Brian F. Wibecan	16-Feb-1994
 *	Fixed setting of all_specified in SelectEvents.
 *
 * X-5	BFW1028		Brian F. Wibecan	16-Feb-1994
 *	Added new argument for fill_event_from_set.
 *
 * X-3	BFW1020		Brian F. Wibecan	02-Feb-1994
 *	Change to CTR_ROUTINES.
 *
 * X-2	BFW1012		Brian F. Wibecan	19-Jan-1994
 *	Added IPL save/restore, changed handling in start/stop/pause/resume.
 *
 * X-1	BFW1000		Brian F. Wibecan	16-Dec-1993
 *	Original version.
 *
 */

#ifdef __NT
#  include "ipr_shr_nt.h"
// includes windows.h for basic type defs for Windows NT
#endif

#include "ev4_routines.h"

#ifdef __NT
#  include "api_routines_nt.h"
#endif

#include <assert.h>

#ifdef __NT

#include <stdio.h>
#include <windows.h>

extern HANDLE hPC0[];
extern HANDLE hPC1[];

#endif





EV4SHARED *Ev4Sh = NULL;

/* Auxiliary routines */

static void ev4_init_events (void)
{
   int
      i;

   (void) CtrClearEventSet (&Ev4Sh -> ctr_events [0]);
   (void) CtrClearEventSet (&Ev4Sh -> ctr_events [1]);
   for (i = 0; i < EV4_MUX0_TOTAL; i++)
      Ev4Sh -> events_0 [i] = 0;
   for (i = 0; i < EV4_MUX1_TOTAL; i++)
      Ev4Sh -> events_1 [i] = 0;
   Ev4Sh -> events_0 [EV4_MUX0_ISSUES] = IPR_EVT_ISSUES;
   Ev4Sh -> events_0 [EV4_MUX0_PIPELINE_DRY] = IPR_EVT_PIPE_DRY;
   Ev4Sh -> events_0 [EV4_MUX0_LOAD_INSTRUCTIONS] = IPR_EVT_LOADS;
   Ev4Sh -> events_0 [EV4_MUX0_PIPELINE_FROZEN] = IPR_EVT_PIPE_FROZEN;
   Ev4Sh -> events_0 [EV4_MUX0_BRANCH_INSTRUCTIONS] = IPR_EVT_BRANCHES;
   Ev4Sh -> events_0 [EV4_MUX0_PAL_MODE] = IPR_EVT_PALCODE_CYCLES;
   Ev4Sh -> events_0 [EV4_MUX0_CYCLES] = IPR_EVT_CYCLES;
   Ev4Sh -> events_0 [EV4_MUX0_NON_ISSUES] = IPR_EVT_NON_ISSUES;
   Ev4Sh -> events_0 [EV4_MUX0_EXTERNAL] = IPR_EVT_BCACHE_VICTIM;

   Ev4Sh -> events_1 [EV4_MUX1_DCACHE_MISSES] = IPR_EVT_DCACHE_MISS;
   Ev4Sh -> events_1 [EV4_MUX1_ICACHE_MISSES] = IPR_EVT_ICACHE_MISS;
   Ev4Sh -> events_1 [EV4_MUX1_DUAL_ISSUE_CYCLES] =
      IPR_EVT_DUAL_ISSUE_CYCLES;
   Ev4Sh -> events_1 [EV4_MUX1_BRANCH_MISPREDICTS] = IPR_EVT_BRANCH_MISPR;
   Ev4Sh -> events_1 [EV4_MUX1_FP_INSTRUCTIONS] = IPR_EVT_FLOAT_OPS;
   Ev4Sh -> events_1 [EV4_MUX1_INTEGER_OPERATE] = IPR_EVT_INTEGER_OPS;
   Ev4Sh -> events_1 [EV4_MUX1_STORE_INSTRUCTIONS] = IPR_EVT_STORES;
   Ev4Sh -> events_1 [EV4_MUX1_EXTERNAL] = IPR_EVT_BCACHE_MISS;

   for (i = 0; i < EV4_MUX0_TOTAL; i++)
      if (Ev4Sh -> events_0 [i])
         (void) CtrInsertEvent (&Ev4Sh -> ctr_events [0],
                                Ev4Sh -> events_0 [i]);
   for (i = 0; i < EV4_MUX1_TOTAL; i++)
      if (Ev4Sh -> events_1 [i])
         (void) CtrInsertEvent (&Ev4Sh -> ctr_events [1],
                                Ev4Sh -> events_1 [i]);
}


/*****************************************************************************/
/* EV4-specific routines                                                     */
/*                                                                           */
/* Many of these routines are simply calls to generic routines with          */
/* appropriate parameters for EV4.                                           */
/*****************************************************************************/

// static IPR_BOOLEAN Ev4ValidEventSet (const IPR_EVENTSET *candidate)
// {
//    return valid_event_set (candidate, Ev4Sh -> ctr_events, EV4_COUNTER_COUNT);
// }

static IPR_STATUS Ev4AvailableEvents (const IPR_EVENTSET *initial_events,
                                    IPR_EVENTSET *more_events)
{
   return available_events (initial_events, more_events, Ev4Sh -> ctr_events,
                            EV4_COUNTER_COUNT);
}

static IPR_STATUS Ev4AvailableFreqs (const IPR_EVENT event, IPR_FREQ *freqs)
{
   return available_freqs (event, freqs, Ev4Sh -> ctr_events,
                           Ev4Sh -> ctr_freqs, EV4_COUNTER_COUNT,
                           Ev4Sh -> adjusted_events, Ev4Sh -> adjusted_shifts,
                           EV4_ADJUSTED_COUNT);
}

static IPR_STATUS Ev4AvailableModes (IPR_MMASK *modes)
{
   *modes = Ev4Sh -> modes;
   return IPR__SUCCESS;
}

static IPR_STATUS Ev4AvailableFuncs (IPR_FMASK *funcs)
{
   *funcs = Ev4Sh -> funcs;
   return IPR__SUCCESS;
}

static IPR_COUNT Ev4NumCounters (void)
{
   return EV4_COUNTER_COUNT;
}

static IPR_STATUS Ev4CurrentEvents (int ctx, IPR_EVENTSET *events)
{
   return current_events (events, Ev4Sh -> context [ctx] . counter,
                          EV4_COUNTER_COUNT);
}

static IPR_STATUS Ev4CounterNumber (int ctx, const IPR_EVENT event,
                               IPR_CTRNUMBER *counter_number)
{
   return get_counter_number (event, counter_number,
                              Ev4Sh -> context [ctx] . counter,
                              EV4_COUNTER_COUNT);
}

static IPR_STATUS Ev4EventCounterMap (int ctx, IPR_EVENT *event_array,
                                    IPR_SIZE asize)
{
   return event_counter_map (event_array, asize,
                             Ev4Sh -> context [ctx] . counter,
                             EV4_COUNTER_COUNT);
}

static IPR_STATUS Ev4FreqCounterMap (int ctx, IPR_FREQ *freq_array,
                                     IPR_SIZE asize)
{
   return freq_counter_map (freq_array, asize,
                            Ev4Sh -> context [ctx] . counter,
                            EV4_COUNTER_COUNT, Ev4Sh -> adjusted_events,
                            Ev4Sh -> adjusted_shifts, EV4_ADJUSTED_COUNT);
}

static IPR_STATUS Ev4CurrentFreqs (int ctx, const IPR_EVENT event,
                              IPR_FREQ *freq)
{
   return get_current_freq (event, freq, Ev4Sh -> context [ctx] . counter,
                            EV4_COUNTER_COUNT, Ev4Sh -> adjusted_events,
                            Ev4Sh -> adjusted_shifts, EV4_ADJUSTED_COUNT);
}

static IPR_STATUS Ev4CurrentModes (int ctx, IPR_MMASK *modes)
{
   *modes = Ev4Sh -> context [ctx] . modes;
   return IPR__SUCCESS;
}

static IPR_STATUS Ev4CurrentState (int ctx, const IPR_EVENT event)
{
   IPR_CTRNUMBER
      ctr;
   IPR_STATUS
      status;

   status = Ev4CounterNumber (ctx, event, &ctr);
   if (!IPR_SUCCESS (status))
      return status;

   return IPR__INTERRUPTING;
}

static IPR_STATUS Ev4ClearContext (int ctx)
{
   int
      low,
      high,
      i;

   if (ctx == IPR_ALL_CONTEXT_AREAS)
   {
      low = 0;
      high = CONTEXT_AREA_COUNT;
   }
   else
   {
      high = ctx;
      low = high++;
   }

   for (i = low; i < high; i++)
   {
      Ev4Sh -> context [i] . all_specified = FALSE;
      Ev4Sh -> context [i] . modes . ipr_mmask_l_bits = 0;
      (void) clear_counter_data (Ev4Sh -> context [i] . counter,
                                 EV4_COUNTER_COUNT);
   }
   return IPR__SUCCESS;
}

static int global_to_code (IPR_EVENT *pE, IPR_COUNT ecnt, IPR_EVENT cand)
{
   IPR_COUNT
      i;

   if (cand)
   {
      for (i = 0; i < ecnt; i++)
         if (cand == pE [i])
            return i;
   }
   return 0;
}

static void ev4_globals_to_codes (CTRCTXDATA *cctx)
{

   /* Translate the events */

   cctx [0] . event_code =
      global_to_code (Ev4Sh -> events_0, EV4_MUX0_TOTAL, cctx [0] . event);
   cctx [1] . event_code =
      global_to_code (Ev4Sh -> events_1, EV4_MUX1_TOTAL, cctx [1] . event);

   /* Translate the frequencies */

   cctx [0] . freq_code = (cctx [0] . freq == Ev4Sh -> ctr_freq_hi [0]);
   cctx [1] . freq_code = (cctx [1] . freq == Ev4Sh -> ctr_freq_hi [1]);
}

// static void ev4_codes_to_globals (CTRCTXDATA *cctx)
// {
//    cctx [0] . event = Ev4Sh -> events_0 [cctx [0] . event_code];
//    cctx [1] . event = Ev4Sh -> events_1 [cctx [1] . event_code];
//    cctx [0] . freq = cctx [0] . freq_code ? Ev4Sh -> ctr_freq_hi [0]
//                                         : Ev4Sh -> ctr_freq_lo [0];
//    cctx [1] . freq = cctx [1] . freq_code ? Ev4Sh -> ctr_freq_hi [1]
//                                         : Ev4Sh -> ctr_freq_lo [1];
// }

static IPR_STATUS Ev4SelectEvents (int ctx, const IPR_EVENTSET *events,
                                   IPR_FREQ freq, uint64 *MUXvalue)
{
   static ev4_context_block
      new_cb;
   ev4_context_block
      *pECB;
   int
      i;
   IPR_STATUS
      status;

   for (i = 0; i < EV4_COUNTER_COUNT; i++)
      new_cb . counter [i] . specified = FALSE;
   status = fill_event_from_set (events, freq,
                                 Ev4Sh -> ctr_events,
                                 Ev4Sh -> ctr_freqs,
                                 EV4_COUNTER_COUNT,
                                 new_cb . counter,
                                 Ev4Sh -> adjusted_events,
                                 Ev4Sh -> adjusted_shifts,
                                 EV4_ADJUSTED_COUNT,
                                 FALSE);
   if (!IPR_SUCCESS (status))
      return status;

   /* Save everything */

   pECB = &Ev4Sh -> context [ctx];
   pECB -> all_specified = TRUE;
   for (i = 0; i < EV4_COUNTER_COUNT; i++)
   {
      if (new_cb . counter [i] . specified)
         pECB -> counter [i] = new_cb . counter [i];
      if (!pECB -> counter [i] . specified)
         pECB -> all_specified = FALSE;
   }

   /* Fill in the code values */

   ev4_globals_to_codes (pECB -> counter);

   /* Return appropriate condition if something is missing */

   if (!pECB -> all_specified)
      return IPR__INCOMPLETE;

   /* Everything is OK.  Return MUX value if needed */

   if (MUXvalue)
      *MUXvalue = ev4_get_mux (pECB -> counter);
   return IPR__SUCCESS;
}

static IPR_STATUS Ev4SelectEventsFromArray (int ctx, const IPR_EVENT *events,
                                            const IPR_FREQ *freqs,
                                            IPR_SIZE asize,
                                            uint64 *MUXvalue)
{
   static ev4_context_block
      new_cb;
   ev4_context_block
      *pECB;
   int
      i;
   IPR_STATUS
      status;

   for (i = 0; i < EV4_COUNTER_COUNT; i++)
      new_cb . counter [i] . specified = FALSE;
   pECB = &Ev4Sh -> context [ctx];
   status = fill_event_from_array (events, freqs, asize,
                                  Ev4Sh -> ctr_events,
                                  Ev4Sh -> ctr_freqs,
                                  EV4_COUNTER_COUNT,
                                  pECB -> counter,
                                  new_cb . counter,
                                  Ev4Sh -> adjusted_events,
                                  Ev4Sh -> adjusted_shifts,
                                  EV4_ADJUSTED_COUNT,
                                  FALSE);
   if (!IPR_SUCCESS (status))
      return status;

   /* Save everything */

   pECB -> all_specified = TRUE;
   for (i = 0; i < EV4_COUNTER_COUNT; i++)
   {
      if (new_cb . counter [i] . specified)
         pECB -> counter [i] = new_cb . counter [i];
      if (!pECB -> counter [i] . specified)
         pECB -> all_specified = FALSE;
   }

   /* Fill in the code values */

   ev4_globals_to_codes (pECB -> counter);
   pECB -> muxvalue = ev4_get_mux (pECB -> counter);

   /* Return appropriate condition if something is missing */

   if (!pECB -> all_specified)
      return IPR__INCOMPLETE;

   /* Everything is OK.  Return MUX value if needed */

   if (MUXvalue)
      *MUXvalue = ev4_get_mux (pECB -> counter);
   return IPR__SUCCESS;
}

static IPR_STATUS Ev4Pause (int ctx, IPR_CTRMASK ctrs)
{
   return Ev4Disable (ctx, ctrs);
}

static IPR_STATUS Ev4Resume (int ctx, IPR_CTRMASK ctrs)
{
   return Ev4Enable (ctx, ctrs);
}

static IPR_STATUS Ev4CounterEvents (IPR_CTRNUMBER ctr, IPR_EVENTSET *e)
{
   return counter_events (ctr, Ev4Sh -> ctr_events, EV4_COUNTER_COUNT, e);
}

static IPR_STATUS Ev4AvailableCounters (IPR_EVENT evt, IPR_CTRMASK *c)
{
   return available_counters (evt, Ev4Sh -> ctr_events, EV4_COUNTER_COUNT, c);
}

extern void Ev4Initialize (void *Areaptr, IPR_BOOLEAN init_shared)
{
   Ev4Sh = Areaptr;
   assert (sizeof (EV4SHARED) < (8192 - 16));
   if (init_shared)
   {
      (void) Ev4ClearContext (IPR_ALL_CONTEXT_AREAS);
      ev4_init_events ();

      (void) CtrClearEventSet (&Ev4Sh -> adjusted_events [0]);
      (void) CtrInsertEvent (&Ev4Sh -> adjusted_events [0], IPR_EVT_ISSUES);
      (void) CtrInsertEvent (&Ev4Sh -> adjusted_events [0],
                             IPR_EVT_NON_ISSUES);
      Ev4Sh -> adjusted_shifts [0] = 1;

      Ev4Sh -> ctr_freq_lo [0] = 1 << EV4_FREQ0_LOW;
      Ev4Sh -> ctr_freq_hi [0] = 1 << EV4_FREQ0_HIGH;
      Ev4Sh -> ctr_freq_lo [1] = 1 << EV4_FREQ1_LOW;
      Ev4Sh -> ctr_freq_hi [1] = 1 << EV4_FREQ1_HIGH;
      Ev4Sh -> ctr_freqs [0] =
         Ev4Sh -> ctr_freq_lo [0] | Ev4Sh -> ctr_freq_hi [0];
      Ev4Sh -> ctr_freqs [1] =
         Ev4Sh -> ctr_freq_lo [1] | Ev4Sh -> ctr_freq_hi [1];

      Ev4Sh -> funcs . ipr_fmask_l_bits = 0;
      Ev4Sh -> funcs . ipr_fmask_v_enable = 1;
      Ev4Sh -> funcs . ipr_fmask_v_disable = 1;
      Ev4Sh -> funcs . ipr_fmask_v_pause = 1;
      Ev4Sh -> funcs . ipr_fmask_v_resume = 1;
      Ev4Sh -> funcs . ipr_fmask_v_interrupt = 1;

      Ev4Sh -> modes . ipr_mmask_l_bits = 0;  // Valid bits set in Ev4OSInit
   }
   PtrAvailableEvents = &Ev4AvailableEvents;
   PtrAvailableFreqs = &Ev4AvailableFreqs;
   PtrAvailableModes = &Ev4AvailableModes;
   PtrAvailableFuncs = &Ev4AvailableFuncs;
   PtrNumCounters = &Ev4NumCounters;
   PtrCounterNumber = &Ev4CounterNumber;
   PtrEventCounterMap = &Ev4EventCounterMap;
   PtrFreqCounterMap = &Ev4FreqCounterMap;
   PtrCurrentEvents = &Ev4CurrentEvents;
   PtrCurrentFreqs = &Ev4CurrentFreqs;
   PtrCurrentModes = &Ev4CurrentModes;
   PtrCurrentState = &Ev4CurrentState;
   PtrClearContext = &Ev4ClearContext;
   PtrSelectEvents = &Ev4SelectEvents;
   PtrSelectEventsFromArray = &Ev4SelectEventsFromArray;
   PtrSetEvents = &Ev4SetEvents;
   PtrSetModes = &Ev4SetModes;
   PtrEnable = &Ev4Enable;
   PtrDisable = &Ev4Disable;
   PtrPause = &Ev4Pause;
   PtrResume = &Ev4Resume;
   PtrCounterEvents = &Ev4CounterEvents;
   PtrAvailableCounters = &Ev4AvailableCounters;
   Ev4OSInit ();
}
