/****************************************************************************/
/*                                                                          */
/*  COPYRIGHT (c) 1993-1999 BY                                              */
/*  Digital Equiptment/Compaq Computer Corporation.                         */
/*  ALL RIGHTS RESERVED.                                                    */
/*                                                                          */
/*  This file is part of the Iprobe performance suite for Alpha processors. */
/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 2 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License       */
/*  along with this program; if not, write to the Free Software             */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307    */
/*                                                                          */
/****************************************************************************/

/*
 * Abstract:	This file contains the EV5-specific performance counter
 *		support routines.
 *
 * Author:		Brian F. Wibecan
 * Creation Date:	18-Jan-1994
 *
 * Modified by:
 *
 * X-24	BFW1230		Brian F. Wibecan	16-Sep-1994
 *	Fix mode_to_option assignment for selected-only.
 *
 * X-23	BFW1200		Brian F. Wibecan	30-Aug-1994
 *	Clear out internal-use variable in Ev5SelectEvents(FromArray).
 *
 * X-22	BFW1182		Brian F. Wibecan	10-Aug-1994
 *	Change bcache_ref to bcache_hit.
 *
 * X-21	BFW1151		Brian F. Wibecan	01-Jul-1994
 *	Removed inclusion of PFMEDEF.
 *
 * X-20	BFW1150		Brian F. Wibecan	01-Jul-1994
 *	Broke up PERFMONDEF.  Changed MUXCTL structure so workaround is not
 *	necessary.
 *
 * X-19	BFW1138		Brian F. Wibecan	23-Jun-1994
 *	Fix workaround again.
 *
 * X-18	BFW1137		Brian F. Wibecan	23-Jun-1994
 *	Fix workaround.
 *
 * X-17 BFW1136		Brian F. Wibecan	23-Jun-1994
 *	Work around EV5 PALcode problem.
 *
 * X-16 BFW1135a	Brian F. Wibecan	22-Jun-1994
 *	Fix typo.
 *
 * X-15	BFW1135		Brian F. Wibecan	22-Jun-1994
 *	Add new events MEM_BARRIER and LOAD_LOCKED.
 *
 * X-14	BFW1131		Brian F. Wibecan	22-Jun-1994
 *	Fix bug in ev5_event_to_mux that prevented conditional events from
 *	being assigned correctly.
 *
 * X-13	BFW1125		Brian F. Wibecan	16-Jun-1994
 *	EV5 non-issue cycles do not count all zero-issue cycles, they
 *	count only the ones where there are instructions at the issue stage.
 *	Therefore, the non-issue cycles event is being mapped to the
 *	pipeline_dry event name.
 *
 * X-12	BFW1120		Brian F. Wibecan	10-Jun-1994
 *	Fixed selection logic again.
 *
 * X-11	BFW1105		Brian F. Wibecan	27-May-1994
 *	Changed "flow change" to "branches".
 *
 * X-10	BFW1101		Brian F. Wibecan	20-May-1994
 *	Fixed issues vs. cycles confusion.
 *
 * X-9a	BFW1099		Brian F. Wibecan	18-May-1994
 *	Added Ev5SelectEventsFromArray.
 *
 * X-9  AJD0010         Anil D'silva		17-May-1994
 *	Added a header to include macros and other header files specific to
 * 	OSF1
 *
 * X-8	BFW1090		Brian F. Wibecan	11-May-1994
 *	Completely reformed event selection logic to handle the fact that
 *	counter 1 "flow change" selection counts different things, depending
 *	upon what counter 2 is counting.
 *
 * X-7	BFW1085		Brian F. Wibecan	09-May-1994
 *	Moved assert.h to OS-specific headers.
 *
 * X-6	BFW1082		Brian F. Wibecan	09-May-1994
 *	Made Ev5AvailableRoutines check for a null returned event set.
 *
 * X-5	BFW1080		Brian F. Wibecan	04-May-1994
 *	Modified #pragma module to be compatible with CC on OSF/1.
 *
 * X-4	BFW1075		Brian F. Wibecan	21-Apr-1994
 *	Changed initialization of adjusted events, since there are none.  Added
 *	instantiation of PtrFreqCounterMap.
 *
 * X-3	BFW1073		Brian F. Wibecan	06-Apr-1994
 *	Added #pragma module statement.  Fix mode selection bug.
 *
 * X-2	BFW1068		Brian F. Wibecan	24-Mar-1994
 *	Add bits to the FUNC mask.  Also fixed initialization.
 *
 * X-1	BFW1010		Brian F. Wibecan	18-Jan-1994
 *	Original version.
 *
*/


#ifdef __NT
#  include "ipr_shr_nt.h"
// includes windows.h for basic type defs for Windows NT
#endif
#include "ev5_routines.h"
#ifdef __NT
#  include "api_routines_nt.h"
#endif

EV5SHARED *Ev5Sh = NULL;

/* Auxiliary routines */

static void ev5_init_events (void)
{
   int
      i;

   (void) CtrClearEventSet (&Ev5Sh -> ctr_events [0]);
   (void) CtrClearEventSet (&Ev5Sh -> ctr_events [1]);
   (void) CtrClearEventSet (&Ev5Sh -> ctr_events [2]);
   for (i = 0; i < EV5_MUX0_TOTAL; i++)
      Ev5Sh -> events_0 [i] = 0;
   for (i = 0; i < EV5_MUX1_TOTAL + EV5_CBOX1_TOTAL; i++)
      Ev5Sh -> events_1 [i] = 0;
   for (i = 0; i < EV5_MUX2_TOTAL + EV5_CBOX2_TOTAL; i++)
      Ev5Sh -> events_2 [i] = 0;
   Ev5Sh -> events_0 [EV5_MUX0_ISSUES] = IPR_EVT_ISSUES;
   Ev5Sh -> events_0 [EV5_MUX0_CYCLES] = IPR_EVT_CYCLES;
   Ev5Sh -> events_1 [EV5_MUX1_NON_ISSUE_CYCLES] =
      IPR_EVT_PIPE_FROZEN;
   Ev5Sh -> events_1 [EV5_MUX1_SPLIT_ISSUE_CYCLES] =
      IPR_EVT_SPLIT_ISSUE_CYCLES;
   Ev5Sh -> events_1 [EV5_MUX1_PIPELINE_DRY] = IPR_EVT_PIPE_DRY;
   Ev5Sh -> events_1 [EV5_MUX1_SINGLE_ISSUE_CYCLES] =
      IPR_EVT_SINGLE_ISSUE_CYCLES;
   Ev5Sh -> events_1 [EV5_MUX1_DUAL_ISSUE_CYCLES] = IPR_EVT_DUAL_ISSUE_CYCLES;
   Ev5Sh -> events_1 [EV5_MUX1_TRIPLE_ISSUE_CYCLES] =
      IPR_EVT_TRIPLE_ISSUE_CYCLES;
   Ev5Sh -> events_1 [EV5_MUX1_QUAD_ISSUE_CYCLES] = IPR_EVT_QUAD_ISSUE_CYCLES;
   Ev5Sh -> events_1 [EV5_MUX1_REPLAY_TRAP] = IPR_EVT_REPLAY_TRAP;
   Ev5Sh -> events_1 [EV5_MUX1_FLOW_CHANGE] = IPR_EVT_BRANCHES;
   Ev5Sh -> events_1 [EV5_MUX1_INTEGER_OPERATE] = IPR_EVT_INTEGER_OPS;
   Ev5Sh -> events_1 [EV5_MUX1_FP_INSTRUCTIONS] = IPR_EVT_FLOAT_OPS;
   Ev5Sh -> events_1 [EV5_MUX1_LOAD_INSTRUCTIONS] = IPR_EVT_LOADS;
   Ev5Sh -> events_1 [EV5_MUX1_STORE_INSTRUCTIONS] = IPR_EVT_STORES;
   Ev5Sh -> events_1 [EV5_MUX1_ICACHE_ACCESS] = IPR_EVT_ICACHE_ACCESS;
   Ev5Sh -> events_1 [EV5_MUX1_DCACHE_ACCESS] = IPR_EVT_DCACHE_ACCESS;
   Ev5Sh -> events_1 [EV5_MUX1_TOTAL + EV5_CBOX1_SCACHE_ACCESS]
      = IPR_EVT_SCACHE_ACCESS;
   Ev5Sh -> events_1 [EV5_MUX1_TOTAL + EV5_CBOX1_SCACHE_READ]
      = IPR_EVT_SCACHE_READ;
   Ev5Sh -> events_1 [EV5_MUX1_TOTAL + EV5_CBOX1_SCACHE_WRITE]
      = IPR_EVT_SCACHE_WRITE;
   Ev5Sh -> events_1 [EV5_MUX1_TOTAL + EV5_CBOX1_SCACHE_VICTIM]
      = IPR_EVT_SCACHE_VICTIM;
   Ev5Sh -> events_1 [EV5_MUX1_TOTAL + EV5_CBOX1_BCACHE_HIT]
      = IPR_EVT_BCACHE_HIT;
   Ev5Sh -> events_1 [EV5_MUX1_TOTAL + EV5_CBOX1_BCACHE_VICTIM]
      = IPR_EVT_BCACHE_VICTIM;
   Ev5Sh -> events_1 [EV5_MUX1_TOTAL + EV5_CBOX1_SYS_REQ]
      = IPR_EVT_SYS_REQ;
   Ev5Sh -> events_2 [EV5_MUX2_LONG_STALLS] = IPR_EVT_LONG_STALLS;
   Ev5Sh -> events_2 [EV5_MUX2_PC_MISPR] = IPR_EVT_PC_MISPR;
   Ev5Sh -> events_2 [EV5_MUX2_BRANCH_MISPREDICTS] = IPR_EVT_BRANCH_MISPR;
   Ev5Sh -> events_2 [EV5_MUX2_ICACHE_MISSES] = IPR_EVT_ICACHE_MISS;
   Ev5Sh -> events_2 [EV5_MUX2_ITB_MISS] = IPR_EVT_ITB_MISS;
   Ev5Sh -> events_2 [EV5_MUX2_DCACHE_MISSES] = IPR_EVT_DCACHE_MISS;
   Ev5Sh -> events_2 [EV5_MUX2_DTB_MISS] = IPR_EVT_DTB_MISS;
   Ev5Sh -> events_2 [EV5_MUX2_LOADS_MERGED] = IPR_EVT_LOADS_MERGED;
   Ev5Sh -> events_2 [EV5_MUX2_LDU_REPLAYS] = IPR_EVT_LDU_REPLAYS;
   Ev5Sh -> events_2 [EV5_MUX2_WB_MAF_FULL_REPLAYS] =
      IPR_EVT_WB_MAF_FULL_REPLAYS;
   Ev5Sh -> events_2 [EV5_MUX2_EXTERNAL] = IPR_EVT_EXTERNAL;
   Ev5Sh -> events_2 [EV5_MUX2_CYCLES] = IPR_EVT_CYCLES;
   Ev5Sh -> events_2 [EV5_MUX2_MEM_BARRIER] = IPR_EVT_MEM_BARRIER_CYCLES;
   Ev5Sh -> events_2 [EV5_MUX2_LOAD_LOCKED] = IPR_EVT_LOAD_LOCKED;
   Ev5Sh -> events_2 [EV5_MUX2_TOTAL + EV5_CBOX2_SCACHE_MISS]
      = IPR_EVT_SCACHE_MISS;
   Ev5Sh -> events_2 [EV5_MUX2_TOTAL + EV5_CBOX2_SCACHE_READ_MISS]
      = IPR_EVT_SCACHE_READ_MISS;
   Ev5Sh -> events_2 [EV5_MUX2_TOTAL + EV5_CBOX2_SCACHE_SH_WRITE]
      = IPR_EVT_SCACHE_SH_WRITE;
   Ev5Sh -> events_2 [EV5_MUX2_TOTAL + EV5_CBOX2_SCACHE_WRITE_MISS]
      = IPR_EVT_SCACHE_WRITE_MISS;
   Ev5Sh -> events_2 [EV5_MUX2_TOTAL + EV5_CBOX2_SCACHE_WRITE]
      = IPR_EVT_SCACHE_WRITE;
   Ev5Sh -> events_2 [EV5_MUX2_TOTAL + EV5_CBOX2_BCACHE_MISS]
      = IPR_EVT_BCACHE_MISS;
   Ev5Sh -> events_2 [EV5_MUX2_TOTAL + EV5_CBOX2_SYS_INV]
      = IPR_EVT_SYS_INV;
   Ev5Sh -> events_2 [EV5_MUX2_TOTAL + EV5_CBOX2_SYS_READ_REQ]
      = IPR_EVT_SYS_READ_REQ;

   for (i = 0; i < EV5_MUX0_TOTAL; i++)
      if (Ev5Sh -> events_0 [i])
         (void) CtrInsertEvent (&Ev5Sh -> ctr_events [0],
                                Ev5Sh -> events_0 [i]);
   for (i = 0; i < EV5_MUX1_TOTAL + EV5_CBOX1_TOTAL; i++)
      if (Ev5Sh -> events_1 [i])
         (void) CtrInsertEvent (&Ev5Sh -> ctr_events [1],
                                Ev5Sh -> events_1 [i]);
   for (i = 0; i < EV5_MUX2_TOTAL + EV5_CBOX2_TOTAL; i++)
      if (Ev5Sh -> events_2 [i])
         (void) CtrInsertEvent (&Ev5Sh -> ctr_events [2],
                                Ev5Sh -> events_2 [i]);
}

static IPR_BOOLEAN ev5_compatible_events (const IPR_EVENTSET *e)
{
   int
      i,
      tcnt,
      rcnt;
   IPR_BOOLEAN
      original_present,
      trigger_present,
      replacement_present;

   /*
    * The "trigger" event causes the "original" event to be replaced with the
    * "replacement" event.  This implies the following:
    *   - If the "original" event is present, we must not have the "trigger"
    *     event, since the "trigger" would have caused replacement of the
    *     "original" event;
    *   - If the "original" event is present, we must not have the
    *     "replacement" event;
    *   - If the "replacement" event is present, the "trigger" event must also
    *     be present.
    * The code below makes sure that these conditions are met.
    */
   tcnt = rcnt = 0;
   for (i = 0; i < EV5_CONDITIONAL_COUNT; i++)
   {
      original_present =
         CtrVerifyEventInSet (e, Ev5Sh -> conditional [i] . original);
      trigger_present =
         CtrVerifyEventInSet (e, Ev5Sh -> conditional [i] . trigger);
      replacement_present =
         CtrVerifyEventInSet (e, Ev5Sh -> conditional [i] . replacement);
      if ((original_present && (replacement_present || trigger_present)) ||
          (replacement_present && !trigger_present))
         return FALSE;
      if (trigger_present)
         tcnt++;
      if (replacement_present)
         rcnt++;
   }

   /* Cannot have more than one replacement or more than one trigger */

   if (tcnt > 1 || rcnt > 1)
      return FALSE;
   return TRUE;
}

static void replace_conditional_event (IPR_EVENT *c1e, IPR_EVENT *c2e)
{
   int
      i;

   for (i = 0; i < EV5_CONDITIONAL_COUNT; i++)
      if (*c2e == Ev5Sh -> conditional [i] . trigger &&
          *c1e == Ev5Sh -> conditional [i] . original)
      {
         *c1e = Ev5Sh -> conditional [i] . replacement;
         return;
      }
}

static IPR_BOOLEAN ev5_restore_conditional (const CTRCTXDATA *prev,
                                          CTRCTXDATA *new,
                                          const IPR_EVENTSET *reqevents)
{
   int
      i;
   IPR_EVENT
      c2event,
      c1event;

   c1event = new [1] . specified ? new [1] . event : prev [1] . event;
   c2event = new [2] . specified ? new [2] . event : prev [2] . event;
   if (new [1] . specified)
   {
      /*
       * Counter 1 is being newly specified.  We need to check for
       * compatibility, and convert the event if necessary.
       */
      if (new [2] . specified || prev [2] . specified)
      {
         /*
          * Counter 2 is being specified.  Replace it if the conditions are
          * correct, and make sure that the replaced value was what was
          * requested in the first place.
          */
         replace_conditional_event (&c1event, &c2event);
         if (CtrVerifyEventInSet (reqevents, c1event))
         {
            new [1] . event = c1event;
            return TRUE;
         }
         else
            return FALSE;
      }
      else
      {
         /*
          * Counter 2 is NOT being specified, so just make sure the counter 1
          * event matches what was requested.
          */
         for (i = 0; i < EV5_CONDITIONAL_COUNT; i++)
         {
            if (new [1] . event == Ev5Sh -> conditional [i] . original &&
                CtrVerifyEventInSet (reqevents,
                                     Ev5Sh -> conditional [i] . replacement))
            {
               new [1] . event = Ev5Sh -> conditional [i] . replacement;
               break;
            }
         }
         return TRUE;
      }
   }
   else
   {
      /*
       * Counter 1 is NOT being newly specified.  We need to check if any
       * new assignment to counter 2 is compatible with the old assignment
       * to counter 1, if there was one.
       */
      if (new [2] . specified && prev [1] . specified)
      {
         /*
          * There are assignments for both counter 1 and counter 2.  Since
          * counter 1 was previously specified, we cannot change the value
          * for counter 1 as a result of this call.  Do the conditional
          * replacement, and if the replaced value is different from the saved
          * value for counter 1, return an error.
          */
         replace_conditional_event (&c1event, &c2event);
         if (c1event != prev [1] . event)
            return FALSE;
      }
   }
   return TRUE;
}

static void ev5_convert_conditional_list (IPR_EVENT *pE, IPR_SIZE asize)
{
   unsigned int
      i,
      j;

   /*
    * If the "trigger" events are present, and the "original" events are
    * present, it converts the "original" into the "replacement" events.
    * To allow assignment, convert the "replacement" back into "original"
    * events.
    */
    for (i = 0; i < EV5_CONDITIONAL_COUNT; i++)
       for (j = 0; j < asize; j++)
         if (pE [j] == Ev5Sh -> conditional [i] . replacement)
            pE [j] = Ev5Sh -> conditional [i] . original;
}

static void ev5_convert_conditional (IPR_EVENTSET *target)
{
   int
      i;

   /*
    * If the "trigger" events are present, and the "original" events are
    * present, it converts the "original" into the "replacement" events.
    * To allow assignment, convert the "replacement" back into "original"
    * events.
    */
    for (i = 0; i < EV5_CONDITIONAL_COUNT; i++)
      if (CtrVerifyEventInSet (target, Ev5Sh -> conditional [i] . replacement))
      {
         (void) CtrDeleteEvent (target,
                                Ev5Sh -> conditional [i] . replacement);
         (void) CtrInsertEvent (target, Ev5Sh -> conditional [i] . original);
      }
}

//static void ev5_remove_incompatible (const IPR_EVENTSET *init, IPR_EVENTSET *added)
//{
//   int
//      i;
//
//   for (i = 0; i < EV5_CONDITIONAL_COUNT; i++)
//      if (CtrVerifyEventInSet (init, Ev5Sh -> conditional [i] . trigger))
//          (void) CtrDeleteEvent (added, Ev5Sh -> conditional [i] . original);
//}


/*****************************************************************************/
/* EV5-specific routines                                                     */
/*                                                                           */
/* Many of these routines are simply calls to generic routines with          */
/* appropriate parameters for EV5.                                           */
/*****************************************************************************/

extern IPR_BOOLEAN Ev5ValidEventSet (const IPR_EVENTSET *candidate)
{
   IPR_EVENTSET
      shadow_events;

   if (!ev5_compatible_events (candidate))
      return FALSE;
   shadow_events = *candidate;
   ev5_convert_conditional (&shadow_events);
   if (!valid_event_set (candidate, Ev5Sh -> ctr_events, EV5_COUNTER_COUNT))
      return FALSE;
   return TRUE;
}

static IPR_STATUS Ev5AvailableEvents (const IPR_EVENTSET *initial_events,
                                    IPR_EVENTSET *more_events)
{
   IPR_STATUS
      status;
   IPR_EVENTSET
      shadow_events,
      shadow_return,
      shadow_ctr_events [EV5_COUNTER_COUNT];
   int
      i,
      tcnt,
      rcnt,
      tmask,
      rmask;
   IPR_BOOLEAN
      original_present,
      trigger_present,
      replacement_present;

   for (i = 0; i < EV5_COUNTER_COUNT; i++)
      shadow_ctr_events [i] = Ev5Sh -> ctr_events [i];

   /* Check first for EV5-specific incompatibilities */

   tcnt = rcnt = 0;
   tmask = rmask = 0;
   for (i = 0; i < EV5_CONDITIONAL_COUNT; i++)
   {

      /* Check for impossible combinations */

      original_present = CtrVerifyEventInSet (initial_events,
         Ev5Sh -> conditional [i] . original);
      trigger_present = CtrVerifyEventInSet (initial_events,
         Ev5Sh -> conditional [i] . trigger);
      replacement_present = CtrVerifyEventInSet (initial_events,
         Ev5Sh -> conditional [i] . replacement);
      if (original_present && (replacement_present || trigger_present))
         return IPR__INVALID;
      if (replacement_present)
      {
         rcnt++;
         rmask |= 1 << i;
      }
      if (trigger_present)
      {
         tcnt++;
         tmask |= 1 << i;
      }

      /* The presence of a replacement forces the presence of the trigger */

      if (replacement_present)
      {
         (void) CtrClearEventSet (&shadow_ctr_events [2]);
         (void) CtrInsertEvent (&shadow_ctr_events [2],
                         Ev5Sh -> conditional [i] . trigger);
      }
   }

   /* Cannot have more than one replacement or more than one trigger */

   if (rcnt > 1 || tcnt > 1)
      return IPR__INVALID;

   /* Cannot have trigger for one conditional and replacement for the other */

   if (tmask && rmask && tmask != rmask)
      return IPR__INVALID;

   /* Copy the set, and convert the special-case events for assignment */

   shadow_events = *initial_events;
   ev5_convert_conditional (&shadow_events);

   /* See if we can get an event assigned to each counter */

   status = available_events (&shadow_events, &shadow_return,
                              shadow_ctr_events, EV5_COUNTER_COUNT);
   if (!IPR_SUCCESS (status))
      return status;
   /*
    * Change the returned event set to account for the presence or absence
    * of conditional events.  Note that we have already forced the use of
    * the trigger for counter 2 if the replacment for counter 1 is in use.
    */
   for (i = 0; i < EV5_CONDITIONAL_COUNT; i++)
   {

      /* "Trigger" replaces "original" with "replacement" */

      if (CtrVerifyEventInSet (initial_events,
                               Ev5Sh -> conditional [i] . trigger) &&
          CtrVerifyEventInSet (&shadow_return,
                               Ev5Sh -> conditional [i] . original))
      {
         (void) CtrDeleteEvent (&shadow_return,
                                Ev5Sh -> conditional [i] . original);
         (void) CtrInsertEvent (&shadow_return,
                                Ev5Sh -> conditional [i] . replacement);
      }
   }
   if (more_events)
      *more_events = shadow_return;
   return status;
}

static IPR_STATUS Ev5AvailableFreqs (const IPR_EVENT event, IPR_FREQ *freqs)
{
   IPR_EVENT new_event;
   int i;

   new_event = event;
   for (i = 0; i < EV5_CONDITIONAL_COUNT; i++)
      if (new_event == Ev5Sh -> conditional [i] . replacement)
         new_event = Ev5Sh -> conditional [i] . original;
   return available_freqs (new_event, freqs, Ev5Sh -> ctr_events,
                           Ev5Sh -> ctr_freqs, EV5_COUNTER_COUNT,
                           Ev5Sh -> adjusted_events, Ev5Sh -> adjusted_shifts,
                           EV5_ADJUSTED_COUNT);
}

static IPR_STATUS Ev5AvailableModes (IPR_MMASK *modes)
{
   *modes = Ev5Sh -> modes;
   return IPR__SUCCESS;
}

static IPR_STATUS Ev5AvailableFuncs (IPR_FMASK *funcs)
{
   *funcs = Ev5Sh -> funcs;
   return IPR__SUCCESS;
}

static IPR_COUNT Ev5NumCounters (void)
{
   return EV5_COUNTER_COUNT;
}

static IPR_STATUS Ev5CurrentEvents (int ctx, IPR_EVENTSET *events)
{
   return current_events (events, Ev5Sh -> context [ctx] . counter,
                          EV5_COUNTER_COUNT);
}

static IPR_STATUS Ev5CounterNumber (int ctx, const IPR_EVENT event,
                               IPR_CTRNUMBER *counter_number)
{
   return get_counter_number (event, counter_number,
                              Ev5Sh -> context [ctx] . counter,
                              EV5_COUNTER_COUNT);
}

static IPR_STATUS Ev5EventCounterMap (int ctx, IPR_EVENT *event_array,
                                    IPR_SIZE asize)
{
   return event_counter_map (event_array, asize,
                             Ev5Sh -> context [ctx] . counter,
                             EV5_COUNTER_COUNT);
}

static IPR_STATUS Ev5FreqCounterMap (int ctx, IPR_FREQ *freq_array, IPR_SIZE asize)
{
   return freq_counter_map (freq_array, asize,
                            Ev5Sh -> context [ctx] . counter,
                            EV5_COUNTER_COUNT, Ev5Sh -> adjusted_events,
                            Ev5Sh -> adjusted_shifts, EV5_ADJUSTED_COUNT);
}

static IPR_STATUS Ev5CurrentFreqs (int ctx, const IPR_EVENT event,
                              IPR_FREQ *freq)
{
   return get_current_freq (event, freq, Ev5Sh -> context [ctx] . counter,
                            EV5_COUNTER_COUNT, Ev5Sh -> adjusted_events,
                            Ev5Sh -> adjusted_shifts, EV5_ADJUSTED_COUNT);
}

static IPR_STATUS Ev5CurrentModes (int ctx, IPR_MMASK *modes)
{
   IPR_STATUS
      status;

   *modes = Ev5Sh -> context [ctx] . modes;
   return IPR__SUCCESS;
}

static IPR_STATUS Ev5CurrentState (int ctx, const IPR_EVENT event)
{
   IPR_CTRNUMBER
      ctr;
   IPR_STATUS
      status;

   status = Ev5CounterNumber (ctx, event, &ctr);
   if (!IPR_SUCCESS (status))
      return status;
   if (!(Ev5Sh -> context [ctx] . enabled_set & (1 << ctr)))
      return IPR__DISABLED;
   if (Ev5Sh -> context [ctx] . counter [ctr] . freq)
      return IPR__INTERRUPTING;
   return IPR__COUNTING;
}

static IPR_STATUS Ev5ClearContext (int ctx)
{
   int
      low,
      high,
      i;

   if (ctx == IPR_ALL_CONTEXT_AREAS)
   {
      low = 0;
      high = CONTEXT_AREA_COUNT;
   }
   else
   {
      high = ctx;
      low = high++;
   }

   for (i = low; i < high; i++)
   {
      Ev5Sh -> context [i] . all_specified = FALSE;
      Ev5Sh -> context [i] . modes . ipr_mmask_l_bits = 0;
      Ev5Sh -> context [i] . muxctl = 0;
      Ev5Sh -> context [i] . fctl = 0;
      (void) clear_counter_data (Ev5Sh -> context [i] . counter,
                                 EV5_COUNTER_COUNT);
   }
   return IPR__SUCCESS;
}

static unsigned int map_freq_to_fctl (IPR_FREQ candidate, IPR_FREQ hif,
                                      IPR_FREQ lof)
{
   if (candidate == hif)
      return EV5_FREQ_M_INTERRUPT | EV5_FREQ_M_HIGH;
   if (candidate == lof)
      return EV5_FREQ_M_INTERRUPT | EV5_FREQ_M_LOW;
   return 0;
}

extern IPR_FREQ map_fctl_to_freq (unsigned int f, IPR_FREQ hi, IPR_FREQ lo)
{
   if (f & EV5_FREQ_M_INTERRUPT)
      return (f & EV5_FREQ_M_HIGH) ? hi : lo;
   return 0;
}

static void ev5_event_to_code (EV5_CONTEXT_BLOCK *cctx)
{
   IPR_EVENT
      e,
      c0event,
      c1event,
      c2event;
   int
      i;

   for (i = 0; i < EV5_COUNTER_COUNT; i++)
   {
      cctx -> counter [i] . event_code = 0;
      cctx -> counter [i] . freq_code = 0;
   }

   /* Save the events in temporaries */

   c0event = cctx -> counter [0] . event;
   c1event = cctx -> counter [1] . event;
   c2event = cctx -> counter [2] . event;

   /* Convert conditional events */

   for (i = 0; i < EV5_CONDITIONAL_COUNT; i++)
      if (c1event == Ev5Sh -> conditional [i] . replacement)
      {
         c1event = Ev5Sh -> conditional [i] . original;
         break;
      }

   /* Assign the code value for counter 0 */

   for (i = 0; i < EV5_MUX0_TOTAL; i++)
      if (Ev5Sh -> events_0 [i] && Ev5Sh -> events_0 [i] == c0event)
      {
         cctx -> counter [0] . event_code = i;
         cctx -> counter [0] . freq_code =
            map_freq_to_fctl (cctx -> counter [0] . freq,
                              Ev5Sh -> ctr_freq_hi [0],
                              Ev5Sh -> ctr_freq_lo [0]);
         break;
      }

   /* Assign the code value for counter 1 */

   for (i = 0; i < EV5_CBOX1_TOTAL + EV5_MUX1_TOTAL; i++)
      if (Ev5Sh -> events_1 [i] && Ev5Sh -> events_1 [i] == c1event)
      {
         cctx -> counter [1] . event_code = i;
         cctx -> counter [1] . freq_code =
            map_freq_to_fctl (cctx -> counter [1] . freq,
                              Ev5Sh -> ctr_freq_hi [1],
                              Ev5Sh -> ctr_freq_lo [1]);
         break;
      }

   /* Assign the code value for counter 2 */

   for (i = 0; i < EV5_CBOX2_TOTAL + EV5_MUX2_TOTAL; i++)
      if (Ev5Sh -> events_2 [i] && Ev5Sh -> events_2 [i] == c2event)
      {
         cctx -> counter [2] . event_code = i;
         cctx -> counter [2] . freq_code =
            map_freq_to_fctl (cctx -> counter [2] . freq,
                              Ev5Sh -> ctr_freq_hi [2],
                              Ev5Sh -> ctr_freq_lo [2]);
         break;
      }
   cctx -> muxctl = ev5_get_mux (cctx -> counter);
   cctx -> fctl = ev5_get_fctl (cctx -> counter);
}

static IPR_STATUS Ev5SelectEvents (int ctx, const IPR_EVENTSET *events,
                                   IPR_FREQ freq, uint64 *MUXvalue)
{
   static EV5_CONTEXT_BLOCK
      new;
   EV5_CONTEXT_BLOCK
      *cctx;
   IPR_EVENTSET
      shadow_events;
   int
      i;
   IPR_STATUS
      status;

   /*
    * Convert the conditional events into their assignment equivalents.
    * This will be undone and tested after the assignments are made.
    */
   shadow_events = *events;
   ev5_convert_conditional (&shadow_events);

   /* Clear out the static context area before filling it */

   for (i = 0; i < EV5_COUNTER_COUNT; i++)
      new . counter [i] . specified = FALSE;
   status = fill_event_from_set (&shadow_events, freq,
                                 Ev5Sh -> ctr_events,
                                 Ev5Sh -> ctr_freqs,
                                 EV5_COUNTER_COUNT,
                                 new . counter,
                                 Ev5Sh -> adjusted_events,
                                 Ev5Sh -> adjusted_shifts,
                                 EV5_ADJUSTED_COUNT, TRUE);
   if (!IPR_SUCCESS (status))
      return status;

   cctx = &Ev5Sh -> context [ctx];
   /*
    * Ensure that we are not left with an incompatible set of events after
    * the assignment, and undo the conditional conversion.
    */
   if (!ev5_restore_conditional (cctx -> counter, new . counter, events))
      return IPR__INVALID;

   /* Save everything */

   cctx -> all_specified = TRUE;
   for (i = 0; i < EV5_COUNTER_COUNT; i++)
   {
      if (new . counter [i] . specified)
         cctx -> counter [i] = new . counter [i];
      if (!cctx -> counter [i] . specified)
         cctx -> all_specified = FALSE;
   }

   /* Fill in the control values */

   ev5_event_to_code (cctx);

   /* Everything is OK.  Return MUX value if needed */

   if (MUXvalue)
      *MUXvalue = Ev5Sh -> context [ctx] . muxctl;
   if (!cctx -> all_specified)
      return IPR__INCOMPLETE;
   return IPR__SUCCESS;
}

static IPR_STATUS Ev5SelectEventsFromArray (int ctx, const IPR_EVENT *events,
                                          const IPR_FREQ *freqs, IPR_SIZE asize,
                                          uint64 *MUXvalue)
{
   static EV5_CONTEXT_BLOCK
      new;
   EV5_CONTEXT_BLOCK
      *cctx;
   IPR_EVENT
      shadow_event_list [EV5_COUNTER_COUNT];
   IPR_EVENTSET
      original_events;
   unsigned int
      i;
   IPR_STATUS
      status;

   cctx = &Ev5Sh -> context [ctx];
   /*
    * Convert the conditional events into their assignment equivalents.
    * This will be undone and tested after the assignments are made.
    */
   if (asize > EV5_COUNTER_COUNT)
      return IPR__INVALID;
   (void) CtrClearEventSet (&original_events);
   for (i = 0; i < asize; i++)
   {
      shadow_event_list [i] = events [i];
      (void) CtrInsertEvent (&original_events, events [i]);
   }
   ev5_convert_conditional_list (shadow_event_list, asize);

   /* Clear out the static context area before filling it */

   for (i = 0; i < EV5_COUNTER_COUNT; i++)
      new . counter [i] . specified = FALSE;
   status = fill_event_from_array (shadow_event_list, freqs, asize,
                                   Ev5Sh -> ctr_events,
                                   Ev5Sh -> ctr_freqs,
                                   EV5_COUNTER_COUNT,
                                   cctx -> counter,
                                   new . counter,
                                   Ev5Sh -> adjusted_events,
                                   Ev5Sh -> adjusted_shifts,
                                   EV5_ADJUSTED_COUNT,
                                   TRUE);
   if (!IPR_SUCCESS (status))
      return status;
   /*
    * Ensure that we are not left with an incompatible set of events after
    * the assignment, and undo the conditional conversion.
    */
   if (!ev5_restore_conditional (cctx -> counter, new . counter,
                                 &original_events))
      return IPR__INVALID;

   /* Save everything */

   cctx -> all_specified = TRUE;
   for (i = 0; i < EV5_COUNTER_COUNT; i++)
   {
      if (new . counter [i] . specified)
         cctx -> counter [i] = new . counter [i];
      if (!cctx -> counter [i] . specified)
         cctx -> all_specified = FALSE;
   }

   /* Fill in the control values */

   ev5_event_to_code (cctx);

   /* Everything is OK.  Return MUX value if needed */

   if (MUXvalue)
      *MUXvalue = Ev5Sh -> context [ctx] . muxctl;
   if (!cctx -> all_specified)
      return IPR__INCOMPLETE;
   return IPR__SUCCESS;
}

static IPR_STATUS Ev5CounterEvents (IPR_CTRNUMBER ctr, IPR_EVENTSET *e)
{
   IPR_STATUS status;
   unsigned int i;

   status = counter_events (ctr, Ev5Sh -> ctr_events, EV5_COUNTER_COUNT, e);
   if (!IPR_SUCCESS (status))
      return status;

   if (ctr == 1)
      for (i = 0; i < EV5_CONDITIONAL_COUNT; i++)
         (void) CtrInsertEvent (e, Ev5Sh -> conditional [i] . replacement);
   return status;
}

static IPR_STATUS Ev5AvailableCounters (IPR_EVENT evt, IPR_CTRMASK *c)
{
   int i;

   for (i = 0; i < EV5_CONDITIONAL_COUNT; i++)
      if (evt == Ev5Sh -> conditional [i] . replacement)
      {
         evt = Ev5Sh -> conditional [i] . original;
         break;
      }
   return available_counters (evt, Ev5Sh -> ctr_events, EV5_COUNTER_COUNT, c);
}

extern void Ev5Initialize (void *Areaptr, IPR_BOOLEAN init_shared)
{
   int
      i;

   Ev5Sh = Areaptr;
   assert (sizeof (EV5SHARED) < (8192 - 16));
   if (init_shared)
   {
      (void) Ev5ClearContext (IPR_ALL_CONTEXT_AREAS);
      ev5_init_events ();

      Ev5Sh -> adjusted_events = NULL;
      Ev5Sh -> adjusted_shifts = NULL;

      for (i = 0; i < EV5_CONDITIONAL_COUNT; i++)
      {
         Ev5Sh -> conditional [i] . trigger = 0;
         Ev5Sh -> conditional [i] . original = 0;
         Ev5Sh -> conditional [i] . replacement = 0;
      }
      Ev5Sh -> conditional [0] . trigger = IPR_EVT_PC_MISPR;
      Ev5Sh -> conditional [0] . original = IPR_EVT_BRANCHES;
      Ev5Sh -> conditional [0] . replacement = IPR_EVT_JSR_RET;

      Ev5Sh -> conditional [1] . trigger = IPR_EVT_BRANCH_MISPR;
      Ev5Sh -> conditional [1] . original = IPR_EVT_BRANCHES;
      Ev5Sh -> conditional [1] . replacement = IPR_EVT_COND_BRANCHES;

      Ev5Sh -> ctr_freq_lo [0] = 1 << EV5_FREQ0_LOW;
      Ev5Sh -> ctr_freq_hi [0] = 1 << EV5_FREQ0_HIGH;
      Ev5Sh -> ctr_freq_lo [1] = 1 << EV5_FREQ1_LOW;
      Ev5Sh -> ctr_freq_hi [1] = 1 << EV5_FREQ1_HIGH;
      Ev5Sh -> ctr_freq_lo [2] = 1 << EV5_FREQ2_LOW;
      Ev5Sh -> ctr_freq_hi [2] = 1 << EV5_FREQ2_HIGH;
      for (i = 0; i < EV5_COUNTER_COUNT; i++)
         Ev5Sh -> ctr_freqs [i] =
            Ev5Sh -> ctr_freq_lo [i] | Ev5Sh -> ctr_freq_hi [i];

      Ev5Sh -> funcs . ipr_fmask_l_bits = 0;
      Ev5Sh -> funcs . ipr_fmask_v_enable = 1;
      Ev5Sh -> funcs . ipr_fmask_v_disable = 1;
      Ev5Sh -> funcs . ipr_fmask_v_pause = 1;
      Ev5Sh -> funcs . ipr_fmask_v_resume = 1;
      Ev5Sh -> funcs . ipr_fmask_v_clear = 1;
      Ev5Sh -> funcs . ipr_fmask_v_write = 1;
      Ev5Sh -> funcs . ipr_fmask_v_read = 1;
      Ev5Sh -> funcs . ipr_fmask_v_count_only = 1;
      Ev5Sh -> funcs . ipr_fmask_v_enable_indiv = 1;
      Ev5Sh -> funcs . ipr_fmask_v_interrupt = 1;

      Ev5Sh -> modes . ipr_mmask_l_bits = EV5_LEGAL_MODES;
   }
   PtrAvailableEvents = &Ev5AvailableEvents;
   PtrAvailableFreqs = &Ev5AvailableFreqs;
   PtrAvailableModes = &Ev5AvailableModes;
   PtrAvailableFuncs = &Ev5AvailableFuncs;
   PtrCounterNumber = &Ev5CounterNumber;
   PtrEventCounterMap = &Ev5EventCounterMap;
   PtrFreqCounterMap = &Ev5FreqCounterMap;
   PtrNumCounters = &Ev5NumCounters;
   PtrCurrentEvents = &Ev5CurrentEvents;
   PtrCurrentFreqs = &Ev5CurrentFreqs;
   PtrCurrentModes = &Ev5CurrentModes;
   PtrCurrentState = &Ev5CurrentState;
   PtrClearContext = &Ev5ClearContext;
   PtrSelectEvents = &Ev5SelectEvents;
   PtrSelectEventsFromArray = &Ev5SelectEventsFromArray;
   PtrSetEvents = &Ev5SetEvents;
   PtrSetModes = &Ev5SetModes;
   PtrEnable = &Ev5Enable;
   PtrDisable = &Ev5Disable;
   PtrPause = &Ev5Pause;
   PtrResume = &Ev5Resume;
   PtrClear = &Ev5Clear;
   PtrWrite = &Ev5Write;
   PtrRead = &Ev5Read;
   PtrReadAll = &Ev5ReadAll;
   PtrCounterEvents = &Ev5CounterEvents;
   PtrAvailableCounters = &Ev5AvailableCounters;
   Ev5OSInit ();
}
