/****************************************************************************/
/*                                                                          */
/*  COPYRIGHT (c) 1993-1999 BY                                              */
/*  Digital Equiptment/Compaq Computer Corporation.                         */
/*  ALL RIGHTS RESERVED.                                                    */
/*                                                                          */
/*  This file is part of the Iprobe performance suite for Alpha processors. */
/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 2 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License       */
/*  along with this program; if not, write to the Free Software             */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307    */
/*                                                                          */
/****************************************************************************/

#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "ipr_events.h"
#include "ipr_events.def"
#include "ipr_base_defs.h"

/***************************************************************************/
/* 9-14-98: Fixed Segfault when accessing an empty event list. -- P. Ezolt */
/***************************************************************************/

extern const char *IprEventNumberToName (IPR_EVENT evt)
{
   return (evt > 0 && evt < IPR_EVT_MAXIMUM) ? event_names [evt] : "<Unknown>";
}

extern IPR_COUNT IprNumberOfEvents (void)
{
   return IPR_EVT_MAXIMUM - 1;
}

extern IPR_STATUS IprEventNameToNumber (char *ename, IPR_EVENT *evtnum,
                                        const char **matches)
{
   IPR_EVENT
      foundnum;
   int
      i,
      len,
      found,
      exact;
   char
      *sname;
   static char
      *found_list = NULL;

   /* If we don't have any event names, don't crash... --Phil*/
   if (event_names==NULL)
         return IPR__INVEVENTNAME;


   if (found_list)
   {
      free (found_list);
      found_list = NULL;
   }
   if (!ename || !evtnum)
      return IPR__NULLPTR;
   *evtnum = 0;
   len = strlen (ename);
   if (len == 0)
      return IPR__INVEVENTNAME;
   sname = malloc (len + 1);
   for (i = 0; i < len + 1; i++)
      sname [i] = _tolower (ename [i]);

   exact = 0;


   for (i = 1, found = 0; i < IPR_EVT_MAXIMUM && !exact; i++)
      if (strncmp (event_names [i], sname, len) == 0)
      {

         // The beginning of the event name matches the requested name,
         // so the requested name is at least a truncation.

         foundnum = i;
         found++;
         if (len == (int)strlen (event_names [i]))
         {

            // An exact match

            exact = i;
            found = 1;
         }
         else
         {

            // Non-exact match.  Add this name to the found list.

            if (found_list)
            {
               found_list = realloc (found_list, strlen (event_names [i]) +
                                     strlen (found_list) + 2);
               (void) strcat (found_list, " ");
               (void) strcat (found_list, event_names [i]);
            }
            else
            {
               found_list = malloc (strlen (event_names [i]) + 1);
               (void) strcpy (found_list, event_names [i]);
            }
         }
      }

   // Get rid of temporary storage

   free (sname);

   // Return the string(s) if requested

   if (matches)
      *matches = (found == 1) ? event_names [foundnum] : found_list;
   switch (found)
   {
      case 0:

         // No match found

         return IPR__INVEVENTNAME;
      case 1:

         // One match found.  Do not need the found list, so get rid of it.
         // In this case, return the event number; all other cases leave it
         // at zero.

         if (found_list)
            free (found_list);
         found_list = NULL;
         *evtnum = foundnum;
         return IPR__SUCCESS;
   }

   // More than one match found

   return IPR__AMBIGUOUS;
}
