/****************************************************************************/
/*                                                                          */
/*  COPYRIGHT (c) 1993-1999 BY                                              */
/*  Digital Equiptment/Compaq Computer Corporation.                         */
/*  ALL RIGHTS RESERVED.                                                    */
/*                                                                          */
/*  This file is part of the Iprobe performance suite for Alpha processors. */
/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 2 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License       */
/*  along with this program; if not, write to the Free Software             */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307    */
/*                                                                          */
/****************************************************************************/

/*
 * Abstract:	This header file contains the OS-indepedent performance
 *		monitoring routines.
  *
 * Author:		Brian F. Wibecan
 * Creation Date:	16-Dec-1993
 *
 * Modified by:
 *
 * X-32	BFW		Brian F. Wibecan	21-Dec-1994
 *	Merged with modified NT source.
 *
 * X-31 BMW             Bradley M. Waters       25-Oct-1994
 *      Fixed bug in delete_histogram_buffers where the end of the
 *      histogram buffer list wasn't being handled correctly.
 *
 * X-30 BMW		Bradley M. Waters	14-Oct-1994
 *	Updated based on code review comments.
 *
 * X-29 BMW		Bradley M. Waters	23-Aug-1994
 *	Changed names from Pfm to Ipr.
 *
 * X-28 BMW		Bradley M. Waters	19-Aug-1994
 *	Major overghaul to integrate with new kernel.
 *
 * X-27 BMW		Bradley M. Waters	18-Jul-1994
 *	Modified based on kernel rationalization.
 *
 * X-26	BMW		Bradley M. Waters	15-Jul-1994
 *	Added include for api_routines_nt.h to define API routines to ctr...
 *
 * X-25	BMW		Bradley M. Waters	06-Jul-1994
 *	Ported to NT.  Added include and other NT specific stuff.
 *
 * X-24	BFW1155		Brian F. Wibecan	05-Jul-1994
 *	Added tear-down to IprDeallocate.
 *
 * X-23			Brian F. Wibecan	28-Jun-1994
 *	Tidying up a bit.
 *
 * X-22a		Anil D'silva		24-Jun-1994
 *	Fixed COPY_IN, COPY_OUT problems.
 *
 * X-22	BFW1112		Brian F. Wibecan	06-Jun-1994
 *	Fixed problem where histogram buffers were being
 *	made too big.  Added code to buffer allocation to
 *	ensure nothing is allocated too big, if it gets that
 *	far.
 *
 * X-21	BFW1110		Brian F. Wibecan	02-Jun-1994
 *	Fix memory leak.
 *
 * X-20	BFW1105		Brian F. Wibecan	27-May-1994
 *	Made the task of finding out no more CPUs exist
 *	more efficient.
 *
 * X-19	BFW1100		Brian F. Wibecan	19-May-1994
 *	Fix argument type in IprSetSampleSkip.
 *
 * X-18	BFW1099		Brian F. Wibecan	18-May-1994
 *	Added IprSelectEventsFromArray.
 *
 * X-17	BFW1088		Brian F. Wibecan	10-May-1994
 *	Made code propagate memory allocation error return values.
 *
 * X-16	BFW1085		Brian F. Wibecan	09-May-1994
 *	Modifications to facilitate OSF/1 merge.
 *
 * X-15	BFW1080		Brian F. Wibecan	04-May-1994
 *	Modified #pragma module to be compatible with CC on OSF/1.
 *
 * X-14	BFW1073		Brian F. Wibecan	06-Apr-1994
 *	Added #pragma module statement.  OSF/1 merge.  Decoupled sample buffer
 *	granularity from the other buffer granularity.
 *
 * X-13	BFW1069		Brian F. Wibecan	28-Mar-1994
 *	Change IprSetValidCounters into IprSetActiveCounters.
 *
 * X-12	BFW1063		Brian F. Wibecan	21-Mar-1994
 *	Another attempt at fixing Pause/resume logic.
 *
 * X-11	BFW1055		Brian F. Wibecan	14-Mar-1994
 *	First attempt at fixing pause/resume logic.  Further work postponed.
 *
 * X-10	BFW1040		Brian F. Wibecan	11-Mar-1994
 *	Add FreqCounterMap.
 *
 * X-9	BFW1034		Brian F. Wibecan	07-Mar-1994
 *	Made first_cpu_with_context use the passed CPU mask instead of
 *	IPR_ALL_CPUS.
 *
 * X-8	BFW1028		Brian F. Wibecan	16-Feb-1994
 *	Added compile-time assertions to ensure that the correct data
 *	size was calculated for sample buffers.  Fixed bug where buffer
 *	sizes were being computed incorrectly.  Added priority to list
 *	of collected items.
 *
 * X-7	BFW1023		Brian F. Wibecan	03-Feb-1994
 *	Put double quotes around ints.h.  Moved builtins.h to VMS-specific
 *	header file.
 *
 * X-6	BFW1020		Brian F. Wibecan	02-Feb-1994
 *	Change name to CTR_ROUTINES.H.
 *
 * X-5	BFW1013		Brian F. Wibecan	21-Jan-1994
 *	Forced data collection mask to clear PC if PC_QUADWORD is set, and
 *	similarly for PS_QUADWORD and PS.  Also added support for valid
 *	counter mask.
 *
 * X-4	BFW1010		Brian F. Wibecan	12-Jan-1994
 *	Modified histogram handling.  Histograms are now organized with the
 *	counter number varying the fastest rather than the data value.  This
 *	corrects a bug where reading a histogram and specifying a minimum
 *	value did not get elements exclusively with that value or higher.
 *	It also makes the index arithmetic simpler.  (Never mind that the
 *	original logic didn't work at all...)
 *
 * X-3	BFW1005		Brian F. Wibecan	05-Jan-1994
 *	Modified to use IPR_KERNEL_ROUTINES_INT.H.
 *
 * X-2	BFW1003		Brian F. Wibecan	03-Jan-1994
 *	Added index to histogram buffers.
 *
 * X-1	BFW1000		Brian F. Wibecan	16-Dec-1993
 *	Original version.
 *
*/

#if defined(__VMS) || defined(__osf__) || defined(__LINUX__)
#  include <stdlib.h>
#endif
#ifdef __NT
#  include <malloc.h>
#  include "ipr_shr_nt.h"
   // includes windows.h for basic type defs for Windows NT
#  include "api_routines_nt.h"
#endif
#include "ipr_api_int_defs.h"

#define DELBUF_SAMPLE 1
#define DELBUF_HISTOGRAM 2
#define DELBUF_DESCRIPTOR 4
#define DELBUF_ALL (DELBUF_SAMPLE | DELBUF_HISTOGRAM | DELBUF_DESCRIPTOR)

static IPR_BOOLEAN
   PME_was_saved = 0,
   saved_PME = 0; 	// This is NOT shared

static IPR_BOOLEAN current_pme_setting;

// This function should be replaced with a call into the kernel to retrieve
// the PME bit setting

static IPR_BOOLEAN get_current_PME (void)
{
   return current_pme_setting;
}

// Various utility routines

// Quadword minimum value

// static uint64 minimum_value (uint64 x, uint64 y)
// {
//    return (x < y) ? x : y;
// }

// Quadword maximum value

// static uint64 maximum_value (uint64 x, uint64 y)
// {
//    return (x > y) ? x : y;
// }

// Determine if owner is valid

extern IPR_BOOLEAN owner_valid (void)
{
   unsigned int Ipid;
   IPR_STATUS status;

   if (!ShareArea)
      return FALSE;
   // call kernel
   status = KIprGetCurrentIpid (&Ipid);
   return (IPR_SUCCESS(status)) ? (ShareArea -> counterowner == Ipid) : FALSE;
}

// Determine if the counters are currently owned

static IPR_BOOLEAN counters_owned (void)
{
    if (!ShareArea)
       return TRUE;
    return (!!ShareArea -> counterowner);
}

// Determine if the counters are currently operating

static IPR_BOOLEAN counters_operating (IN IPR_CPUMASK cpus)
{
   if (!ShareArea)
      return FALSE;

   // Avoid case truncation and possibly very large code generation

   if (cpus == IPR_ALL_CPUS)
      return !!(ShareArea -> countersrunning);
   if (cpus == IPR_CURRENT_CPU)
      return BIprIsSet (ShareArea -> countersrunning, FIND_CURRENT_CPU ());
   return !!(ShareArea -> countersrunning & cpus);
}


/*****************************************************************************/
/* CPU list walking routines                                                 */
/*                                                                           */
/* These routines are designed to do several things:                         */
/*                                                                           */
/*  - Allow simple construction of loops that go through each valid CPU      */
/*    in turn;                                                               */
/*  - Set affinity as appropriate;                                           */
/*  - Provide a mechanism for routines that are called repeatedly, once per  */
/*    CPU, to keep track of which CPU they last worked on.                   */
/*                                                                           */
/* Callers should use loops of the following structure:                      */
/*                                                                           */
/*    for (cpu = first_cpu (...); cpu < IPR_MAX_CPUS; cpu = next_cpu (...)       */
/*                                                                           */
/* The appropriate pair of routines should be used; that is, use first_cpu   */
/* and next_cpu or first_cpu_with_context and next_cpu_with_context.         */
/*                                                                           */
/* Without context, first_cpu_with_context and next_cpu_with_context are the */
/* same as first_cpu and next_cpu.  With context, they do the following:     */
/*                                                                           */
/*  - First call:                                                            */
/*     - first_cpu_with_context returns the first CPU, and saves this as the */
/*       context                                                             */
/*     - next_cpu_with_context returns a loop-terminating value              */
/*  - Subsequent calls                                                       */
/*     - first_cpu_with_context returns the next CPU after the context, and  */
/*       saves this as the new context.  If no more CPUs, it returns a       */
/*       loop-terminating value.                                             */
/*     - next_cpu_with_context returns a loop-terminating value              */
/*****************************************************************************/


// Find the first CPU that matches the mask

static int get_first_cpu (IN IPR_CPUMASK cpus, IN IPR_CPUMASK valid_mask)
{
   int
      i;
   IPR_CPUMASK
      mask;

   // Handle simple case of the current CPU

   if (cpus == IPR_CURRENT_CPU)
      return FIND_CURRENT_CPU ();

   // Check only valid requested CPUs

   if (cpus == IPR_ALL_CPUS)
      mask = valid_mask;
   else
      mask = cpus & valid_mask;
   if (mask)
      return BIprWhich (mask);
   return IPR_MAX_CPUS + 1;
}

// Find the first valid CPU

extern IPR_CPUNUM first_cpu (IN IPR_CPUMASK cpus, IN IPR_BOOLEAN affinity_flag)
{
   IPR_CPUNUM
      cpunum;

   // Find the first CPU, using the appropriate mask

   cpunum = get_first_cpu (cpus, affinity_flag ? AFFINITY_CPU_MASK
                                               : NOAFFINITY_CPU_MASK);

   // Set affinity if necessary
#ifdef __NT
   if (affinity_flag && cpus != IPR_CURRENT_CPU && cpunum < IPR_MAX_CPUS)
      set_affinity (cpunum);
#endif
   return cpunum;
}

// Get the next CPU that matches the mask

static IPR_CPUNUM get_next_cpu (IN IPR_CPUMASK cpus, IN IPR_CPUNUM curcpu,
                                IN IPR_CPUMASK valid_mask)
{
   IPR_CPUMASK
      mask;
   int
      i;

   // Handle the simple case of the current CPU

   if (cpus == IPR_CURRENT_CPU)
      return IPR_MAX_CPUS + 1;

   // Use only valid requested CPUs

   if (cpus == IPR_ALL_CPUS)
      mask = valid_mask;
   else
      mask = cpus & valid_mask;

   mask = (IPR_CPUMASK) BIprClrLo (mask, curcpu + 1);
   if (mask)
      return BIprWhich (mask);
   return IPR_MAX_CPUS + 1;
}

#ifndef __NT
#define set_affinity() 0
#define clear_affinity() 0
#endif

// Find the next valid CPU

extern IPR_CPUNUM next_cpu (IN IPR_CPUMASK cpus, IN IPR_CPUNUM curcpu,
                            IN int affinity_flag)
{
   IPR_CPUNUM
      cpunum;

   // Get the next CPU number

   cpunum = get_next_cpu (cpus, curcpu, affinity_flag ? AFFINITY_CPU_MASK
                                                      : NOAFFINITY_CPU_MASK);

   // Set or clear affinity where appropriate

#ifdef __NT
   if (affinity_flag)
      if (cpunum >= IPR_MAX_CPUS)
         clear_affinity ();
      else
         set_affinity (cpunum);
#endif
   return cpunum;
}

// Find the first (or next) valid CPU, using context information if needed

static IPR_CPUNUM first_cpu_with_context (IN IPR_CPUMASK cpus, IN int *cpuctx,
                                   IN int affinity_flag)
{
   IPR_CPUNUM
      the_cpu;

   // If a regular call, just return the regular first CPU index

   if (!cpuctx)
      return first_cpu (cpus, affinity_flag);

   // A context call

   if (*cpuctx < 0)

      // First call; use the first CPU number available

      the_cpu = first_cpu (cpus, affinity_flag);
   else

      // Subsequent calls; get the next CPU number after the context value

      the_cpu = next_cpu (cpus, *cpuctx, affinity_flag);

   // Save the CPU number for use on the next call, and return it

   *cpuctx = the_cpu;
   return the_cpu;
}

// Find the next valid CPU, using context information as appropriate

static IPR_CPUNUM next_cpu_with_context (IN IPR_CPUMASK cpus, IN int *cpuctx, IN IPR_CPUNUM curcpu,
                                  IN int affinity_flag)
{
   if (cpuctx)
      return IPR_MAX_CPUS + 1;
   return next_cpu (cpus, curcpu, affinity_flag);
}

/*****************************************************************************/
/* Buffer deeallocation routines                                             */
/*****************************************************************************/

static IPR_STATUS delete_sample_buffers (IN IPR_CPUNUM cpu)
{
   IPR_SBUFFER
      *Sptr,
      *SptrN;
   IPR_STATUS status;
   int cnt;
   IPR_SDESC descrip;

   Sptr = NULL;
   // Find 1st sample buffer
   status = KIprSbFind(cpu, &descrip, Sptr, &SptrN);
   if (status == IPR__SBNOTFND)
      return IPR__SUCCESS;
   if (!IPR_SUCCESS(status))
      return status;
   cnt = 0;
   while (SptrN)
   {
      status = KIprSbDelete(cpu,SptrN);
      if (!IPR_SUCCESS(status))
         return status;
      // Find next sample buffer - this will still be the 1st buffer
      status = KIprSbFind(cpu, &descrip, Sptr, &SptrN);
      if (status == IPR__SBNOTFND)
         return IPR__SUCCESS;
      if (!IPR_SUCCESS(status))
         return status;

      cnt++;
      if (cnt > IPR_MAX_LOOP_CNT)
      {
#ifdef TRC0
         TrcPrint("Went through loop %d times before deleting all sample buffers, exiting\n",
            cnt);
#endif
	 return IPR__PTRLOOPLIM;
      }
   }
   return IPR__SUCCESS;
}

static IPR_STATUS delete_histogram_buffers (IN IPR_CPUNUM cpu, IN IPR_HDESC *descrip)
{
   IPR_STATUS status;
   IPR_HBUFFER
      *FirstMatch, *HptrN;
   int cnt;

   cnt = 0;

   // Find the first matching histogram buffer

   status = KIprHbFind (cpu, NULL, descrip, &FirstMatch);
   if (status == IPR__HBNOTFND)
      FirstMatch = NULL;
   else if (!IPR_SUCCESS (status))
      return status;

   // The code below assumes that IPR__HBNOTFND will return
   // FALSE from IPR_SUCCESS.

#if (IPR_SUCCESS(IPR__HBNOTFND))
#  error "IPR__HBNOTFND is not an error code"
#endif

   while (IPR_SUCCESS (status))
   {

      // Find the next matching histogram buffer

      status = KIprHbFind(cpu, FirstMatch, descrip, &HptrN);

      // Delete the next buffer if one was found.  Note that
      // IPR__HBNOTFND is an error code, and that the loop
      // will terminate if the return code from HbDelete is
      // an error.

#if (IPR_SUCCESS(IPR__HBNOTFND))
#  error "IPR__HBNOTFND is not an error code"
#endif

      if (IPR_SUCCESS (status))
         status = KIprHbDelete(cpu,HptrN);

      // Verify that we did not cross the loop limit

      cnt++;
      if (cnt > IPR_MAX_LOOP_CNT)
      {
#ifdef TRC0
         TrcPrint("Went through loop %d times before deleting all histogram buffers, exiting\n",
            cnt);
#endif
         return IPR__PTRLOOPLIM;
      }
   }

   // Convert the status code if we either finished the loop
   // or found nothing in the first place

   if (status == IPR__HBNOTFND)
      status = IPR__SUCCESS;

   // Delete the first matching buffer

   if (FirstMatch && IPR_SUCCESS (status))
      status = KIprHbDelete (cpu, FirstMatch);
   return status;
}

static IPR_STATUS deallocate_buffers_for_cpu (IN IPR_CPUNUM cpu,
                                              IN int delflags)
{

   IPR_STATUS status;
   IPR_HDESC descrip;
   IPR_STATUS retstatus;

  // Delete everything that was requested

   retstatus = IPR__SUCCESS;
   if (delflags & DELBUF_SAMPLE)
   {
      status = delete_sample_buffers (cpu);
      if (IPR_SUCCESS(retstatus)
          && (status != IPR__BDNOTFND)
          && (status != IPR__SBNOTFND)
          && (status != IPR__HBNOTFND)
          && !IPR_SUCCESS(status))
          retstatus = status;
   }
   if (delflags & DELBUF_HISTOGRAM)
   {
      descrip.item.ipr_hmask_l_bits = 0;
      descrip.itemfilter.ipr_hmask_l_bits =
         BIprAll1 (descrip.itemfilter.ipr_hmask_l_bits);
      status = delete_histogram_buffers (cpu,&descrip);
      if (IPR_SUCCESS(retstatus)
          && (status != IPR__BDNOTFND)
          && (status != IPR__SBNOTFND)
          && (status != IPR__HBNOTFND)
          && !IPR_SUCCESS(status))
          retstatus = status;
   }
   if (delflags & DELBUF_DESCRIPTOR)
   {
      status = KIprBdDelete(cpu);
      if (IPR_SUCCESS(retstatus)
          && (status != IPR__BDNOTFND)
          && (status != IPR__SBNOTFND)
          && (status != IPR__HBNOTFND)
          && !IPR_SUCCESS(status))
          retstatus = status;
   }
   return retstatus;
}


/*****************************************************************************/
/* Buffer allocation routines                                                */
/*****************************************************************************/

static IPR_STATUS allocate_sample_buffer (
   IN IPR_CPUNUM cpu,          // CPU number
   IN IPR_SIZE bytes,          // Size of desired buffer
   IN IPR_DMASK datcol,
   IN IPR_DMASK qdatcol,
   OUT IPR_SBUFFER **retptr)
{
   IPR_SBUFFER
      *Sptr;
   IPR_STATUS status;
   IPR_SDESC descrip;

   // Allocate the buffer
   descrip.datamask = datcol;
   descrip.dataquadmask = qdatcol;
   status = KIprSbCreate(cpu,bytes,1,&descrip,&Sptr);
   *retptr = (IPR_SUCCESS (status)) ? Sptr : NULL;
   return status;
}

static IPR_STATUS allocate_buffer_descriptor (
   IN IPR_CPUNUM cpu,            // CPU number
   IN IPR_COUNT ctrcnt,          // Number of counters
   IN int SampleAction,          // Action at sample time (sampling only)
   IN int FullAction,            // Action at buffer full (sampling only)
   IN unsigned int TargetPID,    // Process to be notified (sampling only)
   IN int (*pRoutine) (),        // Buffer full routine (sampling only)
   IN int rangecnt,              // Number of PC ranges
   IN uint64 *ranges)            // PC ranges (alternate start and end)
{
   IPR_BUFDSC
      *BDptr;
   IPR_STATUS status;
   IPR_PC_RANGE pcrange;
   int cnt;

   // If buffer descriptor doesn't exist already, create it

   status = KIprBdCreate(cpu,&BDptr);
   if (!IPR_SUCCESS(status) && (status != IPR__BDEXISTS))
      return status;

   status = KIprBdSetNumCounters(cpu,ctrcnt);
   if (!IPR_SUCCESS(status))
      return status;

   if (FullAction)
   {
      status = KIprBdSetAction(cpu,IPR_ACTION_FULL,FullAction,pRoutine,
                               TargetPID);
      if (!IPR_SUCCESS(status))
         return status;
   }

   if (SampleAction && !FullAction)
   {
      status = KIprBdSetAction(cpu,IPR_ACTION_SAMPLE,SampleAction,NULL,
                               TargetPID);
      if (!IPR_SUCCESS(status))
         return status;
   }

   pcrange.pHBuf = NULL;
   pcrange.sample = TRUE;
   pcrange.histogram = FALSE;
   for (cnt = 0; cnt < rangecnt; cnt++)
   {
      pcrange.StartAddress = ranges[cnt*2];
      pcrange.EndAddress = ranges[(cnt*2)+1];
      status = KIprBdAddPcRange(cpu,&pcrange);
      if (!IPR_SUCCESS(status))
         return status;
   }

   return IPR__SUCCESS;
}

static IPR_STATUS allocate_histo_buffer (
   IN IPR_COUNT CCnt,      	// Number of counters
   IN IPR_HISTVALUE numvalues,  // Number of values represented by the histo
   IN int cpuID,           	// CPU number
   IN unsigned int item,        // Item collected in this histogram
   IN unsigned int index,  	// Index number for this histogram
   IN IPR_HISTVALUE minval,	// Minvalue (VA) for this histogram
   IN unsigned int gran,        // Granularity (shift value) for histogram
   OUT IPR_HBUFFER **retptr)
{
   IPR_HBUFFER
      *HistoPtr;
   IPR_HDESC
      HistoDesc;
   IPR_STATUS status;

   // If histograms created here should not be larger than will fit in a
   // single histogram  -
   // this should be taken care of before this routine is called.

   // Compute number of total elements in histogram and
   // fill in the descrip structure

   (void) IprHbElements (CCnt, numvalues, gran, &HistoDesc.elemcnt);
   HistoDesc.minval = minval;
   HistoDesc.numvalues = numvalues;
   HistoDesc.item.ipr_hmask_l_bits = item;
   HistoDesc.itemfilter.ipr_hmask_l_bits = 0;
   HistoDesc.gran = gran;
   HistoDesc.index = index;

   // Create the buffer
   status = KIprHbCreate(cpuID, &HistoDesc, &HistoPtr);
   if (!IPR_SUCCESS(status))
      return status;

   // Initialize Histogram to 0

   status = KIprHbClear(cpuID, HistoPtr, 0, HistoDesc.elemcnt);
   if (retptr)
      *retptr = HistoPtr;
   return status;
}

static IPR_STATUS add_address_histogram_buffer (
   IN IPR_CPUNUM cpu,         // CPU number
   IN IPR_COUNT ctrcnt,       // Number of counters
   IN IPR_HISTVALUE BeginAddr,// Starting address of range
   IN IPR_HISTVALUE EndAddr,  // Last address in the range
   IN unsigned int gran,      // Granularity in bits
   IN unsigned int index)     // Index number for comparisons
{
   IPR_COUNT
      elems,
      tmpelems,
      max_elems;
   IPR_HBUFFER
      *Hptr;
   IPR_STATUS
      status,
      max_status;
   IPR_HISTVALUE
      minval,
      maxval;
   IPR_PC_RANGE
      pcrange;
   IPR_BOOLEAN
      finished;

   //
   // Allocate as many buffers as needed to cover address range.  Allocate
   // them in increasing address order, so the actual buffers will be in
   // increasing address order when scanned from the buffer descriptor out.
   //

   if (gran < IPR_PCSHIFT)
      gran = IPR_PCSHIFT;

   // Determine the maximum number of elements, rounded down to a multiple of
   // the number of counters

   finished = FALSE;
   status = KIprHbMaxElements(&max_elems);
   max_elems -= max_elems % ctrcnt;

   while (BeginAddr <= EndAddr && !finished)
   {

      // Assume a maximum size buffer to start

      elems = max_elems;

      // Calculate the used ending address for the current buffer.  This is
      // done by calculating the minimum value for the first element beyond
      // the current buffer and subtracting one.  We will use the new beginning
      // address in creating the next buffer.

      minval = BeginAddr;
      max_status = IprHbValue (elems, ctrcnt, minval, gran, &maxval);
      maxval--;

      // Check if the maximum size buffer is too large.  Note that, for some
      // ranges, the value returned by IprHbValue may overflow, so we need
      // to check that the resultant value generates the same number of
      // elements we asked for.

      tmpelems = 0;
      status = IprHbElements (ctrcnt, maxval - minval + 1, gran, &tmpelems);
      if (max_status == IPR__HISTOVFLW ||
          status == IPR__HISTOVFLW ||
          maxval > EndAddr || tmpelems != elems)
      {

         // Too large, reduce to fit.  Note that this must be the last buffer.

         finished = TRUE;
         maxval = EndAddr;
         (void) IprHbElements (ctrcnt, maxval - minval + 1, gran, &elems);
      }

      // Reset the beginning address for the next time through

      (void) IprHbValue (elems, ctrcnt, minval, gran, &BeginAddr);
      if (minval >= BeginAddr)
         finished = TRUE;

      // Allocate the histogram buffer as specified.

      status = allocate_histo_buffer (ctrcnt, (maxval - minval) + 1, cpu,
                 IPR_HMASK_M_ADDRESS, index, minval, gran, &Hptr);

      if (!IPR_SUCCESS(status))
         return status;

      // Create the associated PC range

      pcrange.pHBuf = Hptr;
      pcrange.sample = FALSE;
      pcrange.histogram = TRUE;
      pcrange.StartAddress = minval;
      pcrange.EndAddress = maxval;
      status = KIprBdAddPcRange(cpu,&pcrange);
      if (!IPR_SUCCESS(status))
         return status;
   }
   return IPR__SUCCESS;
}

//
// Miscellaneous routines
//


static IPR_STATUS read_histogram_cpu (
   IN IPR_CPUNUM cpu,      	// CPU number
   OUT IPR_HISTELEM *rdata,	// Data to be returned
   IN IPR_SIZE rsize,      	// Size of returned data array
   IN IPR_HISTVALUE rmin,       // Minimum value to be returned
   IN IPR_HMASK htype,     	// Type of histogram to be read
   IN unsigned int index,  	// Matching index number
   IN IPR_BOOLEAN add_flag)	// TRUE = data is to be added to array, not copied
{
   IPR_HBUFFER
      *Hptr,*HptrN;
   IPR_HISTELEM *outbufP;
   IPR_COUNT
      counters;
   IPR_HDESC descrip;
   IPR_STATUS status;

   unsigned int cnt;

   int elemindex, outindex, numrtnelem, ecnt;
   IPR_COUNT numelem;
   IPR_SIZE elemleft;

   // Verify that the buffers are set up for histogramming

   counters = CtrNumCounters ();

   // Find the histogram buffer(s) of interest

   elemleft = rsize;				// num elem. in output buffer
   outindex = 0;				// index into output buffer

   //
   // Histogram buffers are in a linked list terminated by a pointer to
   // NULL.  Walk this list, returning data for all histogram buffers of
   // the appropriate type.
   //
   Hptr = NULL;
   descrip.itemfilter = htype;
   descrip.index = index;
   // Find 1st histogram buffer
   status = KIprHbFind(cpu, Hptr, &descrip, &HptrN);
   if (!IPR_SUCCESS(status))
      return status;
   cnt = 0;
   while (HptrN && (elemleft != 0))
   {

      // Find where the value is located in this histogram

      status = IprHbIndex(rmin, 0, counters, descrip.minval, descrip.gran,
                          &elemindex);
      if (!IPR_SUCCESS(status))
         return status;

      // check for no values

      if (status != IPR__NOTINHIST)
      {
         status = IprHbElements(counters, descrip.numvalues, descrip.gran,
                                 &numelem);
         if (!IPR_SUCCESS(status))
            return status;

         //  figure out number of elements based on output data size

         numrtnelem = elemleft - elemindex;
         if (numrtnelem > (((int)numelem)-elemindex))
         {
            numrtnelem = numelem - elemindex;
            elemleft -= numrtnelem;
            (void) IprHbValue (numrtnelem, counters, rmin, descrip.gran,
                               &rmin);
         }
         else
         {
            elemleft = 0;
         }
         if (add_flag)
         {
            outbufP = malloc(numrtnelem * sizeof(IPR_HISTELEM));
         }
         else
            outbufP = &rdata[outindex];
         status = KIprHbRead(cpu,HptrN,elemindex,numrtnelem,outbufP);
         if (!IPR_SUCCESS(status))
            return status;
         if (add_flag)
         {
            // add newvalues to old values
            for (ecnt = 0; ecnt < numrtnelem; ecnt++)
               rdata[outindex+ecnt] += outbufP[ecnt];
            // return memory
            free(outbufP);
         }
         outindex += numrtnelem;
         // adjust array size in case this copy spans multiple histograms
      }  // end check to see if the histogram found is the one we are looking for
      else
      {
         if (elemleft != rsize)
            return IPR__NOTALLREAD;
      }
      if (elemleft != 0)
      {
         Hptr = HptrN;
         // Find next histogram buffer
         status = KIprHbFind(cpu, Hptr, &descrip, &HptrN);
         if (!IPR_SUCCESS(status))
            if (elemleft < rsize)
               return IPR__NOTALLREAD;
            else
               return status;
      }

      cnt++;
      if (cnt > IPR_MAX_LOOP_CNT)
      {
#ifdef TRC0
         TrcPrint("Went through loop %d times before reading all histogram buffers, exiting\n",
            cnt);
#endif
	 return IPR__PTRLOOPLIM;
      }
   }
   return (elemleft == 0) ? IPR__SUCCESS : IPR__NOTALLREAD;
}

// Clear the histograms specified for the specified CPU

static IPR_STATUS clear_histogram_buffers_cpu (IN IPR_CPUNUM cpu,
                                               IN IPR_HMASK hmask,
                                               IN unsigned int index)
{
   IPR_HBUFFER
      *Hptr,*HptrN;
   IPR_COUNT
      counters;
   IPR_HDESC descrip;
   IPR_STATUS status;
   unsigned int cnt;
   IPR_COUNT numelem;

   counters = CtrNumCounters ();

   // Find the histogram buffer(s) of interest

   //
   // Histogram buffers are in a linked list terminated by a pointer to
   // NULL.  Walk this list, returning data for all histogram buffers of
   // the appropriate type.
   //
   Hptr = NULL;
   descrip.minval = 0;
   descrip.itemfilter.ipr_hmask_l_bits =
      BIprAll1 (descrip.itemfilter.ipr_hmask_l_bits);
   descrip.index = index;

   // Find 1st histogram buffer
   status = KIprHbFind(cpu, Hptr, &descrip, &HptrN);
   if (!IPR_SUCCESS(status))
      return status;
   cnt = 0;
   while (HptrN)
   {
      // KIprHbFind returns the index in descrip, make sure the
      // index matches if a nonzero index was passed
      if ((index == 0) || (index == descrip.index))
      {
         status = IprHbElements(counters, descrip.numvalues, descrip.gran,
                                 &numelem);
         if (!IPR_SUCCESS(status))
            return status;
         status = KIprHbClear(cpu, HptrN, 0, numelem);
         if (!IPR_SUCCESS(status))
            return status;
      }
      Hptr = HptrN;
      // Find next histogram buffer
      status = KIprHbFind(cpu, Hptr, &descrip, &HptrN);
      if (!IPR_SUCCESS(status))
         return status;

      cnt++;
      if (cnt > IPR_MAX_LOOP_CNT)
      {
#ifdef TRC0
         TrcPrint("Went through loop %d times before clearing all histogram buffers, exiting\n",
            cnt);
#endif
	 return IPR__PTRLOOPLIM;
      }
   }
   return IPR__SUCCESS;
}


/*****************************************************************************/
/* External inner-mode routines                                              */
/*                                                                           */
/* These routines are to be exported in some manner.  The OS-dependent       */
/* portion of the library will cause a transfer to an inner mode before      */
/* execution.  The equivalent name (without the K_ prefix) will either be    */
/* an alias name that runs through a kernel-mode dispatch or an actual       */
/* user-mode routine that saves data and makes an ioctl call.                */
/*****************************************************************************/

extern IPR_STATUS IprAllocate (void)
{
   IPR_STATUS status;

   if (!owner_valid ())
      if (counters_owned ())
         return IPR__NOTOWNER;

   // Set the owner to be the current process

   ShareArea -> counterowner = 0;
   if (!IPR_SUCCESS (allocate_iprdevice ()))
      return IPR__UNAVAILABLE;
   status = KIprGetCurrentIpid(&ShareArea -> counterowner);
   if (!IPR_SUCCESS(status))
      return status;
   return IPR__SUCCESS;
}

extern IPR_STATUS IprDeallocate (void)
{
   if (!owner_valid ())
      return IPR__NOTOWNER;

   if (counters_operating (IPR_ALL_CPUS))
      (void) IprStop (IPR_ALL_CPUS, 0);
   (void) IprDeleteBuffers (IPR_ALL_CPUS);
   (void) deallocate_iprdevice ();
   ShareArea -> counterowner = 0;
   return IPR__SUCCESS;
}

extern IPR_STATUS IprSetActiveCounters (
   IN IPR_CPUMASK cpus,         // CPUs affected
   IN IPR_CTRMASK validmask,    // Mask of valid counters
   IN IPR_CTRMASK *retmask)     // Returned mask for starting counters
{
   int
      current_cpu;
   IPR_FMASK
      funcs;
   IPR_STATUS
      status;

   if (!cpus)
      cpus = IPR_ALL_CPUS;

   // Caller must own the counters

   if (!owner_valid ())
      return IPR__NOTOWNER;

   // Counters must not be operating on the target CPUs

   if (counters_operating (cpus))
      return IPR__ALRACTIVE;

   // Loop through all selected and valid CPUs

   for (current_cpu = first_cpu (cpus, IPR_NOAFFINITY); current_cpu < IPR_MAX_CPUS;
        current_cpu = next_cpu (cpus, current_cpu, IPR_NOAFFINITY))
   {

      // Find the buffer descriptors

      status = KIprBdSetActiveCounters(current_cpu,validmask);
      if (!IPR_SUCCESS(status))
         return status;

   }
   if (retmask)
   {
      (void) CtrAvailableFuncs (&funcs);
      validmask = funcs . ipr_fmask_v_enable_indiv ? validmask : -1;
      *retmask = validmask;
   }

   return IPR__SUCCESS;
}

extern IPR_STATUS IprSetSampleSkip (
   IPR_CPUMASK cpus,         // CPUs affected
   IPR_COUNT *skip_values,   // Array of skip values, one per counter
   IPR_SIZE arrsize)         // Size of the array
{
   IPR_COUNT numcounters,i;
   IPR_CPUNUM
      current_cpu;
   IPR_STATUS status;

   if (!cpus)
      cpus = IPR_ALL_CPUS;

   // Caller must own the counters

   if (!owner_valid ())
      return IPR__NOTOWNER;

   // Counters must not be operating on the target CPUs

   if (counters_operating (cpus))
      return IPR__ALRACTIVE;

   // Loop through all selected and valid CPUs

   numcounters = CtrNumCounters ();

   for (current_cpu = first_cpu (cpus, IPR_NOAFFINITY); current_cpu < IPR_MAX_CPUS;
        current_cpu = next_cpu (cpus, current_cpu, IPR_NOAFFINITY))
   {
      for (i = 0; i < numcounters; i++)
      {
         status = KIprBdSetSkipCount(current_cpu,i,skip_values[i]);
         if (!IPR_SUCCESS(status))
            return status;
      }
   }

   return IPR__SUCCESS;
}

extern IPR_STATUS IprCreateSampleBuffers (
   IPR_CPUMASK cpus,          // Mask of desired CPUs
   IPR_COUNT bufcnt,          // Number of buffers
   IPR_SIZE bufsiz,           // Size of buffer in bytes
   IPR_DMASK datcol,          // Data to be collected
   IPR_DMASK qdatcol,         // Data to be collected
   int sampleaction,          // Action to perform in lieu of sampling
   int fullaction,            // Action to perform when buffer is full
   int (*pRoutine) (),        // Routine to be called
   IPR_COUNT rangecnt,        // Number of valid PC ranges
   uint64 *ranges)            // Valid PC ranges (start-end pairs)
{
   IPR_COUNT
      counters,i;
   IPR_CPUNUM  current_cpu;
   unsigned int current_PID;
   IPR_STATUS
      status;
   IPR_SBUFFER
      *sampBuff;

   if (!cpus)
      cpus = IPR_ALL_CPUS;

   // Caller must own the counters

   if (!owner_valid ())
      return IPR__NOTOWNER;

   // Counters must not be operating on the target CPUs

   if (counters_operating (cpus))
      return IPR__ALRACTIVE;

   counters = CtrNumCounters ();

   // Loop through all selected and valid CPUs

   for (current_cpu = first_cpu (cpus, IPR_NOAFFINITY); current_cpu < IPR_MAX_CPUS;
        current_cpu = next_cpu (cpus, current_cpu, IPR_NOAFFINITY))
   {

      // Destroy existing buffers

      (void) deallocate_buffers_for_cpu (current_cpu, DELBUF_SAMPLE);

      // Allocate and fill in buffer descriptor

      status = KIprGetCurrentIpid(&current_PID);
      if (!IPR_SUCCESS(status))
         return status;
      status =
         allocate_buffer_descriptor (current_cpu, counters,
                                     sampleaction, fullaction,
                                     current_PID, pRoutine, rangecnt,
                                     ranges);
      if (!IPR_SUCCESS(status))
         return status;

      // Allocate and fill in buffers

      for (i = 0; i < bufcnt; i++)
      {
         status = allocate_sample_buffer (current_cpu, bufsiz, datcol, qdatcol, &sampBuff);
         if (!IPR_SUCCESS(status))
            return status;
      }
   }
   return IPR__SUCCESS;
}

extern IPR_STATUS IprAddAddressHistogramBuffers (IN IPR_CPUMASK cpus,
                                                 IN IPR_HISTVALUE BeginAddr,
                                                 IN IPR_HISTVALUE EndAddr,
                                                 unsigned int gran,
                                                 unsigned int index)
{
   IPR_CPUNUM
      current_cpu;
   IPR_COUNT
      counters;
   IPR_STATUS
      status;

   if (!cpus)
      cpus = IPR_ALL_CPUS;

   // Caller must own the counters

   if (!owner_valid ())
      return IPR__NOTOWNER;

   // Counters must not be operating on the target CPUs

   if (counters_operating (cpus))
      return IPR__ALRACTIVE;

   counters = CtrNumCounters ();

   // Loop through all selected and valid CPUs

   for (current_cpu = first_cpu (cpus, IPR_NOAFFINITY); current_cpu < IPR_MAX_CPUS;
        current_cpu = next_cpu (cpus, current_cpu, IPR_NOAFFINITY))
   {
      status = add_address_histogram_buffer (current_cpu,
                                                     counters, BeginAddr,
                                                     EndAddr, gran, index);
      if (!IPR_SUCCESS (status))
         return status;
   }
   return IPR__SUCCESS;
}

extern IPR_STATUS IprCreateHistogramBuffers (IN IPR_CPUMASK cpus,
                                             IN IPR_HMASK histos,
                                             IN IPR_HISTVALUE BeginAddr,
                                             IN IPR_HISTVALUE EndAddr,
                                             IN unsigned int gran,
                                             IN unsigned int index)
{
   IPR_CPUNUM
      current_cpu;
   IPR_COUNT
      counters;
   IPR_STATUS
      status;
   IPR_HBUFFER
      *Hptr;

   if (!cpus)
      cpus = IPR_ALL_CPUS;

   // Caller must own the counters

   if (!owner_valid ())
      return IPR__NOTOWNER;

   // Counters must not be operating on the target CPUs

   if (counters_operating (cpus))
      return IPR__ALRACTIVE;

   counters = CtrNumCounters ();

   // Loop through all selected and valid CPUs

   status = IPR__SUCCESS;
   for (current_cpu = first_cpu (cpus, IPR_NOAFFINITY);
        current_cpu < IPR_MAX_CPUS && IPR_SUCCESS (status);
        current_cpu = next_cpu (cpus, current_cpu, IPR_NOAFFINITY))
   {
      // Destroy existing buffers

      (void) deallocate_buffers_for_cpu (current_cpu, DELBUF_HISTOGRAM);

      // Allocate and fill in buffer descriptor

      status =
         allocate_buffer_descriptor (current_cpu, counters,
                                     0, 0, 0, NULL, 0, NULL);
      if (!IPR_SUCCESS(status))
         return status;

      // Allocate and fill in buffers

      if (histos . ipr_hmask_v_total && IPR_SUCCESS (status))
         status = allocate_histo_buffer (counters, 1, current_cpu,
                                         IPR_HMASK_M_TOTAL, 0,
                                         0,0,&Hptr);

      if (histos . ipr_hmask_v_ipl && IPR_SUCCESS (status))
         status = allocate_histo_buffer (counters, IPR_IPL_COUNT,
                                         current_cpu, IPR_HMASK_M_IPL, 0,
                                         0,0,&Hptr);

      if (histos . ipr_hmask_v_mode && IPR_SUCCESS (status))
         status = allocate_histo_buffer (counters, IPR_MODE_COUNT,
                                         current_cpu, IPR_HMASK_M_MODE, 0,
                                         0,0,&Hptr);

      if (histos . ipr_hmask_v_priority && IPR_SUCCESS (status))
         status = allocate_histo_buffer (counters, IPR_PRIORITY_COUNT,
                                         current_cpu, IPR_HMASK_M_PRIORITY, 0,
                                         0,0,&Hptr);

      if (histos . ipr_hmask_v_address && IPR_SUCCESS (status))
         status = add_address_histogram_buffer (current_cpu,
                                                counters, BeginAddr, EndAddr,
                                                gran, index);
   }
   if (!IPR_SUCCESS (status))
   {
      for (current_cpu = first_cpu (cpus, IPR_NOAFFINITY);
           current_cpu < IPR_MAX_CPUS;
           current_cpu = next_cpu (cpus, current_cpu, IPR_NOAFFINITY))
         (void) deallocate_buffers_for_cpu (current_cpu, DELBUF_HISTOGRAM);
      return status;
   }

   return IPR__SUCCESS;
}

extern IPR_STATUS IprDeleteBuffers (IN IPR_CPUMASK cpus)
{
   IPR_CPUNUM
      current_cpu;
   IPR_STATUS
      status,
      retstatus;

   if (!cpus)
      cpus = IPR_ALL_CPUS;

   if (!owner_valid ())
      return IPR__NOTOWNER;
   if (counters_operating (cpus))
      return IPR__ALRACTIVE;

   // Loop through all selected and valid CPUs

   retstatus = IPR__SUCCESS;
   for (current_cpu = first_cpu (cpus, IPR_NOAFFINITY);
        current_cpu < IPR_MAX_CPUS;
        current_cpu = next_cpu (cpus, current_cpu, IPR_NOAFFINITY))
   {
      status = deallocate_buffers_for_cpu (current_cpu, DELBUF_ALL);
      if (!IPR_SUCCESS (status))
         retstatus = status;
   }
   return retstatus;
}

extern IPR_STATUS IprSelectEvents (IN IPR_CPUMASK cpus,
                                   IN IPR_EVENTSET *events,
                                   IN IPR_FREQ freq, IN uint64 *MUXvalue)
{
   IPR_CPUNUM
      current_cpu;
   IPR_STATUS
      status;

   if (!cpus)
      cpus = IPR_ALL_CPUS;

   if (!owner_valid ())
      return IPR__NOTOWNER;
   if (counters_operating (cpus))
      return IPR__ALRACTIVE;

   for (current_cpu = first_cpu (cpus, IPR_AFFINITY); current_cpu < IPR_MAX_CPUS;
        current_cpu = next_cpu (cpus, current_cpu, IPR_AFFINITY))
   {
      status = CtrSelectEvents (current_cpu, events, freq, MUXvalue);
   }
   return status;
}

extern IPR_STATUS IprSelectEventsFromArray (IN IPR_CPUMASK cpus,
                                            IN const IPR_EVENT *events,
                                            IN const IPR_FREQ *freqs,
                                            IN IPR_SIZE asize,
                                            IN uint64 *MUXvalue)
{
   int
      current_cpu;
   IPR_STATUS
      status;

   if (!cpus)
      cpus = IPR_ALL_CPUS;

   if (!owner_valid ())
      return IPR__NOTOWNER;
   if (counters_operating (cpus))
      return IPR__ALRACTIVE;

   for (current_cpu = first_cpu (cpus, IPR_AFFINITY); current_cpu < IPR_MAX_CPUS;
        current_cpu = next_cpu (cpus, current_cpu, IPR_AFFINITY))
   {
      status = CtrSelectEventsFromArray (current_cpu, events, freqs, asize,
                                         MUXvalue);
   }
   return status;
}

extern IPR_STATUS IprSetEvents (IN IPR_CPUMASK cpus, IN uint64 *MUXvalue)
{
   IPR_CPUNUM
      current_cpu;
   IPR_STATUS
      status;

   if (!cpus)
      cpus = IPR_ALL_CPUS;

   if (!owner_valid ())
      return IPR__NOTOWNER;
   for (current_cpu = first_cpu (cpus, IPR_AFFINITY); current_cpu < IPR_MAX_CPUS;
        current_cpu = next_cpu (cpus, current_cpu, IPR_AFFINITY))
   {
      status = CtrSetEvents (current_cpu, MUXvalue);
   }
   return status;
}

extern IPR_STATUS IprSetModes (IN IPR_CPUMASK cpus, IN IPR_MMASK modes)
{
   int
      current_cpu;
   IPR_STATUS
      status;

   if (!cpus)
      cpus = IPR_ALL_CPUS;

   if (!owner_valid ())
      return IPR__NOTOWNER;
   for (current_cpu = first_cpu (cpus, IPR_AFFINITY); current_cpu < IPR_MAX_CPUS;
        current_cpu = next_cpu (cpus, current_cpu, IPR_AFFINITY))
   {
      status = CtrSetModes (current_cpu, modes);
   }
   return status;

}

extern IPR_STATUS IprEventCounterMap (IN IPR_CPUMASK cpus,
                                      IN IPR_EVENT *event_arr,
                                      IN IPR_SIZE asize)
{
   IPR_STATUS
      status;
   IPR_CPUNUM
      current_cpu;
   IPR_COUNT
      counters;

   if (!cpus)
      cpus = IPR_CURRENT_CPU;

   counters = CtrNumCounters ();

   // Loop through all requested CPUs

   for (current_cpu = first_cpu (cpus, IPR_NOAFFINITY); current_cpu < IPR_MAX_CPUS;
        current_cpu = next_cpu (cpus, current_cpu, IPR_NOAFFINITY))
   {
      status = CtrEventCounterMap (current_cpu, event_arr, asize);
      if (!IPR_SUCCESS (status))
         return status;
      event_arr += counters;
      if (asize > counters)
         asize -= counters;
      else
         asize = 0;
   }
   return IPR__SUCCESS;
}

extern IPR_STATUS IprFreqCounterMap (IN IPR_CPUMASK cpus,
                                     IN IPR_FREQ *freq_arr,
                                     IN IPR_SIZE asize)
{
   IPR_STATUS
      status;
   IPR_CPUNUM
      current_cpu;
   IPR_COUNT
      counters;

   if (!cpus)
      cpus = IPR_CURRENT_CPU;

   if (!owner_valid ())
      return IPR__NOTOWNER;
   counters = CtrNumCounters ();

   // Loop through all requested CPUs

   for (current_cpu = first_cpu (cpus, IPR_NOAFFINITY); current_cpu < IPR_MAX_CPUS;
        current_cpu = next_cpu (cpus, current_cpu, IPR_NOAFFINITY))
   {
      status = CtrFreqCounterMap (current_cpu, freq_arr, asize);
      if (!IPR_SUCCESS (status))
         return status;
      freq_arr += counters;
      if (asize > counters)
         asize -= counters;
      else
         asize = 0;
   }
   return IPR__SUCCESS;
}

extern IPR_STATUS IprCounterNumber (IN IPR_CPUMASK cpus, IN IPR_EVENT event,
                                    IN IPR_CTRNUMBER *cnum)
{
   IPR_STATUS
      status;
   IPR_CPUNUM
      current_cpu;

   if (!cpus)
      cpus = IPR_CURRENT_CPU;

   // Loop through all requested CPUs

   if (!owner_valid ())
      return IPR__NOTOWNER;
   for (current_cpu = first_cpu (cpus, IPR_NOAFFINITY); current_cpu < IPR_MAX_CPUS;
        current_cpu = next_cpu (cpus, current_cpu, IPR_NOAFFINITY))
   {
      status = CtrCounterNumber (current_cpu, event, cnum);
      if (!IPR_SUCCESS (status))
         return status;
   }
   return IPR__SUCCESS;
}

extern IPR_STATUS IprStart (IN IPR_CPUMASK cpus, IN IPR_CTRMASK ctrs)
{
   IPR_CPUNUM
      current_cpu;
   IPR_STATUS
      status;

   if (!cpus)
      cpus = IPR_ALL_CPUS;
   if (!ctrs)
      ctrs = BIprAll1 (ctrs);

   if (!owner_valid ())
      return IPR__NOTOWNER;

   // Loop through all CPUs, setting affinity

   for (current_cpu = first_cpu (cpus, IPR_AFFINITY); current_cpu < IPR_MAX_CPUS;
        current_cpu = next_cpu (cpus, current_cpu, IPR_AFFINITY))
   {
      status = CtrEnable (current_cpu, ctrs);
      if (!IPR_SUCCESS (status))
      {
         clear_affinity ();
         return status;
      }
      ShareArea -> countersrunning =
         (unsigned int) BIprSet1 (ShareArea -> countersrunning, current_cpu);
   }
   return IPR__SUCCESS;
}

extern IPR_STATUS IprStop (IN IPR_CPUMASK cpus, IN IPR_CTRMASK ctrs)
{
   IPR_CPUNUM
      current_cpu;
   IPR_STATUS
      status;

   if (!cpus)
      cpus = IPR_ALL_CPUS;
   if (!ctrs)
      ctrs = BIprAll1 (ctrs);

   if (!owner_valid ())
      return IPR__NOTOWNER;

   // Loop through all CPUs, setting affinity

   for (current_cpu = first_cpu (cpus, IPR_AFFINITY); current_cpu < IPR_MAX_CPUS;
        current_cpu = next_cpu (cpus, current_cpu, IPR_AFFINITY))
   {
      status = CtrDisable (current_cpu, ctrs);
      if (!IPR_SUCCESS (status))
      {
         clear_affinity ();
         return status;
      }
      ShareArea -> countersrunning =
         (unsigned int) BIprClr1 (ShareArea -> countersrunning, current_cpu);
   }
   return IPR__SUCCESS;
}

extern IPR_STATUS IprPause (IN IPR_CPUMASK cpus, IN IPR_CTRMASK ctrs)
{
   IPR_CPUNUM
      current_cpu;
   IPR_STATUS
      status;
   IPR_BOOLEAN old_setting;

   if (!cpus)
      cpus = IPR_CURRENT_CPU;
   if (!ctrs)
      ctrs = BIprAll1 (ctrs);
   if (!counters_operating (cpus))
      return IPR__SUCCESS;

   //
   // So as not to cause inadvertant restart of the counters upon
   // rescheduling, clear the PME bit.  Save a copy of it for use by
   // the RESUME routine.
   //
   saved_PME = get_current_PME ();
   PME_was_saved = TRUE;
   if (saved_PME)
   {
      status = KIprSetPme(0,FALSE,FALSE,&old_setting);
      if (!IPR_SUCCESS (status))
      {
         return status;
      }
      current_pme_setting = FALSE;
   }

   // Loop through all CPUs, setting affinity

   for (current_cpu = first_cpu (cpus, IPR_AFFINITY); current_cpu < IPR_MAX_CPUS;
        current_cpu = next_cpu (cpus, current_cpu, IPR_AFFINITY))
   {
      status = CtrPause (current_cpu, ctrs);
      if (!IPR_SUCCESS (status))
      {
         clear_affinity ();
         return status;
      }
   }
   return IPR__SUCCESS;
}

extern IPR_STATUS IprResume (IN IPR_CPUMASK cpus, IN IPR_CTRMASK ctrs)
{
   IPR_CPUNUM
      current_cpu;
   IPR_STATUS
      status;
   IPR_BOOLEAN old_setting;

   if (!cpus)
      cpus = IPR_CURRENT_CPU;
   if (!ctrs)
      ctrs = BIprAll1 (ctrs);
   if (!counters_operating (cpus))
      return IPR__SUCCESS;

   //
   // The PME bit may have been set before, but cleared by a PAUSE call.
   // In that case, we need to set it again.
   //
   if (PME_was_saved && saved_PME)
   {
      status = KIprSetPme(0,TRUE,FALSE,&old_setting);
      if (!IPR_SUCCESS (status))
      {
         return status;
      }
      current_pme_setting = TRUE;
   }
   PME_was_saved = FALSE;

   // Loop through all CPUs, setting affinity

   for (current_cpu = first_cpu (cpus, IPR_AFFINITY); current_cpu < IPR_MAX_CPUS;
        current_cpu = next_cpu (cpus, current_cpu, IPR_AFFINITY))
   {
      status = CtrResume (current_cpu, ctrs);
      if (!IPR_SUCCESS (status))
      {
         clear_affinity ();
         return status;
      }
   }
   return IPR__SUCCESS;
}

/*****************************************************************************/
/* NOTE WELL:                                                                */
/*                                                                           */
/* The routines below make use of the IPR_HACTION_* constants.  They are     */
/* implemented in a manner that depends on there being exactly three values: */
/* add, sum, and copy.  The two statements of interest are these:            */
/*                                                                           */
/*     add_flag = (action_flags == IPR_HACTION_ADD);                         */
/*                                                                           */
/* in the start of the routine, and                                          */
/*                                                                           */
/*     add_flag = (action_flag != IPR_HACTION_COPY);                         */
/*                                                                           */
/* at the end of the loop.  The first statement will cause the results to be */
/* added into the array the first time through if and only if the requested  */
/* action was ADD.  The second statement will cause subsequent iterations to */
/* add into the array if and only if the action was not COPY, i.e., if it    */
/* was ADD or SUM.  This behavior is correct, but will need to be revisited  */
/* if, for example, a SUBTRACT option is desired.                            */
/*****************************************************************************/

extern IPR_STATUS IprRead (IN IPR_CPUMASK cpus, IN int *cpuctxptr,
                           IN IPR_CTRMASK cmask,
                           OUT int *retdata, IN IPR_SIZE rsize,
                           IN int action_flags)
{
   IPR_BOOLEAN
      add_flag;
   IPR_SIZE
      tsiz;
   IPR_STATUS
      status;
   IPR_COUNT
      counters;
   IPR_SIZE
      i;
   int
      *tptr;
   IPR_CPUNUM
      current_cpu;
   static int
      *tmpdata = NULL;

   if (!cpus)
      cpus = IPR_ALL_CPUS;
   if (!cmask)
      cmask = BIprAll1 (cmask);
   if (!action_flags)
      action_flags = cpuctxptr ? IPR_HACTION_COPY : IPR_HACTION_COPY;

   add_flag = (action_flags == IPR_HACTION_ADD);
   counters = CtrNumCounters ();
   if (!tmpdata)
      tmpdata = malloc (counters * sizeof (*tmpdata));

   // Loop through all CPUs, setting affinity

   for (current_cpu = first_cpu_with_context (cpus, cpuctxptr, IPR_AFFINITY);
        current_cpu < IPR_MAX_CPUS;
        current_cpu = next_cpu_with_context (cpus, cpuctxptr, current_cpu,
                                             IPR_AFFINITY))
   {
      //
      // If just copying the data, have it written directly into the target
      // array.  Otherwise, we need to store it in a temporary location and
      // add it into the target array.
      //
      tptr = add_flag ? tmpdata : retdata;
      tsiz = add_flag ? counters : rsize;
      status = CtrRead (current_cpu, cmask, tptr, tsiz);
      if (!IPR_SUCCESS (status))
      {
         clear_affinity ();
         return status;
      }

      // Add in the saved data if necessary

      if (add_flag)
         for (i = 0; i < counters && i < rsize; i++)
            retdata [i] += tmpdata [i];
      add_flag = (action_flags != IPR_HACTION_COPY);
   }
   return IPR__SUCCESS;
}

extern IPR_STATUS IprReadAll (IN IPR_CPUMASK cpus, IN int *cpuctxptr,
                              OUT int *retdata,
                              IN IPR_SIZE rsize, IN int action_flags)
{
   IPR_BOOLEAN
      add_flag;
   IPR_SIZE
      tsiz;
   IPR_STATUS
      status;
   IPR_COUNT
      counters;
   int
      *tptr;
   IPR_SIZE   i;
   IPR_CPUNUM   current_cpu;
   static int
      *tmpdata = NULL;

   if (!cpus)
      cpus = IPR_ALL_CPUS;
   if (!action_flags)
      action_flags = cpuctxptr ? IPR_HACTION_COPY : IPR_HACTION_SUM;

   add_flag = (action_flags == IPR_HACTION_ADD);
   counters = CtrNumCounters ();
   if (!tmpdata)
      tmpdata = malloc (counters * sizeof (*tmpdata));

   // Loop through all CPUs, setting affinity

   for (current_cpu = first_cpu_with_context (cpus, cpuctxptr, IPR_AFFINITY);
        current_cpu < IPR_MAX_CPUS;
        current_cpu = next_cpu_with_context (cpus, cpuctxptr, current_cpu,
                                             IPR_AFFINITY))
   {
      //
      // If just copying the data, have it written directly into the target
      // array.  Otherwise, we need to store it in a temporary location and
      // add it into the target array.
      //
      tptr = add_flag ? tmpdata : retdata;
      tsiz = add_flag ? sizeof (tmpdata) : rsize;
      status = CtrReadAll (current_cpu, tptr, tsiz);
      if (!IPR_SUCCESS (status))
      {
         clear_affinity ();
         return status;
      }

      // Add in the saved data if necessary

      if (add_flag)
         for (i = 0; i < counters && i < rsize; i++)
            retdata [i] += tmpdata [i];
      add_flag = (action_flags != IPR_HACTION_COPY);
   }
   return IPR__SUCCESS;
}

extern IPR_STATUS IprWrite (IN IPR_CPUMASK cpus, IN int *cpuctxptr,
                            IN IPR_CTRMASK cmask,
                            IN int *srcdata, IN IPR_SIZE srcsize)
{
   IPR_STATUS
      status;
   IPR_CPUNUM
      current_cpu;

   if (!cpus)
      cpus = IPR_CURRENT_CPU;
   if (!cmask)
      cmask = BIprAll1 (cmask);

   // Loop through all CPUs, setting affinity

   for (current_cpu = first_cpu_with_context  (cpus, cpuctxptr, IPR_AFFINITY);
        current_cpu < IPR_MAX_CPUS;
        current_cpu = next_cpu_with_context (cpus, cpuctxptr, current_cpu,
                                             IPR_AFFINITY))
   {
      status = CtrWrite (current_cpu, cmask, srcdata, srcsize);
      if (!IPR_SUCCESS (status))
      {
         clear_affinity ();
         return status;
      }
   }
   return IPR__SUCCESS;
}

extern IPR_STATUS IprClear (IN IPR_CPUMASK cpus, IN IPR_CTRMASK ctrs)
{
   IPR_STATUS
      status;
   IPR_CPUNUM
      current_cpu;

   if (!cpus)
      cpus = IPR_CURRENT_CPU;
   if (!ctrs)
      ctrs = BIprAll1 (ctrs);

   // Loop through all CPUs, setting affinity

   for (current_cpu = first_cpu (cpus, IPR_AFFINITY); current_cpu < IPR_MAX_CPUS;
        current_cpu = next_cpu (cpus, current_cpu, IPR_AFFINITY))
   {
      status = CtrClear (current_cpu, ctrs);
      if (!IPR_SUCCESS (status))
      {
         clear_affinity ();
         return status;
      }
   }
   return IPR__SUCCESS;
}

extern IPR_STATUS IprReadHistogram (IN IPR_CPUMASK cpus, IN int *cpuctxptr,
                                    OUT IPR_HISTELEM *retdata,
                                    IN IPR_SIZE retsize,
                                    IN IPR_HISTVALUE retmin,
                                    IN IPR_HMASK htype,
                                    IN unsigned int index, IN int action_flags)
{
   IPR_STATUS
      status;
   IPR_BOOLEAN
      add_flag;
   IPR_CPUNUM
      current_cpu;

   if (!cpus)
      cpus = IPR_ALL_CPUS;
   if (!action_flags)
      action_flags = cpuctxptr ? IPR_HACTION_COPY : IPR_HACTION_SUM;

   if (!owner_valid ())
      return IPR__NOTOWNER;

   add_flag = (action_flags == IPR_HACTION_ADD);

   // Loop through all CPUs

   for (current_cpu = first_cpu_with_context (cpus, cpuctxptr, IPR_NOAFFINITY);
        current_cpu < IPR_MAX_CPUS;
        current_cpu = next_cpu_with_context (cpus, cpuctxptr, current_cpu,
                                             IPR_NOAFFINITY))
   {
      status = read_histogram_cpu (current_cpu, retdata, retsize, retmin,
                                   htype, index, add_flag);
      if (!IPR_SUCCESS (status))
         return status;
      add_flag = (action_flags != IPR_HACTION_COPY);
   }
   return IPR__SUCCESS;
}

extern IPR_STATUS IprClearHistograms (IN IPR_CPUMASK cpus, IN IPR_HMASK hmask,
                                      IN unsigned int index)
{
   IPR_CPUNUM
      current_cpu;
   IPR_STATUS
      status;

   if (!cpus)
      cpus = IPR_ALL_CPUS;

   if (!owner_valid ())
      return IPR__NOTOWNER;

   // Loop through all the CPUs

   for (current_cpu = first_cpu (cpus, IPR_NOAFFINITY);
        current_cpu < IPR_MAX_CPUS;
        current_cpu = next_cpu (cpus, current_cpu, IPR_NOAFFINITY))
   {
      status = clear_histogram_buffers_cpu (current_cpu, hmask, index);
      if (!IPR_SUCCESS (status) && status != IPR__HBNOTFND)
         return status;
   }
   return IPR__SUCCESS;
}

/*****************************************************************************/
/* Routines to facilitate writing sample buffers                             */
/*                                                                           */
/* These routines, plus IprWriteToSampleFile, allow a user-mode routine to   */
/* write out the sample buffers.  They incorporate all functions that must   */
/* be done in kernel mode.                                                   */
/*****************************************************************************/

// Mark a sample buffer as empty

extern IPR_STATUS IprMarkEmptySampleBuffer (OUT IPR_SBUFFER *Sptr)
{
   return KIprSbSetFull(Sptr,IPR_BUFFER_EMPTY);
}

/*****************************************************************************/
/* IprNextFullSampleBuffer is intended to be called multiple times.  Each    */
/* call will continue from where it left off.  The routine will examine each */
/* sample buffer in turn for each CPU in turn, and find ones that should be  */
/* written out according to the needs of the caller.  Each call returns one  */
/* sample buffer address, or returns a status that indicates no more full    */
/* sample buffers can be found.                                              */
/*****************************************************************************/

extern IPR_STATUS IprNextFullSampleBuffer (
   IN IPR_CPUMASK cpus,         // CPUs to be checked
   OUT IPR_SBUFFER **bctx,      // Returned pointer and previous context
   OUT IPR_SIZE *bsize,         // Returned size of buffer
   IN IPR_BOOLEAN write_flag)   // Shows whether to check for partial buffers
{
   static IPR_CPUNUM
      current_cpu;      // CPU context saved internally
   IPR_BOOLEAN
      partwrite,        // Indicates a partial buffer write is to be done
      new_cpu,          // Indicates a new CPU number is needed
      write_this;       // Indicates the current buffer is to be written
   static IPR_BUFDSC
      *BDptr;
   static IPR_SBUFFER
      *first_buffer,
      *sentinel,
      *Sptr,
      *SptrP;
   IPR_STATUS status;
   IPR_SDESC descrip;

   if (!cpus)
      cpus = IPR_ALL_CPUS;

   if (!owner_valid ())
      return IPR__NOTOWNER;

   partwrite = FALSE;
   write_this = FALSE;

   // If no context, this is first call, so prepare to get a CPU number

   new_cpu = (!*bctx);
   //
   // Loop through the CPUs, starting either where we left off or at the
   // very first matching CPU
   //
   for (current_cpu = (new_cpu ? first_cpu (cpus, IPR_NOAFFINITY)
                               : current_cpu);
        current_cpu < IPR_MAX_CPUS;
        current_cpu = next_cpu (cpus, current_cpu, IPR_NOAFFINITY))
   {
      if (new_cpu)
      {
         //
         // We have a new CPU, so get new values for CPU-dependent data.
         // Skip this CPU if it is not set up for sampling.
         //
         SptrP = NULL;
         status = KIprSbFind(current_cpu,&descrip,NULL,&SptrP);
         if (!IPR_SUCCESS(status))
             return status;
         first_buffer = SptrP;
         status = KIprSbFind(current_cpu,&descrip,SptrP,&Sptr);
         if (!IPR_SUCCESS(status))
             return status;
         sentinel = NULL;
      }
      else
      {

         // Not a new CPU, so start with the buffer after the last one seen

         SptrP = *bctx;
         status = KIprSbFind(current_cpu,&descrip,SptrP,&Sptr);
         if (!IPR_SUCCESS(status))
            return status;
      }

      new_cpu = TRUE;
      //
      // Walk the buffer list until we reach the sentinel again.  To
      // force going through it the first time, the sentinel is initially
      // null.
      //
      while (Sptr != sentinel)
      {
         if (!sentinel)
            sentinel = Sptr;

         // Always write out the buffer if it is full

         if (descrip.full == IPR_BUFFER_FULL)
            write_this = TRUE;
         //
         // Write out a partial buffer only if:
         //  - Partial buffer writes are requested;
         //  - The buffer is one currently pointed to by the buffer descriptor;
         //  - There is something in it (non-zero area offset);
         //  - The area offset is not greater than the area size, since a
         //    greater area size and an unmarked buffer is only seen at buffer
         //    initialization, not in actual use.
         //
         else if ((write_flag == IPR_WRITE_PARTIAL && Sptr == first_buffer)
                    && descrip.areaoffset
                    && (descrip.areaoffset <= descrip.areasize))
         {
            write_this = TRUE;
            //
            // If the buffer is really full, just not marked yet (i.e., the
            // next sample collection would have caused it to be marked), mark
            // it and treat it as full.
            //
            // NOTE: This action is based on the assumption that partial buffer
            // writing is only done with the counters deactivated.
            //
            if (descrip.areaoffset == descrip.areasize)
            {
               status = KIprSbSetFull(Sptr,IPR_BUFFER_FULL);
               if (!IPR_SUCCESS(status))
                  return status;
            }
            else
               partwrite = TRUE;
         }

         // If OK, return the buffer

         if (write_this)
         {
            *bctx = Sptr;
            *bsize = descrip.size;
            if (partwrite)
               return IPR__PARTBUFF;
            else
               return IPR__SUCCESS;
         }

         // Buffer is not to be written, move on to next one

         SptrP = Sptr;
         status = KIprSbFind(current_cpu,&descrip,SptrP,&Sptr);
         if (!IPR_SUCCESS(status))
            return status;
      }
   }

   // Went through the whole list, so return empty

   *bctx = NULL;
   *bsize = 0;
   return IPR__NOMOREBUFF;
}

extern IPR_STATUS IprClearContext (IN IPR_CPUMASK cpus)
{
   IPR_STATUS
      status;
   int
      current_cpu;

   if (!cpus)
      cpus = IPR_ALL_CPUS;

   for (current_cpu = first_cpu (cpus, IPR_NOAFFINITY); current_cpu < IPR_MAX_CPUS;
        current_cpu = next_cpu (cpus, current_cpu, IPR_NOAFFINITY))
   {
      status = CtrClearContext (current_cpu);
      if (!IPR_SUCCESS (status))
         return status;
   }
   return IPR__SUCCESS;
}


/*****************************************************************************/
/*                                                                           */
/* Alias routines                                                            */
/*                                                                           */
/* Here are listed Ipr routines that will simply be created as aliases for   */
/* Ctr routines.  In some cases, they will dispatch to the appropriate       */
/* mode.                                                                     */
/*                                                                           */
/* IprNumCounters                                                            */
/* IprClearEventSet                                                          */
/* IprInsertEvent                                                            */
/* IprMaskEventSet                                                           */
/* IprRemoveEventSet                                                         */
/* IprVerifyEventInSet                                                       */
/* IprEventToNumber                                                          */
/* IprNumberEventsInSet                                                      */
/* IprExpandEventSet                                                         */
/* IprMergeEventSet                                                          */
/*                                                                           */
/*****************************************************************************/

