/****************************************************************************/
/*                                                                          */
/*  COPYRIGHT (c) 1993-1999 BY                                              */
/*  Digital Equiptment/Compaq Computer Corporation.                         */
/*  ALL RIGHTS RESERVED.                                                    */
/*                                                                          */
/*  This file is part of the Iprobe performance suite for Alpha processors. */
/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 2 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License       */
/*  along with this program; if not, write to the Free Software             */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307    */
/*                                                                          */
/****************************************************************************/

/*
 * Abstract:	This file contains user mode performance monitoring
 *		routines.  It expects to be in an image that allows use
 *		of the C run-time library.
 *
 * Author:		Brian F. Wibecan
 * Creation Date:	16-Dec-1993
 *
 * Modified by:
 *
 * X-18 BMW		Bradley M. Waters	23-Aug-1994
 *	Changed names from Pfm to Ipr.
 *
 * X-17	BMW		Bradley M. Waters	18-Jul-1994
 *	Modified based on kernel rationalization.
 *
 * X-16	BMW		Bradley M. Waters	15-Jul-1994
 *	Added include for api_routines_nt.h to define API routines to ctr...
 *
 * X-15	BFW1100		Brian F. Wibecan	19-May-1994
 *	Fix IprSpecifyOperations fill-in index problem.
 *
 * X-14	BFW1099		Brian F. Wibecan	18-May-1994
 *	Added use of IprSelectEventsFromArray in IprSpecifyOperations.
 *
 * X-13	BFW1095		Brian F. Wibecan	17-May-1994
 *	Made IprWriteSampleBuffers use new function IprWriteOneSampleBuffer.
 *
 * X-12	BFW1080		Brian F. Wibecan	04-May-1994
 *	Modified #pragma module to be compatible with CC on OSF/1.
 *
 * X-11	BFW1071		Brian F. Wibecan	06-Apr-1994
 *	Changed user routines to have simple names.  Added #pragma module
 *	statement.
 *
 * X-10	BFW1067		Brian F. Wibecan	23-Mar-1994
 *	Added argument to IprSpecifyOperations to return the valid counter
 *	mask.
 *
 * X-9	???		Brian F. Wibecan	???
 *
 * X-8	BFW1066		Brian F. Wibecan	23-Mar-1994
 *	Made IprSpecifyOperations add more events if the event set
 *	is incomplete, and set the valid counter mask accordingly.
 *
 * X-7	BFW1034		Brian F. Wibecan	07-Mar-1994
 *	Fixed loop logic in IprReadMultipleHistograms.
 *
 * X-6	BFW1030		Brian F. Wibecan	24-Feb-1994
 *	Fix SpecifyOperations to handle multiple frequency specs correctly.
 *
 * X-5	BFW1020		Brian F. Wibecan	02-Feb-1994
 *	Change header file to CTR_ROUTINES.  Change name to Ipr_USER_ROUTINES.
 *
 * X-4	BFW1005		Brian F. Wibecan	05-Jan-1993
 *	Added use of header files to define function prototypes.
 *
 * X-3	BFW1003		Brian F. Wibecan	03-Jan-1993
 *	Added index to histogram buffers.
 *
 * X-2	???		Brian F. Wibecan	???
 *
 * X-1	BFW1000		Brian F. Wibecan	16-Dec-1993
 *	Original version.
 *
*/

#ifdef __NT
#  include "ipr_shr_nt.h"
// includes windows.h for basic type defs for Windows NT
#endif
#include <stdlib.h>
#include "ipr_api_ext_defs.h"

/*****************************************************************************/
/*                                                                           */
/* Performance monitoring user mode routines                                 */
/*                                                                           */
/*****************************************************************************/

extern IPR_STATUS IprWriteSampleBuffers (
   IN IPR_CPUMASK cpus,     // Valid CPU mask
   IN int write_flag,       // Indicates whether to write partial buffers
   OUT int *rbufwrites,     // Optional returned number of buffers written
   OUT int *rpartwrites)    // Optional returned number of partial writes
{
   int
      bufwrites,
      partwrites;
   IPR_STATUS
      status,
      wstat;
   IPR_SIZE
      Ssize;

   IPR_SBUFFER
      *Sptr;

   Sptr = NULL;
   bufwrites = 0;
   partwrites = 0;

   // Go through all full buffers, writing out and marking empty as we go

   do
   {
      status = IprNextFullSampleBuffer (cpus, &Sptr, &Ssize, write_flag);

      // Indicate partial write if we come across a partially full buffer

      if (status == IPR__PARTBUFF)
         partwrites++;

         // Write the buffer and mark it empty

      if (status == IPR__PARTBUFF || status == IPR__SUCCESS)
      {
         wstat = IprWriteOneSampleBuffer (Sptr, Ssize);
         if (!IPR_SUCCESS (wstat))
            status = wstat;
         bufwrites++;
         wstat = IprMarkEmptySampleBuffer (Sptr);
         if (IPR_SUCCESS (status) && !IPR_SUCCESS (wstat))
            status = wstat;
      }
   } while (IPR_SUCCESS (status) && status != IPR__NOMOREBUFF);
   if (!IPR_SUCCESS (status))
      return status;

   // Return the buffer write counts if the arguments were passed

   if (rbufwrites)
      *rbufwrites = bufwrites;
   if (rpartwrites)
      *rpartwrites = partwrites;
   return IPR__SUCCESS;
}

extern IPR_STATUS IprTerminateSampling (
   OUT int *rbufwrites,
   OUT int *rpartwrites)
{
   int
      i, j;
   IPR_STATUS
      status [4];
   int sbuffwrites,spartwrites;

   i = 0;
   status [i++] = IprStop (IPR_ALL_CPUS, 0);
   status [i++] = IprWriteSampleBuffers (IPR_ALL_CPUS, IPR_WRITE_PARTIAL,
                                         rbufwrites, rpartwrites);
   status [i++] = IprCloseSampleFile (&sbuffwrites,&spartwrites);
   status [i++] = IprDeleteBuffers (IPR_ALL_CPUS);
   for (j = 0; j < i; j++)
   if (!IPR_SUCCESS (status [j]))
      return status [j];

   if (rbufwrites)
      *rbufwrites += sbuffwrites;
   if (rpartwrites)
      *rpartwrites += spartwrites;

   return IPR__SUCCESS;

}

extern IPR_STATUS IprSpecifyOperations (
   IN IPR_CPUMASK cpus,
   IN IPR_EVENT *selected_event,
   IN IPR_FREQ *selected_frequency,
   IN IPR_COUNT ecount,
   IN IPR_MMASK selected_modes,
   OUT uint64 *MUXvalue,
   OUT IPR_CTRMASK *ctrmask)
{
   IPR_FREQ
      ignored_frequency,
      *pF;
   IPR_COUNT
      i,j,
      intcnt,
      ctrcnt;
   IPR_FMASK
      funcs;
   IPR_STATUS
      status;
   void
      *e_set,
      *n_set;
   IPR_EVENT
      e,
      *pE,
      *pEMap;

   ctrcnt = IprNumCounters ();
   if (ecount <= 0 || ecount > ctrcnt)
      return IPR__INVALID;

   // Create shadow arrays to hold the desired items

   pE = malloc (sizeof (IPR_EVENT) * ctrcnt);
   pF = malloc (sizeof (IPR_FREQ) * ctrcnt);
   e_set = malloc (IprEventSetSize ());
   n_set = malloc (IprEventSetSize ());
   (void) IprClearEventSet (e_set);
   (void) IprClearEventSet (n_set);
   intcnt = ecount;

   // Copy the data into the shadow arrays

   for (i = 0; i < intcnt; i++)
   {
      pE [i] = selected_event [i];
      pF [i] = selected_frequency [i];
      (void) IprInsertEvent (e_set, selected_event [i]);
   }
   //
   // Get a list of all events available with the initial set.  It may be
   // a bit optimistic, so handle that later.
   //
   status = IprAvailableEvents (e_set, n_set);
   if (!IPR_SUCCESS (status))
      return status;

   // Get a frequency to use for any added events

   (void) IprAvailableFuncs (&funcs);
   ignored_frequency = funcs . ipr_fmask_v_count_only ? 0
                                                      : IPR_FREQUENCY_LOWEST;

   // Clear the slate and attempt the selection

   (void) IprClearContext (cpus);
   status = IprSelectEventsFromArray (cpus, pE, pF, intcnt, MUXvalue);

   // If a counter map is desired, get it now

   if ((IPR_SUCCESS (status) || status == IPR__INCOMPLETE) && ctrmask)
   {
      pEMap = malloc (ctrcnt * sizeof (IPR_EVENT));
      (void) IprEventCounterMap (cpus, pEMap, ctrcnt);
      *ctrmask = 0;
      for (i = 0; i < ctrcnt; i++)
      {
         for (j = 0; j < ecount; j++)
            if (pE [j] && pE [j] == pEMap [i])
               *ctrmask |= (1 << i);
      }
      free (pEMap);
   }

   // If we are not finished, try to fill in if possible

   while (!IPR_SUCCESS (status) || status == IPR__INCOMPLETE)
   {
      //
      // It did not work.  If it was a pure failure, and we have not added any
      // events, then we cannot do anything about it, so return the failure
      // status.
      //
      if (status != IPR__INCOMPLETE)
         if (intcnt <= ecount)
            return status;
         else
            intcnt--;
      //
      // Attempt to add an event to the list.  Pick one of the ones from the
      // available list.  Make sure there is at least one there.  Remove it
      // from the available list so we do not pick it again.
      //
      e = IprEventToNumber (n_set);
      if (!e)
         return IPR__INVALID;
      (void) IprDeleteEvent (n_set, e);
      pE [intcnt] = e;
      pF [intcnt++] = ignored_frequency;

      // Attempt the assignment again

      (void) IprClearContext (cpus);
      status = IprSelectEventsFromArray (cpus, pE, pF, intcnt, MUXvalue);
   }
   //
   // To get out of the loop above, we must have succeeded.  Everything is
   // OK, so set the events in the counters and set the modes.
   //
   if (IPR_SUCCESS (status))
      status = IprSetModes (cpus, selected_modes);
   if (IPR_SUCCESS (status))
      status = IprSetEvents (cpus, 0);
   free (e_set);
   free (n_set);
   free (pE);
   free (pF);
   return status;
}

extern IPR_STATUS IprReadMultipleHistograms (
   IN IPR_CPUMASK cpus,         // CPU mask
   IN int *cpuctxptr,           // CPU context
   OUT IPR_HISTELEM **retdata,  // Array of array pointers for return data
   OUT IPR_SIZE *retsize,       // Array of array sizes
   OUT IPR_HISTVALUE *retmin,   // Array of minimum values
   OUT IPR_HMASK *htype,        // Array of histogram types
   OUT unsigned int *index,     // Array of buffer index values
   IN int action_flags,         // Indicates how to store data
   IN IPR_COUNT itemcnt)        // Number of arrays
{
   IPR_COUNT
      idx;
   IPR_STATUS
      status;
   IPR_CPUMASK
      cpumsk;
   IPR_BOOLEAN
      first_iteration;
   int
      tmpctx,
      starting_ctx;

   cpumsk = cpus;
   first_iteration = TRUE;
   //
   // Save the initial context value for use in repeated calls to
   // IprReadHistogram.
   //
   if (cpuctxptr)
      starting_ctx = *cpuctxptr;
   for (idx = 0; idx < itemcnt; idx++)
   {

      // Read each requested histogram using the same context value

      if (cpuctxptr)
         tmpctx = starting_ctx;
      status = IprReadHistogram (cpumsk,
                                 cpuctxptr ? &tmpctx : NULL,
                                 retdata [idx],
                                 retsize [idx],
                                 retmin ? retmin [idx] : 0,
                                 htype [idx],
                                 index ? index [idx] : 0,
                                 action_flags);
      if (!IPR_SUCCESS (status))
         return status;
      if (first_iteration && cpuctxptr)
      {

         // Save the next CPU number the first time through

         *cpuctxptr = tmpctx;
         //
         // If we are walking the list of CPUs, the context value will be
         // the same as the last CPU visited.  If we have already visited
         // the last CPU, the context value will be at least IPR_MAX_CPUS, and
         // we are done.
         //
         if (tmpctx >= IPR_MAX_CPUS)
         {
            return IPR__SUCCESS;
         }
         //
         // We are not done yet.  We want to make sure to get the same CPU
         // value for each additional histogram, so we create an artificial
         // temporary CPU mask that contains only the last CPU seen.
         //
         cpumsk = 1 << tmpctx;
      }
      first_iteration = FALSE;
   }
   return IPR__SUCCESS;
}
