/****************************************************************************/
/*                                                                          */
/*  COPYRIGHT (c) 1993-1999 BY                                              */
/*  Digital Equiptment/Compaq Computer Corporation.                         */
/*  ALL RIGHTS RESERVED.                                                    */
/*                                                                          */
/*  This file is part of the Iprobe performance suite for Alpha processors. */
/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 2 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License       */
/*  along with this program; if not, write to the Free Software             */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307    */
/*                                                                          */
/****************************************************************************/


/*
 * Abstract:	This file contains VMS-specific routines needed by all chip
 *		types.
 *
 * Author:		Brian F. Wibecan
 * Creation Date:	12-Jan-1994
 *
 * Modified by:
 *
 * X-7	BFW1080		Brian F. Wibecan	04-May-1994
 *	Modified #pragma module to be compatible with CC on OSF/1.
 *
 * X-6	BFW1073		Brian F. Wibecan	06-Apr-1994
 *	Added #pragma module statement.
 *
 * X-5	BFW1065		Brian F. Wibecan	22-Mar-1994
 *	Add EV45 support for free.
 *
 * X-4	BFW1063		Brian F. Wibecan	21-Mar-1994
 *	Put initialization code in here.  Use chip definitions from HWRPBDEF.
 *	Support EV5.
 *
 * X-3	BFW1032		Brian F. Wibecan	04-Mar-1994
 *	Updates for V6.1 HWRPB.
 *
 * X-2	BFW1020		Brian F. Wibecan	02-Feb-1994
 *	Renamed to CTR_ROUTINES_VMS.C.
 *
 * X-1	BFW1009		Brian F. Wibecan	12-Jan-1994
 *	Original version.
 *
 */

#include "ctr_int_defs.h"
//#include <machine/cpuconf.h>

static IPR_CHIPINFO
   chipdata;
static IPR_BOOLEAN
   chipdata_found = FALSE;

STATUSFUNCPTR PtrGetStopCode = &NullUnavailableRoutine;

int chip_major (void)
{
   if (!chipdata_found)
   {
      KIprGetChipInfo (&chipdata);
      chipdata_found = TRUE;
   }
   return chipdata.majorIdent;
}

int chip_minor (void)
{
   if (!chipdata_found)
   {
      KIprGetChipInfo (&chipdata);
      chipdata_found = TRUE;
   }
   return chipdata.minorIdent;
}


/*****************************************************************************/
/* Initialization routines                                                   */
/*****************************************************************************/

/*
 * Determine chip type and initialize function pointers and other data
 * accordingly
 */

typedef void IniFunction (void *, IPR_BOOLEAN);

IPR_STATUS CtrInitialize (void *Areaptr, IPR_BOOLEAN init_shared)
{
   extern IniFunction
      Ev4Initialize,
      Ev5Initialize;
   extern int
      chip_major (void),
      chip_minor (void);

   switch (chip_major ())
   {
     case 2:
     case 4:
     case 6:
         Ev4Initialize (Areaptr, init_shared);
         break;
     case 5:
     case 7:
         Ev5Initialize (Areaptr, init_shared);
         break;
     default:
   	 printf("\033[1mError : Chip not recognized. IPROBE may not work properly\033[0m\n");
   }
   return IPR__SUCCESS;
}
