/****************************************************************************/
/*                                                                          */
/*  COPYRIGHT (c) 1993-1999 BY                                              */
/*  Digital Equiptment/Compaq Computer Corporation.                         */
/*  ALL RIGHTS RESERVED.                                                    */
/*                                                                          */
/*  This file is part of the Iprobe performance suite for Alpha processors. */
/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 2 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License       */
/*  along with this program; if not, write to the Free Software             */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307    */
/*                                                                          */
/****************************************************************************/

/*
 * Abstract:	This file contains the NT-specific performance monitoring
 * 		routines.
 *
 * Author:		Bradley M. Waters
 * Creation Date:	05-Jul-1994
 *
 * Modified by:
 *
 * X-4  BMW		Bradley M. Waters	23-Aug-1994
 *	Changed names from Pfm to Ipr.
 *
 * X-3	BMW		Bradley M. Waters	18-Jul-1994
 *	Modified based on kernel rationalization.
 *
 * X-2	BMW		Bradley M. Waters	15-Jul-1994
 *	Added api_routines_nt.h include to define api routines to ctr...
 *
 * X-1	BMW		Bradley M. Waters	05-Jul-1994
 *	Original version.
 *
 */


#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include <sys/ioctl.h>
#include <errno.h>
#include "ipr_api_ext_defs.h"
#include "ipr_api_int_defs.h"
#include "evx_vms_osf.h"
#include "ipr_driver_linux.h"


static int
   exit_handler_block [6],
   exit_status;
int ipr_fd,ipr_udbg=0, shmem_id;
IPR_CPUMASK ipr_cp_mask,ipr_is_cp_mask;
FILE *hndl;
#define SHARE_SIZE 65536

IPR_SHAREDATA *ShareArea = NULL;

IPR_STATUS create_shared_area(void);


static IPR_BOOLEAN
   sample_file_open = FALSE;

static int exit_handler (int cond_value, int argument)
{
   IprTerminateSampling (NULL,NULL);
   IprDeallocate ();
   return 0;
}

extern void init_ipr_lib(void)
{
    /*
     * open the device and turn it on
     */
    IPR_STATUS status;
#if defined DEBGG
    ipr_udbg=1;			/* if compiled with DEBGG set the debug falg */
#endif
    if( (ipr_fd=open("/dev/ipr", O_RDONLY )) < 0 ){
	perror("couldn't open the counter device");
	exit(1);
    }
    status = create_shared_area();
}

extern int cpu_number()
{
    return 0;
}

// This routine will create shared memory area if one does not already exists
// This shared area is used to store info on the counters owner and if
// it is running.

IPR_STATUS create_shared_area(void)
{	
   key_t ipr_key;
   int i,osf_status, flag=0;
   struct shmid_ds shmdata;
   extern void
      CtrInitialize (void *, IPR_BOOLEAN);
   if(ipr_udbg)
	   printf ("Inside CREATE_SHARED_AREA\n");
// get a key using the shared library as a start value

   ipr_key=ftok("/usr/shlib/libipr.so",'i');
// Create a mem area if one does not exist else get the shmem_id of it

   shmem_id= shmget(ipr_key,8192,IPC_CREAT);
// Get the permission setting
   shmdata.shm_nattch=0;
   osf_status = shmctl(shmem_id,IPC_STAT,&shmdata);

// If this is the first process attached to it then set the mode

   if(shmdata.shm_nattch==0){
       shmdata.shm_perm.mode=00666;
       osf_status = shmctl(shmem_id,IPC_SET,&shmdata);
       flag = 1;
   }
// Attached to the area

   ShareArea = (IPR_SHAREDATA *) shmat(shmem_id,0,0);
   if(ShareArea == (void *)-1){
       perror(" Error attaching to shared area");
       exit(1);
   }
	   
   if (flag == 1) {
	ShareArea -> counterowner = 0;
	ShareArea -> countersrunning = 0;
    }
	CtrInitialize (ShareArea+1,TRUE);    
}

extern ulong IprClose()
{
    int ipr_status;
    struct shmid_ds shmdata, *shmptr;
    shmptr = &shmdata;
    ipr_status = shmctl(shmem_id,IPC_STAT,&shmdata);
    if(shmptr->shm_nattch>1){
	ipr_status = shmdt(ShareArea);
    }
    else if(shmptr->shm_nattch==1){
	ipr_status=shmctl(shmem_id,IPC_RMID,NULL);
    }
    close(ipr_fd);
    ipr_fd=0;
}

extern IPR_STATUS KIprAllocate(void)
{
    IPR_ARGAllocate udata;
    if(ioctl(ipr_fd, IOCTL_IPR_Allocate, &udata) ){
	perror("ioctl to ipr driver failed");
	exit(1);
    }
    ipr_is_cp_mask = udata.ipr_is_cp_mask;
    ipr_cp_mask = udata.ipr_cp_mask;
    if(!IPR_SUCCESS(udata.status)){
	return udata.status;
    }
    return IPR__SUCCESS;
}    

extern IPR_STATUS KIprDeallocate(void)
{
    IPR_ARGDeallocate udata;
    if(ioctl(ipr_fd, IOCTL_IPR_Deallocate, &udata) ){
	perror("ioctl to ipr driver failed");
	exit(1);
    }
    if(!IPR_SUCCESS(udata.status)){
	return udata.status;
    }
    return IPR__SUCCESS;
}

extern IPR_STATUS allocate_iprdevice(void)
{
   IPR_STATUS status;
   if (ShareArea->counterowner == 0){
       status = KIprAllocate();
       if (!IPR_SUCCESS(status))
         return status;
       return IPR__SUCCESS;
   }
   else
       return IPR__ALRACTIVE;	
}


extern IPR_STATUS deallocate_iprdevice(void)
{
   IPR_STATUS status;
   unsigned int ipr_ipid;
   status =KIprGetCurrentIpid(&ipr_ipid);
       if (!IPR_SUCCESS(status))
         return status;
   if(ShareArea->counterowner == ipr_ipid){
       ShareArea->counterowner = 0;
   }
   status = KIprDeallocate();
   if (!IPR_SUCCESS(status))
       return status;
   return IPR__SUCCESS;
}
/*
static void cpystr (char **dst, unsigned char *dsiz, char *src)
{
   int sz;

   sz = strlen (src);   // @@@ uses string function, present in SYSTEM_PRIM.
   *dst = malloc (sz);  // @@@ uses malloc
   strncpy (*dst, src, sz);
   *dsiz = sz;
}
*/

extern IPR_STATUS IprCloseSampleFile (OUT int *buffwrites,
                                      OUT int *partwrites)
{
   IPR_STATUS
      status;

   if (buffwrites)
      *buffwrites = 0;
   if (partwrites)
      *partwrites = 0;
   status = fflush(hndl);
   if(ipr_udbg)
        printf("Status of the flush is : %d\n",status);
   if(status=fclose(hndl))
     return IPR__INVALID;
   else
     return IPR__SUCCESS;
}



extern IPR_STATUS IprOpenSampleFile (char *fname, char *defname)
{
   IPR_STATUS
      status;

   // Verify the correct user is doing this
   if (!owner_valid ())
      return IPR__NOTOWNER;

   //fname=strcat(fname,defname);

   hndl=fopen(fname, "w" );

   if(hndl!=NULL)
     return IPR__SUCCESS;
   else{
     return IPR__OPENFAIL;
   }

   return IPR__SUCCESS;


}



extern IPR_STATUS IprWriteToSampleFile (IN void *bptr, IN IPR_SIZE bsize)
{
   IPR_STATUS
      status;

 /*  The fwrite is a success if more than 1 byte has been written (status) */
   if((status=fwrite(bptr, 1, bsize, hndl))==0)
        return IPR__INVACTION;
   else
   {   if(ipr_udbg)                   /* Debug statement */
	       printf("Wrote 1 sample buffer\n"); 
       return IPR__SUCCESS;
   }
}


extern IPR_STATUS IprWriteOneSampleBuffer (IPR_SBUFFER *bptr, IPR_SIZE bsize)
{
   IPR_STATUS status;
   IPR_SIZE usize = bsize;
   IPR_SBUFFER *uptr;
   if(ipr_udbg)
          printf("At Buffer Write time the bsize is %d\n",bsize);
   uptr = (IPR_SBUFFER *) malloc(bsize);
 
   status = KIprGetBuffer(bptr,uptr);
   if((status=fwrite(uptr, 1, usize, hndl))==0){
       free(uptr);
       uptr = NULL;
        return IPR__INVACTION;
   }
   else{
       free(uptr);
       uptr = NULL;
       return IPR__SUCCESS;
   }
}

// The three functions below all cal KIprSetPme with different state 
// arguments:
// -1 : To get the PME bit
//  0 : To set the PME bit 
//  1 : To clear the PME bit

extern IPR_STATUS IprGetPme(IN unsigned int pid, OUT IPR_BOOLEAN *value)
{    	
    return KIprSetPme(pid,-1,0,value);
}

extern IPR_STATUS IprSetPme(IN unsigned int pid, IN IPR_BOOLEAN child)
{
    IPR_BOOLEAN prev;
    return KIprSetPme(pid,0,child,&prev);
}

extern IPR_STATUS IprClearPme(IN unsigned int pid, IN IPR_BOOLEAN child)
{
    IPR_BOOLEAN prev;
    return KIprSetPme(pid,1,child,&prev);
}

extern IPR_STATUS IprExit(void)
{
    IPR_STATUS status=0;
    uint64 iprdev;
    if(ipr_fd){
	status = IprStop(IPR_ALL_CPUS, -1);
	status = IprDeleteBuffers(IPR_ALL_CPUS);
	status = IprDeallocate ();
	iprdev = IprClose();
	return status;
    }
    else
	return IPR__INVALID;
}

extern IPR_SIZE IprEventSetSize (void)
{
    return CtrEventSetSize();
}

extern IPR_EVENT IprEventToNumber (IN const IPR_EVENTSET *event)
{
    return CtrEventToNumber(event);
}

extern IPR_STATUS IprClearEventSet (OUT IPR_EVENTSET *e)
{
    return CtrClearEventSet(e);
}


extern IPR_STATUS IprInsertEvent (OUT IPR_EVENTSET *s, IN const IPR_EVENT e)
{
    return CtrInsertEvent (s, e);
}

extern IPR_STATUS IprDeleteEvent (OUT IPR_EVENTSET *s, IN const IPR_EVENT e)
{
    return CtrDeleteEvent(s,e);
}

extern IPR_STATUS IprAvailableEvents (IN const IPR_EVENTSET *initial_events,
                             OUT IPR_EVENTSET *more_events)
{
   return CtrAvailableEvents (initial_events, more_events);
}

extern IPR_STATUS IprAvailableFreqs (IN const IPR_EVENT event, OUT IPR_FREQ *freqs)
{
   return CtrAvailableFreqs (event, freqs);
}

extern IPR_STATUS IprAvailableFuncs (OUT IPR_FMASK *funcs)
{
   return CtrAvailableFuncs (funcs);
}

extern IPR_COUNT IprNumCounters (void)
{
   return CtrNumCounters ();
}

extern IPR_STATUS IprAvailableCounters(IN IPR_EVENT evt,
				       OUT IPR_CTRMASK *cmask)
{
    return CtrAvailableCounters(evt, cmask);
}

extern IPR_STATUS IprExpandEventSet (IN const IPR_EVENTSET *s, 
				     OUT IPR_EVENT *a, IN IPR_SIZE size)
{
    return CtrExpandEventSet (s, a, size);
}

extern IPR_COUNT IprNumberEventsInSet (IN const IPR_EVENTSET *x)
{
     return CtrNumberEventsInSet (x);
}

extern IPR_BOOLEAN IprVerifyEventInSet (IN const IPR_EVENTSET *s,
                                        IN const IPR_EVENT e)
{
    return CtrVerifyEventInSet (s,e);
}

extern IPR_STATUS IprAvailableModes (OUT IPR_MMASK *modes)
{
   return CtrAvailableModes (modes);
}

extern IPR_STATUS KIprGetChipInfo(struct _ipr_chipinfo  *chip_info)
{
    IPR_ARGChipInfo udata;
    if (ioctl(ipr_fd, IOCTL_IPR_GetChipInfo, &udata) == -1){
	printf("%d ",errno);
	perror("ioctl to ipr driver failed");
	exit(1);
    }
    if(!IPR_SUCCESS(udata.status)){
	return udata.status;
    }
    memcpy(chip_info,&(udata.chip_info),sizeof(IPR_CHIPINFO));
    return IPR__SUCCESS;
}

extern IPR_STATUS KIprUpdatePerfmon(IPR_CPUNUM cpu, uint64 arg1, 
				    uint64 arg2, uint64 *retval)
{
    IPR_ARGUPerfmon udata;
    udata.cpu=cpu;
    udata.arg1=arg1;
    udata.arg2=arg2;
    if ( ioctl(ipr_fd, IOCTL_IPR_UpdatePERFMON, &udata) ){
	perror("ioctl to ipr driver failed");
	exit(1);
    }
    if(!IPR_SUCCESS(udata.status)){
	return udata.status;
    }
//    printf(" Call to PAL for cpu %d: arg1=%d, arg2 = %d:: status = %d\n",
//	   cpu, arg1,arg2,udata.status);
    if(retval != NULL){
	*retval = udata.retval;
    }
    return IPR__SUCCESS;
}

extern IPR_STATUS KIprGetCurrentIpid(unsigned int *Ipid)
{
    IPR_ARGIpid udata;
    if ( ioctl(ipr_fd, IOCTL_IPR_GetCurrentIpid, &udata) ){
	perror("ioctl to ipr driver failed");
	exit(1);
    }
    if(!IPR_SUCCESS(udata.status)){
	return udata.status;
    }
    *Ipid = udata.Ipid;
    return IPR__SUCCESS;
}

extern IPR_STATUS KIprSetAffinity(IPR_CPUNUM cpu, IPR_BOOLEAN setting)
{
    IPR_ARGAffinity udata;
    udata.cpu =cpu;
    udata.setting = setting;
    if ( ioctl(ipr_fd, IOCTL_IPR_SetAffinity, &udata) ){
	perror("ioctl to ipr driver failed");
	exit(1);
    }
    if(!IPR_SUCCESS(udata.status)){
	return udata.status;
    }
    return IPR__SUCCESS;
}

extern IPR_STATUS KIprSetPme(IN unsigned int pid, IN int state,
			    IN IPR_BOOLEAN child, OUT IPR_BOOLEAN *prev_state)
{
    IPR_ARGPme udata;
    udata.pid=pid;	
    udata.state=state;
    udata.child=child;
    if ( ioctl(ipr_fd, IOCTL_IPR_SetPme, &udata) ){
	perror("ioctl to ipr driver failed");
	exit(1);
    }
    if(!IPR_SUCCESS(udata.status)){
	return udata.status;
    }
    *prev_state=udata.prev_state;
    return IPR__SUCCESS;
}

extern IPR_STATUS KIprBdCreate(IN IPR_CPUNUM cpu, OUT IPR_BUFDSC **retptr)
{
    IPR_ARGBDCreate udata;
    udata.cpu=cpu;
    if ( ioctl(ipr_fd, IOCTL_IPR_BdCreate, &udata) ){
	perror("ioctl to ipr driver failed");
	exit(1);
    }
    if(!IPR_SUCCESS(udata.status)){
	return udata.status;
    }
    *retptr = udata.retptr;
    return IPR__SUCCESS;
}

extern IPR_STATUS KIprBdDelete(IN IPR_CPUNUM cpu)
{
    IPR_ARGBDelete udata;
    udata.cpu=cpu;
    if ( ioctl(ipr_fd, IOCTL_IPR_BdDelete, &udata) ){
	perror("ioctl to ipr driver failed");
	exit(1);
    }
    if(!IPR_SUCCESS(udata.status)){
	return udata.status;
    }
    return IPR__SUCCESS;
}
 

extern IPR_STATUS KIprBdSetNumCounters(IN IPR_CPUNUM cpu, IN IPR_COUNT numctrs)
{
    IPR_ARGBDNumctrs udata;
    udata.cpu=cpu;
    udata.numctrs=numctrs;
    if ( ioctl(ipr_fd, IOCTL_IPR_BdSetNumCounters, &udata) ){
	perror("ioctl to ipr driver failed");
	exit(1);
    }
    if(!IPR_SUCCESS(udata.status)){
	return udata.status;
    }
    return IPR__SUCCESS;
}

extern IPR_STATUS KIprBdSetAction(IN IPR_CPUNUM cpu, IN int action_type,
				 IN int action, IN int (*tgtroutine)(),
				 IN unsigned int tgtpid)
{
    IPR_ARGBDAction udata;
    udata.cpu=cpu;
    udata.action_type = action_type;
    udata.action = action;
    udata.targetroutine = tgtroutine;
    udata.targetpid = tgtpid;
    if ( ioctl(ipr_fd, IOCTL_IPR_BdSetAction, &udata) ){
	perror("ioctl to ipr driver failed");
	exit(1);
    }
    if(!IPR_SUCCESS(udata.status)){
	return udata.status;
    }
    return IPR__SUCCESS;
}

extern IPR_STATUS KIprBdSetActiveCounters(IN IPR_CPUNUM cpu, 
					 IN IPR_CTRMASK ctrmask)
{
    IPR_ARGBDActctrs udata;
    udata.cpu=cpu; 
    udata.ctrmask = ctrmask;
    if ( ioctl(ipr_fd, IOCTL_IPR_BdSetActiveCounters, &udata) ){
	perror("ioctl to ipr driver failed");
	exit(1);
    }
    if(!IPR_SUCCESS(udata.status)){
	return udata.status;
    }
    return IPR__SUCCESS;
}

extern IPR_STATUS KIprBdAddPcRange(IN IPR_CPUNUM cpu, IN IPR_PC_RANGE *pcrange)
{
    IPR_ARGBDPcrng udata;
    udata.cpu = cpu;
    memcpy(&(udata.pcrange),pcrange,sizeof(IPR_PC_RANGE));
    if ( ioctl(ipr_fd, IOCTL_IPR_BdAddPcRange, &udata) ){
	perror("ioctl to ipr driver failed");
	exit(1);
    }
    if(!IPR_SUCCESS(udata.status)){
	return udata.status;
    }
    return IPR__SUCCESS;
}

extern IPR_STATUS KIprBdSetSkipCount(IN IPR_CPUNUM cpu, IN IPR_CTRNUMBER ctr,
				    IN int skipvalue)
{
    IPR_ARGBDSkip udata;
    udata.cpu = cpu;
    udata.ctr = ctr;
    udata.skipvalue = skipvalue;
    if ( ioctl(ipr_fd, IOCTL_IPR_BdSetSkipCount, &udata) ){
	perror("ioctl to ipr driver failed");
	exit(1);
    }
    if(!IPR_SUCCESS(udata.status)){
	return udata.status;
    }
    return IPR__SUCCESS;
}

extern IPR_STATUS KIprSbCreate(IN IPR_CPUNUM cpu, IN IPR_SIZE size,
			      IN unsigned int numbuffs, 
			      IN IPR_SDESC *descrip, OUT IPR_SBUFFER **retptr)
{
    IPR_ARGSBCreate udata;
    udata.cpu = cpu;
    udata.size = size;
    udata.numbuffs = numbuffs;
    memcpy(&(udata.descrip),descrip, sizeof(IPR_SDESC));
    if ( ioctl(ipr_fd, IOCTL_IPR_SbCreate, &udata) ){
	perror("ioctl to ipr driver failed");
	exit(1);
    }
    memcpy(descrip, &(udata.descrip), sizeof(IPR_SDESC));
    if(!IPR_SUCCESS(udata.status)){
	return udata.status;
    }
    *retptr=udata.retptr;
    return IPR__SUCCESS;
}

extern IPR_STATUS KIprSbDelete(IN IPR_CPUNUM cpu, IN IPR_SBUFFER *bufptr)
{
    IPR_ARGSBDelete udata;
    udata.cpu = cpu;
    udata.bufptr = bufptr;
    if ( ioctl(ipr_fd, IOCTL_IPR_SbDelete, &udata) ){
	perror("ioctl to ipr driver failed");
	exit(1);
    }
    if(!IPR_SUCCESS(udata.status)){
	return udata.status;
    }
    return IPR__SUCCESS;
}

extern IPR_STATUS KIprSbSetFull(IN IPR_SBUFFER *bufptr,
			       IN IPR_FULL_STATE full_val)
{
    IPR_ARGSBSetFull udata;
    udata.bufptr = bufptr;
    udata.full_val = full_val;
    if ( ioctl(ipr_fd, IOCTL_IPR_SbSetFull, &udata) ){
	perror("ioctl to ipr driver failed");
	exit(1);
    }
    if(!IPR_SUCCESS(udata.status)){
	return udata.status;
    }
    return IPR__SUCCESS;
}

extern IPR_STATUS KIprSbGetInfo(IN IPR_SBUFFER *bufptr,
			       IN IPR_SDESC *descrip)
{
    IPR_ARGSBGetInfo udata;
    udata.bufptr = bufptr;
    memcpy(&(udata.descrip),descrip, sizeof(IPR_SDESC));    
    if ( ioctl(ipr_fd, IOCTL_IPR_SbGetInfo, &udata) ){
	perror("ioctl to ipr driver failed");
	exit(1);
    }
    if(!IPR_SUCCESS(udata.status)){
	return udata.status;
    }
    memcpy(descrip, &(udata.descrip), sizeof(IPR_SDESC));
    return IPR__SUCCESS;
}

extern IPR_STATUS KIprSbFind (IN IPR_CPUNUM cpu, IN IPR_SDESC *descrip,
			     IN IPR_SBUFFER *prev_buffer, 
			     OUT IPR_SBUFFER **retptr)
{
    IPR_ARGSBFind udata;
    udata.cpu = cpu;
    memcpy(&(udata.descrip),descrip, sizeof(IPR_SDESC));
    udata.previous_buffer=prev_buffer;
    if ( ioctl(ipr_fd, IOCTL_IPR_SbFind, &udata) ){
	perror("ioctl to ipr driver failed");
	exit(1);
    }
    if(!IPR_SUCCESS(udata.status)){
	return udata.status;
    }
    memcpy(descrip, &(udata.descrip), sizeof(IPR_SDESC));
    *retptr = udata.retptr;
    return IPR__SUCCESS;
}

/*
extern IPR_STATUS KIprHbElements(IN IPR_COUNT numctrs, IN IPR_COUNT numval,
				IN int gran, IN IPR_COUNT *numelem)
{
    IPR_ARGHBElements udata;
    udata.numctrs = numctrs;
    udata.numval = numval;
    udata.gran = gran;
    if ( ioctl(ipr_fd, IOCTL_IPR_HbElements, &udata) ){
	perror("ioctl to ipr driver failed");
	exit(1);
    }
    if(!IPR_SUCCESS(udata.status)){
	return udata.status;
    }
    *numelem = udata.numelem;
    return IPR__SUCCESS;
}

*/

extern IPR_STATUS KIprHbMaxElements(OUT IPR_COUNT *elements)
{
    IPR_ARGHBMaxElem udata;
    if ( ioctl(ipr_fd, IOCTL_IPR_HbMaxElements, &udata) ){
	perror("ioctl to ipr driver failed");
	exit(1);
    }
    if(!IPR_SUCCESS(udata.status)){
	return udata.status;
    }
    *elements=udata.elem;
    return IPR__SUCCESS;
}
   
extern IPR_STATUS KIprHbCreate(IN IPR_CPUNUM cpu, OUT IPR_HDESC *descrip,
			      OUT IPR_HBUFFER **retptr)
{
    IPR_ARGHBCreate udata;
    udata.cpu = cpu;
    memcpy(&(udata.descrip),descrip,sizeof(IPR_HDESC));
    if ( ioctl(ipr_fd, IOCTL_IPR_HbCreate, &udata) ){
	perror("ioctl to ipr driver failed");
	exit(1);
    }
    if(!IPR_SUCCESS(udata.status)){
	return udata.status;
    }
    memcpy(descrip, &(udata.descrip), sizeof(IPR_HDESC));
    *retptr=udata.retptr;
    return IPR__SUCCESS;
}

extern IPR_STATUS KIprHbDelete(IN IPR_CPUNUM cpu, IN IPR_HBUFFER *bufptr)
{
    IPR_ARGHBDelete udata;
    udata.cpu = cpu;
    udata.bufptr = bufptr;
    if ( ioctl(ipr_fd, IOCTL_IPR_HbDelete, &udata) ){
	perror("ioctl to ipr driver failed");
	exit(1);
    }
    if(!IPR_SUCCESS(udata.status)){
	return udata.status;
    }
    return IPR__SUCCESS;
}

extern IPR_STATUS KIprHbFind(IN IPR_CPUNUM cpu,
			IN IPR_HBUFFER *previous_buffer,
			IN IPR_HDESC *descrip,
			OUT IPR_HBUFFER **retptr)
{
    IPR_ARGHBFind udata;
    udata.cpu = cpu;
    udata.previous_buffer = previous_buffer;
    memcpy(&(udata.descrip),descrip, sizeof(IPR_HDESC));
    if ( ioctl(ipr_fd, IOCTL_IPR_HbFind, &udata) ){
	perror("ioctl to ipr driver failed");
	exit(1);
    }
    if(!IPR_SUCCESS(udata.status)){
	return udata.status;
    }
    memcpy(descrip, &(udata.descrip), sizeof(IPR_HDESC));
    *retptr=udata.retptr;
    return IPR__SUCCESS;
}   


extern IPR_STATUS KIprHbRead(IN IPR_CPUNUM cpu, IPR_HBUFFER *bufptr,
			    IN int start, IN IPR_COUNT cnt, 
			    OUT IPR_HISTELEM *retptr)
{	
    IPR_ARGHBRead udata;
    udata.cpu = cpu;
    udata.bufptr = bufptr;
    udata.start = start;
    udata.cnt = cnt;
    udata.retptr=retptr;
    if ( ioctl(ipr_fd, IOCTL_IPR_HbRead, &udata) ){
	perror("ioctl to ipr driver failed");
	exit(1);
    }
    if(!IPR_SUCCESS(udata.status)){
	return udata.status;
    }
    retptr = udata.retptr;
    return IPR__SUCCESS;
}

extern IPR_STATUS KIprHbClear(IN IPR_CPUNUM cpu, IPR_HBUFFER *bufptr,
			     IN int start, IN IPR_COUNT count)
{

    IPR_ARGHBRead udata;
    udata.status = IPR__SUCCESS;
    udata.cpu = cpu;
    udata.bufptr = bufptr;
    udata.start = start;
    udata.cnt = count;
    if ( ioctl(ipr_fd, IOCTL_IPR_HbClear, &udata) ){
	perror("ioctl to ipr driver failed");
	exit(1);
    }
    return udata.status;
}

extern IPR_STATUS KIprGetBuffer(IN IPR_SBUFFER *srcbuffer,  
			       IN IPR_SBUFFER *usrbuffer)
{
    IPR_ARGGetBuffer udata;
    udata.srcbuffer = srcbuffer;
    udata.usrbuffer = usrbuffer;
    if ( ioctl(ipr_fd, IOCTL_IPR_GetBuffer, &udata) ){
	perror("ioctl to ipr driver failed");
	exit(1);
    }
    if(!IPR_SUCCESS(udata.status)){
	return udata.status;
    }
    return IPR__SUCCESS;
}


extern IPR_STATUS IprGetSysInfo(IPR_CPU_CONF *Usysstrc)
{
    IPR_ARGCpuConf udata;
    if ( ioctl(ipr_fd, IOCTL_IPR_CpuConf, &udata) ){
	perror("ioctl to ipr driver failed");
	exit(1);
    }
    memcpy(Usysstrc,&(udata.Usysstrc), sizeof(IPR_CPU_CONF));
    if(!IPR_SUCCESS(udata.status)){
	return udata.status;
    }
    return IPR__SUCCESS;
}





