#if defined(__DECC) || defined(__DECCXX)
# pragma module EV4_ROUTINES_VMS_OSF "X-1"
#endif

// EV4_ROUTINES_VMS_OSF.C

/****************************************************************************/
/*									    */
/*  COPYRIGHT (c) 1993, 1994 BY						    */
/*  DIGITAL EQUIPMENT CORPORATION, MAYNARD, MASSACHUSETTS.		    */
/*  ALL RIGHTS RESERVED.						    */
/* 									    */
/*  THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND COPIED   */
/*  ONLY IN  ACCORDANCE WITH  THE  TERMS  OF  SUCH  LICENSE  AND WITH THE   */
/*  INCLUSION OF THE ABOVE COPYRIGHT NOTICE. THIS SOFTWARE OR  ANY  OTHER   */
/*  COPIES THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE AVAILABLE TO ANY   */
/*  OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF  THE  SOFTWARE IS  HEREBY   */
/*  TRANSFERRED.							    */
/* 									    */
/*  THE INFORMATION IN THIS SOFTWARE IS  SUBJECT TO CHANGE WITHOUT NOTICE   */
/*  AND  SHOULD  NOT  BE  CONSTRUED AS  A COMMITMENT BY DIGITAL EQUIPMENT   */
/*  CORPORATION.							    */
/* 									    */
/*  DIGITAL ASSUMES NO RESPONSIBILITY FOR THE USE  OR  RELIABILITY OF ITS   */
/*  SOFTWARE ON EQUIPMENT WHICH IS NOT SUPPLIED BY DIGITAL.		    */
/* 									    */
/*									    */
/****************************************************************************/

//++
// Abstract:	This file contains the EV4-specific performance counter
//		support routines.
//
// Author:		Brian F. Wibecan
// Creation Date:	26-Oct-1994
//
// Modified by:
//
// X-1	BFW		Brian F. Wibecan 	26-Oct-1994
//	Original version.
//--

#include "ev4_routines.h"
#include "ev4_masks.h"

extern EV4SHARED
   *Ev4Sh;

typedef enum
{
   EV4_PERFMON_DISABLE,               /* Disable the interrupts           */
   EV4_PERFMON_ENABLE,                /* Enable the interrupts            */
   EV4_PERFMON_MUX_SELECT,            /* Select desired function          */
   EV4_PERFMON_OPTIONS,               /* Specify option desired           */
   EV4_PERFMON_PAL_CONTROL,           /* Turn PAL counters on/off         */
   EV4_PERFMON_PAL_ASSIGN             /* Assign PAL counter array         */
} EV4_PERFMON;

typedef enum
{
   EV4_OPTIONS_NONE,
   EV4_OPTIONS_ALL_PROC,
   EV4_OPTIONS_SEL_PROC
} EV4_OPTIONS;

extern uint64 ev4_get_mux (CTRCTXDATA *cctx)
{
   EV4_MUXCTL
      muxctl;

   muxctl . ev4_muxctl_q_bits = 0;
   muxctl . ev4_muxctl_v_pcmux0 = cctx [0] . event_code;
   muxctl . ev4_muxctl_v_pcmux1 = cctx [1] . event_code;
   muxctl . ev4_muxctl_v_pc0 = cctx [0] . freq_code;
   muxctl . ev4_muxctl_v_pc1 = cctx [1] . freq_code;
   return muxctl . ev4_muxctl_q_bits;
}

#define ev4_mode_to_option(m) \
   ((m) . ipr_mmask_v_selected ? EV4_OPTIONS_SEL_PROC : EV4_OPTIONS_ALL_PROC)

extern IPR_STATUS Ev4SetModes (int ctx, IPR_MMASK modes)
{
   if (modes . ipr_mmask_l_bits
       && (modes . ipr_mmask_l_bits & Ev4Sh -> modes . ipr_mmask_l_bits)
          != modes . ipr_mmask_l_bits)
   return IPR__INVALID_MODES;
   Ev4Sh -> context [ctx] . modes = modes;
   return KIprUpdatePerfmon (ctx, EV4_PERFMON_OPTIONS,
      ev4_mode_to_option (Ev4Sh -> context [ctx] . modes), NULL);
}

static IPR_EVENT code_to_global (IPR_CTRNUMBER ctr, unsigned int code)
{
   switch (ctr)
   {
      case 0 :
         return (code < EV4_MUX0_TOTAL) ? Ev4Sh -> events_0 [code] : 0;
      case 1 :
         return (code < EV4_MUX1_TOTAL) ? Ev4Sh -> events_1 [code] : 0;
   }
   return 0;
}

extern IPR_STATUS Ev4SetEvents (int ctx, uint64 *MUXvalue)
{
   CTRCTXDATA
      *c0,
      *c1;
   EV4_MUXCTL
      mux;

   /*
    * If a MUX value was passed, save it and convert the information
    * to the normal form in the counter context area
    */
   if (MUXvalue)
      mux . ev4_muxctl_q_bits = *MUXvalue;
   if (MUXvalue && mux . ev4_muxctl_q_bits !=
                   Ev4Sh -> context [ctx] . muxvalue)
   {
      Ev4Sh -> context [ctx] . muxvalue = mux . ev4_muxctl_q_bits = *MUXvalue;

      /* Save temp values to make code below easier to read */

      c0 = &Ev4Sh -> context [ctx] . counter [0];
      c1 = &Ev4Sh -> context [ctx] . counter [1];

      /* Convert the MUX events back to event numbers */

      c0 -> event_code = mux . ev4_muxctl_v_pcmux0;
      c1 -> event_code = mux . ev4_muxctl_v_pcmux1;
      c0 -> freq_code = mux . ev4_muxctl_v_pc0;
      c1 -> freq_code = mux . ev4_muxctl_v_pc1;

      c0 -> event = code_to_global (0, c0 -> event_code);
      c1 -> event = code_to_global (1, c1 -> event_code);

      /* Convert the MUX frequencies back to frequency values */

      c0 -> freq = c0 -> freq_code ? Ev4Sh -> ctr_freq_hi [0]
                                   : Ev4Sh -> ctr_freq_lo [0];
      c1 -> freq = c1 -> freq_code ? Ev4Sh -> ctr_freq_hi [1]
                                   : Ev4Sh -> ctr_freq_lo [1];
   }
   else
      mux . ev4_muxctl_q_bits = Ev4Sh -> context [ctx] . muxvalue;

   /* Set the MUX selection as requested */

   return KIprUpdatePerfmon (ctx, EV4_PERFMON_MUX_SELECT,
                             mux . ev4_muxctl_q_bits, NULL);
}

static IPR_STATUS Ev4StartStop (int ctx, IPR_CTRMASK ctrs, uint64 option)
{
   IPR_STATUS
      status;

   if (!ctrs)
      ctrs = BIprAll1 (ctrs);
   ctrs &= EV4_COUNTER_MASK;

   if (ctrs != EV4_COUNTER_MASK)
      return IPR__INCOMPLETE_ERROR;

   status = KIprUpdatePerfmon (ctx, option, 0, NULL);
   // RESET_IPL ();
   return status;
}

extern IPR_STATUS Ev4Enable (int ctx, IPR_CTRMASK ctrs)
{
   return Ev4StartStop (ctx, ctrs, EV4_PERFMON_ENABLE);
}

extern IPR_STATUS Ev4Disable (int ctx, IPR_CTRMASK ctrs)
{
   return Ev4StartStop (ctx, ctrs, EV4_PERFMON_DISABLE);
}

extern IPR_STATUS Ev4GetStopCode (uint64 *arg1, uint64 *arg2)
{
   *arg1 = EV4_PERFMON_DISABLE;
   *arg2 = 0;
   return IPR__SUCCESS;
}

extern void Ev4OSInit (void)
{
   PtrGetStopCode = &Ev4GetStopCode;
   Ev4Sh -> modes . ipr_mmask_v_selected = 1;
}
