#include "ev5_routines.h"
#include "ev5_masks_vms_osf.h"

extern EV5SHARED
   *Ev5Sh;

typedef enum
{
   EV5_PERFMON_DISABLE,                // Disable counters
   EV5_PERFMON_ENABLE,                 // Enable counters
   EV5_PERFMON_MUX_SELECT,             // Select desired events
   EV5_PERFMON_OPTIONS,                // Select special options
   EV5_PERFMON_FREQ_SELECT,            // Select desired interrupt frequencies
   EV5_PERFMON_READ,                   // Read the counters
   EV5_PERFMON_WRITE,                  // Write the counters
   EV5_PERFMON_ENABLE_RESET            // Start the counters from zero
} EV5_PERFMON;

extern uint64 ev5_get_fctl (CTRCTXDATA *ctrctx)
{
   EV5_FCTL
      f;

   f . ev5_fctl_q_bits = 0;
   f . ev5_fctl_v_pcfreq0 = ctrctx [0] . freq_code;
   f . ev5_fctl_v_pcfreq1 = ctrctx [1] . freq_code;
   f . ev5_fctl_v_pcfreq2 = ctrctx [2] . freq_code;
   return f . ev5_fctl_q_bits;
}

extern uint64 ev5_get_mux (CTRCTXDATA *ctrctx)
{
   EV5_MUXCTL
      m;

   m . ev5_muxctl_q_bits = 0;
   m . ev5_muxctl_v_pcsel0 = ctrctx [0] . event_code;
   if (ctrctx [1] . event_code < EV5_MUX1_TOTAL)
   {
      m . ev5_muxctl_v_pcsel1 = ctrctx [1] . event_code;
      m . ev5_muxctl_v_cbox1 = 0;
   }
   else
   {
      m . ev5_muxctl_v_pcsel1 = EV5_MUX1_CBOX1;
      m . ev5_muxctl_v_cbox1 = ctrctx [1] . event_code - EV5_MUX1_TOTAL;
   }
   if (ctrctx [2] . event_code < EV5_MUX2_TOTAL)
   {
      m . ev5_muxctl_v_pcsel2 = ctrctx [2] . event_code;
      m . ev5_muxctl_v_cbox2 = 0;
   }
   else
   {
      m . ev5_muxctl_v_pcsel2 = EV5_MUX2_CBOX2;
      m . ev5_muxctl_v_cbox2 = ctrctx [2] . event_code - EV5_MUX2_TOTAL;
   }
   return m . ev5_muxctl_q_bits;
}

extern void ev5_mux_to_event (EV5_CONTEXT_BLOCK *cctx)
{
   EV5_MUXCTL
      ic;
   CTRCTXDATA
      *pCtr;
   int
      i;

   /* First fill in the basic events according to the mux/event array */

   ic . ev5_muxctl_q_bits = cctx -> muxctl;

   pCtr = &cctx -> counter [0];
   if (ic . ev5_muxctl_v_pcsel0 < EV5_MUX0_TOTAL)
   {
      pCtr -> event_code = ic . ev5_muxctl_v_pcsel0;
      pCtr -> event = Ev5Sh -> events_0 [pCtr -> event_code];
   }
   else
   {
      pCtr -> event_code = 0;
      pCtr -> event = 0;
   }

   pCtr = &cctx -> counter [1];
   if (ic . ev5_muxctl_v_pcsel1 < EV5_MUX1_CBOX1)
   {
      pCtr -> event_code = ic . ev5_muxctl_v_pcsel1;
      pCtr -> event = Ev5Sh -> events_1 [pCtr -> event_code];
   }
   else if (ic . ev5_muxctl_v_pcsel1 == EV5_MUX1_CBOX1 &&
            ic . ev5_muxctl_v_cbox1 < EV5_CBOX1_TOTAL)
   {
      pCtr -> event_code = EV5_MUX1_TOTAL + ic . ev5_muxctl_v_cbox1;
      pCtr -> event = Ev5Sh -> events_1 [pCtr -> event_code];
   }
   else
   {
      pCtr -> event_code = 0;
      pCtr -> event = 0;
   }

   pCtr = &cctx -> counter [2];
   if (ic . ev5_muxctl_v_pcsel2 < EV5_MUX2_CBOX2)
   {
      pCtr -> event_code = ic . ev5_muxctl_v_pcsel2;
      pCtr -> event = Ev5Sh -> events_2 [pCtr -> event_code];
   }
   else if (ic . ev5_muxctl_v_pcsel2 == EV5_MUX2_CBOX2 &&
            ic . ev5_muxctl_v_cbox2 < EV5_CBOX2_TOTAL)
   {
      pCtr -> event_code = EV5_MUX2_TOTAL + ic . ev5_muxctl_v_cbox2;
      pCtr -> event = Ev5Sh -> events_2 [pCtr -> event_code];
   }
   else
   {
      pCtr -> event_code = 0;
      pCtr -> event = 0;
   }

   /* Next handle the special cases for the conditional events */

   for (i = 0; i < EV5_CONDITIONAL_COUNT; i++)
      if (cctx -> counter [2] . event == Ev5Sh -> conditional [i] . trigger &&
          cctx -> counter [1] . event == Ev5Sh -> conditional [i] . original)
      {
         cctx -> counter [1] . event = Ev5Sh -> conditional [i] . replacement;
         break;
      }
}

static uint64 ev5_mode_to_option (IPR_MMASK m)
{
   EV5_MODESEL
      ms;

   /*
    * The mode mask is assumed to be valid; otherwise, the assignments below
    * will not be correct.
    */
   ms . ev5_modesel_q_bits = 0;
   if (m . ipr_mmask_l_bits)
   {
      ms . ev5_modesel_v_processes = m . ipr_mmask_v_selected;
      if (m . ipr_mmask_l_bits & ~IPR_MMASK_M_SELECTED)
      {
         /*
          * There are bits other than the selected bit.  That means that
          * the caller wants to restrict modes based on processor mode,
          * so set the kill bits.  Again, the mode combinations are assumed
          * to be valid.
          */
         ms . ev5_modesel_v_kill_user = !m . ipr_mmask_v_user;
         ms . ev5_modesel_v_kill_pal = !m . ipr_mmask_v_pal;
         ms . ev5_modesel_v_kill_kernel = !m . ipr_mmask_v_kernel;
      }
   }
   return ms . ev5_modesel_q_bits;
}

static IPR_BOOLEAN ev5_valid_modes (IPR_MMASK m)
{
   int i;
   static IPR_MMASK legal_modes [] =
   {
#ifdef __VMS
      (IPR_MMASK_M_KERNEL | IPR_MMASK_M_EXECUTIVE | IPR_MMASK_M_SUPERVISOR
         | IPR_MMASK_M_USER  | IPR_MMASK_M_PAL),
      (IPR_MMASK_M_PAL),
      (IPR_MMASK_M_KERNEL | IPR_MMASK_M_EXECUTIVE | IPR_MMASK_M_SUPERVISOR),
      (IPR_MMASK_M_USER),
      (IPR_MMASK_M_KERNEL | IPR_MMASK_M_EXECUTIVE | IPR_MMASK_M_SUPERVISOR
         | IPR_MMASK_M_USER),
      (IPR_MMASK_M_KERNEL | IPR_MMASK_M_EXECUTIVE | IPR_MMASK_M_SUPERVISOR
         | IPR_MMASK_M_PAL),
      (IPR_MMASK_M_USER | IPR_MMASK_M_PAL),
      (IPR_MMASK_M_EXECUTIVE | IPR_MMASK_M_SUPERVISOR)
#endif
#if  defined(__osf__) || defined(__LINUX__)   
      (IPR_MMASK_M_KERNEL | IPR_MMASK_M_USER | IPR_MMASK_M_PAL),
      (IPR_MMASK_M_KERNEL | IPR_MMASK_M_USER),
      (IPR_MMASK_M_KERNEL | IPR_MMASK_M_PAL),
      (IPR_MMASK_M_USER | IPR_MMASK_M_PAL),
      (IPR_MMASK_M_KERNEL),
      (IPR_MMASK_M_USER),
      (IPR_MMASK_M_PAL)
#endif

   };

   /* First check that all set bits are legal */

   if ((m . ipr_mmask_l_bits & Ev5Sh -> modes . ipr_mmask_l_bits)
       != m . ipr_mmask_l_bits)
      return FALSE;
   m . ipr_mmask_v_selected = 0;
   if (m . ipr_mmask_l_bits == 0)
      return TRUE;
   for (i = 0; i < (sizeof (legal_modes) / sizeof (IPR_MMASK)); i++)
      if (m . ipr_mmask_l_bits == legal_modes [i] . ipr_mmask_l_bits)
         return TRUE;
   return FALSE;
}

extern IPR_STATUS Ev5SetModes (int ctx, IPR_MMASK modes)
{

   /* Check that all specified modes are valid */

   if (!ev5_valid_modes (modes))
      return IPR__INVALID_MODES;

   /* Save the mode settings and use them */

   Ev5Sh -> context [ctx] . modes = modes;
   return KIprUpdatePerfmon (ctx, EV5_PERFMON_OPTIONS,
      ev5_mode_to_option (Ev5Sh -> context [ctx] . modes), NULL);
}

extern IPR_STATUS Ev5SetEvents (int ctx, uint64 *MUXvalue)
{
   IPR_STATUS
      status;

   /*
    * If a MUX value was passed, save it and convert the information
    * to the normal form in the counter context area
    */
   if (MUXvalue)
   {

      /* Convert the MUX events back to event numbers */

      Ev5Sh -> context [ctx] . muxctl = *MUXvalue;
      ev5_mux_to_event (&Ev5Sh -> context [ctx]);
   }

   /* Set the MUX selection as requested */

   status = KIprUpdatePerfmon (ctx, EV5_PERFMON_MUX_SELECT,
      Ev5Sh -> context [ctx] . muxctl, NULL);
   if (!IPR_SUCCESS (status))
      return status;
   return KIprUpdatePerfmon (ctx, EV5_PERFMON_FREQ_SELECT,
      Ev5Sh -> context [ctx] . fctl, NULL);
   return IPR__SUCCESS;
}

extern IPR_STATUS Ev5Enable (int ctx, IPR_CTRMASK ctrs)
{
   IPR_STATUS
      status;
   IPR_BOOLEAN
      prev_pme;

   if (!ctrs)
      ctrs = BIprAll1 (ctrs);
   ctrs = BIprBoth (ctrs, EV5_COUNTER_MASK);
   Ev5Sh -> context [ctx] . enabled_set |= ctrs;

   // Workaround EV5 PALcode problem

   if (Ev5Sh -> context [ctx] . modes . ipr_mmask_v_selected)
      (void) KIprSetPme (0, 1, FALSE, &prev_pme);

   // Enable the counters

   status = KIprUpdatePerfmon (ctx, EV5_PERFMON_ENABLE_RESET, ctrs, NULL);

   // Finish EV5 PALcode workaround

   if (Ev5Sh -> context [ctx] . modes . ipr_mmask_v_selected && prev_pme)
      (void) KIprSetPme (0, 0, FALSE, NULL);
   return status;
}

extern IPR_STATUS Ev5Disable (int ctx, IPR_CTRMASK ctrs)
{
   if (!ctrs)
      ctrs = BIprAll1 (ctrs);
   ctrs = BIprBoth (ctrs, EV5_COUNTER_MASK);
   Ev5Sh -> context [ctx] . enabled_set &= ~ctrs;
   return KIprUpdatePerfmon (ctx, EV5_PERFMON_DISABLE, ctrs, NULL);
}

extern IPR_STATUS Ev5Clear (int ctx, const IPR_CTRMASK ctrs)
{
   EV5_RDFMT
      tmp;
   IPR_STATUS
      status;
   IPR_CTRMASK
      tmpctrs;

   tmpctrs = (ctrs ? ctrs : EV5_COUNTER_MASK) & EV5_COUNTER_MASK;
   if (tmpctrs == EV5_COUNTER_MASK)
      tmp . ev5_rdfmt_q_bits = 0;
   else
   {
      status = KIprUpdatePerfmon (ctx, EV5_PERFMON_READ, 0,
                                  &tmp . ev5_rdfmt_q_bits);
      if (!IPR_SUCCESS (status))
         return status;
      if (tmpctrs & (1 << 0))
         tmp . ev5_rdfmt_v_ctr0 = 0;
      if (tmpctrs & (1 << 1))
         tmp . ev5_rdfmt_v_ctr1 = 0;
      if (tmpctrs & (1 << 2))
         tmp . ev5_rdfmt_v_ctr2 = 0;
   }
   return KIprUpdatePerfmon (ctx, EV5_PERFMON_WRITE, tmp . ev5_rdfmt_q_bits,
                             NULL);
}

extern IPR_STATUS Ev5Read (int ctx, const IPR_CTRMASK ctrs, int *values,
                           IPR_SIZE vsize)
{
   EV5_RDFMT
      tmp;
   IPR_STATUS
      status;
   IPR_CTRMASK
      tmpctrs;

   status = KIprUpdatePerfmon (ctx, EV5_PERFMON_READ, 0,
                               &tmp . ev5_rdfmt_q_bits);
   if (!IPR_SUCCESS (status))
      return status;
   tmpctrs = (ctrs ? ctrs : EV5_COUNTER_MASK) & EV5_COUNTER_MASK;
   if ((tmpctrs & (1 << 0)) && vsize > 0)
      values [0] = tmp . ev5_rdfmt_v_ctr0;
   if ((tmpctrs & (1 << 1)) && vsize > 1)
      values [1] = tmp . ev5_rdfmt_v_ctr1;
   if ((tmpctrs & (1 << 2)) && vsize > 2)
      values [2] = tmp . ev5_rdfmt_v_ctr2;
   return IPR__SUCCESS;
}

extern IPR_STATUS Ev5Pause (int ctx, IPR_CTRMASK ctrs)
{
   if (!ctrs)
      ctrs = BIprAll1 (ctrs);
   ctrs = BIprBoth (ctrs, EV5_COUNTER_MASK);
   Ev5Sh -> context [ctx] . enabled_set &= ~ctrs;
   return KIprUpdatePerfmon (ctx, EV5_PERFMON_DISABLE, ctrs, NULL);
}

extern IPR_STATUS Ev5Resume (int ctx, IPR_CTRMASK ctrs)
{
   IPR_STATUS
      status;
   IPR_BOOLEAN
      prev_pme;

   if (!ctrs)
      ctrs = BIprAll1 (ctrs);
   ctrs = BIprBoth (ctrs, EV5_COUNTER_MASK);
   Ev5Sh -> context [ctx] . enabled_set |= ctrs;


   // Workaround EV5 PALcode problem

   if (Ev5Sh -> context [ctx] . modes . ipr_mmask_v_selected)
      (void) KIprSetPme (0, 1, FALSE, &prev_pme);

   // Enable the counters

   status = KIprUpdatePerfmon (ctx, EV5_PERFMON_ENABLE, ctrs, NULL);

   // Finish EV5 PALcode workaround

   if (Ev5Sh -> context [ctx] . modes . ipr_mmask_v_selected && prev_pme)
      (void) KIprSetPme (0, 0, FALSE, NULL);
   return status;
}

extern IPR_STATUS Ev5ReadAll (int ctx, int *values, IPR_SIZE vsize)
{
   EV5_RDFMT
      tmp;
   int
      i;
   IPR_STATUS
      status;

   tmp . ev5_rdfmt_q_bits = 0;
   status = KIprUpdatePerfmon (ctx, EV5_PERFMON_READ, 0,
                               &tmp . ev5_rdfmt_q_bits);
   if (!IPR_SUCCESS (status))
      return status;
   if (vsize > 0)
   {
      switch (vsize)
      {
         default :
            values [2] = tmp . ev5_rdfmt_v_ctr2;
         case 2 :
            values [1] = tmp . ev5_rdfmt_v_ctr1;
         case 1 :
            values [0] = tmp . ev5_rdfmt_v_ctr0;
      }
   }
   return IPR__SUCCESS;
}

extern IPR_STATUS Ev5Write (int ctx, const IPR_CTRMASK ctrs, int *values,
                            IPR_SIZE vsize)
{
   EV5_RDFMT
      tmp;
   IPR_STATUS
      status;
   IPR_CTRMASK
      tmpctrs;

   tmp . ev5_rdfmt_q_bits = 0;
   tmpctrs = (ctrs ? ctrs : EV5_COUNTER_MASK) & EV5_COUNTER_MASK;
   if (tmpctrs != EV5_COUNTER_MASK)
   {
      status = KIprUpdatePerfmon (ctx, EV5_PERFMON_READ, 0,
                                  &tmp . ev5_rdfmt_q_bits);
      if (!IPR_SUCCESS (status))
         return status;
   }
   if ((tmpctrs & (1 << 0)) && vsize > 0)
      tmp . ev5_rdfmt_v_ctr0 = values [0];
   if ((tmpctrs & (1 << 1)) && vsize > 1)
      tmp . ev5_rdfmt_v_ctr1 = values [1];
   if ((tmpctrs & (1 << 2)) && vsize > 2)
      tmp . ev5_rdfmt_v_ctr2 = values [2];
   return KIprUpdatePerfmon (ctx, EV5_PERFMON_WRITE, tmp . ev5_rdfmt_q_bits,
                             NULL);
}

extern IPR_STATUS Ev5GetStopCode (uint64 *arg1, uint64 *arg2)
{
   *arg1 = EV5_PERFMON_DISABLE;
   *arg2 = EV5_COUNTER_MASK;
   return IPR__SUCCESS;
}

extern void Ev5OSInit (void)
{
   PtrGetStopCode = &Ev5GetStopCode;
}
