/****************************************************************************/
/*                                                                          */
/*  COPYRIGHT (c) 1993-1999 BY                                              */
/*  Digital Equiptment/Compaq Computer Corporation.                         */
/*  ALL RIGHTS RESERVED.                                                    */
/*                                                                          */
/*  This file is part of the Iprobe performance suite for Alpha processors. */
/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 2 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License       */
/*  along with this program; if not, write to the Free Software             */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307    */
/*                                                                          */
/****************************************************************************/
/*
 * Program: REP for all platforms, REPCOMMON.h module
 *
 * Abstract: This header file is part of rep from the IProbe suite.  It was written to 
 *           define structures that are common through-out all 3 OS versions of REP.
 *
 * Author: Phillip Ezolt
 * Creation Date: 17-June-97
 *
 *
 * MODIFICATION HISTORY:
 *
 * J. Goddard		10-Sep-1997
 *				Add new types and protos for functions that support the
 *				dynamically allocated symbol table.
 *
 *				Moved defines etc from code modules to this file
 *				as per the C coding spec.
 *
 *				Changed OS identifier macros to a uniform set supported
 *				out of the box by the C compilers:
 *						vms
 *						nt
 *						unix
 *
 */
#include "structs.h"
  
#ifdef nt
#define SCAN_INPUT  "%s %s %I64X %I64X"
#define SCAN_SHORT_INPUT  "%s %I64X %I64X"
#define SCAN_OUTPUT "%s %s %016I64X %016I64X\n"
#define REPLINEFORMAT "%-64s %016LX %016LX\n" 
#define MAX_FUNCNAME_SIZE 256
#define MAX_LIBRARY_SIZE 256
#define PROCESS_INDEX_NOT_FOUND 0xffffffff
#define MAX_PROCESS_NAME_SIZE 1024
#define MIN_NUMBER_OF_PROCESSES 1
#endif

#ifdef vms
#include <climsgdef.h>
#include <file.h>
#include <unixio.h>
#include <strings.h>
#define REPLINEFORMAT "%s %016LX %016LX\n" 
#define SCAN_INPUT  "%s %s %LX %LX"
#define SCAN_OUTPUT "%s %s %016LX %016LX\n"
#define MAX_FUNCNAME_SIZE 32
#define MAX_LIBRARY_SIZE 64
#endif

#ifdef unix
#include <strings.h>
#define SCAN_INPUT  "%s %s %16lX %16lX"
#define SCAN_OUTPUT "%s %s %016lX %016lX\n"
#define REPLINEFORMAT "%s %016lX %016lX\n" 
#define MAX_FUNCNAME_SIZE 256
#define MAX_LIBRARY_SIZE 256
#endif

#define DEFAULT_RESOLVED_FILENAME "address.res"
#define DEFAULT_UNRESOLVED_FILENAME "address.unr"

#define TEMP_BUFFER_SIZE 8192
#define REP_ARGS "efhi:klmno:p:qru?" 
#define NOT_FOUND "\?\?\?-NOT_FOUND"

#define INPUT_LENGTH 1024

#define TEMP_BUFFER_SIZE 8192
#define TEMP_STRING_SIZE 256

#define A_LESS_THAN_B -1
#define A_GREATER_THAN_B 1
#define A_EQUALS_B 0

#define PATHNAMESIZE 256

#define POOL_CHUNK 64 * 1024

/*
 * List element used for lists of file names in the command table
 */ 

typedef struct _filelist_elem
{
  char	FileName[PATHNAMESIZE];
  void	*Next;
} FILELIST, *PFILELIST;

/*
 * Command table -
 * Definition of global structure containing flags and file names
 * set by command line processing to control REP actions
 */

typedef struct _cmd_struct
{
  PFILELIST		InputFileNames;
  char OutputFileName[PATHNAMESIZE]; /* The name of the Output File. */
  char ArgStr[PATHNAMESIZE]; /* The complete Command line string to be run.. Really an NT ism. */
  uint32 ProcId; /* The process ID of the Process we want to manipulate. */
  uint32 FirstNonFlagArgumentNumber; /* The number of the first non flag argument. */
  boolean ExecMode; /* Executive Mode. VMS */
  boolean FResolve; /* Only Resolve Files... This should be renamed to something clearer.. Maybe OnlyResolve */
  boolean Help; /* We need help. */
  boolean KMode; /* Kernel Mode. */
  boolean KUMode; /*Temp Variable. */
  boolean Logging; /* Turn on logging. */
  boolean ModeSelect; /*This is for the PMEBit. */
  boolean NoResolve; /* Don't resolve the addresses. */
  boolean ResidentMode; /* Resident Mode.. VMS. */
  boolean Quiet; /* Nothing printed out. */
  boolean UMode; /* User Mode. */
  boolean Valid; /* The CommandStructure is Valid. */
  
} CMD_TABLE, *PCMD_TABLE;

typedef enum 
{
  RECORD_EOF,
  RECORD_COMMENT,
  RECORD_VALID,
  RECORD_NOT_FOUND,
  RECORD_INVALID
} RecordInfo;

typedef struct
{
	doublylinkedlist Links;
	char String;													// first character of string...strings
																				// with a zero length still consume a single
																				// char for the NUL character
} string;

typedef struct
{
	doublylinkedlist Links;
	uint64 BeginningAddress;								// function's beginning address
	uint64 EndingAddress;									//    "       ending       "
	char FunctionAndModuleNames;							// the function and module names packed
																	// together...function name first separated
																	// from the moduel name by a NUL...both fields
																	// are NUL terminated...see PointToFunctionName
																	// and PointToModuleName
} symbolnode;

/* Function prototypes */

PCMD_TABLE GetArgsAndSwitches(int argc, char** argv);
void SendHelp(void);

/* From RepFiles */

boolean OpenRepFile(char * FileName, PCMD_TABLE CommandTable);
boolean AppendRepFile(char * FileName, PCMD_TABLE CommandTable);
void CloseRepFile();
boolean WriteRep(PCMD_TABLE CommandTable);
boolean AddSymbolToList(char * RepStr, char * ImgStr, uint64 ba, uint64 ea);
uint64 FixAddresses(uint64 Value);
doublylinkedlist *GetSymbolListHeadPointer(void);
char *PointToFunctionName(symbolnode *pSymbolNode);
char *PointToModuleName(symbolnode *pSymbolNode);
symbolnode *AllocateSymbolNode(uint32 LengthOfNames);
boolean OrderSymbolList(void);

/* From Resolve Addresses */

void ResolveAddress(boolean Logging, char *OutputFileName);


/* From cmdln */

void CheckValidSwitches(PCMD_TABLE CommandTable);
void BreakUpInputFileList(PCMD_TABLE CommandTable, char *FileNameChunk);
void SendHelp(void);
void InitializeCommandTable(PCMD_TABLE CommandTable);
PCMD_TABLE GetArgsAndSwitches(int argc, char **argv);
CMD_TABLE *GetCommandTablePointer(void);
