/****************************************************************************/
/*                                                                          */
/*  COPYRIGHT (c) 1993-1999 BY                                              */
/*  Digital Equiptment/Compaq Computer Corporation.                         */
/*  ALL RIGHTS RESERVED.                                                    */
/*                                                                          */
/*  This file is part of the Iprobe performance suite for Alpha processors. */
/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 2 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License       */
/*  along with this program; if not, write to the Free Software             */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307    */
/*                                                                          */
/****************************************************************************/
/*
 * Program: REP (Read Entry Points) for Windows NT
 *
 * Abstract: This module contains the command line syntax checking and processing routines.
 *
 * Author:  Michael Dempsey
 *
 * Creation Date: 14-July-94
 *
 * Modification history:
 *
 * J. Goddard			11-Sep-1997
 *	Added the current date/time and version/rev to the help line and
 *	reformatted it slightly.
 *
 * Phil Ezolt: 17-June-97 
 *     Problem:  Did Not use GNU getopt, and was not necesarily platfrom independent.
 *     Solution: Made code use GNU getopt, rewrote BreakUpInputFileList, so that it wouldn't
 *               Use pointer additon/subtraction.  Slighty modified return structure to point to
 *               the first non flag argv[] from the command line. Made OS independent.
 *
 * Phil Ezolt: 27-May-97 
 *     Problem:  Code hard to read and some functions not commented. 
 *     Solution: Renamed varibles and added block comments before functions, moved 
 *               some constants to  #defines.
 *
 * Jim Goddard 5-March-1998
 *		Problem: wrong copyright date in help output
 *    Solution: update it
 *
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#ifdef nt
#include "getopt.def"
#include "getopt1.def"
#include <windows.h>
#endif

#ifdef unix
#include <unistd.h>
#include <getopt.h>
#endif

#ifdef vms
#include <unistd.h>
#endif

#include "stdtypes.h"
#include "rep_common.h"

/************************************************************************************/
/* Name: SendHelp                                                                   */
/*                                                                                  */
/* Purpose: This prints out the all of the options for rep.                         */
/*                                                                                  */
/* Arguments: None                                                                  */
/*                                                                                  */
/* Output: None                                                                     */
/*                                                                                  */
/* Function Value: None                                                             */
/*                                                                                  */
/************************************************************************************/
void SendHelp(void)
{
	printf("REP V%d.%d compiled on: " __DATE__ " at " __TIME__ 
	"\n\tDigital Equipment Corporation,"
	"\n\tCopyright (c) 1993, 1994, 1997, 1998\n"
 
	"\nUsage: rep [[{switch}]...]\n" 
	"Switch definitions:\n"
	"\t-e Read and resolve the images in Executive space. (Open VMS Only)\n"
	"\t-f Resolve input file(s) only. Must be used with -i\n"
	"\t-h Provides information on how to use rep\n"
	"\t-i Specifies a comma separated list of input files containing unresolved symbols.\n"
	"\t   (Unresolved symbol files are produced using the -n switch.)\n"
	"\t   Input files will be merged with REP output and resolved. If the\n"
	"\t   -f switch is also specified, normal REP processing is disabled and\n"
	"\t   only the input files will be merged and resolved.\n"
	"\t-k To read and resolve kernel address space only. (Default) \n"
	"\t-l Turns on logging, printing the step by step proceedings.\n"
	"\t   Use of this switch is recommended when reporting a bug\n"
	"\t   (Copious amounts of information in Open VMS)\n"
	"\t-m Sets the PME bit for the process started by rep.\n"
	"\t   (No longer availible) \n"
	"\t-n Disable resolve pass. Default output file is %s.\n"
	"\t-o Specifies a name for the output file, default is %s.\n"
	"\t-p The address space of the given PID is read. \n"
	"\t-q This will silence almost all messages. (NT Only)\n"
	"\t-r Read and resolve resident images. (Open VMS Only)\n"
	"\t-u To read and resolve user address space only.\n"
	"\t-? Provides information on how to use rep. (same as -h)\n"
    
	"\nNotes:\n"
	"\t1. It's illegal to provide both a process ID and a command verb.\n"
	"\t2. The command verb and its modifiers, if any, must appear AFTER\n"
	"\t   all other REP switches and arguments\n"
    
	"\nExamples:\n"
	"\t1. To create a file named myfile.dat containing only kernel\n"
	"\t   entry points: rep -k -o myfile.dat\n"
	"\t2. To activate a process with <command> and create a file\n"
	"\t   containing entry points in the user address space for the\n"
	"\t   process:      rep -u <command>\n"
	"\t3. To create a file with both user and kernel entry\n"
	"\t   points:       rep -uk\n"
	"\t4. To produce a file combining the entry points from a\n"
	"\t   running process with a previously collected set of entry\n"
	"\t   points:       rep -ku -i address1.res,ddress2.res -o address.res\n\n"
	,IPROBE_MAJOR_VER, IPROBE_MINOR_VER, DEFAULT_UNRESOLVED_FILENAME, DEFAULT_RESOLVED_FILENAME);

	return;
}

/************************************************************************************/
/* Name: InitializeCommandTable                                                     */
/*                                                                                  */
/* Purpose: This sets all of the fields in the Command parsing return structure to  */
/*          their defaults.                                                         */
/*                                                                                  */
/* Arguments: CommandTable: A pointer to the command table to be initialized.       */
/*                                                                                  */
/* Output: CommandTable: Contains the correctly initialized defaults.               */
/*                                                                                  */
/* Function Value: None.                                                            */
/*                                                                                  */
/************************************************************************************/
void InitializeCommandTable(CMD_TABLE *CommandTable)
{
  CommandTable->InputFileNames = NULL;  
 
  strcpy(CommandTable->OutputFileName, DEFAULT_RESOLVED_FILENAME);

  CommandTable->ArgStr[0] = NUL;
  CommandTable->ExecMode = FALSE;
  CommandTable->FirstNonFlagArgumentNumber = 0;  
  CommandTable->FResolve = FALSE;
  CommandTable->Help = FALSE;

  CommandTable->KMode = TRUE;			/* Collect kernel addresses by default */
  CommandTable->KUMode = FALSE;
  CommandTable->Logging = FALSE;
  CommandTable->ModeSelect = FALSE;	/* present for compatibility, not supported for Windows NT */
  CommandTable->NoResolve = FALSE;

  CommandTable->ProcId = 0;
  CommandTable->Quiet = FALSE;
  CommandTable->ResidentMode = FALSE;

  CommandTable->UMode = FALSE;
  CommandTable->Valid = TRUE;

	return;
}

/************************************************************************************/
/* Name: GetArgsAndSwitches                                                         */
/*                                                                                  */
/* Purpose: This procedure parses the command line and returns a pointer to         */
/*          a command table containing that contains a flag for each user option    */
/*                                                                                  */
/* Arguments: argc: This contains the number of command line arguments. (from main) */
/*            argv: This contains the vector of pointers to the arguments. (main)   */
/*                                                                                  */
/* Outputs: none                                                                    */
/*                                                                                  */
/* Function Value:  Returns a pointer to the next option for parsing.               */
/*                                                                                  */
/************************************************************************************/
CMD_TABLE *GetArgsAndSwitches(int argc, char **argv)
{
	boolean Error = FALSE;
  char CurrentOption, TempString[TEMP_BUFFER_SIZE];
  int64 ArgumentNumber;
  CMD_TABLE *CommandTable = GetCommandTablePointer();
  
  if (CommandTable == NULL)
  {
    if (CommandTable->Logging)
    {
      printf("Couldn't allocate memory for the new command table. \n");
    }
    exit(EXIT_FAILURE);
  }

  InitializeCommandTable(CommandTable);

  optarg = NULL;

  while (!Error && ((CurrentOption = getopt(argc, argv, REP_ARGS)) != -1))
  {
    switch (CurrentOption)
		{
	    case 'e':			// VMS specific
	    case 'E':
	      CommandTable->ExecMode=TRUE;
	      break;

	    case 'f':
	    case 'F':
	      CommandTable->KMode = CommandTable->KUMode;
	      CommandTable->FResolve = TRUE;
	      break;

	    case 'h':
	    case 'H':
	      CommandTable->Help = TRUE;
	      break;

	    case 'i':      
	    case 'I':      
	      BreakUpInputFileList(CommandTable, (char *)optarg);
	      break;

	    case 'k':
	    case 'K':
	      CommandTable->KMode = TRUE;
	      CommandTable->UMode = CommandTable->KUMode;
	      CommandTable->KUMode = TRUE;
	      break;

	    case 'l':
	    case 'L':
	      CommandTable->Logging = TRUE;
	      break;

		case 'm':
    case 'M':
      fprintf(stderr,"PME Bit mode no longer supported. \nContinuing in normal mode. \n");
      CommandTable->ModeSelect = TRUE;
      break;

    case 'n':
    case 'N':
      CommandTable->NoResolve = TRUE;
      strcpy(CommandTable->OutputFileName, DEFAULT_UNRESOLVED_FILENAME);
      break;

    case 'o':
    case 'O':
      strcpy(CommandTable->OutputFileName, optarg);
      break;

    case 'p':
    case 'P':
      CommandTable->ProcId = strtoul(optarg, NULL, 0); // Convert PID from a string to uint32

      if (CommandTable->ProcId == 0)
      {
				CommandTable->Valid = FALSE;
				fprintf(stderr, "Invalid PID specified. \n");
      }

      break;

    case 'q':
    case 'Q':
      CommandTable->Quiet = TRUE;
      break;

    case 'r':				// VMS specific
    case 'R':
      CommandTable->ResidentMode = TRUE;
      break;

    case 'u':
    case 'U':
      CommandTable->UMode = TRUE;
      CommandTable->KMode = CommandTable->KUMode;
      CommandTable->KUMode = TRUE;
      break;

    case '?':
      CommandTable->Help = TRUE;
      break;

    default :
      printf("Unknown FLAG.\n");
      Error = TRUE;
      CommandTable->Valid = FALSE;
			break;
    }
  }

	if (!Error)
	{
	  CommandTable->FirstNonFlagArgumentNumber = optind;  

	  if (optind < argc)
    {
      if (!strcmp(argv[optind], argv[0]))
			{
			  printf("Rep cannot be run on itself. \n");
			  CommandTable->Valid = FALSE;
			}
    }

		/* This is only needed for NT.  However, it doesn't hurt any of the other operating systems. */
		/* This turns the rest of the vector into a single string. (The image command line. ) */

		for (ArgumentNumber = optind, TempString[0]='\0'; ArgumentNumber < argc; ArgumentNumber++)
		{
			strcat(TempString, argv[ArgumentNumber]);
			strcat(TempString, " ");
		}

		strcpy(CommandTable->ArgStr, TempString);
  
		CheckValidSwitches(CommandTable);
	}

  return CommandTable;
}

/************************************************************************************/
/* Name:  CheckValidSwitches                                                        */
/*                                                                                  */
/* Purpose: This will make sure that all of the given flags can co-exist.           */
/*                                                                                  */
/* Arguments: CommandTable: A pointer to the command table to be initialized.       */
/*                                                                                  */
/* Output: CommandTable: If the commandtable is invalid, the CommandTable->Valid    */
/*         flag is set to false.                                                    */
/*                                                                                  */
/* Function Value: None.                                                            */
/*                                                                                  */
/************************************************************************************/
void CheckValidSwitches(CMD_TABLE *CommandTable)
{
	//
	// Resolve switch is incompatible with -k -n -u and -p
	//
  if ((CommandTable->FResolve) && ((CommandTable->KUMode) || (CommandTable->ProcId) ||
	(CommandTable->NoResolve)))
  {
    fprintf(stderr, "Error, the \"-f\" switch is incompatible with the \"k, n, u, p\" switches\n");
    CommandTable->Valid = FALSE;
  }

	//
	// Resolve switch must be used with at least one input file
	//
  if ((CommandTable->FResolve) && (!CommandTable->InputFileNames))
  {
    fprintf(stderr, "Error, please specify an input file,\"-i filename\", when using the \"-f\" switch\n");
    CommandTable->Valid = FALSE;
  }
    
	//
	// It's illegal to have an image name argument and also specify the -p switch
	//
  if (CommandTable->ProcId && CommandTable->ArgStr[0] != NUL)
  {
    CommandTable->Valid = FALSE;
    fprintf(stderr, "Usage error, the -p switch and an image name argument cannot be used together\n");
  }
    
	//
	// Using the -p switch implies -u
	//
  if (CommandTable->ProcId)
  {
    CommandTable->UMode = TRUE;
  }

	//
	// Providing an image name argument implies -u
	//
  CommandTable->UMode |= (CommandTable->ArgStr[0] != NUL);
    
	//
	// If the user mode switch, -u is used, then either a file
	// name argument or a process ID is required
	//
  if (CommandTable->UMode && CommandTable->ArgStr[0] == NUL && CommandTable->ProcId == 0)
  {
    fprintf(stderr, "Command line switch -u must be accompanied by an image name or a process ID\n");
    CommandTable->Valid = FALSE;
  }

	return;
}

/************************************************************************************/
/* Name:  BreakUpInputFileList                                                      */
/*                                                                                  */
/* Purpose:  This will take a string of comma delimited Files names, and insert     */
/*           into the Command Table, a linked list of FileNames.                    */
/*                                                                                  */
/* Arguments: CommandTable: A pointer to the command table to be initialized.       */
/*                                                                                  */
/* Output: CommandTable: A linked list of filenames within                          */
/*         CommandTable->InputFileNames                                             */
/*                                                                                  */
/* Function Value: None.                                                            */
/*                                                                                  */
/************************************************************************************/
void BreakUpInputFileList(CMD_TABLE *CommandTable, char *FileNameChunk)
{
  char *CurrentToken, TempBuffer[TEMP_BUFFER_SIZE];
  int FileNameLength;
  PFILELIST pFileListElement;
  
  strcpy(TempBuffer, FileNameChunk);
  
	//
	// Now extract the file names and put them into a list
	//
  if ((CurrentToken = strtok(TempBuffer, ",")) != NULL)
  {     
    pFileListElement = (PFILELIST) malloc(sizeof(FILELIST));

    if (pFileListElement == NULL)
    {
      if (CommandTable->Logging)
      {
				printf("Couldn't allocate memory for the new file list element. \n");
      }
      exit(EXIT_FAILURE);
    }

    pFileListElement->Next = CommandTable->InputFileNames;
    strcpy(pFileListElement->FileName, CurrentToken);
    
    CommandTable->InputFileNames = pFileListElement;    
    
    while ((CurrentToken = strtok(NULL, ",")) != NULL)
    {
      pFileListElement = (PFILELIST) malloc(sizeof(FILELIST));
      if (pFileListElement == NULL)
      {
				if (CommandTable->Logging)
				{
				  printf("Couldn't allocate memory for the new file list element. \n");
				}

				exit(EXIT_FAILURE);
      }

      pFileListElement->Next = CommandTable->InputFileNames;
      strcpy(pFileListElement->FileName, CurrentToken);
      CommandTable->InputFileNames = pFileListElement;
    }      
  }  
}

CMD_TABLE *GetCommandTablePointer(void)
{
	static CMD_TABLE *pCommandTable = NULL;

  if (pCommandTable == NULL)
	{
		pCommandTable = (CMD_TABLE *) malloc(sizeof(CMD_TABLE));
	}

	return pCommandTable;
}
