#ifndef __BIT_LIBRARY_LOADED
#define __BIT_LIBRARY_LOADED 1
/****************************************************************************/
/*                                                                          */
/*  COPYRIGHT (c) 1993-1999 BY                                              */
/*  Digital Equiptment/Compaq Computer Corporation.                         */
/*  ALL RIGHTS RESERVED.                                                    */
/*                                                                          */
/*  This file is part of the Iprobe performance suite for Alpha processors. */
/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 2 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License       */
/*  along with this program; if not, write to the Free Software             */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307    */
/*                                                                          */
/****************************************************************************/

/*
 *             Patent Pending, Digital Equipment Corporation
 *
 * Abstract:	This header file contains bit manipulation macros and
 *		function prototypes.
 *
 * Author:		Brian F. Wibecan
 * Creation Date:	20-Jun-1995
 *
 * Modified by:
 *
 * X-1	BFW		Brian F. Wibecan	20-Jun-1995
 *	Original version.
 */

#include "stdtypes.h"

/*
 * Modify the argument to set all bits in the argument
 */
#define BIprAll1(b) ((b) | ~(b))

/*
 * Create a bit mask with only the specified bit set
 */
extern uint64 BIprB2M (unsigned int b);

/*
 * Return TRUE if bit is set in the mask, FALSE otherwise
 */
#define BIprIsSet(m,b) (((m) & BIprB2M(b)) != 0)

/*
 * Return a mask with bits set that are set in either mask
 */
#define BIprSet(m,c) ((m) | (c))

/*
 * Return a mask with bits set that are set in both masks
 */
#define BIprBoth(m,c) ((m) & (c))

/*
 * Return a mask with bits set that are set only in the first mask, and not
 * in the second
 */
#define BIprClr(m,c) ((m) & ~(c))

/*
 * Create a bit mask with all bits set from the specified one down
 */
#define BIprSetLo(b) (BIprB2M((b) + 1) - 1)

/*
 * Create a bit mask with all bits from the specified one up set
 */
#define BIprSetHi(b) (~(BIprB2M(b) - 1))

/*
 * Set the specified bit in the mask
 */
#define BIprSet1(m,b) BIprSet(m,BIprB2M(b))

/*
 * Clear the specified bit in the mask
 */
#define BIprClr1(m,b) BIprClr(m,BIprB2M(b))

/*
 * Return a mask consisting only the lowest set bit in the passed mask
 *
 * *** NOTE *** This macro uses the passed argument more than once
 */
#define BIprLowest(m) ((m) & (~(m) + 1))

/*
 * Return TRUE if exactly one bit is set in the passed mask
 *
 * *** NOTE *** This macro uses the passed argument more than once
 */
#define BIprIs1(m) ((m) && BIprLowest(m) == (m))

extern int BIprCnt64 (uint64 x);

#define BIprCnt(x) BIprCnt64(x)

/*
 * Determine the bit number of the lowest set bit
 */
extern int BIprWhich (uint64 m);

/*
 * Clear all the bits in the mask below the specified bit
 */
#define BIprClrLo(m,b) BIprClr (m, BIprSetLo ((b) - 1))

/*
 * Clear all the bits in the mask above the specified bit
 */
#define BIprClrHi(m,b) (BIprClr (m, BIprSetHi ((b) + 1)))

/*
 * Find the relative position of the specified bit within the set mask bits
 */
extern int BIprRelPos (uint64 m, unsigned int b);

#endif // #ifndef __BIT_LIBRARY_LOADED
