#ifndef __EV4_ROUTINES_LOADED
#define __EV4_ROUTINES_LOADED 1

/****************************************************************************/
/*									    */
/*  COPYRIGHT (c) 1993-1999 BY			                            */
/*  Digital Equiptment/Compaq Computer Corporation.       		    */
/*  ALL RIGHTS RESERVED.						    */
/*                                                                          */
/*  This file is part of the Iprobe performance suite for Alpha processors. */
/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 2 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License       */
/*  along with this program; if not, write to the Free Software             */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307    */
/*									    */
/****************************************************************************/
/*
 * Abstract:	This header file contains EV4-specific definitions used
 * 		by the EV4 performance counter support code.
 *
 * Author:		Brian F. Wibecan
 * Creation Date:	16-Dec-1993
 *
 * Modified by:
 *
 * X-6	BMW		Bradley M. Waters	18-Jul-1994
 *	Modified based on kernel rationalization.
 *
 * X-5	BFW1051		Brian F. Wibecan	13-Mar-1994
 *	Fixed what I left out last time.
 *
 * X-4	BFW1051		Brian F. Wibecan	13-Mar-1994
 *	Moved terminator key for MUX lists to PERFMONDEF.
 *
 * X-3	BFW1035		Brian F. Wibecan	08-Mar-1994
 *	Modified EV4SHARED to include array of events for each counter.
 *
 * X-2	???		Brian F. Wibecan	???
 *	Added EV$SHARED typedef.
 *
 * X-1	BFW1000		Brian F. Wibecan	16-Dec-1993
 *	Original version.
 *
*/

#include "ctr_int_defs.h"

typedef enum
{
   //
   // MUX selections for counter 0
   //
   EV4_MUX0_ISSUES,              // Total issues divided by 2
   EV4_MUX0_UNUSED_1,            // Unused value
   EV4_MUX0_PIPELINE_DRY,        // Nothing issued, no valid Istream data
   EV4_MUX0_UNUSED_3,            // Unused value
   EV4_MUX0_LOAD_INSTRUCTIONS,   // All load instructions
   EV4_MUX0_UNUSED_5,            // Unused value
   EV4_MUX0_PIPELINE_FROZEN,     // Nothing issued, resource conflict
   EV4_MUX0_UNUSED_7,            // Unused value
   EV4_MUX0_BRANCH_INSTRUCTIONS, // All brances (cond, uncond, JSR, HW_REI)
   EV4_MUX0_UNUSED_9,            // Unused value
   EV4_MUX0_CYCLES,              // Total cycles
   EV4_MUX0_PAL_MODE,            // Cycles while in PAL mode
   EV4_MUX0_NON_ISSUES,          // Total non-issues divided by 2
   EV4_MUX0_UNUSED_13,           // Unused value
   EV4_MUX0_EXTERNAL,            // External event supplied by pin
   EV4_MUX0_UNUSED_15,           // Unused value
   EV4_MUX0_TOTAL
} EV4_MUX0;

typedef enum
{
   //
   // MUX selections for counter 1
   //
   EV4_MUX1_DCACHE_MISSES,       // Total Dcache misses
   EV4_MUX1_ICACHE_MISSES,       // Total Icache misses
   EV4_MUX1_DUAL_ISSUE_CYCLES,   // Cycles of dual issue
   EV4_MUX1_BRANCH_MISPREDICTS,  // Branch mispredicts (cond, JSR, HW_REI)
   EV4_MUX1_FP_INSTRUCTIONS,     // FP operate instr (not br, load, store)
   EV4_MUX1_INTEGER_OPERATE,     // Integer op (incl lda, ldah into R0-R30)
   EV4_MUX1_STORE_INSTRUCTIONS,  // Total store instructions
   EV4_MUX1_EXTERNAL,            // External event supplied by pin
   EV4_MUX1_TOTAL
} EV4_MUX1;

//
// Counter 0 frequency selections
//
#define EV4_FREQ0_LOW 16
#define EV4_FREQ0_HIGH 12

//
// Counter 1 frequency selections
//
#define EV4_FREQ1_LOW 12
#define EV4_FREQ1_HIGH 8

#define EV4_COUNTER_COUNT 2
#define EV4_COUNTER_MASK 3
#define EV4_ADJUSTED_COUNT 1

typedef struct _ev4_context_block
   {
      IPR_MMASK modes;
      int all_specified;
      uint64 muxvalue;
      CTRCTXDATA counter [EV4_COUNTER_COUNT];
   } ev4_context_block;

typedef struct {
   ev4_context_block
      context [CONTEXT_AREA_COUNT];
   IPR_EVENTSET
      ctr_events [EV4_COUNTER_COUNT],
      adjusted_events [EV4_ADJUSTED_COUNT];
   IPR_FMASK
      funcs;
   IPR_MMASK
      modes;
   int
      adjusted_shifts [EV4_ADJUSTED_COUNT];
   IPR_FREQ
      ctr_freqs [EV4_COUNTER_COUNT],
      ctr_freq_hi [EV4_COUNTER_COUNT],
      ctr_freq_lo [EV4_COUNTER_COUNT];
   IPR_EVENT
      events_0 [EV4_MUX0_TOTAL],
      events_1 [EV4_MUX1_TOTAL];
} EV4SHARED;

// OS specific routines
extern uint64
   ev4_get_mux (CTRCTXDATA *cctx);
extern IPR_STATUS
   Ev4SetModes (int ctx, IPR_MMASK modes),
   Ev4SetEvents (int ctx, uint64 *MUXvalue),
   Ev4Enable (int ctx, IPR_CTRMASK ctrs),
   Ev4Disable (int ctx, IPR_CTRMASK ctrs);
extern void
   Ev4OSInit (void);

#ifdef __VMS
#  include "ev4_routines_vms.h"
#endif
#ifdef __osf__
#  include "ev4_routines_osf1.h"
#endif
#ifdef __LINUX__
#  include "ev4_routines_linux.h"
#endif
#ifdef __NT
#  include "ev4_routines_nt.h"
#endif

#endif /* ifndef __EV4_ROUTINES_LOADED */
