#ifndef __EV5_ROUTINES_LOADED
#define __EV5_ROUTINES_LOADED 1


/****************************************************************************/
/*                                                                          */
/*  COPYRIGHT (c) 1993-1999 BY                                              */
/*  Digital Equiptment/Compaq Computer Corporation.                         */
/*  ALL RIGHTS RESERVED.                                                    */
/*                                                                          */
/*  This file is part of the Iprobe performance suite for Alpha processors. */
/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 2 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License       */
/*  along with this program; if not, write to the Free Software             */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307    */
/*                                                                          */
/****************************************************************************/
//
// 
//
//             Patent Pending, Digital Equipment Corporation
//
//
// Abstract:	This header file contains EV5-specific API definitions.
//
// Author:		Brian F. Wibecan
// Creation Date:	20-Jun-1995
//
// Modified by:
//
// X-1	BFW		Brian F. Wibecan	20-Jun-1995
//	Original version.
//

#include "ctr_int_defs.h"

#define EV5_COUNTER_MASK 7
#define EV5_ADJUSTED_COUNT 0
#define EV5_CONDITIONAL_COUNT 2
#define EV5_COUNTER_COUNT 3

typedef enum
{
   //
   // Counter 0 events
   //
   EV5_MUX0_CYCLES,
   EV5_MUX0_ISSUES,
   EV5_MUX0_TOTAL
} EV5_MUX0;

//
// Counter 1 events
//
typedef enum
{
   EV5_MUX1_NON_ISSUE_CYCLES,
   EV5_MUX1_SPLIT_ISSUE_CYCLES,
   EV5_MUX1_PIPELINE_DRY,
   EV5_MUX1_REPLAY_TRAP,
   EV5_MUX1_SINGLE_ISSUE_CYCLES,
   EV5_MUX1_DUAL_ISSUE_CYCLES,
   EV5_MUX1_TRIPLE_ISSUE_CYCLES,
   EV5_MUX1_QUAD_ISSUE_CYCLES,
   EV5_MUX1_FLOW_CHANGE,
   EV5_MUX1_INTEGER_OPERATE,
   EV5_MUX1_FP_INSTRUCTIONS,
   EV5_MUX1_LOAD_INSTRUCTIONS,
   EV5_MUX1_STORE_INSTRUCTIONS,
   EV5_MUX1_ICACHE_ACCESS,
   EV5_MUX1_DCACHE_ACCESS,
   EV5_MUX1_CBOX1,
   EV5_MUX1_TOTAL
} EV5_MUX1;

//
// Counter 1 CBOX events
//
typedef enum
{
   EV5_CBOX1_SCACHE_ACCESS,
   EV5_CBOX1_SCACHE_READ,
   EV5_CBOX1_SCACHE_WRITE,
   EV5_CBOX1_SCACHE_VICTIM,
   EV5_CBOX1_RESERVED4,
   EV5_CBOX1_BCACHE_HIT,
   EV5_CBOX1_BCACHE_VICTIM,
   EV5_CBOX1_SYS_REQ,
   EV5_CBOX1_TOTAL
} EV5_CBOX1;

//
// Counter 2 events
//
typedef enum
{
   EV5_MUX2_LONG_STALLS,
   EV5_MUX2_RESERVED1,
   EV5_MUX2_PC_MISPR,
   EV5_MUX2_BRANCH_MISPREDICTS,
   EV5_MUX2_ICACHE_MISSES,
   EV5_MUX2_ITB_MISS,
   EV5_MUX2_DCACHE_MISSES,
   EV5_MUX2_DTB_MISS,
   EV5_MUX2_LOADS_MERGED,
   EV5_MUX2_LDU_REPLAYS,
   EV5_MUX2_WB_MAF_FULL_REPLAYS,
   EV5_MUX2_EXTERNAL,
   EV5_MUX2_CYCLES,
   EV5_MUX2_MEM_BARRIER,
   EV5_MUX2_LOAD_LOCKED,
   EV5_MUX2_CBOX2,
   EV5_MUX2_TOTAL
} EV5_MUX2;

//
// Counter 2 CBOX events
//
typedef enum
{
   EV5_CBOX2_SCACHE_MISS,
   EV5_CBOX2_SCACHE_READ_MISS,
   EV5_CBOX2_SCACHE_WRITE_MISS,
   EV5_CBOX2_SCACHE_SH_WRITE,
   EV5_CBOX2_SCACHE_WRITE,
   EV5_CBOX2_BCACHE_MISS,
   EV5_CBOX2_SYS_INV,
   EV5_CBOX2_SYS_READ_REQ,
   EV5_CBOX2_TOTAL
} EV5_CBOX2;

//
// Counter 0 frequency selections
//
#define EV5_FREQ0_LOW 16
#define EV5_FREQ0_HIGH 8

//
// Counter 1 frequency selections
//
#define EV5_FREQ1_LOW 16
#define EV5_FREQ1_HIGH 8

//
// Counter 2 frequency selections
//
#define EV5_FREQ2_LOW 14
#define EV5_FREQ2_HIGH 8

// Note that the definitions below are not an enum, even thought they
// coincidentally could be defined via an enum, because they are masks,
// not absolute values.

#define EV5_FREQ_M_LOW 0
#define EV5_FREQ_M_HIGH 1
#define EV5_FREQ_M_INTERRUPT 2

typedef struct _ev5_context_block
{
   IPR_MMASK modes;
   IPR_BOOLEAN all_specified;
   IPR_CTRMASK enabled_set;
   CTRCTXDATA counter [EV5_COUNTER_COUNT];
   uint64 muxctl;
   uint64 fctl;
} EV5_CONTEXT_BLOCK;

typedef struct
{
   IPR_FMASK
      funcs;
   IPR_MMASK
      modes;
   IPR_EVENTSET
      ctr_events [EV5_COUNTER_COUNT],
      *adjusted_events;
   int
      *adjusted_shifts;
   EV5_CONTEXT_BLOCK
      context [CONTEXT_AREA_COUNT];
   IPR_FREQ
      ctr_freqs [EV5_COUNTER_COUNT],
      ctr_freq_hi [EV5_COUNTER_COUNT],
      ctr_freq_lo [EV5_COUNTER_COUNT];
   struct
   {
      IPR_EVENT trigger;
      IPR_EVENT replacement;
      IPR_EVENT original;
   } conditional [EV5_CONDITIONAL_COUNT];
   IPR_EVENT
      events_0 [EV5_MUX0_TOTAL],
      events_1 [EV5_MUX1_TOTAL + EV5_CBOX1_TOTAL],
      events_2 [EV5_MUX2_TOTAL + EV5_CBOX2_TOTAL];
} EV5SHARED;

extern uint64 ev5_get_mux (CTRCTXDATA *ctrctx);
extern uint64 ev5_get_fctl (CTRCTXDATA *ctrctx);
extern IPR_STATUS Ev5SetModes (int ctx, IPR_MMASK modes);
extern IPR_STATUS Ev5SetEvents (int ctx, uint64 *MUXvalue);
extern IPR_STATUS Ev5Enable (int ctx, IPR_CTRMASK ctrs);
extern IPR_STATUS Ev5Disable (int ctx, IPR_CTRMASK ctrs);
extern IPR_STATUS Ev5Clear (int ctx, const IPR_CTRMASK ctrs);
extern IPR_STATUS Ev5Pause (int ctx, IPR_CTRMASK ctrs);
extern IPR_STATUS Ev5Resume (int ctx, IPR_CTRMASK ctrs);
extern IPR_STATUS Ev5Read (int ctx, const IPR_CTRMASK ctrs, int *values,
                           IPR_SIZE vsize);
extern IPR_STATUS Ev5ReadAll (int ctx, int *values, IPR_SIZE vsize);
extern IPR_STATUS Ev5Write (int ctx, const IPR_CTRMASK ctrs, int *values,
                            IPR_SIZE vsize);
extern void
   Ev5OSInit (void);

#ifdef __VMS
#  include "ev5_routines_vms.h"
#endif
#ifdef __osf__
#  include "ev5_routines_osf1.h"
#endif
#ifdef __LINUX__
#  include "ev5_routines_osf1.h"
#endif
#ifdef __NT
#  include "ev5_routines_nt.h"
#endif

#endif // ifndef __EV5_ROUTINES_LOADED
