#ifndef __IPR_API_EXT_DEFS_LOADED
#define __IPR_API_EXT_DEFS_LOADED
/****************************************************************************/
/*                                                                          */
/*  COPYRIGHT (c) 1993-1999 BY                                              */
/*  Digital Equiptment/Compaq Computer Corporation.                         */
/*  ALL RIGHTS RESERVED.                                                    */
/*                                                                          */
/*  This file is part of the Iprobe performance suite for Alpha processors. */
/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 2 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License       */
/*  along with this program; if not, write to the Free Software             */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307    */
/*                                                                          */
/****************************************************************************/
//
//             Patent Pending, Digital Equipment Corporation
//
// Abstract:	This header file contains exported IPROBE API definitions.
//
// Author:		Brian F. Wibecan
// Creation Date:	20-Jun-1995
//
// Modified by:
//
// X-1	BFW		Brian F. Wibecan	20-Jun-1995
//	Original version.
//

#ifdef __osf__
#ifndef KERNEL
#include <fcntl.h>
#include <stdio.h>
#endif
#endif

#include "ipr_api_base_defs.h"
#include "hb_library.h"

#define IPR_ALL_CPUS    (0xffffffff)
#define IPR_CURRENT_CPU (0xfffffffe)
typedef enum
{
   IPR_HACTION_COPY = 1,
   IPR_HACTION_ADD,
   IPR_HACTION_SUM
} IPR_COPYACTION;

#define IPR_WRITE_PARTIAL 1
#define IPR_NOWRITE_PARTIAL 0

#define IPR_PCSHIFT 2

#if defined (__cplusplus)
extern "c"
{
#endif
extern IPR_STATUS IprAllocate (void);
extern IPR_STATUS IprDeallocate (void);
extern IPR_STATUS IprSetActiveCounters (
   IN IPR_CPUMASK cpus,         // CPUs affected
   IN IPR_CTRMASK validmask,    // Mask of valid counters
   IN IPR_CTRMASK *retmask);    // Returned mask for starting counters
extern IPR_STATUS IprSetSampleSkip (
   IPR_CPUMASK cpus,         // CPUs affected
   IPR_COUNT *skip_values,   // Array of skip values, one per counter
   IPR_SIZE arrsize);        // Size of the array
extern IPR_STATUS IprCreateSampleBuffers (
   IPR_CPUMASK cpus,          // Mask of desired CPUs
   IPR_COUNT bufcnt,          // Number of buffers
   IPR_SIZE bufsiz,           // Size of buffer in bytes
   IPR_DMASK datcol,          // Data to be collected
   IPR_DMASK qdatcol,         // Data to be collected
   int sampleaction,          // Action to perform in lieu of sampling
   int fullaction,            // Action to perform when buffer is full
   int (*pRoutine) (),        // Routine to be called
   IPR_COUNT rangecnt,        // Number of valid PC ranges
   uint64 *ranges);           // Valid PC ranges (start-end pairs)
extern IPR_STATUS IprAddAddressHistogramBuffers (
   IN IPR_CPUMASK cpus,
   IN IPR_HISTVALUE BeginAddr,
   IN IPR_HISTVALUE EndAddr,
   unsigned int gran,
   unsigned int index);
extern IPR_STATUS IprCreateHistogramBuffers (
   IN IPR_CPUMASK cpus,
   IN IPR_HMASK histos,
   IN IPR_HISTVALUE BeginAddr,
   IN IPR_HISTVALUE EndAddr,
   IN unsigned int gran,
   IN unsigned int index);
extern IPR_STATUS IprDeleteBuffers (IN IPR_CPUMASK cpus);
extern IPR_STATUS IprSelectEvents (
   IN IPR_CPUMASK cpus,
   IN IPR_EVENTSET *events,
   IN IPR_FREQ freq,
   IN uint64 *MUXvalue);
extern IPR_STATUS IprSelectEventsFromArray (
   IN IPR_CPUMASK cpus,
   IN const IPR_EVENT *events,
   IN const IPR_FREQ *freqs,
   IN IPR_SIZE asize,
   IN uint64 *MUXvalue);
// 10
extern IPR_STATUS IprSetEvents (
   IN IPR_CPUMASK cpus,
   IN uint64 *MUXvalue);
extern IPR_STATUS IprSetModes (
   IN IPR_CPUMASK cpus,
   IN IPR_MMASK modes);
extern IPR_STATUS IprEventCounterMap (
   IN IPR_CPUMASK cpus,
   IN IPR_EVENT *event_arr,
   IN IPR_SIZE asize);
extern IPR_STATUS IprFreqCounterMap (
   IN IPR_CPUMASK cpus,
   IN IPR_FREQ *freq_arr,
   IN IPR_SIZE asize);
extern IPR_STATUS IprCounterNumber (
   IN IPR_CPUMASK cpus,
   IN IPR_EVENT event,
   IN IPR_CTRNUMBER *cnum);
extern IPR_STATUS IprStart (
   IN IPR_CPUMASK cpus,
   IN IPR_CTRMASK ctrs);
extern IPR_STATUS IprStop (
   IN IPR_CPUMASK cpus,
   IN IPR_CTRMASK ctrs);
extern IPR_STATUS IprPause (
   IN IPR_CPUMASK cpus,
   IN IPR_CTRMASK ctrs);
extern IPR_STATUS IprResume (
   IN IPR_CPUMASK cpus,
   IN IPR_CTRMASK ctrs);
extern IPR_STATUS IprRead (
   IN IPR_CPUMASK cpus,
   IN int *cpuctxptr,
   IN IPR_CTRMASK cmask,
   OUT int *retdata,
   IN IPR_SIZE rsize,
   IN int action_flags);
//20
extern IPR_STATUS IprReadAll (
   IN IPR_CPUMASK cpus,
   IN int *cpuctxptr,
   OUT int *retdata,
   IN IPR_SIZE rsize,
   IN int action_flags);
extern IPR_STATUS IprWrite (
   IN IPR_CPUMASK cpus,
   IN int *cpuctxptr,
   IN IPR_CTRMASK cmask,
   IN int *srcdata,
   IN IPR_SIZE srcsize);
extern IPR_STATUS IprClear (
   IN IPR_CPUMASK cpus,
   IN IPR_CTRMASK ctrs);
extern IPR_STATUS IprReadHistogram (
   IN IPR_CPUMASK cpus,
   IN int *cpuctxptr,
   OUT IPR_HISTELEM *retdata,
   IN IPR_SIZE retsize,
   IN IPR_HISTVALUE retmin,
   IN IPR_HMASK htype,
   IN unsigned int index,
   IN int action_flags);
extern IPR_STATUS IprClearHistograms (
   IN IPR_CPUMASK cpus,
   IN IPR_HMASK hmask,
   IN unsigned int index);
extern IPR_STATUS IprMarkEmptySampleBuffer (OUT IPR_SBUFFER *Sptr);
extern IPR_STATUS IprNextFullSampleBuffer (
   IN IPR_CPUMASK cpus,         // CPUs to be checked
   OUT IPR_SBUFFER **bctx,      // Returned pointer and previous context
   OUT IPR_SIZE *bsize,         // Returned size of buffer
   IN IPR_BOOLEAN write_flag);  // Shows whether to check for partial buffers
extern IPR_STATUS IprClearContext (IN IPR_CPUMASK cpus);

// OS specific IPR_ROUTINES

extern IPR_STATUS IprWriteToSampleFile (
   IN void *bptr,
   IN IPR_SIZE bsize);
extern IPR_STATUS IprWriteOneSampleBuffer (
   IN IPR_SBUFFER *bptr,
   IN IPR_SIZE bsize);
//30
// General IPR_USER_ROUTINES

extern IPR_STATUS IprWriteSampleBuffers (
   IN IPR_CPUMASK cpus,     // Valid CPU mask
   IN int write_flag,       // Indicates whether to write partial buffers
   OUT int *rbufwrites,     // Optional returned number of buffers written
   OUT int *rpartwrites);   // Optional returned number of partial writes
extern IPR_STATUS IprTerminateSampling (
   OUT int *rbufwrites,
   OUT int *rpartwrites);
extern IPR_STATUS IprSpecifyOperations (
   IN IPR_CPUMASK cpus,
   IN IPR_EVENT *selected_event,
   IN IPR_FREQ *selected_frequency,
   IN IPR_COUNT ecount,
   IN IPR_MMASK selected_modes,
   OUT uint64 *MUXvalue,
   OUT IPR_CTRMASK *ctrmask);
extern IPR_STATUS IprReadMultipleHistograms (
   IN IPR_CPUMASK cpus,         // CPU mask
   IN int *cpuctxptr,           // CPU context
   OUT IPR_HISTELEM **retdata,  // Array of array pointers for return data
   OUT IPR_SIZE *retsize,       // Array of array sizes
   OUT IPR_HISTVALUE *retmin,   // Array of minimum values
   OUT IPR_HMASK *htype,        // Array of histogram types
   OUT unsigned int *index,     // Array of buffer index values
   IN int action_flags,         // Indicates how to store data
   IN IPR_COUNT itemcnt);        // Number of arrays

// OS specific IPR_USER_ROUTINES

extern IPR_STATUS IprCloseSampleFile (
   OUT int *buffwrites,
   OUT int *partwrites);

extern IPR_STATUS IprOpenSampleFile (IN char *fname, IN char *defname);

extern IPR_STATUS IprPrintMessage (IN char *tag, IN IPR_STATUS errcode,
                                   IN void (*callback)(char *));

extern const char *IprEventNumberToName (IPR_EVENT evt);

extern IPR_COUNT IprNumberOfEvents (void);

extern IPR_STATUS IprEventNameToNumber (IN char *ename, OUT IPR_EVENT *evtnum,
                                        OUT const char **matches);

extern IPR_STATUS IprSetAffinity (IN IPR_CPUMASK newmask, IN IPR_BOOLEAN flag,
                                  OUT IPR_CPUMASK *oldmask);

extern IPR_STATUS IprSetPme (IN unsigned int pid, IN IPR_BOOLEAN child);
extern IPR_STATUS IprGetPme (IN unsigned int pid, OUT IPR_BOOLEAN *value);
extern IPR_STATUS IprClearPme (IN unsigned int pid, IN IPR_BOOLEAN child);

#define CRECTRNAME1(z) Ipr ## z
#define CRECTRNAME(x) CRECTRNAME1(x)
#include "ctr_names.h"
#undef CRECTRNAME
#undef CRECTRNAME1

// OS Specific CTR_ROUTINES

#if defined (__cplusplus)
}
#endif

#endif // ifndef __IPR_API_EXT_DEFS_LOADED
