#ifndef __IPR_KERNEL_EXT_DEFS_LOADED
#define __IPR_KERNEL_EXT_DEFS_LOADED 1
/****************************************************************************/
/*                                                                          */
/*  COPYRIGHT (c) 1993-1999 BY                                              */
/*  Digital Equiptment/Compaq Computer Corporation.                         */
/*  ALL RIGHTS RESERVED.                                                    */
/*                                                                          */
/*  This file is part of the Iprobe performance suite for Alpha processors. */
/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 2 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License       */
/*  along with this program; if not, write to the Free Software             */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307    */
/*                                                                          */
/****************************************************************************/

//
//             Patent Pending, Digital Equipment Corporation
//

/*
 * Abstract:	This header file contains definitions for the IPROBE API
 *		kernel.
 *
 * Author:		Brian F. Wibecan
 * Creation Date:	10-Jul-1994
 *
 * Modified by:
 *
 * X-3	BMW1002		Bradley M. Waters	22-Sep-1994
 *	Added identifiers for write structure (WBUFFER) and
 *      deferred sampling (DBUFFER) - these structures
 *	are currently used only be NT but it seems best to maintain
 *	all buffer identifiers in one place.  The actual write strucuture
 *      is currently defined in ipr_driver_nt.h.
 *
 * X-2	BMW1001		Bradley M. Waters	18-Jul-1994
 *	Modified based on kernel rationalization.
 *
 * X-1	BFW1000		Brian F. Wibecan	08-Jul-1994
 *	Original version.
 *
*/
#include "ipr_api_base_defs.h"
#include "ipr_sample.h"

#define IPR_ACTION_EMPTY    	0
#define IPR_ACTION_FULL         1
#define IPR_ACTION_SAMPLE       2
#define IPR_ACTION_HISTOGRAM    3

// defines max. times through a pointer traverse to avoid a hang due
// to pointer corruption
#define IPR_MAX_LOOP_CNT 1000

struct _ipr_bufdsc;
typedef struct _ipr_bufdsc IPR_BUFDSC;

// This structure needs to be modified to create a linked list... - BMW
#define IPR_PCRangeGroup 20		// Number of PC range groups allocated at a time

// Some number of instances - initially IPR_PCRangeGroup instances
typedef struct _ipr_pc_range {
    uint64 StartAddress;                // Starting legal PC value
    uint64 EndAddress;                  // Ending legal PC value
    IPR_HBUFFER *pHBuf;
    unsigned sample : 1;
    unsigned histogram : 1;
    } IPR_PC_RANGE;

typedef struct _ipr_hdesc
{
      IPR_HISTVALUE minval;      // Minimum legal value
      IPR_HISTVALUE numvalues;   // Number of stored values
      IPR_COUNT elemcnt;         // Number of elements
      IPR_HMASK item;            // Item saved in this histogram
      IPR_HMASK itemfilter;
      unsigned int gran;
      unsigned int index;
} IPR_HDESC;

typedef struct _ipr_sdesc
{
    uint64 begintime;                   // Buffer started at this time
    unsigned short int size;            // Total buff bytes (incl header)
    unsigned short int filler;          // (structure alignment padding)
    unsigned int areasize;              // Size of data area in bytes
    int full;                           // Flag indicating buffer is full
    IPR_CPUNUM cpuid;                   // Current CPU ID
    IPR_DMASK datamask;                 // Data to be collected in a sample
    IPR_DMASK dataquadmask;             // Quadword data filter
    unsigned int missedcnt;             // Samples missed due to buffer full
    unsigned int areaoffset;            // Offset in bytes to next sample
} IPR_SDESC;

typedef int *  IPR_TARGET_ROUTINE;

extern IPR_STATUS KIprSetPme(IN unsigned int pid, IN int state,
                             IN IPR_BOOLEAN children,
                             OUT IPR_BOOLEAN *prev_state);
extern IPR_STATUS KIprGetCurrentIpid(OUT unsigned int *Ipid);
extern IPR_STATUS KIprBdCreate (IN IPR_CPUNUM cpu, OUT IPR_BUFDSC **retptr);
extern IPR_STATUS KIprBdDelete (IN IPR_CPUNUM cpu);
extern IPR_STATUS KIprBdSetNumCounters (IN IPR_CPUNUM cpu,
                                        IN IPR_COUNT numctrs);
extern IPR_STATUS KIprBdSetAction (IN IPR_CPUNUM cpu,
			IN int action_type, // need to define
			IN int action,
			IN int (*targetroutine) (),
			IN unsigned int targetpid);
extern IPR_STATUS KIprBdSetActiveCounters (IN IPR_CPUNUM cpu,
                                           IN IPR_CTRMASK ctrmask);
extern IPR_STATUS KIprBdAddPcRange (IN IPR_CPUNUM cpu, 
                                    IN IPR_PC_RANGE *pcrange);
extern IPR_STATUS KIprBdSetSkipCount (IN IPR_CPUNUM cpu, IN IPR_CTRNUMBER ctr, 
                                      IN int skipvalue);
extern IPR_STATUS KIprSbCreate (IN IPR_CPUNUM cpu,
			IN IPR_SIZE size,
			IN IPR_COUNT numbuffs,
			OUT IPR_SDESC *descrip,
			OUT IPR_SBUFFER **retptr);
extern IPR_STATUS KIprSbDelete (IN IPR_CPUNUM cpu, IN IPR_SBUFFER *bufptr);
extern IPR_STATUS KIprSbSetFull(IN IPR_SBUFFER *bufptr,
                                IN IPR_FULL_STATE full_val);
extern IPR_STATUS KIprSbGetInfo(IN IPR_SBUFFER *bufptr,
                                OUT IPR_SDESC *descrip);
extern IPR_STATUS KIprSbFind(IN IPR_CPUNUM cpu,
			OUT IPR_SDESC *descrip,
			IN IPR_SBUFFER *previous_buffer,
			OUT IPR_SBUFFER **retptr);

extern IPR_STATUS KIprHbCreate (IN IPR_CPUNUM cpu, OUT IPR_HDESC *descrip,
                                OUT IPR_HBUFFER **retptr);
extern IPR_STATUS KIprHbDelete (IN IPR_CPUNUM cpu, IN IPR_HBUFFER *bufptr);
extern IPR_STATUS KIprHbFind (IN IPR_CPUNUM cpu,
			IN IPR_HBUFFER *previous_buffer,
			OUT IPR_HDESC *descrip,
			OUT IPR_HBUFFER **retptr);
extern IPR_STATUS KIprHbRead(IN IPR_CPUNUM cpu,
		      IN IPR_HBUFFER *bufptr,
		      IN int start,
		      IN IPR_COUNT cnt,
		      OUT IPR_HISTELEM *retptr);
extern IPR_STATUS KIprHbClear (IN IPR_CPUNUM cpu,
			IN IPR_HBUFFER *bufptr,
			IN int start,
			IN IPR_COUNT cnt);
extern IPR_STATUS KIprHbMaxElements(OUT IPR_COUNT *elem);

#ifdef __NT
#  include "ipr_kernel_ext_nt.h"
#endif
#ifdef __osf__
#  include "ipr_kernel_ext_osf.h"
#endif
#ifdef __VMS
#  include "ipr_kernel_ext_vms.h"
#endif
#ifdef __LINUX__
#  include "ipr_kernel_ext_linux.h"
#endif



#endif // ifndef __IPR_KERNEL_EXT_DEFS_LOADED
