#ifndef __IPR_SAMPLE_LOADED
#define __IPR_SAMPLE_LOADED
/****************************************************************************/
/*                                                                          */
/*  COPYRIGHT (c) 1993-1999 BY                                              */
/*  Digital Equiptment/Compaq Computer Corporation.                         */
/*  ALL RIGHTS RESERVED.                                                    */
/*                                                                          */
/*  This file is part of the Iprobe performance suite for Alpha processors. */
/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 2 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License       */
/*  along with this program; if not, write to the Free Software             */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307    */
/*                                                                          */
/****************************************************************************/
//
//             Patent Pending, Digital Equipment Corporation
//
// Abstract:	This header file contains IPROBE API sampling data structures.
//
// Author:		Brian F. Wibecan
// Creation Date:	20-Jun-1995
//
// Modified by:
//
// X-1	BFW		Brian F. Wibecan	20-Jun-1995
//	Original version.
//

#include "ipr_api_base_defs.h"
// All the buffer types are 19, so they look like buffered I/O on VMS

#define IPR_SBUFFER_TYPE 19
#define IPR_HBUFFER_TYPE 19
#define IPR_BUFDSC_TYPE 19
#define IPR_PBUFFER_TYPE 19
#define IPR_IBUFFER_TYPE 19
#define IPR_WBUFFER_TYPE 19
#define IPR_DBUFFER_TYPE 19

// The subtypes are all zero, as well

#define IPR_SBUFFER_SUBTYPE 0
#define IPR_HBUFFER_SUBTYPE 0
#define IPR_BUFDSC_SUBTYPE 0
#define IPR_PBUFFER_SUBTYPE 0
#define IPR_IBUFFER_SUBTYPE 0
#define IPR_WBUFFER_SUBTYPE 0
#define IPR_DBUFFER_SUBTYPE 0

// Differentiate with the "kind" field

#define IPR_SBUFFER_KIND 101
#define IPR_HBUFFER_KIND 102
#define IPR_BUFDSC_KIND 103
#define IPR_PBUFFER_KIND 104
#define IPR_IBUFFER_KIND 105
#define IPR_WBUFFER_KIND 106
#define IPR_DBUFFER_KIND 107

//
// The IPR_PROD_IDENT definitions below will create the string " IPROBE " in
// an eight-byte integer field.
//
#define IPR_PROD_IDENT 0x2045424F52504920

typedef enum
{
   IPR_BUFFER_EMPTY,
   IPR_BUFFER_FULL,
   IPR_BUFFER_PART_FULL } IPR_FULL_STATE;

typedef struct _ipr_bufhdr {
//
// Note: prod_ident is uint64 to force alignment on a quadword boundary
//
    uint64 prod_ident;              // Product identification
    unsigned short int size;        // Buffer size in bytes
    unsigned char type;             // Buffer type
    unsigned char subtype;          // Buffer subtype
    unsigned int bufkind;           // Internal-use buffer ident
} IPR_BUFHDR;

struct _ipr_sbuffer {
    IPR_BUFHDR header;
    uint64 begintime;                   // Buffer started at this time
    struct _ipr_sbuffer *pNext;         // Next buffer in the ring
    unsigned int areasize;              // Size of data area in bytes
    IPR_FULL_STATE full;                // Flag indicating buffer is full
    IPR_CPUNUM cpuid;                   // Current CPU ID
    IPR_DMASK data;                     // Data to be collected in a sample
    IPR_DMASK dataquad;                 // Quadword data filter
    unsigned int missedcnt;             // Samples missed due to buffer full
    unsigned int areaoffset;            // Offset in bytes to next sample
    unsigned int pad;			// padding required (in bytes) for each sample (so
    					// samples containing quadwords are at quad boundaries)
    void *wrtQEntry;			// Optional entry for the write queue (used by NT)
    };

#endif // ifndef __IPR_SAMPLE_LOADED
