/****************************************************************************/
/*									    */
/*  COPYRIGHT (c) 1993-1999 BY			                            */
/*  Digital Equiptment/Compaq Computer Corporation.       		    */
/*  ALL RIGHTS RESERVED.						    */
/*                                                                          */
/*  This file is part of the Iprobe performance suite for Alpha processors. */
/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 2 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License       */
/*  along with this program; if not, write to the Free Software             */
/*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307    */
/*									    */
/****************************************************************************/

#include "hb_library.h"

/*****************************************************************************/
/* Name: IprHbIndex                                                          */
/*                                                                           */
/* Purpose: Given a value, this will tell under what index that value will   */
/*          fall in the histogram.                                           */
/*                                                                           */
/*                                                                           */
/* Arguments:  value: The number to be examined.                             */
/*	       ctr: (??) Current event counter number                        */
/*	       numctrs: The number of counters that this buffer represents.  */
/*	       minval: This is the minimum value of the histogram.           */
/*	       gran: This is the granularity of the histogram.               */
/*	       index: This is the index into the histogram for the given value.*/
/* */
/* Output:  index: This is the index under which the given value would fall.*/
/**/
/* Function Value: This contains whether or not the call succeeded or failed.*/
/*                                                                           */
/*****************************************************************************/



extern IPR_STATUS IprHbIndex (IPR_HISTVALUE value,
		IPR_CTRNUMBER ctr,
		IPR_COUNT numctrs,
		IPR_HISTVALUE minval,
		int gran,
		int *index)
{
   IPR_HISTVALUE
      tmpidx;

   if (!index)
      return IPR__NULLPTR;
    if (value < minval)
      return IPR__NOTINHIST;
   tmpidx = ((((value - minval) >> gran) * numctrs) + ctr);
   *index = (int) tmpidx;
   if (*index < 0 || *index != tmpidx)
      return IPR__NOTINHIST;
   return IPR__SUCCESS;
}

extern IPR_STATUS IprHbValue (int index,
		IPR_COUNT numctrs,
		IPR_HISTVALUE minval,
		int gran,
		IPR_HISTVALUE *value)
{
   IPR_HISTVALUE
      tval,
      tval2;

   if (!value)
      return IPR__NULLPTR;

   // Get the first value into a 64-bit form so as to avoid truncation
   // later.

   tval = index / numctrs;
   tval2 = tval << gran;

   // Check for overflow on the shift.

   if (tval2 >> gran != tval)
      return IPR__HISTOVFLW;

   // Check for overflow on the addition.

   tval = tval2 + minval;
   if (tval < minval)
      return IPR__HISTOVFLW;

   // All done, return result.

   *value = tval;
   return IPR__SUCCESS;
}

extern IPR_STATUS IprHbElements (IPR_COUNT numctrs,
		IPR_HISTVALUE numval,
		int gran,
		IPR_COUNT *numelem)
{
   IPR_HISTVALUE
      tmpelems;

   if (!numelem)
      return IPR__NULLPTR;
   tmpelems = (((numval - 1) >> gran) + 1) * numctrs;
   *numelem = (IPR_COUNT) tmpelems;
   if (*numelem != tmpelems)
      return IPR__HISTOVFLW;
   return IPR__SUCCESS;
}

/*
 *
 *
 */

extern IPR_STATUS IprHbMaxValue (int index,
		IPR_COUNT numctrs,
		IPR_HISTVALUE minval,
		IPR_HISTVALUE numvals,
		int gran,
		IPR_HISTVALUE *hival)
{
   IPR_HISTVALUE tval;
   IPR_HISTVALUE maxidx;

   if (!hival)
      return IPR__NULLPTR;

   // Verify that index is within histogram bounds

   if ((numvals >> gran) < (index / numctrs))
      return IPR__HISTOVFLW;

   // Check for badly constructed histogram

   if (minval + numvals - 1 < minval)
      return IPR__HISTOVFLW;

   // First do the division and get into 64-bit format

   tval = (index / numctrs) + 1;

   // Next do the shift and addition

   tval = minval + (tval << gran) - 1;

   // Save the result

   *hival = tval;

   // Bound the result

   if (*hival > (minval + numvals - 1) || *hival < minval)
      *hival = minval + numvals - 1;

   // All done

   return IPR__SUCCESS;
}
